@@ -80,9 +80,10 @@ function getVersionFromLog()
 function printHelp()
 {
   echo "Usage:"                                                1>&2
-  echo "${0} [--help|--print-only] [SOURCEPATH]"               1>&2
+  echo "${0} [--help|--print-only|--version] [SOURCEPATH]"     1>&2
   echo "  --help       prints this message"                    1>&2
   echo "  --print-only prints the version to stdout and quits" 1>&2
+  echo "  --version    VERSION sets the version manually"      1>&2 
 }
 
 #-------------------------------------------------------------------------------
@@ -93,6 +94,13 @@ while test ${#} -ne 0; do
     PRINTHELP=1
   elif test x${1} = x--print-only; then
     PRINTONLY=1
+  elif test x${1} = x--version; then
+    if test ${#} -lt 2; then
+      echo "--version parameter needs an argument" 1>&2
+      exit 1
+    fi
+    USER_VERSION=${2}
+    shift
   else
     SOURCEPATH=${1}
   fi
@@ -122,7 +130,10 @@ if test ! -d ${SOURCEPATH}.git; then
   # We cannot figure out what version we are
   #----------------------------------------------------------------------------
   echo "[I] No git repository info found. Trying to interpret VERSION_INFO" 1>&2
-  if test ! -r ${SOURCEPATH}VERSION_INFO; then
+  if test -f src/XrdVersion.hh; then
+    echo "[I] The XrdVersion.hh file already exists" 1>&2
+    exit 0
+  elif test ! -r ${SOURCEPATH}VERSION_INFO; then
     echo "[!] VERSION_INFO file absent. Unable to determine the version. Using \"unknown\"" 1>&2
   elif test x"`grep Format ${SOURCEPATH}VERSION_INFO`" != x; then
     echo "[!] VERSION_INFO file invalid. Unable to determine the version. Using \"unknown\"" 1>&2
@@ -143,6 +154,12 @@ if test ! -d ${SOURCEPATH}.git; then
     fi
   fi
 
+#-------------------------------------------------------------------------------
+# Check if the version has been specified by the user
+#-------------------------------------------------------------------------------
+elif test x$USER_VERSION != x; then
+  VERSION=$USER_VERSION
+
 #-------------------------------------------------------------------------------
 # We're in a git repo so we can try to determine the version using that
 #-------------------------------------------------------------------------------
@@ -184,7 +201,7 @@ fi
 #-------------------------------------------------------------------------------
 # Make sure the version string is not longer than 25 characters
 #-------------------------------------------------------------------------------
-if test ${#VERSION} -gt 25; then
+if [ ${#VERSION} -gt 25 ] && [ x$USER_VERSION == x ] ; then
   VERSION="${VERSION:0:19}...${VERSION: -3}"
 fi
 
