@@ -25,14 +25,15 @@ function findProg()
 #-------------------------------------------------------------------------------
 function printHelp()
 {
-  echo "Usage:"                                              1>&2
-  echo "${0} [--help] [--source PATH] [--output PATH]"       1>&2
-  echo "  --help        prints this message"                 1>&2
-  echo "  --source PATH specify the root of the source tree" 1>&2
-  echo "                defaults to ../"                     1>&2
-  echo "  --output PATH the directory where the source rpm"  1>&2
-  echo "                should be stored, defaulting to ."   1>&2
-  echo "  --version VERSION the version provided by user"    1>&2
+  echo "Usage:"                                               1>&2
+  echo "${0} [--help] [--source PATH] [--output PATH]"        1>&2
+  echo "  --help        prints this message"                  1>&2
+  echo "  --source  PATH specify the root of the source tree" 1>&2
+  echo "                defaults to ../"                      1>&2
+  echo "  --output  PATH the directory where the source rpm"  1>&2
+  echo "                should be stored, defaulting to ."    1>&2
+  echo "  --version VERSION the version provided by user"     1>&2
+  echo "  --define  'MACRO EXPR'"                             1>&2
 }
 
 #-------------------------------------------------------------------------------
@@ -66,6 +67,19 @@ while test ${#} -ne 0; do
     fi
     USER_VERSION="--version ${2}"
     shift
+  elif test x${1} = x--define; then
+    if test ${#} -lt 2; then
+      echo "--define parameter needs an argument" 1>&2
+      exit 1
+    fi
+    USER_DEFINE="$USER_DEFINE --define \""${2}"\""
+    shift
+  elif test x${1} = x-D; then
+    if test ${#} -lt 2; then
+      echo "-D parameter needs an argument" 1>&2
+      exit 1
+    fi
+    USER_D="$USER_D -D \""${2}"\""
   fi
   shift
 done
@@ -213,14 +227,81 @@ if test $? -ne 0; then
   exit 5
 fi
 
-git archive --prefix=xrootd/ --format=tar $COMMIT | gzip -9fn > \
-      $RPMSOURCES/xrootd.tar.gz
+git describe --tags --abbrev=0 --exact-match >/dev/null 2>&1
+if test ${?} -eq 0; then
+  TAG="`git describe --tags --abbrev=0 --exact-match`"
+fi
+
+git archive --prefix=xrootd/ --format=tar $COMMIT > $RPMSOURCES/xrootd.tar
 
 if test $? -ne 0; then
   echo "[!] Unable to create the source tarball" 1>&2
   exit 6
 fi
 
+#-------------------------------------------------------------------------------
+# Make sure submodules are in place
+#-------------------------------------------------------------------------------
+git submodule init
+git submodule update -- src/XrdClHttp
+git submodule update -- src/XrdCeph
+#git submodule foreach git pull origin master
+
+#-------------------------------------------------------------------------------
+# Add XrdClHttp sub-module to our tarball
+#-------------------------------------------------------------------------------
+cd src/XrdClHttp
+
+if [ -z ${TAG+x} ]; then
+  COMMIT=`git log --pretty=format:"%H" -1`
+else
+  COMMIT=$TAG
+fi
+
+git archive --prefix=xrootd/src/XrdClHttp/ --format=tar $COMMIT > $RPMSOURCES/xrdcl-http.tar
+if test $? -ne 0; then
+  echo "[!] Unable to create the xrdcl-http source tarball" 1>&2
+  exit 6
+fi
+
+tar --concatenate --file $RPMSOURCES/xrootd.tar $RPMSOURCES/xrdcl-http.tar
+if test $? -ne 0; then
+  echo "[!] Unable to add xrdcl-http to xrootd tarball" 1>&2
+  exit 6
+fi
+
+cd - > /dev/null
+
+#-------------------------------------------------------------------------------
+# Add XrdCeph sub-module to our tarball
+#-------------------------------------------------------------------------------
+cd src/XrdCeph
+
+if [ -z ${TAG+x} ]; then
+  COMMIT=`git log --pretty=format:"%H" -1`
+else
+  COMMIT=$TAG
+fi
+
+git archive --prefix=xrootd/src/XrdCeph/ --format=tar $COMMIT > $RPMSOURCES/xrootd-ceph.tar
+if test $? -ne 0; then
+  echo "[!] Unable to create the xrootd-ceph source tarball" 1>&2
+  exit 6
+fi
+
+tar --concatenate --file $RPMSOURCES/xrootd.tar $RPMSOURCES/xrootd-ceph.tar
+if test $? -ne 0; then
+  echo "[!] Unable to add xrootd-ceph to xrootd tarball" 1>&2
+  exit 6
+fi
+
+cd - > /dev/null
+
+#-------------------------------------------------------------------------------
+# gzip the tarball
+#-------------------------------------------------------------------------------
+gzip -9fn $RPMSOURCES/xrootd.tar
+
 #-------------------------------------------------------------------------------
 # Check if we need some other versions
 #-------------------------------------------------------------------------------
@@ -249,12 +330,14 @@ echo "[i] Creating the source RPM..."
 
 # Dirty, dirty hack!
 echo "%_sourcedir $RPMSOURCES" >> $TEMPDIR/rpmmacros
-rpmbuild --define "_topdir $TEMPDIR/rpmbuild"    \
-         --define "%_sourcedir $RPMSOURCES"      \
-         --define "%_srcrpmdir %{_topdir}/SRPMS" \
-         --define "_source_filedigest_algorithm md5" \
-         --define "_binary_filedigest_algorithm md5" \
-  -bs $TEMPDIR/xrootd.spec > $TEMPDIR/log
+eval "rpmbuild --define \"_topdir $TEMPDIR/rpmbuild\"    \
+               --define \"%_sourcedir $RPMSOURCES\"      \
+               --define \"%_srcrpmdir %{_topdir}/SRPMS\" \
+               --define \"_source_filedigest_algorithm md5\" \
+               --define \"_binary_filedigest_algorithm md5\" \
+               ${USER_DEFINE} \
+               ${USER_D} \
+               -bs $TEMPDIR/xrootd.spec > $TEMPDIR/log"
 if test $? -ne 0; then
   echo "[!] RPM creation failed" 1>&2
   exit 8
