@@ -28,9 +28,11 @@
 /******************************************************************************/
 
 #include <errno.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <stdio.h>
 #include <sys/resource.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #ifdef __APPLE__
@@ -40,6 +42,7 @@
 #include "Xrd/XrdJob.hh"
 #include "Xrd/XrdScheduler.hh"
 #include "XrdSys/XrdSysError.hh"
+#include "XrdSys/XrdSysLogger.hh"
 
 #define XRD_TRACE XrdTrace->
 #include "Xrd/XrdTrace.hh"
@@ -95,50 +98,95 @@ XrdScheduler::XrdScheduler(XrdSysError *eP, XrdOucTrace *tP,
               : XrdJob("underused thread monitor"),
                 WorkAvail(0, "sched work")
 {
-    struct rlimit rlim;
 
-    XrdLog      =  eP;
-    XrdTrace    =  tP;
-    min_Workers =  minw;
-    max_Workers =  maxw;
-    max_Workidl =  maxi;
-    num_Workers =  0;
-    num_JobsinQ =  0;
-    stk_Workers =  maxw - (maxw/4*3);
-    idl_Workers =  0;
-    num_Jobs    =  0;
-    max_QLength =  0;
-    num_TCreate =  0;
-    num_TDestroy=  0;
-    num_Layoffs =  0;
-    num_Limited =  0;
-    firstPID    =  0;
-    WorkFirst = WorkLast = TimerQueue = 0;
+// Perform common initialization
+//
+   XrdLog      =  eP;
+   XrdTrace    =  tP;
+   Init(minw, maxw, maxi);
 
 // Make sure we are using the maximum number of threads allowed (Linux only)
 //
 #if defined(__linux__) && defined(RLIMIT_NPROC)
 
+   struct rlimit rlim;
+
+// First determine the absolute maximum we can have
+//
+   rlim_t theMax = MAX_SCHED_PROCS;
+   int pdFD, rdsz;
+   if ((pdFD = open("/proc/sys/kernel/pid_max", O_RDONLY)) >= 0)
+      {char pmBuff[32];
+       if ((rdsz = read(pdFD, pmBuff, sizeof(pmBuff))) > 0)
+          {rdsz = atoi(pmBuff);
+           if (rdsz < 16384) theMax = 16384; // This is unlikely
+              else if (rdsz < MAX_SCHED_PROCS)
+                      theMax = static_cast<rlim_t>(rdsz-2000);
+          }
+       close(pdFD);
+      }
+
 // Get the resource thread limit and set to maximum. In Linux this may be -1
 // to indicate useless infnity, so we have to come up with a number, sigh.
 //
    if (!getrlimit(RLIMIT_NPROC, &rlim))
-      {     if (rlim.rlim_max == RLIM_INFINITY) rlim.rlim_cur = MAX_SCHED_PROCS;
-       else if (rlim.rlim_cur  < rlim.rlim_max) rlim.rlim_cur = rlim.rlim_max;
-       if (rlim.rlim_cur != rlim.rlim_max) setrlimit(RLIMIT_NPROC, &rlim);
+      {if (rlim.rlim_max == RLIM_INFINITY || rlim.rlim_max > theMax)
+          {rlim.rlim_cur = theMax;
+           setrlimit(RLIMIT_NPROC, &rlim);
+          } else {
+           if (rlim.rlim_cur != rlim.rlim_max)
+              {rlim.rlim_cur = rlim.rlim_max;
+               setrlimit(RLIMIT_NPROC, &rlim);
+              }
+          }
       }
 
 // Readjust our internal maximum to be the actual maximum
 //
    if (!getrlimit(RLIMIT_NPROC, &rlim))
-      {if (rlim.rlim_cur == RLIM_INFINITY || rlim.rlim_cur > MAX_SCHED_PROCS)
-          max_Workers = MAX_SCHED_PROCS;
+      {if (rlim.rlim_cur == RLIM_INFINITY || rlim.rlim_cur > theMax)
+               max_Workers = static_cast<int>(theMax);
           else max_Workers = static_cast<int>(rlim.rlim_cur);
       }
 #endif
 
 }
  
+/******************************************************************************/
+
+// This constructor creates a self contained scheduler.
+//
+XrdScheduler::XrdScheduler(int minw, int maxw, int maxi)
+              : XrdJob("underused thread monitor"),
+                WorkAvail(0, "sched work")
+{
+   XrdSysLogger *Logger;
+   int eFD;
+
+// Get a file descriptor mirroring standard error
+//
+#if defined(__linux__) && defined(O_CLOEXEC)
+   eFD = fcntl(STDERR_FILENO, F_DUPFD_CLOEXEC, 0);
+#else
+   eFD = dup(STDERR_FILENO);
+   fcntl(eFD, F_SETFD, FD_CLOEXEC);
+#endif
+
+// Now we need to get a logger object. We make this a real dumb one.
+//
+   Logger = new XrdSysLogger(eFD, 0);
+   XrdLog = new XrdSysError(Logger);
+
+// Now get a trace object
+//
+   XrdTrace = new XrdOucTrace(XrdLog);
+   if (getenv("XRDDEBUG") != 0) XrdTrace->What = TRACE_SCHED;
+
+// Set remaining values. We do no use maximum possible threads here.
+//
+   Init(minw, maxw, maxi);
+}
+
 /******************************************************************************/
 /*                            D e s t r u c t o r                             */
 /******************************************************************************/
@@ -625,6 +673,29 @@ void XrdScheduler::hireWorker(int dotrace)
       } else if (dotrace) TRACE(SCHED, "Now have " <<num_Workers <<" workers" );
 }
  
+/******************************************************************************/
+/*                                  I n i t                                   */
+/******************************************************************************/
+
+void XrdScheduler::Init(int minw, int maxw, int maxi)
+{
+   min_Workers =  minw;
+   max_Workers =  maxw;
+   max_Workidl =  maxi;
+   num_Workers =  0;
+   num_JobsinQ =  0;
+   stk_Workers =  maxw - (maxw/4*3);
+   idl_Workers =  0;
+   num_Jobs    =  0;
+   max_QLength =  0;
+   num_TCreate =  0;
+   num_TDestroy=  0;
+   num_Layoffs =  0;
+   num_Limited =  0;
+   firstPID    =  0;
+   WorkFirst = WorkLast = TimerQueue = 0;
+}
+
 /******************************************************************************/
 /*                             t r a c e E x i t                              */
 /******************************************************************************/
