@@ -275,6 +275,9 @@ int XrdAccConfig::ConfigFile(XrdSysError &Eroute, const char *ConfigFN) {
 // Now start reading records until eof.
 //
    ConfigDefaults(); Config.Attach(cfgFD); Config.Tabs(0);
+   static const char *cvec[] = { "*** acc plugin config:", 0 };
+   Config.Capture(cvec);
+
    while((var = Config.GetMyFirstWord()))
         {if (!strncmp(var, "acc.", 2))
             {recs++;
