@@ -24,6 +24,7 @@
 #include "XrdCl/XrdClXRootDTransport.hh"
 #include "XrdCl/XrdClXRootDMsgHandler.hh"
 #include "XrdCl/XrdClOptimizers.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include <netinet/tcp.h>
 
 namespace XrdCl
@@ -31,7 +32,8 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Constructor
   //----------------------------------------------------------------------------
-  AsyncSocketHandler::AsyncSocketHandler( Poller           *poller,
+  AsyncSocketHandler::AsyncSocketHandler( const URL        &url,
+                                          Poller           *poller,
                                           TransportHandler *transport,
                                           AnyObject        *channelData,
                                           uint16_t          subStreamNum ):
@@ -54,7 +56,9 @@ namespace XrdCl
     pOutMsgDone( false ),
     pOutHandler( 0 ),
     pIncMsgSize( 0 ),
-    pOutMsgSize( 0 )
+    pOutMsgSize( 0 ),
+    pUrl( url ),
+    pTlsHandShakeOngoing( false )
   {
     Env *env = DefaultEnv::GetEnv();
 
@@ -81,7 +85,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Connect to given address
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::Connect( time_t timeout )
+  XRootDStatus AsyncSocketHandler::Connect( time_t timeout )
   {
     Log *log = DefaultEnv::GetLog();
     pLastActivity = pConnectionStarted = ::time(0);
@@ -90,7 +94,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Initialize the socket
     //--------------------------------------------------------------------------
-    Status st = pSocket->Initialize( pSockAddr.Family() );
+    XRootDStatus st = pSocket->Initialize( pSockAddr.Family() );
     if( !st.IsOK() )
     {
       log->Error( AsyncSockMsg, "[%s] Unable to initialize socket: %s",
@@ -108,8 +112,8 @@ namespace XrdCl
     env->GetInt( "TCPKeepAlive", keepAlive );
     if( keepAlive )
     {
-      int    param = 1;
-      Status st    = pSocket->SetSockOpt( SOL_SOCKET, SO_KEEPALIVE, &param,
+      int          param = 1;
+      XRootDStatus st    = pSocket->SetSockOpt( SOL_SOCKET, SO_KEEPALIVE, &param,
                                           sizeof(param) );
       if( !st.IsOK() )
         log->Error( AsyncSockMsg, "[%s] Unable to turn on keepalive: %s",
@@ -167,31 +171,31 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( !pPoller->AddSocket( pSocket, this ) )
     {
-      Status st( stFatal, errPollerError );
+      XRootDStatus st( stFatal, errPollerError );
       pSocket->Close();
       return st;
     }
 
     if( !pPoller->EnableWriteNotification( pSocket, true, pTimeoutResolution ) )
     {
-      Status st( stFatal, errPollerError );
+      XRootDStatus st( stFatal, errPollerError );
       pPoller->RemoveSocket( pSocket );
       pSocket->Close();
       return st;
     }
 
-    return Status();
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
   // Close the connection
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::Close()
+  XRootDStatus AsyncSocketHandler::Close()
   {
     Log *log = DefaultEnv::GetLog();
     log->Debug( AsyncSockMsg, "[%s] Closing the socket", pStreamName.c_str() );
 
-    pTransport->Disconnect( *pChannelData, pStream->GetStreamNumber(),
+    pTransport->Disconnect( *pChannelData,
                             pSubStreamNum );
 
     pPoller->RemoveSocket( pSocket );
@@ -201,7 +205,7 @@ namespace XrdCl
       delete pIncoming;
 
     pIncoming = 0;
-    return Status();
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
@@ -212,7 +216,6 @@ namespace XrdCl
     pStream    = stream;
     std::ostringstream o;
     o << pStream->GetURL()->GetHostId();
-    o << " #" << pStream->GetStreamNumber();
     o << "." << pSubStreamNum;
     pStreamName = o.str();
   }
@@ -222,13 +225,22 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void AsyncSocketHandler::Event( uint8_t type, XrdCl::Socket */*socket*/ )
   {
+//    //--------------------------------------------------------------------------
+//    // First check if the socket itself wants to apply some mapping on the
+//    // event. E.g. in case of TLS socket it might want to map read events to
+//    // write events and vice-versa.
+//    //--------------------------------------------------------------------------
+    type = pSocket->MapEvent( type );
+
     //--------------------------------------------------------------------------
     // Read event
     //--------------------------------------------------------------------------
     if( type & ReadyToRead )
     {
       pLastActivity = time(0);
-      if( likely( pHandShakeDone ) )
+      if( unlikely( pTlsHandShakeOngoing ) )
+        OnTLSHandShake();
+      else if( likely( pHandShakeDone ) )
         OnRead();
       else
         OnReadWhileHandshaking();
@@ -253,6 +265,8 @@ namespace XrdCl
       pLastActivity = time(0);
       if( unlikely( pSocket->GetStatus() == Socket::Connecting ) )
         OnConnectionReturn();
+      else if( unlikely( pTlsHandShakeOngoing ) )
+        OnTLSHandShake();
       else if( likely( pHandShakeDone ) )
         OnWrite();
       else
@@ -285,7 +299,7 @@ namespace XrdCl
 
     int errorCode = 0;
     socklen_t optSize = sizeof( errorCode );
-    Status st = pSocket->GetSockOpt( SOL_SOCKET, SO_ERROR, &errorCode,
+    XRootDStatus st = pSocket->GetSockOpt( SOL_SOCKET, SO_ERROR, &errorCode,
                                      &optSize );
 
     //--------------------------------------------------------------------------
@@ -296,9 +310,9 @@ namespace XrdCl
     {
       log->Error( AsyncSockMsg, "[%s] Unable to get the status of the "
                   "connect operation: %s", pStreamName.c_str(),
-                  strerror( errno ) );
+                  XrdSysE2T( errno ) );
       pStream->OnConnectError( pSubStreamNum,
-                               Status( stFatal, errSocketOptError, errno ) );
+                               XRootDStatus( stFatal, errSocketOptError, errno ) );
       return;
     }
 
@@ -308,18 +322,27 @@ namespace XrdCl
     if( errorCode )
     {
       log->Error( AsyncSockMsg, "[%s] Unable to connect: %s",
-                  pStreamName.c_str(), strerror( errorCode ) );
+                  pStreamName.c_str(), XrdSysE2T( errorCode ) );
       pStream->OnConnectError( pSubStreamNum,
-                               Status( stError, errConnectionError ) );
+                               XRootDStatus( stError, errConnectionError ) );
       return;
     }
     pSocket->SetStatus( Socket::Connected );
 
+    //--------------------------------------------------------------------------
+    // Cork the socket
+    //--------------------------------------------------------------------------
+    st = pSocket->Cork();
+    if( !st.IsOK() )
+    {
+      pStream->OnConnectError( pSubStreamNum, st );
+      return;
+    }
+
     //--------------------------------------------------------------------------
     // Initialize the handshake
     //--------------------------------------------------------------------------
     pHandShakeData = new HandShakeData( pStream->GetURL(),
-                                        pStream->GetStreamNumber(),
                                         pSubStreamNum );
     pHandShakeData->serverAddr = &pSocket->GetServerAddress();
     pHandShakeData->clientName = pSocket->GetSockName();
@@ -351,7 +374,7 @@ namespace XrdCl
     if( !pPoller->EnableReadNotification( pSocket, true, pTimeoutResolution ) )
     {
       pStream->OnConnectError( pSubStreamNum,
-                               Status( stFatal, errPollerError ) );
+                               XRootDStatus( stFatal, errPollerError ) );
       return;
     }
   }
@@ -381,22 +404,21 @@ namespace XrdCl
       // Secure the message if necessary
       //------------------------------------------------------------------------
       delete pSignature; pSignature = 0;
-      XRootDStatus st = GetSignature( pOutgoing, pSignature );
+      XRootDStatus st = pTransport->GetSignature( pOutgoing, pSignature, *pChannelData );
       if( !st.IsOK() )
       {
         OnFault( st );
         return;
       }
+
+      if( pSignature )
+        pOutMsgSize += pSignature->GetSize();
     }
 
     //--------------------------------------------------------------------------
-    // Try to write everything at once: signature, request and raw data
-    // (this is only supported if pOutHandler is an instance of XRootDMsgHandler)
+    // Write everything at once: signature, request and raw data
     //--------------------------------------------------------------------------
-    Status st = WriteMessageAndRaw( pOutgoing, pSignature );
-    if( !st.IsOK() && st.code == errNotSupported )   //< this part should go away
-      st = WriteSeparately( pOutgoing, pSignature ); //< once we can add GetMsgBody
-                                                     //< to OutgoingMsgHandler interface !!!
+    XRootDStatus st = WriteMessageAndRaw( pOutgoing, pSignature );
     if( !st.IsOK() )
     {
       OnFault( st );
@@ -425,7 +447,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void AsyncSocketHandler::OnWriteWhileHandshaking()
   {
-    Status st;
+    XRootDStatus st;
     if( !pHSOutgoing )
     {
       if( !(st = DisableUplink()).IsOK() )
@@ -439,20 +461,28 @@ namespace XrdCl
       return;
     }
 
-    if( st.code != suRetry )
+    if( st.code == suRetry ) return;
+
+    delete pHSOutgoing;
+    pHSOutgoing = 0;
+
+    st = pSocket->Flash();
+    if( !st.IsOK() )
     {
-      delete pHSOutgoing;
-      pHSOutgoing = 0;
-      if( !(st = DisableUplink()).IsOK() )
-        OnFaultWhileHandshaking( st );
-      return;
+      Log *log = DefaultEnv::GetLog();
+      log->Error( AsyncSockMsg, "[%s] Unable to flash the socket: %s",
+                  pStreamName.c_str(), XrdSysE2T( st.errNo ) );
+      OnFaultWhileHandshaking( st );
     }
+
+    if( !(st = DisableUplink()).IsOK() )
+      OnFaultWhileHandshaking( st );
   }
 
   //----------------------------------------------------------------------------
   // Write the current message
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::WriteCurrentMessage( Message *toWrite )
+  XRootDStatus AsyncSocketHandler::WriteCurrentMessage( Message *toWrite )
   {
     Log *log = DefaultEnv::GetLog();
 
@@ -460,28 +490,23 @@ namespace XrdCl
     // Try to write down the current message
     //--------------------------------------------------------------------------
     Message  *msg             = toWrite;
-    uint32_t  leftToBeWritten = msg->GetSize()-msg->GetCursor();
+    size_t    leftToBeWritten = msg->GetSize()-msg->GetCursor();
 
     while( leftToBeWritten )
     {
-      int status = pSocket->Send( msg->GetBufferAtCursor(), leftToBeWritten );
-      if( status <= 0 )
-      {
-        //----------------------------------------------------------------------
-        // Writing operation would block! So we are done for now, but we will
-        // return
-        //----------------------------------------------------------------------
-        if( errno == EAGAIN || errno == EWOULDBLOCK )
-          return Status( stOK, suRetry );
+      int bytesWritten = 0;
+      XRootDStatus st = pSocket->Send( msg->GetBufferAtCursor(), leftToBeWritten, bytesWritten );
 
-        //----------------------------------------------------------------------
-        // Actual socket error error!
-        //----------------------------------------------------------------------
+      if( !st.IsOK() )
+      {
         toWrite->SetCursor( 0 );
-        return Status( stError, errSocketError, errno );
+        return st;
       }
-      msg->AdvanceCursor( status );
-      leftToBeWritten -= status;
+
+      if( st.code == suRetry ) return st;
+
+      msg->AdvanceCursor( bytesWritten );
+      leftToBeWritten -= bytesWritten;
     }
 
     //--------------------------------------------------------------------------
@@ -490,173 +515,47 @@ namespace XrdCl
     log->Dump( AsyncSockMsg, "[%s] Wrote a message: %s (0x%x), %d bytes",
                pStreamName.c_str(), toWrite->GetDescription().c_str(),
                toWrite, toWrite->GetSize() );
-    return Status();
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
   // Write the message and its signature
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::WriteVMessage( Message   *toWrite,
-                                            Message   *&sign,
-                                            ChunkList *chunks,
-                                            uint32_t  *asyncOffset )
+  XRootDStatus AsyncSocketHandler::WriteMessageAndRaw( Message *toWrite, Message *&sign )
   {
-    if( !sign && !chunks ) return WriteCurrentMessage( toWrite );
-
-    Log *log = DefaultEnv::GetLog();
-
-    const size_t iovcnt = 1 + ( sign ? 1 : 0 ) + ( chunks ? chunks->size() : 0 );
-    iovec iov[iovcnt];
-    uint32_t leftToBeWritten = 0;
-    size_t i = 0;
+    XRootDStatus st;
 
     if( sign )
     {
-      ToIov( *sign, iov[i] );
-      leftToBeWritten += iov[i].iov_len;
-      ++i;
+      st = WriteCurrentMessage( sign );
+      if( !st.IsOK() || st.code == suRetry )
+        return st;
     }
 
-    ToIov( *toWrite, iov[i] );
-    leftToBeWritten += iov[i].iov_len;
-    ++i;
-
-    uint32_t rawSize = 0;
-    if( chunks )
-    {
-      rawSize = ToIov( chunks, asyncOffset, iov + i );
-      leftToBeWritten += rawSize;
-    }
-
-    while( leftToBeWritten )
-    {
-      int bytesWritten = pSocket->WriteV( iov, iovcnt );
-      if( bytesWritten <= 0 )
-      {
-        //----------------------------------------------------------------------
-        // Writing operation would block! So we are done for now, but we will
-        // return
-        //----------------------------------------------------------------------
-        if( errno == EAGAIN || errno == EWOULDBLOCK )
-          return Status( stOK, suRetry );
-
-        //----------------------------------------------------------------------
-        // Actual socket error error!
-        //----------------------------------------------------------------------
-        if( sign ) sign->SetCursor( 0 );
-        toWrite->SetCursor( 0 );
-        return Status( stError, errSocketError, errno );
-      }
-
-      leftToBeWritten -= bytesWritten;
-      if( sign )
-        UpdateAfterWrite( *sign, iov[0], bytesWritten );
-
-      i = sign ? 1 : 0;
-      UpdateAfterWrite( *toWrite, iov[i], bytesWritten );
-
-      if( chunks && asyncOffset )
-        UpdateAfterWrite( chunks, asyncOffset, iov + i + 1, bytesWritten );
-    }
-
-    //--------------------------------------------------------------------------
-    // We have written the message successfully
-    //--------------------------------------------------------------------------
-    if( sign )
-      log->Dump( AsyncSockMsg, "[%s] WroteV a message signature : %s (0x%x), "
-                 "%d bytes",
-                 pStreamName.c_str(), sign->GetDescription().c_str(),
-                 sign, sign->GetSize() );
-
-    log->Dump( AsyncSockMsg, "[%s] WroteV a message: %s (0x%x), %d bytes",
-               pStreamName.c_str(), toWrite->GetDescription().c_str(),
-               toWrite, toWrite->GetSize() );
-
-    if( chunks )
-      log->Dump( AsyncSockMsg, "[%s] WroteV raw data:  %d bytes",
-                 pStreamName.c_str(), rawSize );
-
-    return Status();
-  }
-
-  Status AsyncSocketHandler::WriteMessageAndRaw( Message *toWrite, Message *&sign )
-  {
-    // once we can add 'GetMessageBody' to OutgoingMsghandler
-    // interface we can get rid of the ugly dynamic_cast
-    static XRootDMsgHandler *xrdHandler = 0;
-    ChunkList *chunks = 0;
-    uint32_t  *asyncOffset = 0;
+    st = WriteCurrentMessage( toWrite );
+    if( !st.IsOK() || st.code == suRetry )
+      return st;
 
     if( pOutHandler->IsRaw() )
     {
-      if( xrdHandler != pOutHandler )
-        xrdHandler = dynamic_cast<XRootDMsgHandler*>( pOutHandler );
-
-      if( !xrdHandler )
-        return Status( stError, errNotSupported );
-
-      chunks = xrdHandler->GetMessageBody( asyncOffset );
-      Log    *log = DefaultEnv::GetLog();
-      log->Dump( AsyncSockMsg, "[%s] Will write the payload in one go with "
-                 "the header for message: %s (0x%x).", pStreamName.c_str(),
-                 pOutgoing->GetDescription().c_str(), pOutgoing );
+      uint32_t bytesWritten = 0;
+      st = pOutHandler->WriteMessageBody( pSocket, bytesWritten );
+      pOutMsgSize += bytesWritten;
+      if( !st.IsOK() || st.code == suRetry )
+        return st;
     }
 
-    Status st = WriteVMessage( toWrite, sign, chunks, asyncOffset );
-    if( st.IsOK() && st.code == suDone )
+    st = pSocket->Flash();
+    if( !st.IsOK() )
     {
-      if( asyncOffset )
-        pOutMsgSize += *asyncOffset;
-      pOutMsgDone  = true;
+      Log *log = DefaultEnv::GetLog();
+      log->Error( AsyncSockMsg, "[%s] Unable to flash the socket: %s",
+                  pStreamName.c_str(), XrdSysE2T( st.errNo ) );
     }
 
     return st;
   }
 
-  Status AsyncSocketHandler::WriteSeparately( Message *toWrite, Message *&sign )
-  {
-    //------------------------------------------------------------------------
-    // Write the message if not already written
-    //------------------------------------------------------------------------
-    Status st;
-    if( !pOutMsgDone )
-    {
-      if( !(st = WriteVMessage( toWrite, sign, 0, 0 )).IsOK() )
-        return st;
-
-      if( st.code == suRetry )
-        return st;
-
-      Log *log = DefaultEnv::GetLog();
-
-      if( pOutHandler && pOutHandler->IsRaw() )
-      {
-        log->Dump( AsyncSockMsg, "[%s] Will call raw handler to write payload "
-                   "for message: %s (0x%x).", pStreamName.c_str(),
-                   pOutgoing->GetDescription().c_str(), pOutgoing );
-      }
-
-      pOutMsgDone = true;
-    }
-
-    //------------------------------------------------------------------------
-    // Check if the handler needs to be called
-    //------------------------------------------------------------------------
-    if( pOutHandler && pOutHandler->IsRaw() )
-    {
-      uint32_t bytesWritten = 0;
-      st = pOutHandler->WriteMessageBody( pSocket->GetFD(), bytesWritten );
-      pOutMsgSize += bytesWritten;
-      if( !st.IsOK() )
-        return st;
-
-      if( st.code == suRetry )
-        return st;
-    }
-
-    return Status();
-  }
-
   //----------------------------------------------------------------------------
   // Got a read readiness event
   //----------------------------------------------------------------------------
@@ -674,7 +573,7 @@ namespace XrdCl
       pIncMsgSize  = 0;
     }
 
-    Status  st;
+    XRootDStatus  st;
     Log    *log = DefaultEnv::GetLog();
 
     //--------------------------------------------------------------------------
@@ -682,7 +581,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( !pHeaderDone )
     {
-      st = pTransport->GetHeader( pIncoming, pSocket->GetFD() );
+      st = pTransport->GetHeader( pIncoming, pSocket );
       if( !st.IsOK() )
       {
         OnFault( st );
@@ -712,7 +611,7 @@ namespace XrdCl
     if( pIncHandler.first )
     {
       uint32_t bytesRead = 0;
-      st = pIncHandler.first->ReadMessageBody( pIncoming, pSocket->GetFD(),
+      st = pIncHandler.first->ReadMessageBody( pIncoming, pSocket,
                                                bytesRead );
       if( !st.IsOK() )
       {
@@ -729,7 +628,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     else
     {
-      st = pTransport->GetBody( pIncoming, pSocket->GetFD() );
+      st = pTransport->GetBody( pIncoming, pSocket );
       if( !st.IsOK() )
       {
         OnFault( st );
@@ -761,7 +660,7 @@ namespace XrdCl
     // Read the message and let the transport handler look at it when
     // reading has finished
     //--------------------------------------------------------------------------
-    Status st = ReadMessage( pHSIncoming );
+    XRootDStatus st = ReadMessage( pHSIncoming );
     if( !st.IsOK() )
     {
       OnFaultWhileHandshaking( st );
@@ -771,12 +670,20 @@ namespace XrdCl
     if( st.code != suDone )
       return;
 
+    HandleHandShake();
+  }
+
+  //------------------------------------------------------------------------
+  // Handle the handshake message
+  //------------------------------------------------------------------------
+  void AsyncSocketHandler::HandleHandShake()
+  {
     //--------------------------------------------------------------------------
     // OK, we have a new message, let's deal with it;
     //--------------------------------------------------------------------------
     pHandShakeData->in = pHSIncoming;
     pHSIncoming = 0;
-    st = pTransport->HandShake( pHandShakeData, *pChannelData );
+    XRootDStatus st = pTransport->HandShake( pHandShakeData, *pChannelData );
 
     //--------------------------------------------------------------------------
     // Deal with wait responses
@@ -807,7 +714,7 @@ namespace XrdCl
                     "reach connection timeout.",
                     pStreamName.c_str() );
 
-        OnFaultWhileHandshaking( Status( stError, errSocketTimeout ) );
+        OnFaultWhileHandshaking( XRootDStatus( stError, errSocketTimeout ) );
       }
       else
       {
@@ -819,52 +726,67 @@ namespace XrdCl
       return;
     }
 
+    //--------------------------------------------------------------------------
+    // If now is the time enable encryption
+    //--------------------------------------------------------------------------
+    if( !pSocket->IsEncrypted() &&
+         pTransport->NeedEncryption( pHandShakeData, *pChannelData ) )
+    {
+      XRootDStatus st = DoTlsHandShake();
+      if( !st.IsOK() || st.code == suRetry ) return;
+    }
+
+    //--------------------------------------------------------------------------
+    // Now prepare the next step of the hand-shake procedure
+    //--------------------------------------------------------------------------
+    HandShakeNextStep( st.IsOK() && st.code == suDone );
+  }
+
+  //------------------------------------------------------------------------
+  // Prepare the next step of the hand-shake procedure
+  //------------------------------------------------------------------------
+  void AsyncSocketHandler::HandShakeNextStep( bool done )
+  {
     //--------------------------------------------------------------------------
     // We successfully proceeded to the next step
     //--------------------------------------------------------------------------
     ++pHandShakeData->step;
 
+    //--------------------------------------------------------------------------
+    // The hand shake process is done
+    //--------------------------------------------------------------------------
+    if( done )
+    {
+      delete pHandShakeData;
+      XRootDStatus st;
+      if( !(st = EnableUplink()).IsOK() )
+      {
+        OnFaultWhileHandshaking( st );
+        return;
+      }
+      pHandShakeDone = true;
+      pStream->OnConnect( pSubStreamNum );
+    }
     //--------------------------------------------------------------------------
     // The transport handler gave us something to write
     //--------------------------------------------------------------------------
-    if( pHandShakeData->out )
+    else if( pHandShakeData->out )
     {
       pHSOutgoing = pHandShakeData->out;
       pHandShakeData->out = 0;
-      Status st;
+      XRootDStatus st;
       if( !(st = EnableUplink()).IsOK() )
       {
         OnFaultWhileHandshaking( st );
         return;
       }
     }
-
-    //--------------------------------------------------------------------------
-    // The hand shake process is done
-    //--------------------------------------------------------------------------
-    if( st.IsOK() && st.code == suDone )
-    {
-      delete pHandShakeData;
-      if( !(st = EnableUplink()).IsOK() )
-      {
-        OnFaultWhileHandshaking( Status( stFatal, errPollerError ) );
-        return;
-      }
-      pHandShakeDone = true;
-      pStream->OnConnect( pSubStreamNum );
-    }
-
-    if( !st.IsOK() )
-    {
-      OnFaultWhileHandshaking( st );
-      return;
-    }
   }
 
   //----------------------------------------------------------------------------
   // Read a message
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::ReadMessage( Message *&toRead )
+  XRootDStatus AsyncSocketHandler::ReadMessage( Message *&toRead )
   {
     if( !toRead )
     {
@@ -872,11 +794,11 @@ namespace XrdCl
       toRead      = new Message();
     }
 
-    Status  st;
+    XRootDStatus  st;
     Log    *log = DefaultEnv::GetLog();
     if( !pHeaderDone )
     {
-      st = pTransport->GetHeader( toRead, pSocket->GetFD() );
+      st = pTransport->GetHeader( toRead, pSocket );
       if( st.IsOK() && st.code == suDone )
       {
         log->Dump( AsyncSockMsg,
@@ -888,7 +810,7 @@ namespace XrdCl
         return st;
     }
 
-    st = pTransport->GetBody( toRead, pSocket->GetFD() );
+    st = pTransport->GetBody( toRead, pSocket );
     if( st.IsOK() && st.code == suDone )
     {
       log->Dump( AsyncSockMsg, "[%s] Received a message of %d bytes",
@@ -900,7 +822,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Handle fault
   //----------------------------------------------------------------------------
-  void AsyncSocketHandler::OnFault( Status st )
+  void AsyncSocketHandler::OnFault( XRootDStatus st )
   {
     Log *log = DefaultEnv::GetLog();
     log->Error( AsyncSockMsg, "[%s] Socket error encountered: %s",
@@ -919,7 +841,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Handle fault while handshaking
   //----------------------------------------------------------------------------
-  void AsyncSocketHandler::OnFaultWhileHandshaking( Status st )
+  void AsyncSocketHandler::OnFaultWhileHandshaking( XRootDStatus st )
   {
     Log *log = DefaultEnv::GetLog();
     log->Error( AsyncSockMsg, "[%s] Socket error while handshaking: %s",
@@ -954,6 +876,9 @@ namespace XrdCl
       // called from inside of Stream::OnReadTimeout, this
       // in turn means that the ownership of following
       // pointers, has been transfered to the inQueue
+      if( !pIncHandler.second )
+        delete pIncoming;
+
       pIncoming   = 0;
       pOutgoing   = 0;
       pOutHandler = 0;
@@ -967,99 +892,53 @@ namespace XrdCl
   {
     time_t now = time(0);
     if( now > pConnectionStarted+pConnectionTimeout )
-      OnFaultWhileHandshaking( Status( stError, errSocketTimeout ) );
+      OnFaultWhileHandshaking( XRootDStatus( stError, errSocketTimeout ) );
   }
 
   //------------------------------------------------------------------------
-  // Get signature for given message
+  // Carry out the TLS hand-shake
   //------------------------------------------------------------------------
-  Status AsyncSocketHandler::GetSignature( Message *toSign, Message *&sign )
+  XRootDStatus AsyncSocketHandler::DoTlsHandShake()
   {
-    // ideally the 'GetSignature' method should be in  TransportHandler interface
-    // however due to ABI compatibility for the time being this workaround has to
-    // be employed
-    XRootDTransport *xrootdTransport = dynamic_cast<XRootDTransport*>( pTransport );
-    if( !xrootdTransport ) return Status( stError, errNotSupported );
-    return xrootdTransport->GetSignature( toSign, sign, *pChannelData );
-  }
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( AsyncSockMsg, "[%s] TLS hand-shake exchange.", pStreamName.c_str() );
 
-  //------------------------------------------------------------------------
-  // Initialize the iovec with given message
-  //------------------------------------------------------------------------
-  void AsyncSocketHandler::ToIov( Message &msg, iovec &iov )
-  {
-    iov.iov_base = msg.GetBufferAtCursor();
-    iov.iov_len  = msg.GetSize() - msg.GetCursor();
-  }
-
-  //------------------------------------------------------------------------
-  // Update iovec after write
-  //------------------------------------------------------------------------
-  void AsyncSocketHandler::UpdateAfterWrite( Message  &msg,
-                                             iovec    &iov,
-                                             int      &bytesWritten )
-  {
-    size_t advance = ( bytesWritten < (int)iov.iov_len ) ? bytesWritten : iov.iov_len;
-    bytesWritten -= advance;
-    msg.AdvanceCursor( advance );
-    ToIov( msg, iov );
-  }
-
-  //------------------------------------------------------------------------
-  // Add chunks to the given iovec
-  //------------------------------------------------------------------------
-  uint32_t AsyncSocketHandler::ToIov( ChunkList       *chunks,
-                                      const uint32_t  *offset,
-                                      iovec           *iov )
-  {
-    if( !chunks || !offset ) return 0;
-
-    uint32_t off  = *offset;
-    uint32_t size = 0;
-
-    for( auto itr = chunks->begin(); itr != chunks->end(); ++itr )
+    XRootDStatus st;
+    if( !( st = pSocket->TlsHandShake( this, pUrl.GetHostName() ) ).IsOK() )
     {
-      auto &chunk = *itr;
-      if( off > chunk.length )
-      {
-        iov->iov_len = 0;
-        iov->iov_base = 0;
-        off -= chunk.length;
-      }
-      else if( off > 0 )
-      {
-        iov->iov_len  = chunk.length - off;
-        iov->iov_base = reinterpret_cast<char*>( chunk.buffer ) + off;
-        size += iov->iov_len;
-        off = 0;
-      }
-      else
-      {
-        iov->iov_len  = chunk.length;
-        iov->iov_base = chunk.buffer;
-        size += iov->iov_len;
-      }
-      ++iov;
+      OnFaultWhileHandshaking( st );
+      pTlsHandShakeOngoing = false;
+      return st;
     }
 
-    return size;
+    if( st.code == suRetry )
+    {
+      pTlsHandShakeOngoing = true;
+      return st;
+    }
+
+    pTlsHandShakeOngoing = false;
+    log->Info( AsyncSockMsg, "[%s] TLS hand-shake done.", pStreamName.c_str() );
+
+    return st;
   }
 
-  void AsyncSocketHandler::UpdateAfterWrite( ChunkList  *chunks,
-                                             uint32_t   *offset,
-                                             iovec      *iov,
-                                             int        &bytesWritten )
+  //------------------------------------------------------------------------
+  // Handle read/write event if we are in the middle of a TLS hand-shake
+  //------------------------------------------------------------------------
+  inline void AsyncSocketHandler::OnTLSHandShake()
   {
-    *offset += bytesWritten;
-    bytesWritten = 0;
-    ToIov( chunks, offset, iov );
-  }
+    XRootDStatus st = DoTlsHandShake();
+    if( !st.IsOK() || st.code == suRetry ) return;
 
+    HandShakeNextStep( pTransport->HandShakeDone( pHandShakeData,
+                                                  *pChannelData ) );
+  }
 
   void AsyncSocketHandler::RetryHSMsg( Message *msg )
   {
     pHSOutgoing = msg;
-    Status st;
+    XRootDStatus st;
     if( !(st = EnableUplink()).IsOK() )
     {
       OnFaultWhileHandshaking( st );
