@@ -30,6 +30,9 @@
 #include "XrdCl/XrdClLog.hh"
 #include "XrdCl/XrdClUglyHacks.hh"
 #include "XrdCl/XrdClRedirectorRegistry.hh"
+#include "XrdCl/XrdClXRootDTransport.hh"
+
+#include "XrdSys/XrdSysPthread.hh"
 
 #include <ctime>
 
@@ -76,9 +79,8 @@ namespace
       //------------------------------------------------------------------------
       // Handle a fault
       //------------------------------------------------------------------------
-      virtual uint8_t OnStreamEvent( StreamEvent   event,
-                                     uint16_t      streamNum,
-                                     XrdCl::Status status )
+      virtual uint8_t OnStreamEvent( StreamEvent         event,
+                                     XrdCl::XRootDStatus status )
       {
         if( event == Ready )
           return 0;
@@ -90,7 +92,7 @@ namespace
       //------------------------------------------------------------------------
       // Wait for a status of the message
       //------------------------------------------------------------------------
-      XrdCl::Status WaitForStatus()
+      XrdCl::XRootDStatus WaitForStatus()
       {
         pSem->Wait();
         return pStatus;
@@ -122,7 +124,7 @@ namespace
       XrdCl::Semaphore     *pSem;
       XrdCl::MessageFilter *pFilter;
       XrdCl::Message       *pMsg;
-      XrdCl::Status         pStatus;
+      XrdCl::XRootDStatus   pStatus;
   };
 
   //----------------------------------------------------------------------------
@@ -150,7 +152,7 @@ namespace
       // Handle the status information
       //------------------------------------------------------------------------
       void OnStatusReady( const XrdCl::Message *message,
-                          XrdCl::Status         status )
+                          XrdCl::XRootDStatus   status )
       {
         if( pMsg == message )
           pStatus = status;
@@ -160,7 +162,7 @@ namespace
       //------------------------------------------------------------------------
       // Wait for the status to be ready
       //------------------------------------------------------------------------
-      XrdCl::Status WaitForStatus()
+      XrdCl::XRootDStatus WaitForStatus()
       {
         pSem->Wait();
         return pStatus;
@@ -170,11 +172,15 @@ namespace
       StatusHandler(const StatusHandler &other);
       StatusHandler &operator = (const StatusHandler &other);
 
-      XrdCl::Semaphore *pSem;
-      XrdCl::Status     pStatus;
-      XrdCl::Message   *pMsg;
+      XrdCl::Semaphore    *pSem;
+      XrdCl::XRootDStatus  pStatus;
+      XrdCl::Message      *pMsg;
   };
 
+}
+
+namespace XrdCl
+{
   class TickGeneratorTask: public XrdCl::Task
   {
     public:
@@ -194,6 +200,9 @@ namespace
       //------------------------------------------------------------------------
       time_t Run( time_t now )
       {
+        XrdSysMutexHelper lck( pMtx );
+        if( !pChannel ) return 0;
+
         using namespace XrdCl;
         pChannel->Tick( now );
 
@@ -202,13 +211,17 @@ namespace
         env->GetInt( "TimeoutResolution", timeoutResolution );
         return now+timeoutResolution;
       }
+
+      void Invalidate()
+      {
+        XrdSysMutexHelper lck( pMtx );
+        pChannel = 0;
+      }
+
     private:
       XrdCl::Channel *pChannel;
+      XrdSysMutex     pMtx;
   };
-}
-
-namespace XrdCl
-{
 
   //----------------------------------------------------------------------------
   // Constructor
@@ -231,33 +244,29 @@ namespace XrdCl
     int  timeoutResolution = DefaultTimeoutResolution;
     env->GetInt( "TimeoutResolution", timeoutResolution );
 
-    pTransport->InitializeChannel( pChannelData );
-    uint16_t numStreams = transport->StreamNumber( pChannelData );
-    log->Debug( PostMasterMsg, "Creating new channel to: %s %d stream(s)",
-                                url.GetHostId().c_str(), numStreams );
+    pTransport->InitializeChannel( url, pChannelData );
+    log->Debug( PostMasterMsg, "Creating new channel to: %s",
+                                url.GetChannelId().c_str() );
 
     pUrl.SetParams( url.GetParams() );
+    pUrl.SetProtocol( url.GetProtocol() );
 
     //--------------------------------------------------------------------------
-    // Create the streams
+    // Create the stream
     //--------------------------------------------------------------------------
-    pStreams.resize( numStreams );
-    for( uint16_t i = 0; i < numStreams; ++i )
-    {
-      pStreams[i] = new Stream( &pUrl, i );
-      pStreams[i]->SetTransport( transport );
-      pStreams[i]->SetPoller( poller );
-      pStreams[i]->SetIncomingQueue( &pIncoming );
-      pStreams[i]->SetTaskManager( taskManager );
-      pStreams[i]->SetJobManager( jobManager );
-      pStreams[i]->SetChannelData( &pChannelData );
-      pStreams[i]->Initialize();
-    }
+      pStream = new Stream( &pUrl );
+      pStream->SetTransport( transport );
+      pStream->SetPoller( poller );
+      pStream->SetIncomingQueue( &pIncoming );
+      pStream->SetTaskManager( taskManager );
+      pStream->SetJobManager( jobManager );
+      pStream->SetChannelData( &pChannelData );
+      pStream->Initialize();
 
     //--------------------------------------------------------------------------
     // Register the task generating timeout events
     //--------------------------------------------------------------------------
-    pTickGenerator = new TickGeneratorTask( this, pUrl.GetHostId() );
+    pTickGenerator = new TickGeneratorTask( this, pUrl.GetChannelId() );
     pTaskManager->RegisterTask( pTickGenerator, ::time(0)+timeoutResolution );
   }
 
@@ -266,19 +275,19 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   Channel::~Channel()
   {
+    pTickGenerator->Invalidate();
     pTaskManager->UnregisterTask( pTickGenerator );
-    for( uint32_t i = 0; i < pStreams.size(); ++i )
-      delete pStreams[i];
+    delete pStream;
     pTransport->FinalizeChannel( pChannelData );
   }
 
   //----------------------------------------------------------------------------
   // Send a message synchronously
   //----------------------------------------------------------------------------
-  Status Channel::Send( Message *msg, bool stateful, time_t expires )
+  XRootDStatus Channel::Send( Message *msg, bool stateful, time_t expires )
   {
     StatusHandler sh( msg );
-    Status sc = Send( msg, &sh, stateful, expires );
+    XRootDStatus sc = Send( msg, &sh, stateful, expires );
     if( !sc.IsOK() )
       return sc;
     sc = sh.WaitForStatus();
@@ -288,22 +297,13 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Send the message asynchronously
   //----------------------------------------------------------------------------
-  Status Channel::Send( Message              *msg,
-                        OutgoingMsgHandler   *handler,
-                        bool                  stateful,
-                        time_t                expires,
-                        VirtualRedirector    *redirector )
+  XRootDStatus Channel::Send( Message              *msg,
+                              OutgoingMsgHandler   *handler,
+                              bool                  stateful,
+                              time_t                expires )
 
   {
-    PathID path = pTransport->Multiplex( msg, pChannelData );
-
-    if( redirector )
-    {
-      handler->OnStatusReady( msg, Status() );
-      return redirector->HandleRequest( msg, pStreams[path.down] );
-    }
-
-    return pStreams[path.up]->Send( msg, handler, stateful, expires );
+    return pStream->Send( msg, handler, stateful, expires );
   }
 
   //----------------------------------------------------------------------------
@@ -338,9 +338,36 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void Channel::Tick( time_t now )
   {
-    std::vector<Stream *>::iterator it;
-    for( it = pStreams.begin(); it != pStreams.end(); ++it )
-      (*it)->Tick( now );
+    pStream->Tick( now );
+  }
+
+  //----------------------------------------------------------------------------
+  // Force disconnect of all streams
+  //----------------------------------------------------------------------------
+  Status Channel::ForceDisconnect()
+  {
+    //--------------------------------------------------------------------------
+    // Disconnect and recreate the streams
+    //--------------------------------------------------------------------------
+    pStream->ForceError( Status( stError, errOperationInterrupted ) );
+
+    return Status();
+  }
+
+  //------------------------------------------------------------------------
+  // Get the number of connected data streams
+  //------------------------------------------------------------------------
+  uint16_t Channel::NbConnectedStrm()
+  {
+    return XRootDTransport::NbConnectedStrm( pChannelData );
+  }
+
+  //------------------------------------------------------------------------
+  //! Set the on-connect handler for data streams
+  //------------------------------------------------------------------------
+  void Channel::SetOnDataConnectHandler( std::shared_ptr<Job> &onConnJob )
+  {
+    pStream->SetOnDataConnectHandler( onConnJob );
   }
 
   //----------------------------------------------------------------------------
@@ -356,9 +383,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void Channel::RegisterEventHandler( ChannelEventHandler *handler )
   {
-    std::vector<Stream *>::iterator it;
-    for( it = pStreams.begin(); it != pStreams.end(); ++it )
-      (*it)->RegisterEventHandler( handler );
+    pStream->RegisterEventHandler( handler );
   }
 
   //------------------------------------------------------------------------
@@ -366,8 +391,6 @@ namespace XrdCl
   //------------------------------------------------------------------------
   void Channel::RemoveEventHandler( ChannelEventHandler *handler )
   {
-    std::vector<Stream *>::iterator it;
-    for( it = pStreams.begin(); it != pStreams.end(); ++it )
-      (*it)->RemoveEventHandler( handler );
+    pStream->RemoveEventHandler( handler );
   }
 }
