@@ -31,6 +31,7 @@
 #include "XrdCl/XrdClFileSystem.hh"
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClDlgEnv.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysPthread.hh"
 
 #include <stdio.h>
@@ -489,8 +490,10 @@ int main( int argc, char **argv )
   bool         makedir       = false;
   bool         dynSrc        = false;
   bool         delegate      = false;
+  bool         preserveXAttr = false;
   bool         rmOnBadCksum  = false;
   bool         continue_     = false;
+  bool         recurse       = false;
   std::string thirdParty = "none";
 
   if( config.Want( XrdCpConfig::DoPosc ) )     posc       = true;
@@ -510,9 +513,14 @@ int main( int argc, char **argv )
   else
     DlgEnv::Instance().Disable();
 
-  if( config.Want( XrdCpConfig::DoRecurse ) )      makedir       = true;
+  if( config.Want( XrdCpConfig::DoRecurse ) )
+  {
+    makedir = true;
+    recurse = true;
+  }
   if( config.Want( XrdCpConfig::DoPath    ) )      makedir       = true;
   if( config.Want( XrdCpConfig::DoDynaSrc ) )      dynSrc        = true;
+  if( config.Want( XrdCpConfig::DoXAttr ) )        preserveXAttr = true;
   if( config.Want( XrdCpConfig::DoRmOnBadCksum ) ) rmOnBadCksum  = true;
   if( config.Want( XrdCpConfig::DoContinue ) )     continue_     = true;
 
@@ -593,6 +601,15 @@ int main( int argc, char **argv )
   if( config.nStrm != 0 )
     env->PutInt( "SubStreamsPerChannel", config.nStrm + 1 /*stands for the control stream*/ );
 
+  if( config.Want( XrdCpConfig::DoNoTlsOK ) )
+    env->PutInt( "NoTlsOK", 1 );
+
+  if( config.Want( XrdCpConfig::DoTlsNoData ) )
+    env->PutInt( "TlsNoData", 1 );
+
+  if( config.Want( XrdCpConfig::DoTlsMLF ) )
+    env->PutInt( "TlsMetalink", 1 );
+
   if( config.Want( XrdCpConfig::DoZipMtlnCksum ) )
     env->PutInt( "ZipMtlnCksum", 1 );
 
@@ -614,6 +631,13 @@ int main( int argc, char **argv )
     return 50; // generic error
   }
 
+  if( !preserveXAttr )
+  {
+    int val = DefaultPreserveXAttrs;
+    env->GetInt( "PreserveXAttrs", val );
+    if( val ) preserveXAttr = true;
+  }
+
   log->Dump( AppMsg, "Chunk size: %d, parallel chunks %d, streams: %d",
              chunkSize, parallelChunks, config.nStrm + 1 );
 
@@ -635,7 +659,7 @@ int main( int argc, char **argv )
       char *cwd = getcwd( buf, FILENAME_MAX );
       if( !cwd )
       {
-        XRootDStatus st( stError, XProtocol::mapError( errno ), errno, strerror( errno ) );
+        XRootDStatus st( stError, XProtocol::mapError( errno ), errno, XrdSysE2T( errno ) );
         std::cerr <<  st.GetErrorMessage() << std::endl;
         return st.GetShellCode();
       }
@@ -652,25 +676,41 @@ int main( int argc, char **argv )
   //    * we can accept multiple sources
   //    * we need to append the source name
   //----------------------------------------------------------------------------
-  bool targetIsDir = false;
+  bool targetIsDir  = false;
+  bool targetExists = false;
   if( config.dstFile->Protocol == XrdCpFile::isDir )
     targetIsDir = true;
-  else if( config.dstFile->Protocol == XrdCpFile::isXroot )
+  else if( config.dstFile->Protocol == XrdCpFile::isXroot ||
+           config.dstFile->Protocol == XrdCpFile::isXroots )
   {
     URL target( dest );
     FileSystem fs( target );
     StatInfo *statInfo = 0;
     XRootDStatus st = fs.Stat( target.GetPath(), statInfo );
     if( st.IsOK() )
-      {if (statInfo->TestFlags( StatInfo::IsDir ) ) targetIsDir = true;}
-      else if (st.errNo == kXR_NotFound && config.Want( XrdCpConfig::DoPath ))
-              {int n = strlen(config.dstFile->Path);
-               if (config.dstFile->Path[n-1] == '/') targetIsDir = true;
-              }
+    {
+      if( statInfo->TestFlags( StatInfo::IsDir ) )
+        targetIsDir = true;
+      targetExists = true;
+    }
+    else if( st.errNo == kXR_NotFound && config.Want( XrdCpConfig::DoPath ) )
+    {
+      int n = strlen(config.dstFile->Path);
+      if( config.dstFile->Path[n-1] == '/' )
+        targetIsDir = true;
+    }
 
     delete statInfo;
   }
 
+  if( !targetIsDir && targetExists && !force && !recurse )
+  {
+    XRootDStatus st( stError, errInvalidOp, EEXIST );
+    // Unable to create /tmp/test.txt; file exists
+    log->Error( AppMsg, "%s (destination)", st.ToString().c_str() );
+    return st.GetShellCode();
+  }
+
   //----------------------------------------------------------------------------
   // If we have multiple sources and target is neither a directory nor stdout
   // then we cannot proceed
@@ -739,7 +779,7 @@ int main( int argc, char **argv )
         char *cwd = getcwd( buf, FILENAME_MAX );
         if( !cwd )
         {
-          XRootDStatus st( stError, XProtocol::mapError( errno ), errno, strerror( errno ) );
+          XRootDStatus st( stError, XProtocol::mapError( errno ), errno, XrdSysE2T( errno ) );
           std::cerr <<  st.GetErrorMessage() << std::endl;
           return st.GetShellCode();
         }
@@ -757,8 +797,11 @@ int main( int argc, char **argv )
     // Set up the job
     //--------------------------------------------------------------------------
     std::string target = dest;
+
+    // if the source was a file Dlen and Doff overlap
+    bool srcIsDir = std::string( sourceFile->Path ).size() == size_t( sourceFile->Doff + sourceFile->Dlen );
     // if this is a recursive copy make sure we preserve the directory structure
-    if( config.Want( XrdCpConfig::DoRecurse ) )
+    if( config.Want( XrdCpConfig::DoRecurse ) && srcIsDir )
     {
       // get the source directory
       std::string srcDir( sourceFile->Path, sourceFile->Doff );
@@ -793,6 +836,7 @@ int main( int argc, char **argv )
     properties.Set( "xcpBlockSize",   blockSize      );
     properties.Set( "delegate",       delegate       );
     properties.Set( "targetIsDir",    targetIsDir    );
+    properties.Set( "preserveXAttr",  preserveXAttr  );
     properties.Set( "xrate",          config.xRate   );
     properties.Set( "rmOnBadCksum",   rmOnBadCksum   );
     properties.Set( "continue",       continue_      );
