@@ -121,12 +121,27 @@ namespace
 
 namespace XrdCl
 {
+  struct CopyProcessImpl
+  {
+    std::vector<PropertyList>   pJobProperties;
+    std::vector<PropertyList*>  pJobResults;
+    std::vector<CopyJob*>       pJobs;
+  };
+
+  //----------------------------------------------------------------------------
+  // Destructor
+  //----------------------------------------------------------------------------
+  CopyProcess::CopyProcess() : pImpl( new CopyProcessImpl() )
+  {
+  }
+
   //----------------------------------------------------------------------------
   // Destructor
   //----------------------------------------------------------------------------
   CopyProcess::~CopyProcess()
   {
     CleanUpJobs();
+    delete pImpl;
   }
 
   //----------------------------------------------------------------------------
@@ -143,17 +158,17 @@ namespace XrdCl
     if( properties.HasProperty( "jobType" ) &&
         properties.Get<std::string>( "jobType" ) == "configuration" )
     {
-      if( pJobProperties.size() > 0 &&
-          pJobProperties.rbegin()->HasProperty( "jobType" ) &&
-          pJobProperties.rbegin()->Get<std::string>( "jobType" ) == "configuration" )
+      if( pImpl->pJobProperties.size() > 0 &&
+          pImpl->pJobProperties.rbegin()->HasProperty( "jobType" ) &&
+          pImpl->pJobProperties.rbegin()->Get<std::string>( "jobType" ) == "configuration" )
       {
-        PropertyList &config = *pJobProperties.rbegin();
+        PropertyList &config = *pImpl->pJobProperties.rbegin();
         PropertyList::PropertyMap::const_iterator it;
         for( it = properties.begin(); it != properties.end(); ++it )
           config.Set( it->first, it->second );
       }
       else
-        pJobProperties.push_back( properties );
+        pImpl->pJobProperties.push_back( properties );
       return XRootDStatus();
     }
 
@@ -166,10 +181,11 @@ namespace XrdCl
     if( !properties.HasProperty( "target" ) )
       return XRootDStatus( stError, errInvalidArgs, 0, "target not specified" );
 
-    pJobProperties.push_back( properties );
-    PropertyList &p = pJobProperties.back();
+    pImpl->pJobProperties.push_back( properties );
+    PropertyList &p = pImpl->pJobProperties.back();
 
-    const char *bools[] = {"target", "force", "posc", "coerce", "makeDir", "zipArchive", "xcp", 0};
+    const char *bools[] = {"target", "force", "posc", "coerce", "makeDir", "zipArchive",
+                           "xcp", "preserveXAttr", "rmOnBadCksum", "continue", 0};
     for( int i = 0; bools[i]; ++i )
       if( !p.HasProperty( bools[i] ) )
         p.Set( bools[i], false );
@@ -183,7 +199,7 @@ namespace XrdCl
     {
       if( !p.HasProperty( "checkSumType" ) )
       {
-        pJobProperties.pop_back();
+        pImpl->pJobProperties.pop_back();
         return XRootDStatus( stError, errInvalidArgs, 0,
                              "checkSumType not specified" );
       }
@@ -238,13 +254,16 @@ namespace XrdCl
     if( !p.HasProperty( "dynamicSource" ) )
       p.Set( "dynamicSource", false );
 
+    if( !p.HasProperty( "xrate" ) )
+      p.Set( "xrate", 0 );
+
     //--------------------------------------------------------------------------
     // Insert the properties
     //--------------------------------------------------------------------------
     Log *log = DefaultEnv::GetLog();
     Utils::LogPropertyList( log, UtilityMsg, "Adding job with properties: %s",
                             p );
-    pJobResults.push_back( results );
+    pImpl->pJobResults.push_back( results );
     return XRootDStatus();
   }
 
@@ -257,11 +276,11 @@ namespace XrdCl
     std::vector<PropertyList>::iterator it;
 
     log->Debug( UtilityMsg, "CopyProcess: %d jobs to prepare",
-                pJobProperties.size() );
+                pImpl->pJobProperties.size() );
 
     std::map<std::string, uint32_t> targetFlags;
     int i = 0;
-    for( it = pJobProperties.begin(); it != pJobProperties.end(); ++it, ++i )
+    for( it = pImpl->pJobProperties.begin(); it != pImpl->pJobProperties.end(); ++it, ++i )
     {
       PropertyList &props = *it;
 
@@ -269,7 +288,7 @@ namespace XrdCl
           props.Get<std::string>( "jobType" ) == "configuration" )
         continue;
 
-      PropertyList *res   = pJobResults[i];
+      PropertyList *res   = pImpl->pJobResults[i];
       std::string tmp;
 
       props.Get( "source", tmp );
@@ -277,6 +296,16 @@ namespace XrdCl
       if( !source.IsValid() )
         return XRootDStatus( stError, errInvalidArgs, 0, "invalid source" );
 
+      //--------------------------------------------------------------------------
+      // Create a virtual redirector if it is a Metalink file
+      //--------------------------------------------------------------------------
+      if( source.IsMetalink() )
+      {
+        RedirectorRegistry &registry = RedirectorRegistry::Instance();
+        XRootDStatus st = registry.RegisterAndWait( source );
+        if( !st.IsOK() ) return st;
+      }
+
       // handle UNZIP CGI
       const URL::ParamsMap &cgi = source.GetParams();
       URL::ParamsMap::const_iterator itr = cgi.find( "xrdcl.unzip" );
@@ -294,47 +323,37 @@ namespace XrdCl
       if( target.GetProtocol() != "stdio" )
       {
         // handle directories
-        FileSystem fs( target );
-        StatInfo *infoptr = 0;
-        XRootDStatus st = fs.Stat( target.GetPath(), infoptr );
+        bool targetIsDir = false;
+        props.Get( "targetIsDir", targetIsDir );
 
-        if( !st.IsOK() )
+        if( targetIsDir )
         {
-          if( st.code != errNotFound && st.errNo != kXR_NotFound )
-            return st;
-        }
-        else
-        {
-          std::unique_ptr<StatInfo> info( infoptr );
-          if( info->TestFlags( StatInfo::IsDir) )
+          std::string path = target.GetPath() + '/';
+          std::string fn;
+
+          bool isZip = false;
+          props.Get( "zipArchive", isZip );
+          if( isZip )
           {
-            std::string path = target.GetPath() + '/';
-            std::string fn;
-
-            bool isZip = false;
-            props.Get( "zipArchive", isZip );
-            if( isZip )
-            {
-              props.Get( "zipSource", fn );
-            }
-            else if( source.IsMetalink() )
-            {
-              RedirectorRegistry &registry = XrdCl::RedirectorRegistry::Instance();
-              VirtualRedirector *redirector = registry.Get( source );
-              fn = redirector->GetTargetName();
-            }
-            else
-            {
-              fn = source.GetPath();
-            }
-
-            size_t pos = fn.rfind( '/' );
-            if( pos != std::string::npos )
-              fn = fn.substr( pos + 1 );
-            path += fn;
-            target.SetPath( path );
-            props.Set( "target", target.GetURL() );
+            props.Get( "zipSource", fn );
           }
+          else if( source.IsMetalink() )
+          {
+            RedirectorRegistry &registry = XrdCl::RedirectorRegistry::Instance();
+            VirtualRedirector *redirector = registry.Get( source );
+            fn = redirector->GetTargetName();
+          }
+          else
+          {
+            fn = source.GetPath();
+          }
+
+          size_t pos = fn.rfind( '/' );
+          if( pos != std::string::npos )
+            fn = fn.substr( pos + 1 );
+          path += fn;
+          target.SetPath( path );
+          props.Set( "target", target.GetURL() );
         }
       }
 
@@ -372,11 +391,14 @@ namespace XrdCl
       CopyJob *job = 0;
 
       if( tpc == true )
+      {
+        MarkTPC( props );
         job = new TPFallBackCopyJob( i+1, &props, res );
+      }
       else
         job = new ClassicCopyJob( i+1, &props, res );
 
-      pJobs.push_back( job );
+      pImpl->pJobs.push_back( job );
     }
     return XRootDStatus();
   }
@@ -390,11 +412,11 @@ namespace XrdCl
     // Get the configuration
     //--------------------------------------------------------------------------
     uint8_t parallelThreads = 1;
-    if( pJobProperties.size() > 0 &&
-        pJobProperties.rbegin()->HasProperty( "jobType" ) &&
-        pJobProperties.rbegin()->Get<std::string>( "jobType" ) == "configuration" )
+    if( pImpl->pJobProperties.size() > 0 &&
+        pImpl->pJobProperties.rbegin()->HasProperty( "jobType" ) &&
+        pImpl->pJobProperties.rbegin()->Get<std::string>( "jobType" ) == "configuration" )
     {
-      PropertyList &config = *pJobProperties.rbegin();
+      PropertyList &config = *pImpl->pJobProperties.rbegin();
       if( config.HasProperty( "parallel" ) )
         parallelThreads = (uint8_t)config.Get<int>( "parallel" );
     }
@@ -404,7 +426,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     std::vector<CopyJob *>::iterator it;
     uint16_t currentJob = 1;
-    uint16_t totalJobs  = pJobs.size();
+    uint16_t totalJobs  = pImpl->pJobs.size();
 
     //--------------------------------------------------------------------------
     // Single thread
@@ -413,7 +435,7 @@ namespace XrdCl
     {
       XRootDStatus err;
 
-      for( it = pJobs.begin(); it != pJobs.end(); ++it )
+      for( it = pImpl->pJobs.begin(); it != pImpl->pJobs.end(); ++it )
       {
         QueuedCopyJob j( *it, progress, currentJob, totalJobs );
         j.Run(0);
@@ -434,7 +456,7 @@ namespace XrdCl
     else
     {
       uint16_t workers = std::min( (uint16_t)parallelThreads,
-                                   (uint16_t)pJobs.size() );
+                                   (uint16_t)pImpl->pJobs.size() );
       JobManager jm( workers );
       jm.Initialize();
       if( !jm.Start() )
@@ -443,7 +465,7 @@ namespace XrdCl
 
       Semaphore *sem = new Semaphore(0);
       std::vector<QueuedCopyJob*> queued;
-      for( it = pJobs.begin(); it != pJobs.end(); ++it )
+      for( it = pImpl->pJobs.begin(); it != pImpl->pJobs.end(); ++it )
       {
         QueuedCopyJob *j = new QueuedCopyJob( *it, progress, currentJob,
                                               totalJobs, sem );
@@ -465,7 +487,7 @@ namespace XrdCl
       for( itQ = queued.begin(); itQ != queued.end(); ++itQ )
         delete *itQ;
 
-      for( it = pJobs.begin(); it != pJobs.end(); ++it )
+      for( it = pImpl->pJobs.begin(); it != pImpl->pJobs.end(); ++it )
       {
         XRootDStatus st = (*it)->GetResults()->Get<XRootDStatus>( "status" );
         if( !st.IsOK() ) return st;
@@ -477,8 +499,17 @@ namespace XrdCl
   void CopyProcess::CleanUpJobs()
   {
     std::vector<CopyJob*>::iterator itJ;
-    for( itJ = pJobs.begin(); itJ != pJobs.end(); ++itJ )
-      delete *itJ;
-    pJobs.clear();
+    for( itJ = pImpl->pJobs.begin(); itJ != pImpl->pJobs.end(); ++itJ )
+    {
+      CopyJob *job = *itJ;
+      URL src = job->GetSource();
+      if( src.IsMetalink() )
+      {
+        RedirectorRegistry &registry = RedirectorRegistry::Instance();
+        registry.Release( src );
+      }
+      delete job;
+    }
+    pImpl->pJobs.clear();
   }
 }
