@@ -72,7 +72,6 @@ extern "C"
     env->GetInt( "RunForkHandler", runForkHandler );
     if( runForkHandler )
       forkHandler->Prepare();
-    env->WriteLock();
   }
 
   //----------------------------------------------------------------------------
@@ -84,7 +83,6 @@ extern "C"
     Log         *log         = DefaultEnv::GetLog();
     Env         *env         = DefaultEnv::GetEnv();
     ForkHandler *forkHandler = DefaultEnv::GetForkHandler();
-    env->UnLock();
 
     pid_t pid = getpid();
     log->Debug( UtilityMsg, "In the parent fork handler for process %d", pid );
@@ -111,7 +109,7 @@ extern "C"
     Log         *log         = DefaultEnv::GetLog();
     Env         *env         = DefaultEnv::GetEnv();
     ForkHandler *forkHandler = DefaultEnv::GetForkHandler();
-    env->ReInitializeLock();
+    env->RecreateLock();
 
     pid_t pid = getpid();
     log->Debug( UtilityMsg, "In the child fork handler for process %d", pid );
@@ -153,6 +151,7 @@ namespace
       masks["AsyncSockMsg"]       = XrdCl::AsyncSockMsg;
       masks["JobMgrMsg"]          = XrdCl::JobMgrMsg;
       masks["PlugInMgrMsg"]       = XrdCl::PlugInMgrMsg;
+      masks["ExDbgMsg"]           = XrdCl::ExDbgMsg;
     }
 
     //--------------------------------------------------------------------------
@@ -262,42 +261,53 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     std::vector<EnvVarHolder<int> >         varsInt;
     std::vector<EnvVarHolder<std::string> > varsStr;
-    REGISTER_VAR_INT( varsInt, "ConnectionWindow",     DefaultConnectionWindow     );
-    REGISTER_VAR_INT( varsInt, "ConnectionRetry",      DefaultConnectionRetry      );
-    REGISTER_VAR_INT( varsInt, "RequestTimeout",       DefaultRequestTimeout       );
-    REGISTER_VAR_INT( varsInt, "StreamTimeout",        DefaultStreamTimeout        );
-    REGISTER_VAR_INT( varsInt, "SubStreamsPerChannel", DefaultSubStreamsPerChannel );
-    REGISTER_VAR_INT( varsInt, "TimeoutResolution",    DefaultTimeoutResolution    );
-    REGISTER_VAR_INT( varsInt, "StreamErrorWindow",    DefaultStreamErrorWindow    );
-    REGISTER_VAR_INT( varsInt, "RunForkHandler",       DefaultRunForkHandler       );
-    REGISTER_VAR_INT( varsInt, "RedirectLimit",        DefaultRedirectLimit        );
-    REGISTER_VAR_INT( varsInt, "WorkerThreads",        DefaultWorkerThreads        );
-    REGISTER_VAR_INT( varsInt, "CPChunkSize",          DefaultCPChunkSize          );
-    REGISTER_VAR_INT( varsInt, "CPParallelChunks",     DefaultCPParallelChunks     );
-    REGISTER_VAR_INT( varsInt, "DataServerTTL",        DefaultDataServerTTL        );
-    REGISTER_VAR_INT( varsInt, "LoadBalancerTTL",      DefaultLoadBalancerTTL      );
-    REGISTER_VAR_INT( varsInt, "CPInitTimeout",        DefaultCPInitTimeout        );
-    REGISTER_VAR_INT( varsInt, "CPTPCTimeout",         DefaultCPTPCTimeout         );
-    REGISTER_VAR_INT( varsInt, "TCPKeepAlive",         DefaultTCPKeepAlive         );
-    REGISTER_VAR_INT( varsInt, "TCPKeepAliveTime",     DefaultTCPKeepAliveTime     );
-    REGISTER_VAR_INT( varsInt, "TCPKeepAliveInterval", DefaultTCPKeepAliveInterval );
-    REGISTER_VAR_INT( varsInt, "TCPKeepProbes",        DefaultTCPKeepAliveProbes   );
-    REGISTER_VAR_INT( varsInt, "MultiProtocol",        DefaultMultiProtocol        );
-    REGISTER_VAR_INT( varsInt, "ParallelEvtLoop",      DefaultParallelEvtLoop      );
-    REGISTER_VAR_INT( varsInt, "MetalinkProcessing",   DefaultMetalinkProcessing   );
-    REGISTER_VAR_INT( varsInt, "LocalMetalinkFile",    DefaultLocalMetalinkFile    );
-    REGISTER_VAR_INT( varsInt, "XCpBlockSize",         DefaultXCpBlockSize         );
-    REGISTER_VAR_INT( varsInt, "NoDelay",              DefaultNoDelay              );
+    REGISTER_VAR_INT( varsInt, "ConnectionWindow",        DefaultConnectionWindow        );
+    REGISTER_VAR_INT( varsInt, "ConnectionRetry",         DefaultConnectionRetry         );
+    REGISTER_VAR_INT( varsInt, "RequestTimeout",          DefaultRequestTimeout          );
+    REGISTER_VAR_INT( varsInt, "StreamTimeout",           DefaultStreamTimeout           );
+    REGISTER_VAR_INT( varsInt, "SubStreamsPerChannel",    DefaultSubStreamsPerChannel    );
+    REGISTER_VAR_INT( varsInt, "TimeoutResolution",       DefaultTimeoutResolution       );
+    REGISTER_VAR_INT( varsInt, "StreamErrorWindow",       DefaultStreamErrorWindow       );
+    REGISTER_VAR_INT( varsInt, "RunForkHandler",          DefaultRunForkHandler          );
+    REGISTER_VAR_INT( varsInt, "RedirectLimit",           DefaultRedirectLimit           );
+    REGISTER_VAR_INT( varsInt, "WorkerThreads",           DefaultWorkerThreads           );
+    REGISTER_VAR_INT( varsInt, "CPChunkSize",             DefaultCPChunkSize             );
+    REGISTER_VAR_INT( varsInt, "CPParallelChunks",        DefaultCPParallelChunks        );
+    REGISTER_VAR_INT( varsInt, "DataServerTTL",           DefaultDataServerTTL           );
+    REGISTER_VAR_INT( varsInt, "LoadBalancerTTL",         DefaultLoadBalancerTTL         );
+    REGISTER_VAR_INT( varsInt, "CPInitTimeout",           DefaultCPInitTimeout           );
+    REGISTER_VAR_INT( varsInt, "CPTPCTimeout",            DefaultCPTPCTimeout            );
+    REGISTER_VAR_INT( varsInt, "TCPKeepAlive",            DefaultTCPKeepAlive            );
+    REGISTER_VAR_INT( varsInt, "TCPKeepAliveTime",        DefaultTCPKeepAliveTime        );
+    REGISTER_VAR_INT( varsInt, "TCPKeepAliveInterval",    DefaultTCPKeepAliveInterval    );
+    REGISTER_VAR_INT( varsInt, "TCPKeepProbes",           DefaultTCPKeepAliveProbes      );
+    REGISTER_VAR_INT( varsInt, "MultiProtocol",           DefaultMultiProtocol           );
+    REGISTER_VAR_INT( varsInt, "ParallelEvtLoop",         DefaultParallelEvtLoop         );
+    REGISTER_VAR_INT( varsInt, "MetalinkProcessing",      DefaultMetalinkProcessing      );
+    REGISTER_VAR_INT( varsInt, "LocalMetalinkFile",       DefaultLocalMetalinkFile       );
+    REGISTER_VAR_INT( varsInt, "XCpBlockSize",            DefaultXCpBlockSize            );
+    REGISTER_VAR_INT( varsInt, "NoDelay",                 DefaultNoDelay                 );
+    REGISTER_VAR_INT( varsInt, "AioSignal",               DefaultAioSignal               );
+    REGISTER_VAR_INT( varsInt, "PreferIPv4",              DefaultPreferIPv4              );
+    REGISTER_VAR_INT( varsInt, "MaxMetalinkWait",         DefaultMaxMetalinkWait         );
+    REGISTER_VAR_INT( varsInt, "PreserveLocateTried",     DefaultPreserveLocateTried     );
+    REGISTER_VAR_INT( varsInt, "NotAuthorizedRetryLimit", DefaultNotAuthorizedRetryLimit );
+    REGISTER_VAR_INT( varsInt, "PreserveXAttrs",          DefaultPreserveXAttrs          );
+    REGISTER_VAR_INT( varsInt, "NoTlsOK",                 DefaultNoTlsOK                 );
+    REGISTER_VAR_INT( varsInt, "TlsNoData",               DefaultTlsNoData               );
+    REGISTER_VAR_INT( varsInt, "TlsMetalink",             DefaultTlsMetalink             );
+    REGISTER_VAR_INT( varsInt, "ZipMtlnCksum",            DefaultZipMtlnCksum            );
 
-    REGISTER_VAR_STR( varsStr, "PollerPreference",     DefaultPollerPreference     );
-    REGISTER_VAR_STR( varsStr, "ClientMonitor",        DefaultClientMonitor        );
-    REGISTER_VAR_STR( varsStr, "ClientMonitorParam",   DefaultClientMonitorParam   );
-    REGISTER_VAR_STR( varsStr, "NetworkStack",         DefaultNetworkStack         );
-    REGISTER_VAR_STR( varsStr, "PlugIn",               DefaultPlugIn               );
-    REGISTER_VAR_STR( varsStr, "PlugInConfDir",        DefaultPlugInConfDir        );
-    REGISTER_VAR_STR( varsStr, "ReadRecovery",         DefaultReadRecovery         );
-    REGISTER_VAR_STR( varsStr, "WriteRecovery",        DefaultWriteRecovery        );
-    REGISTER_VAR_STR( varsStr, "GlfnRedirector",       DefaultGlfnRedirector       );
+    REGISTER_VAR_STR( varsStr, "ClientMonitor",           DefaultClientMonitor           );
+    REGISTER_VAR_STR( varsStr, "ClientMonitorParam",      DefaultClientMonitorParam      );
+    REGISTER_VAR_STR( varsStr, "NetworkStack",            DefaultNetworkStack            );
+    REGISTER_VAR_STR( varsStr, "PlugIn",                  DefaultPlugIn                  );
+    REGISTER_VAR_STR( varsStr, "PlugInConfDir",           DefaultPlugInConfDir           );
+    REGISTER_VAR_STR( varsStr, "ReadRecovery",            DefaultReadRecovery            );
+    REGISTER_VAR_STR( varsStr, "WriteRecovery",           DefaultWriteRecovery           );
+    REGISTER_VAR_STR( varsStr, "OpenRecovery",            DefaultOpenRecovery            );
+    REGISTER_VAR_STR( varsStr, "GlfnRedirector",          DefaultGlfnRedirector          );
+    REGISTER_VAR_STR( varsStr, "TlsDbgLvl",               DefaultTlsDbgLvl               );
 
     //--------------------------------------------------------------------------
     // Process the configuration files
@@ -819,9 +829,11 @@ namespace XrdCl
     log->SetTopicName( AsyncSockMsg,       "AsyncSock" );
     log->SetTopicName( JobMgrMsg,          "JobMgr" );
     log->SetTopicName( PlugInMgrMsg,       "PlugInMgr" );
+    log->SetTopicName( ExDbgMsg,           "ExDbgMsg" );
+    log->SetTopicName( TlsMsg,             "TlsMsg" );
   }
-}
 
+}
 
 //------------------------------------------------------------------------------
 // Static initialization and finalization
