@@ -32,11 +32,13 @@
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClCopyProcess.hh"
 #include "XrdCl/XrdClFile.hh"
+#include "XrdSys/XrdSysE2T.hh"
 
 #include <cstdlib>
 #include <cstdio>
 #include <iostream>
 #include <iomanip>
+#include <cmath>
 
 #ifdef HAVE_READLINE
 #include <readline/readline.h>
@@ -205,6 +207,14 @@ XRootDStatus DoCD( FileSystem                      *fs,
   return XRootDStatus();
 }
 
+//------------------------------------------------------------------------------
+// Helper function to calculate number of digits in a number
+//------------------------------------------------------------------------------
+int nbDigits( uint64_t nb )
+{
+  return int( log10( double(nb) ) + 1);
+}
+
 //------------------------------------------------------------------------------
 // List a directory
 //------------------------------------------------------------------------------
@@ -246,6 +256,11 @@ XRootDStatus DoLS( FileSystem                      *fs,
       // show duplicates
       flags &= ~DirListFlags::Merge;
     }
+    else if( args[i] == "-Z" )
+    {
+      // check if file is a ZIP archive if yes list content
+      flags |= DirListFlags::Zip;
+    }
     else
       path = args[i];
   }
@@ -301,29 +316,53 @@ XRootDStatus DoLS( FileSystem                      *fs,
       }
       else
       {
-        if( info->TestFlags( StatInfo::IsDir ) )
-          std::cout << "d";
+        if( info->ExtendedFormat() )
+        {
+          if( info->TestFlags( StatInfo::IsDir ) )
+            std::cout << "d";
+          else
+            std::cout << "-";
+          std::cout << info->GetModeAsOctString();
+
+          std::cout << " " << info->GetOwner();
+          std::cout << " " << info->GetGroup();
+
+          uint64_t size = info->GetSize();
+          int width = nbDigits( size ) + 2;
+          if( width < 12 ) width = 12;
+
+          std::cout << std::setw( width ) << info->GetSize();
+          std::cout << " " << info->GetModTimeAsString() << " ";
+        }
         else
-          std::cout << "-";
+        {
+          if( info->TestFlags( StatInfo::IsDir ) )
+            std::cout << "d";
+          else
+            std::cout << "-";
 
-        if( info->TestFlags( StatInfo::IsReadable ) )
-          std::cout << "r";
-        else
-          std::cout << "-";
+          if( info->TestFlags( StatInfo::IsReadable ) )
+            std::cout << "r";
+          else
+            std::cout << "-";
 
-        if( info->TestFlags( StatInfo::IsWritable ) )
-          std::cout << "w";
-        else
-          std::cout << "-";
+          if( info->TestFlags( StatInfo::IsWritable ) )
+            std::cout << "w";
+          else
+            std::cout << "-";
 
-        if( info->TestFlags( StatInfo::XBitSet ) )
-          std::cout << "x";
-        else
-          std::cout << "-";
+          if( info->TestFlags( StatInfo::XBitSet ) )
+            std::cout << "x";
+          else
+            std::cout << "-";
 
-        std::cout << " " << info->GetModTimeAsString();
+          std::cout << " " << info->GetModTimeAsString();
 
-        std::cout << std::setw(12) << info->GetSize() << " ";
+          uint64_t size = info->GetSize();
+          int width = nbDigits( size ) + 2;
+          if( width < 12 ) width = 12;
+          std::cout << std::setw( width ) << info->GetSize() << " ";
+        }
       }
     }
     if( showUrls )
@@ -660,6 +699,11 @@ XRootDStatus DoLocate( FileSystem                      *fs,
       flags |= OpenFlags::Force;
     else if( args[i] == "-d" )
       doDeepLocate = true;
+    else if( args[i] == "-p" )
+    {
+      Env *env = DefaultEnv::GetEnv();
+      env->PutInt( "PreserveLocateTried", 0 );
+    }
     else if( !hasPath )
     {
       path = args[i];
@@ -903,11 +947,27 @@ XRootDStatus DoStat( FileSystem                      *fs,
   if( !flags.empty() )
     flags.erase( flags.length()-1, 1 );
 
-  std::cout << "Path:   " << fullPath << std::endl;
-  std::cout << "Id:     " << info->GetId() << std::endl;
-  std::cout << "Size:   " << info->GetSize() << std::endl;
-  std::cout << "MTime:  " << info->GetModTimeAsString() << std::endl;
+  std::cout <<   "Path:   " << fullPath << std::endl;
+  std::cout <<   "Id:     " << info->GetId() << std::endl;
+  std::cout <<   "Size:   " << info->GetSize() << std::endl;
+  std::cout <<   "MTime:  " << info->GetModTimeAsString() << std::endl;
+  // if extended stat information is available we can print also
+  // change time and access time
+  if( info->ExtendedFormat() )
+  {
+    std::cout << "CTime:  " << info->GetChangeTimeAsString() << std::endl;
+    std::cout << "ATime:  " << info->GetAccessTimeAsString() << std::endl;
+  }
   std::cout << "Flags:  " << info->GetFlags() << " (" << flags << ")";
+
+  // check if extended stat information is available
+  if( info->ExtendedFormat() )
+  {
+    std::cout << "\nMode:   " << info->GetModeAsString() << std::endl;
+    std::cout << "Owner:  " << info->GetOwner() << std::endl;
+    std::cout << "Group:  " << info->GetGroup();
+  }
+
   std::cout << std::endl;
   if( query.length() != 0 )
   {
@@ -994,7 +1054,7 @@ XRootDStatus DoQuery( FileSystem                      *fs,
   Log         *log     = DefaultEnv::GetLog();
   uint32_t     argc    = args.size();
 
-  if( argc != 3 )
+  if( !( argc >= 3 ) )
   {
     log->Error( AppMsg, "Wrong number of arguments." );
     return XRootDStatus( stError, errInvalidArgs );
@@ -1025,30 +1085,57 @@ XRootDStatus DoQuery( FileSystem                      *fs,
     return XRootDStatus( stError, errInvalidArgs );
   }
 
-  std::string strArg = args[2];
-  if( qCode == QueryCode::ChecksumCancel ||
-      qCode == QueryCode::Checksum       ||
-      qCode == QueryCode::XAttr )
+  if( !( qCode & QueryCode::Prepare ) && argc != 3  )
   {
-    if( !BuildPath( strArg, env, args[2] ).IsOK() )
+    log->Error( AppMsg, "Wrong number of arguments." );
+    return XRootDStatus( stError, errInvalidArgs );
+  }
+
+  std::string strArg = args[2];
+  if( qCode & QueryCode::Prepare )
+  {
+    // strArg is supposed to contain already the request ID
+
+    for( size_t i = 3; i < args.size(); ++i )
     {
-      log->Error( AppMsg, "Invalid path." );
-      return XRootDStatus( stError, errInvalidArgs );
+      std::string path = args[i];
+      if( !BuildPath( path, env, path ).IsOK() )
+      {
+        log->Error( AppMsg, "Invalid path." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+      // we use new line character as delimiter
+      strArg += '\n';
+      strArg += path;
+    }
+  }
+  else
+  {
+    std::string strArg = args[2];
+    if( qCode == QueryCode::ChecksumCancel ||
+        qCode == QueryCode::Checksum       ||
+        qCode == QueryCode::XAttr )
+    {
+      if( !BuildPath( strArg, env, args[2] ).IsOK() )
+      {
+        log->Error( AppMsg, "Invalid path." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
     }
   }
 
   //----------------------------------------------------------------------------
   // Run the query
   //----------------------------------------------------------------------------
-  Buffer *response = 0;
-  Buffer  arg( strArg.size() );
+  Buffer arg( strArg.size() );
   arg.FromString( strArg );
+  Buffer *response = 0;
   XRootDStatus st = fs->Query( qCode, arg, response );
 
   if( !st.IsOK() )
   {
-    log->Error( AppMsg, "Unable run query %s %s: %s",
-                        args[1].c_str(), strArg.c_str(),
+    log->Error( AppMsg, "Unable run query %s: %s",
+                        args[1].c_str(),
                         st.ToStr().c_str() );
     return st;
   }
@@ -1083,6 +1170,7 @@ XRootDStatus DoPrepare( FileSystem                      *fs,
   PrepareFlags::Flags      flags    = PrepareFlags::None;
   std::vector<std::string> files;
   uint8_t                  priority = 0;
+  std::string              reqid;
 
   for( uint32_t i = 1; i < args.size(); ++i )
   {
@@ -1115,6 +1203,24 @@ XRootDStatus DoPrepare( FileSystem                      *fs,
       flags |= PrepareFlags::Stage;
     else if( args[i] == "-w" )
       flags |= PrepareFlags::WriteMode;
+    else if( args[i] == "-e" )
+      flags |= PrepareFlags::Evict;
+    else if( args[i] == "-a" )
+    {
+      flags |= PrepareFlags::Cancel;
+      if( i < args.size()-1 )
+      {
+        // by convention the request ID appears as the the first token
+        // in the list of files
+        files.push_back( args[i+1] );
+        ++i;
+      }
+      else
+      {
+        log->Error( AppMsg, "Parameter '-a' requires an argument." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+    }
     else
       files.push_back( args[i] );
   }
@@ -1135,6 +1241,12 @@ XRootDStatus DoPrepare( FileSystem                      *fs,
     log->Error( AppMsg, "Prepare request failed: %s", st.ToStr().c_str() );
     return st;
   }
+
+  if( ( flags & PrepareFlags::Stage ) && response )
+  {
+    std::cout << response->ToString() << '\n';
+  }
+
   delete response;
   return XRootDStatus();
 }
@@ -1401,7 +1513,7 @@ XRootDStatus DoTail( FileSystem                      *fs,
     if( ret == -1 )
     {
       log->Error( AppMsg, "Unable to write to stdout: %s",
-                  strerror(errno) );
+                  XrdSysE2T(errno) );
       delete [] buffer;
       return st;
     }
@@ -1463,6 +1575,170 @@ XRootDStatus DoSpaceInfo( FileSystem                      *fs,
   return XRootDStatus();
 }
 
+//------------------------------------------------------------------------------
+// Carry out xattr operation
+//------------------------------------------------------------------------------
+XRootDStatus DoXAttr( FileSystem                      *fs,
+                      Env                             *env,
+                      const FSExecutor::CommandParams &args )
+{
+  //----------------------------------------------------------------------------
+  // Check up the args
+  //----------------------------------------------------------------------------
+  Log         *log     = DefaultEnv::GetLog();
+  uint32_t     argc    = args.size();
+
+  if( argc < 3 )
+  {
+    log->Error( AppMsg, "Wrong number of arguments." );
+    return XRootDStatus( stError, errInvalidArgs );
+  }
+
+  kXR_char code = 0;
+  if( args[2] == "set")
+    code = kXR_fattrSet;
+  else if( args[2] == "get" )
+    code = kXR_fattrGet;
+  else if( args[2] == "del" )
+    code = kXR_fattrDel;
+  else if( args[2] == "list" )
+    code = kXR_fattrList;
+  else
+  {
+    log->Error( AppMsg, "Invalid xattr code." );
+    return XRootDStatus( stError, errInvalidArgs );
+  }
+
+  std::string path;
+  if( !BuildPath( path, env, args[1] ).IsOK() )
+  {
+    log->Error( AppMsg, "Invalid path." );
+    return XRootDStatus( stError, errInvalidArgs );
+  }
+
+  //----------------------------------------------------------------------------
+  // Issue the xattr operation
+  //----------------------------------------------------------------------------
+  XRootDStatus status;
+  switch( code )
+  {
+    case kXR_fattrSet:
+    {
+      if( argc != 4 )
+      {
+        log->Error( AppMsg, "Wrong number of arguments." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+
+      std::string key_value = args[3];
+      size_t pos = key_value.find( '=' );
+      std::string key   = key_value.substr( 0, pos );
+      std::string value = key_value.substr( pos + 1 );
+      std::vector<xattr_t> attrs;
+      attrs.push_back( std::make_tuple( key, value ) );
+
+      std::vector<XAttrStatus> result;
+      XRootDStatus status = fs->SetXAttr( path, attrs, result );
+      XAttrStatus xst = result.front();
+
+      if( !xst.status.IsOK() )
+        status = xst.status;
+
+      if( !status.IsOK() )
+        log->Error( AppMsg, "Unable to xattr set %s %s: %s",
+                            key.c_str(), value.c_str(),
+                            status.ToStr().c_str() );
+      return status;
+    }
+
+    case kXR_fattrGet:
+    {
+      if( argc != 4 )
+      {
+        log->Error( AppMsg, "Wrong number of arguments." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+
+      std::string key = args[3];
+      std::vector<std::string> attrs;
+      attrs.push_back( key );
+
+      std::vector<XAttr> result;
+      XRootDStatus status = fs->GetXAttr( path, attrs, result );
+      XAttr xattr = result.front();
+
+      if( !xattr.status.IsOK() )
+        status = xattr.status;
+
+      if( !status.IsOK() )
+        log->Error( AppMsg, "Unable to xattr get %s : %s",
+                            key.c_str(),
+                            status.ToStr().c_str() );
+      else
+      {
+        std::cout << "# file: " << path << '\n';
+        std::cout << xattr.name << "=\"" << xattr.value << "\"\n";
+      }
+
+      return status;
+    }
+
+    case kXR_fattrDel:
+    {
+      if( argc != 4 )
+      {
+        log->Error( AppMsg, "Wrong number of arguments." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+
+      std::string key = args[3];
+      std::vector<std::string> attrs;
+      attrs.push_back( key );
+
+      std::vector<XAttrStatus> result ;
+      XRootDStatus status = fs->DelXAttr( path, attrs, result );
+      XAttrStatus xst = result.front();
+
+      if( !xst.status.IsOK() )
+        status = xst.status;
+
+      if( !status.IsOK() )
+        log->Error( AppMsg, "Unable to xattr del %s : %s",
+                            key.c_str(),
+                            status.ToStr().c_str() );
+      return status;
+    }
+
+    case kXR_fattrList:
+    {
+      if( argc != 3 )
+      {
+        log->Error( AppMsg, "Wrong number of arguments." );
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+
+      std::vector<XAttr> result;
+      XRootDStatus status = fs->ListXAttr( path, result );
+
+      if( !status.IsOK() )
+        log->Error( AppMsg, "Unable to xattr list : %s",
+                            status.ToStr().c_str() );
+      else
+      {
+        std::cout << "# file: " << path << '\n';
+        auto itr = result.begin();
+        for( ; itr != result.end(); ++itr )
+          std::cout << itr->name << "=\"" << itr->value << "\"\n";
+      }
+
+      return status;
+    }
+
+    default:
+      return XRootDStatus( stError, errInvalidAddr );
+  }
+}
+
 //------------------------------------------------------------------------------
 // Print help
 //------------------------------------------------------------------------------
@@ -1492,19 +1768,23 @@ XRootDStatus PrintHelp( FileSystem *, Env *,
   printf( "     Modify permissions. Permission string example:\n"             );
   printf( "     rwxr-x--x\n\n"                                                );
 
-  printf( "   ls [-l] [-u] [dirname]\n"                                       );
+  printf( "   ls [-l] [-u] [-R] [-D] [-Z] [dirname]\n"                        );
   printf( "     Get directory listing.\n"                                     );
   printf( "     -l stat every entry and pring long listing\n"                 );
-  printf( "     -u print paths as URLs\n\n"                                   );
+  printf( "     -u print paths as URLs\n"                                     );
+  printf( "     -R list subdirectories recursively\n"                         );
+  printf( "     -D show duplicate entries"                                    );
+  printf( "     -Z if a ZIP archive list its content\n\n"                     );
 
-  printf( "   locate [-n] [-r] [-d] <path>\n"                                 );
+  printf( "   locate [-n] [-r] [-d] [-m] [-i] [-p] <path>\n"                  );
   printf( "     Get the locations of the path.\n"                             );
   printf( "     -r refresh, don't use cached locations\n"                     );
   printf( "     -n make the server return the response immediately even\n"    );
   printf( "        though it may be incomplete\n"                             );
   printf( "     -d do a recursive (deep) locate\n"                            );
   printf( "     -m|-h prefer host names to IP addresses\n"                    );
-  printf( "     -i ignore network dependencies\n\n"                           );
+  printf( "     -i ignore network dependencies\n"                             );
+  printf( "     -p be passive: ignore tried/triedrc cgi opaque info\n\n"      );
 
   printf( "   mkdir [-p] [-m<user><group><other>] <dirname>\n"                );
   printf( "     Creates a directory/tree of directories.\n\n"                 );
@@ -1560,7 +1840,8 @@ XRootDStatus PrintHelp( FileSystem *, Env *,
   printf( "                               i - server identification\n"      );
   printf( "                               z - synchronized statistics\n"    );
   printf( "                               l - connection statistics\n"      );
-  printf( "     xattr          <path>   Extended attributes\n\n"            );
+  printf( "     xattr          <path>   Extended attributes\n"            );
+  printf( "     prepare        <reqid> [filenames]  Prepare request status\n\n" );
 
   printf( "   rm <filename>\n"                                              );
   printf( "     Remove a file.\n\n"                                         );
@@ -1571,13 +1852,15 @@ XRootDStatus PrintHelp( FileSystem *, Env *,
   printf( "   truncate <filename> <length>\n"                               );
   printf( "     Truncate a file.\n\n"                                       );
 
-  printf( "   prepare [-c] [-f] [-s] [-w] [-p priority] filenames\n"        );
+  printf( "   prepare [-c] [-f] [-s] [-w] [-e] [-p priority] [-a requestid] filenames\n"   );
   printf( "     Prepare one or more files for access.\n"                    );
   printf( "     -c co-locate staged files if possible\n"                    );
   printf( "     -f refresh file access time even if the location is known\n" );
   printf( "     -s stage the files to disk if they are not online\n"        );
   printf( "     -w the files will be accessed for modification\n"           );
-  printf( "     -p priority of the request, 0 (lowest) - 3 (highest)\n\n"   );
+  printf( "     -p priority of the request, 0 (lowest) - 3 (highest)\n"     );
+  printf( "     -a abort stage request\n"                                   );
+  printf( "     -e evict the file from disk cache\n\n"                      );
 
   printf( "   cat [-o local file] file\n"                                   );
   printf( "     Print contents of a file to stdout.\n"                      );
@@ -1591,6 +1874,14 @@ XRootDStatus PrintHelp( FileSystem *, Env *,
   printf( "   spaceinfo path\n"                                             );
   printf( "     Get space statistics for given path.\n\n"                   );
 
+  printf( "   xattr <path> <code> <params> \n"                              );
+  printf( "     Operation on extended attributes. Codes:\n\n"               );
+  printf( "     set   <attr>          Set extended attribute; <attr> is\n"  );
+  printf( "                             string of form name=value\n"        );
+  printf( "     get   <name>          Get extended attribute\n"             );
+  printf( "     del   <name>          Delete extended attribute\n"          );
+  printf( "     list                  List extended attributes\n\n"         );
+
   return XRootDStatus();
 }
 
@@ -1619,6 +1910,7 @@ FSExecutor *CreateExecutor( const URL &url )
   executor->AddCommand( "cat",         DoCat        );
   executor->AddCommand( "tail",        DoTail       );
   executor->AddCommand( "spaceinfo",   DoSpaceInfo  );
+  executor->AddCommand( "xattr",       DoXAttr      );
   return executor;
 }
 
