@@ -117,7 +117,7 @@ namespace XrdCl
                            uint16_t           timeout )
   {
     SyncResponseHandler handler;
-    Status st = Open( url, flags, mode, &handler, timeout );
+    XRootDStatus st = Open( url, flags, mode, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -143,7 +143,7 @@ namespace XrdCl
   XRootDStatus File::Close( uint16_t timeout )
   {
     SyncResponseHandler handler;
-    Status st = Close( &handler, timeout );
+    XRootDStatus st = Close( &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -171,7 +171,7 @@ namespace XrdCl
                            uint16_t   timeout )
   {
     SyncResponseHandler handler;
-    Status st = Stat( force, &handler, timeout );
+    XRootDStatus st = Stat( force, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -204,7 +204,7 @@ namespace XrdCl
                            uint16_t  timeout )
   {
     SyncResponseHandler handler;
-    Status st = Read( offset, size, buffer, &handler, timeout );
+    XRootDStatus st = Read( offset, size, buffer, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -218,6 +218,31 @@ namespace XrdCl
     return status;
   }
 
+  //------------------------------------------------------------------------
+  // Read number of pages at a given offset - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::PgRead( uint64_t         offset,
+                             uint32_t         size,
+                             void            *buffer,
+                             ResponseHandler *handler,
+                             uint16_t         timeout )
+  {
+    return XRootDStatus( stError, errNotImplemented );
+  }
+
+  //------------------------------------------------------------------------
+  // Read number of pages at a given offset - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::PgRead( uint64_t               offset,
+                             uint32_t               size,
+                             void                  *buffer,
+                             uint32_t              &bytesRead,
+                             std::vector<uint32_t> &cksums,
+                             uint16_t               timeout )
+  {
+    return XRootDStatus( stError, errNotImplemented );
+  }
+
   //----------------------------------------------------------------------------
   // Write a data chunk at a given offset - async
   //----------------------------------------------------------------------------
@@ -242,7 +267,7 @@ namespace XrdCl
                             uint16_t    timeout )
   {
     SyncResponseHandler handler;
-    Status st = Write( offset, size, buffer, &handler, timeout );
+    XRootDStatus st = Write( offset, size, buffer, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -250,6 +275,31 @@ namespace XrdCl
     return status;
   }
 
+  //------------------------------------------------------------------------
+  // Write number of pages at a given offset - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::PgWrite( uint64_t               offset,
+                              uint32_t               size,
+                              const void            *buffer,
+                              std::vector<uint32_t> &cksums,
+                              ResponseHandler       *handler,
+                              uint16_t               timeout )
+  {
+    return XRootDStatus( stError, errNotImplemented );
+  }
+
+  //------------------------------------------------------------------------
+  // Write number of pages at a given offset - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::PgWrite( uint64_t               offset,
+                              uint32_t               size,
+                              const void            *buffer,
+                              std::vector<uint32_t> &cksums,
+                              uint16_t               timeout )
+  {
+    return XRootDStatus( stError, errNotImplemented );
+  }
+
   //----------------------------------------------------------------------------
   // Commit all pending disk writes - async
   //----------------------------------------------------------------------------
@@ -268,7 +318,7 @@ namespace XrdCl
   XRootDStatus File::Sync( uint16_t timeout )
   {
     SyncResponseHandler handler;
-    Status st = Sync( &handler, timeout );
+    XRootDStatus st = Sync( &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -296,7 +346,7 @@ namespace XrdCl
   XRootDStatus File::Truncate( uint64_t size, uint16_t timeout )
   {
     SyncResponseHandler handler;
-    Status st = Truncate( size, &handler, timeout );
+    XRootDStatus st = Truncate( size, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -327,13 +377,72 @@ namespace XrdCl
                                  uint16_t          timeout )
   {
     SyncResponseHandler handler;
-    Status st = VectorRead( chunks, buffer, &handler, timeout );
+    XRootDStatus st = VectorRead( chunks, buffer, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
     return MessageUtils::WaitForResponse( &handler, vReadInfo );
   }
 
+  //------------------------------------------------------------------------
+  // Write scattered data chunks in one operation - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::VectorWrite( const ChunkList &chunks,
+                            ResponseHandler *handler,
+                            uint16_t         timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->VectorWrite( chunks, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Read scattered data chunks in one operation - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::VectorWrite( const ChunkList  &chunks,
+                           uint16_t          timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = VectorWrite( chunks, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    return MessageUtils::WaitForStatus( &handler );
+  }
+
+  //------------------------------------------------------------------------
+  // Write scattered buffers in one operation - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::WriteV( uint64_t            offset,
+                                  const struct iovec *iov,
+                                  int                 iovcnt,
+                                  ResponseHandler    *handler,
+                                  uint16_t            timeout )
+  {
+    // TODO check pPlugIn
+
+    return pStateHandler->WriteV( offset, iov, iovcnt, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Write scattered buffers in one operation - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::WriteV( uint64_t            offset,
+                                  const struct iovec *iov,
+                                  int                 iovcnt,
+                                  uint16_t            timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = WriteV( offset, iov, iovcnt, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    XRootDStatus status = MessageUtils::WaitForStatus( &handler );
+    return status;
+  }
+
+
   //----------------------------------------------------------------------------
   // Performs a custom operation on an open file, server implementation
   // dependent - async
@@ -357,7 +466,7 @@ namespace XrdCl
                             uint16_t          timeout )
   {
     SyncResponseHandler handler;
-    Status st = Fcntl( arg, &handler, timeout );
+    XRootDStatus st = Fcntl( arg, &handler, timeout );
     if( !st.IsOK() )
       return st;
 
@@ -383,13 +492,144 @@ namespace XrdCl
                            uint16_t   timeout )
   {
     SyncResponseHandler handler;
-    Status st = Visa( &handler, timeout );
+    XRootDStatus st = Visa( &handler, timeout );
     if( !st.IsOK() )
       return st;
 
     return MessageUtils::WaitForResponse( &handler, visa );
   }
 
+  //------------------------------------------------------------------------
+  // Set extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::SetXAttr( const std::vector<xattr_t>  &attrs,
+                               ResponseHandler             *handler,
+                               uint16_t                     timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->SetXAttr( attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Set extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::SetXAttr( const std::vector<xattr_t>  &attrs,
+                               std::vector<XAttrStatus>    &result,
+                               uint16_t                     timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = SetXAttr( attrs, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttrStatus> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+  //------------------------------------------------------------------------
+  // Get extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::GetXAttr( const std::vector<std::string>  &attrs,
+                               ResponseHandler                 *handler,
+                               uint16_t                         timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->GetXAttr( attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Get extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::GetXAttr( const std::vector<std::string>  &attrs,
+                               std::vector<XAttr>              &result,
+                               uint16_t                         timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = GetXAttr( attrs, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttr> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+  //------------------------------------------------------------------------
+  // Delete extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::DelXAttr( const std::vector<std::string>  &attrs,
+                               ResponseHandler                 *handler,
+                               uint16_t                         timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->DelXAttr( attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Delete extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::DelXAttr( const std::vector<std::string>  &attrs,
+                               std::vector<XAttrStatus>        &result,
+                               uint16_t                         timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = DelXAttr( attrs, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttrStatus> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+  //------------------------------------------------------------------------
+  // List extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::ListXAttr( ResponseHandler  *handler,
+                                uint16_t          timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->ListXAttr( handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // List extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::ListXAttr( std::vector<XAttr>  &result,
+                                uint16_t             timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = ListXAttr( &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttr> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+
   //----------------------------------------------------------------------------
   // Check if the file is open
   //----------------------------------------------------------------------------
