@@ -532,7 +532,7 @@ namespace XrdCl
     params.followRedirects = pFollowRedirects;
     MessageUtils::ProcessSendParams( params );
 
-    Status st = IssueRequest( *pFileUrl, msg, openHandler, params );
+    XRootDStatus st = IssueRequest( *pFileUrl, msg, openHandler, params );
 
     if( !st.IsOK() )
     {
@@ -591,7 +591,7 @@ namespace XrdCl
     params.stateful        = true;
     MessageUtils::ProcessSendParams( params );
 
-    Status st = IssueRequest( *pDataServer, msg, closeHandler, params );
+    XRootDStatus st = IssueRequest( *pDataServer, msg, closeHandler, params );
 
     if( !st.IsOK() )
     {
@@ -751,6 +751,30 @@ namespace XrdCl
     return SendOrQueue( *pDataServer, msg, stHandler, params );
   }
 
+  //------------------------------------------------------------------------
+  // Write a data chunk at a given offset - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileStateHandler::Write( uint64_t          offset,
+                                        Buffer          &&buffer,
+                                        ResponseHandler  *handler,
+                                        uint16_t          timeout )
+  {
+    return XRootDStatus( stError, errNotImplemented );
+  }
+
+  //------------------------------------------------------------------------
+  // Write a data from a given file descriptor at a given offset - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileStateHandler::Write( uint64_t            offset,
+                                        uint32_t            size,
+                                        Optional<uint64_t>  fdoff,
+                                        int                 fd,
+                                        ResponseHandler    *handler,
+                                        uint16_t            timeout )
+  {
+    return XRootDStatus( stError, errNotImplemented );
+  }
+
   //----------------------------------------------------------------------------
   // Commit all pending disk writes - async
   //----------------------------------------------------------------------------
@@ -1101,6 +1125,99 @@ namespace XrdCl
     return SendOrQueue( *pDataServer, msg, stHandler, params );
   }
 
+  //------------------------------------------------------------------------
+  // Set extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileStateHandler::SetXAttr( const std::vector<xattr_t> &attrs,
+                                           ResponseHandler            *handler,
+                                           uint16_t                    timeout )
+  {
+    XrdSysMutexHelper scopedLock( pMutex );
+
+    if( pFileState != Opened && pFileState != Recovering )
+      return XRootDStatus( stError, errInvalidOp );
+
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( FileMsg, "[0x%x@%s] Sending a fattr set command for handle 0x%x to "
+                "%s", this, pFileUrl->GetURL().c_str(),
+                *((uint32_t*)pFileHandle), pDataServer->GetHostId().c_str() );
+
+    //--------------------------------------------------------------------------
+    // Issue a new fattr get request
+    //--------------------------------------------------------------------------
+    return XAttrOperationImpl( kXR_fattrSet, 0, attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Get extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileStateHandler::GetXAttr( const std::vector<std::string> &attrs,
+                                           ResponseHandler                *handler,
+                                           uint16_t                        timeout )
+  {
+    XrdSysMutexHelper scopedLock( pMutex );
+
+    if( pFileState != Opened && pFileState != Recovering )
+      return XRootDStatus( stError, errInvalidOp );
+
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( FileMsg, "[0x%x@%s] Sending a fattr get command for handle 0x%x to "
+                "%s", this, pFileUrl->GetURL().c_str(),
+                *((uint32_t*)pFileHandle), pDataServer->GetHostId().c_str() );
+
+    //--------------------------------------------------------------------------
+    // Issue a new fattr get request
+    //--------------------------------------------------------------------------
+    return XAttrOperationImpl( kXR_fattrGet, 0, attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Delete extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileStateHandler::DelXAttr( const std::vector<std::string> &attrs,
+                                           ResponseHandler                *handler,
+                                           uint16_t                        timeout )
+  {
+    XrdSysMutexHelper scopedLock( pMutex );
+
+    if( pFileState != Opened && pFileState != Recovering )
+      return XRootDStatus( stError, errInvalidOp );
+
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( FileMsg, "[0x%x@%s] Sending a fattr del command for handle 0x%x to "
+                "%s", this, pFileUrl->GetURL().c_str(),
+                *((uint32_t*)pFileHandle), pDataServer->GetHostId().c_str() );
+
+    //--------------------------------------------------------------------------
+    // Issue a new fattr del request
+    //--------------------------------------------------------------------------
+    return XAttrOperationImpl( kXR_fattrDel, 0, attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // List extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileStateHandler::ListXAttr( ResponseHandler  *handler,
+                                            uint16_t          timeout )
+  {
+    XrdSysMutexHelper scopedLock( pMutex );
+
+    if( pFileState != Opened && pFileState != Recovering )
+      return XRootDStatus( stError, errInvalidOp );
+
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( FileMsg, "[0x%x@%s] Sending a fattr list command for handle 0x%x to "
+                "%s", this, pFileUrl->GetURL().c_str(),
+                *((uint32_t*)pFileHandle), pDataServer->GetHostId().c_str() );
+
+    //--------------------------------------------------------------------------
+    // Issue a new fattr get request
+    //--------------------------------------------------------------------------
+    static const std::vector<std::string> nothing;
+    return XAttrOperationImpl( kXR_fattrList, ClientFattrRequest::aData,
+                               nothing, handler, timeout );
+  }
+
   //----------------------------------------------------------------------------
   // Check if the file is open
   //----------------------------------------------------------------------------
@@ -1325,11 +1442,14 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( !status->IsOK() && status->code == errRedirect && pFollowRedirects )
     {
-      static const std::string root = "root", xroot = "xroot", file = "file";
+      static const std::string root  = "root",  xroot  = "xroot", file = "file",
+                               roots = "roots", xroots = "xroots";
       std::string msg = status->GetErrorMessage();
-      if( !msg.compare( 0, root.size(), root ) ||
-          !msg.compare( 0, xroot.size(), xroot ) ||
-          !msg.compare( 0, file.size(), file ) )
+      if( !msg.compare( 0, root.size(),   root )  ||
+          !msg.compare( 0, xroot.size(),  xroot ) ||
+          !msg.compare( 0, file.size(),   file )  ||
+          !msg.compare( 0, roots.size(),  roots ) ||
+          !msg.compare( 0, xroots.size(), xroots ) )
       {
         OnStateRedirection( msg, message, userHandler, sendParams );
         return;
@@ -1360,12 +1480,10 @@ namespace XrdCl
       ClientRequest *req = (ClientRequest*)message->GetBuffer();
       switch( req->header.requestid )
       {
-        case kXR_read:   i.opCode = Monitor::ErrorInfo::ErrRead;  break;
-        case kXR_readv:  i.opCode = Monitor::ErrorInfo::ErrReadV; break;
-        case kXR_write:  i.opCode = Monitor::ErrorInfo::ErrWrite; break;
-        // TODO
-        // once we do major release we can replace this with 'ErrWriteV'
-        case kXR_writev: i.opCode = Monitor::ErrorInfo::ErrWrite; break;
+        case kXR_read:   i.opCode = Monitor::ErrorInfo::ErrRead;   break;
+        case kXR_readv:  i.opCode = Monitor::ErrorInfo::ErrReadV;  break;
+        case kXR_write:  i.opCode = Monitor::ErrorInfo::ErrWrite;  break;
+        case kXR_writev: i.opCode = Monitor::ErrorInfo::ErrWriteV; break;
         default: i.opCode = Monitor::ErrorInfo::ErrUnc;
       }
 
@@ -1577,6 +1695,43 @@ namespace XrdCl
       pFileState = Error;
   }
 
+  //------------------------------------------------------------------------
+  // Generic implementation of xattr operation
+  //------------------------------------------------------------------------
+  template<typename T>
+  Status FileStateHandler::XAttrOperationImpl( kXR_char              subcode,
+                                               kXR_char              options,
+                                               const std::vector<T> &attrs,
+                                               ResponseHandler      *handler,
+                                               uint16_t              timeout )
+  {
+    //--------------------------------------------------------------------------
+    // Issue a new fattr request
+    //--------------------------------------------------------------------------
+    Message            *msg;
+    ClientFattrRequest *req;
+    MessageUtils::CreateRequest( msg, req );
+
+    req->requestid = kXR_fattr;
+    req->subcode   = subcode;
+    req->numattr   = attrs.size();
+    req->options   = options;
+    memcpy( req->fhandle, pFileHandle, 4 );
+    XRootDStatus st = MessageUtils::CreateXAttrBody( msg, attrs );
+    if( !st.IsOK() ) return st;
+
+    MessageSendParams params;
+    params.timeout         = timeout;
+    params.followRedirects = false;
+    params.stateful        = true;
+    MessageUtils::ProcessSendParams( params );
+
+    XRootDTransport::SetDescription( msg );
+    StatefulHandler *stHandler = new StatefulHandler( this, handler, msg, params );
+
+    return SendOrQueue( *pDataServer, msg, stHandler, params );
+  }
+
   //----------------------------------------------------------------------------
   // Send a message to a host or put it in the recovery queue
   //----------------------------------------------------------------------------
@@ -1599,7 +1754,7 @@ namespace XrdCl
     if( pFileState == Opened )
     {
       msg->SetSessionId( pSessionId );
-      Status st = IssueRequest( *pDataServer, msg, handler, sendParams );
+      XRootDStatus st = IssueRequest( *pDataServer, msg, handler, sendParams );
 
       //------------------------------------------------------------------------
       // Invalid session id means that the connection has been broken while we
@@ -1711,7 +1866,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Send a close and ignore the response
   //----------------------------------------------------------------------------
-  Status FileStateHandler::SendClose( uint16_t timeout )
+  XRootDStatus FileStateHandler::SendClose( uint16_t timeout )
   {
     Message            *msg;
     ClientCloseRequest *req;
@@ -1736,7 +1891,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Re-open the current file at a given server
   //----------------------------------------------------------------------------
-  Status FileStateHandler::ReOpenFileAtServer( const URL &url, uint16_t timeout )
+  XRootDStatus FileStateHandler::ReOpenFileAtServer( const URL &url, uint16_t timeout )
   {
     Log *log = DefaultEnv::GetLog();
     log->Dump( FileMsg, "[0x%x@%s] Sending a recovery open command to %s",
@@ -1792,7 +1947,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Issue the open request
     //--------------------------------------------------------------------------
-    Status st = IssueRequest( url, msg, openHandler, params );
+    XRootDStatus st = IssueRequest( url, msg, openHandler, params );
 
     // if there was a problem destroy the open handler
     if( !st.IsOK() )
@@ -1859,7 +2014,7 @@ namespace XrdCl
     {
       it->request->SetSessionId( pSessionId );
       ReWriteFileHandle( it->request );
-      Status st = IssueRequest( *pDataServer, it->request,
+      XRootDStatus st = IssueRequest( *pDataServer, it->request,
                                  it->handler, it->params );
       if( !st.IsOK() )
         FailMessage( *it, st );
@@ -1939,14 +2094,15 @@ namespace XrdCl
       i.file = pFileUrl;
       i.oTOD = pOpenTime;
       gettimeofday( &i.cTOD, 0 );
-      i.rBytes = pRBytes;
-      i.vBytes = pVRBytes;
-      i.wBytes = pWBytes + pVWBytes; //TODO once we can break ABI compatibility
-      i.vSegs  = pVSegs;             // we will add a special field for WriteV
-      i.rCount = pRCount;
-      i.vCount = pVRCount;
-      i.wCount = pWCount;
-      i.status = status;
+      i.rBytes  = pRBytes;
+      i.vrBytes = pVRBytes;
+      i.wBytes  = pWBytes;
+      i.vwBytes = pVWBytes;
+      i.vSegs   = pVSegs;
+      i.rCount  = pRCount;
+      i.vCount  = pVRCount;
+      i.wCount  = pWCount;
+      i.status  = status;
       mon->Event( Monitor::EvClose, &i );
     }
   }
