@@ -36,6 +36,7 @@
 #include "XrdCl/XrdClPlugInManager.hh"
 #include "XrdCl/XrdClLocalFileTask.hh"
 #include "XrdCl/XrdClZipListHandler.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysPthread.hh"
 
 #include <sys/stat.h>
@@ -62,10 +63,10 @@ namespace
         struct stat ssp;
         if( stat( path.c_str(), &ssp ) == -1 )
         {
-          log->Error( FileMsg, "Stat: failed: %s", strerror( errno ) );
+          log->Error( FileMsg, "Stat: failed: %s", XrdSysE2T( errno ) );
           XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
                                                   XProtocol::mapError( errno ),
-                                                  strerror( errno ) );
+                                                  XrdSysE2T( errno ) );
           return QueueTask( error, 0, handler );
         }
 
@@ -91,6 +92,25 @@ namespace
         return QueueTask( new XRootDStatus(), resp, handler );
       }
 
+      XrdCl::XRootDStatus Rm( const std::string       &path,
+                              XrdCl::ResponseHandler  *handler,
+                              uint16_t                 timeout )
+      {
+        using namespace XrdCl;
+
+        Log *log = DefaultEnv::GetLog();
+        if( unlink( path.c_str() ) )
+        {
+          log->Error( FileMsg, "Rm: failed: %s", XrdSysE2T( errno ) );
+          XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
+                                                  XProtocol::mapError( errno ),
+                                                  XrdSysE2T( errno ) );
+          return QueueTask( error, 0, handler );
+        }
+
+        return QueueTask( new XRootDStatus(), 0, handler );
+      }
+
       static LocalFS& Instance()
       {
         static LocalFS instance;
@@ -852,7 +872,7 @@ namespace XrdCl
       //------------------------------------------------------------------------
       // Constructor and destructor
       //------------------------------------------------------------------------
-      AssignLBHandler( FileSystem *fs, ResponseHandler *userHandler ):
+      AssignLBHandler( FileSystemImpl *fs, ResponseHandler *userHandler ):
         pFS(fs), pUserHandler(userHandler) {}
 
       virtual ~AssignLBHandler() {}
@@ -862,41 +882,166 @@ namespace XrdCl
       //------------------------------------------------------------------------
       virtual void HandleResponseWithHosts( XRootDStatus *status,
                                             AnyObject    *response,
-                                            HostList     *hostList )
-      {
-        if( status->IsOK() )
-        {
-          HostList::reverse_iterator it;
-          for( it = hostList->rbegin(); it != hostList->rend(); ++it )
-            if( it->loadBalancer )
-            {
-              pFS->AssignLoadBalancer( it->url );
-              break;
-            }
-        }
-
-        bool finalrsp = !( status->IsOK() && status->code == suContinue );
-        pUserHandler->HandleResponseWithHosts( status, response, hostList );
-        if( finalrsp )
-          delete this;
-      }
+                                            HostList     *hostList );
 
     private:
-      FileSystem      *pFS;
+      FileSystemImpl  *pFS;
       ResponseHandler *pUserHandler;
   };
 
+  //----------------------------------------------------------------------------
+  //! Wrapper class used to assign last URL
+  //----------------------------------------------------------------------------
+  class AssignLastURLHandler: public ResponseHandler
+  {
+    public:
+      //------------------------------------------------------------------------
+      // Constructor and destructor
+      //------------------------------------------------------------------------
+      AssignLastURLHandler( FileSystemImpl *fs, ResponseHandler *userHandler ):
+        pFS(fs), pUserHandler(userHandler) {}
+
+      virtual ~AssignLastURLHandler() {}
+
+      //------------------------------------------------------------------------
+      // Response callback
+      //------------------------------------------------------------------------
+      virtual void HandleResponseWithHosts( XRootDStatus *status,
+                                            AnyObject    *response,
+                                            HostList     *hostList );
+
+    private:
+      FileSystemImpl  *pFS;
+      ResponseHandler *pUserHandler;
+  };
+
+  //----------------------------------------------------------------------------
+  //! Implementation holding the data members
+  //----------------------------------------------------------------------------
+  struct FileSystemImpl
+  {
+    FileSystemImpl( const URL &url ) :
+      pLoadBalancerLookupDone( false ),
+      pFollowRedirects( true ),
+      pUrl( new URL( url.GetURL() ) )
+    {
+    }
+
+    ~FileSystemImpl()
+    {
+      delete pUrl;
+    }
+
+    //------------------------------------------------------------------------
+    // Send a message in a locked environment
+    //------------------------------------------------------------------------
+    XRootDStatus Send( Message                 *msg,
+                       ResponseHandler         *handler,
+                       MessageSendParams       &params )
+    {
+      Log *log = DefaultEnv::GetLog();
+      XrdSysMutexHelper scopedLock( pMutex );
+
+      log->Dump( FileSystemMsg, "[0x%x@%s] Sending %s", this,
+                 pUrl->GetHostId().c_str(), msg->GetDescription().c_str() );
+
+      handler = new AssignLastURLHandler( this, handler );
+
+      if( !pLoadBalancerLookupDone && pFollowRedirects )
+        handler = new AssignLBHandler( this, handler ); // TODO
+
+      params.followRedirects = pFollowRedirects;
+
+      return MessageUtils::SendMessage( *pUrl, msg, handler, params, 0 );
+    }
+
+    //----------------------------------------------------------------------------
+    // Assign a load balancer if it has not already been assigned
+    //----------------------------------------------------------------------------
+    void AssignLoadBalancer( const URL &url )
+    {
+      Log *log = DefaultEnv::GetLog();
+      XrdSysMutexHelper scopedLock( pMutex );
+
+      if( pLoadBalancerLookupDone )
+        return;
+
+      log->Dump( FileSystemMsg, "[0x%x@%s] Assigning %s as load balancer", this,
+                 pUrl->GetHostId().c_str(), url.GetHostId().c_str() );
+
+      delete pUrl;
+      pUrl = new URL( url );
+      pLoadBalancerLookupDone = true;
+    }
+
+    //----------------------------------------------------------------------------
+    // Assign last URL
+    //----------------------------------------------------------------------------
+    void AssignLastURL( const URL &url )
+    {
+      Log *log = DefaultEnv::GetLog();
+      XrdSysMutexHelper scopedLock( pMutex );
+
+      log->Dump( FileSystemMsg, "[0x%x@%s] Assigning %s as last URL", this,
+                 pUrl->GetHostId().c_str(), url.GetHostId().c_str() );
+
+      pLastUrl.reset( new URL( url ) );
+    }
+
+    XrdSysMutex           pMutex;
+    bool                  pLoadBalancerLookupDone;
+    bool                  pFollowRedirects;
+    URL                  *pUrl;
+    std::unique_ptr<URL>  pLastUrl;
+  };
+
+  //------------------------------------------------------------------------
+  // Response callback
+  //------------------------------------------------------------------------
+  void AssignLBHandler::HandleResponseWithHosts( XRootDStatus *status,
+                                                 AnyObject    *response,
+                                                 HostList     *hostList )
+  {
+    if( status->IsOK() )
+    {
+      HostList::reverse_iterator it;
+      for( it = hostList->rbegin(); it != hostList->rend(); ++it )
+        if( it->loadBalancer )
+        {
+          pFS->AssignLoadBalancer( it->url );
+          break;
+        }
+    }
+
+    bool finalrsp = !( status->IsOK() && status->code == suContinue );
+    pUserHandler->HandleResponseWithHosts( status, response, hostList );
+    if( finalrsp )
+      delete this;
+  }
+
+  //------------------------------------------------------------------------
+  // Response callback
+  //------------------------------------------------------------------------
+  void AssignLastURLHandler::HandleResponseWithHosts( XRootDStatus *status,
+                                               AnyObject    *response,
+                                               HostList     *hostList )
+  {
+    if( status->IsOK() && hostList )
+      pFS->AssignLastURL( hostList->front().url );
+
+    bool finalrsp = !( status->IsOK() && status->code == suContinue );
+    pUserHandler->HandleResponseWithHosts( status, response, hostList );
+    if( finalrsp )
+      delete this;
+  }
 
   //----------------------------------------------------------------------------
   // Constructor
   //----------------------------------------------------------------------------
   FileSystem::FileSystem( const URL &url, bool enablePlugIns ):
-    pLoadBalancerLookupDone( false ),
-    pFollowRedirects( true ),
+    pImpl( new FileSystemImpl( url ) ),
     pPlugIn(0)
   {
-    pUrl = new URL( url.GetURL() );
-
     //--------------------------------------------------------------------------
     // Check if we need to install a plug-in for this URL
     //--------------------------------------------------------------------------
@@ -931,8 +1076,8 @@ namespace XrdCl
         DefaultEnv::GetForkHandler()->UnRegisterFileSystemObject( this );
     }
 
-    delete pUrl;
     delete pPlugIn;
+    delete pImpl;
   }
 
   //----------------------------------------------------------------------------
@@ -961,7 +1106,7 @@ namespace XrdCl
 
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1039,7 +1184,7 @@ namespace XrdCl
 
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1080,7 +1225,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1124,7 +1269,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1152,6 +1297,9 @@ namespace XrdCl
     if( pPlugIn )
       return pPlugIn->Rm( path, handler, timeout );
 
+    if( pImpl->pUrl->IsLocalFile() )
+      return LocalFS::Instance().Rm( path, handler, timeout );
+
     std::string fPath = FilterXrdClCgi( path );
 
     Message         *msg;
@@ -1165,7 +1313,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1209,7 +1357,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1251,7 +1399,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1293,7 +1441,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1329,7 +1477,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1355,7 +1503,7 @@ namespace XrdCl
     if( pPlugIn )
       return pPlugIn->Stat( path, handler, timeout );
 
-    if( pUrl->IsLocalFile() )
+    if( pImpl->pUrl->IsLocalFile() )
       return LocalFS::Instance().Stat( path, handler, timeout );
 
     std::string fPath = FilterXrdClCgi( path );
@@ -1372,7 +1520,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1414,7 +1562,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1451,7 +1599,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1486,7 +1634,7 @@ namespace XrdCl
     {
       // stat the file to check if it is a directory or a file
       // the ZIP handler will take care of the rest
-      ZipListHandler *zipHandler = new ZipListHandler( *pUrl, path, flags, handler, timeout );
+      ZipListHandler *zipHandler = new ZipListHandler( *pImpl->pUrl, path, flags, handler, timeout );
       XRootDStatus st = Stat( path, zipHandler, timeout );
       if( !st.IsOK() )
         delete zipHandler;
@@ -1504,7 +1652,7 @@ namespace XrdCl
       req->options[0] = kXR_dstat;
 
     if( flags & DirListFlags::Recursive )
-      handler = new RecursiveDirListHandler( *pUrl, url.GetPath(), flags, handler, timeout );
+      handler = new RecursiveDirListHandler( *pImpl->pUrl, url.GetPath(), flags, handler, timeout );
 
     if( flags & DirListFlags::Merge )
       handler = new MergeDirListHandler( flags & DirListFlags::Chunked, handler );
@@ -1516,7 +1664,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1578,7 +1726,10 @@ namespace XrdCl
 
       for( uint32_t i = 0; i < locations->GetSize(); ++i )
       {
-        fs = new FileSystem( locations->At(i).GetAddress() );
+        URL locationURL( locations->At(i).GetAddress() );
+        // make sure the original protocol is preserved (root vs roots)
+        locationURL.SetProtocol( pImpl->pUrl->GetProtocol() );
+        fs = new FileSystem( locationURL );
         st = fs->DirList( path, flags, currentResp, timeout );
         if( !st.IsOK() )
         {
@@ -1686,7 +1837,7 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //----------------------------------------------------------------------------
@@ -1741,11 +1892,11 @@ namespace XrdCl
     MessageUtils::ProcessSendParams( params );
     XRootDTransport::SetDescription( msg );
 
-    return Send( msg, handler, params );
+    return pImpl->Send( msg, handler, params );
   }
 
   //------------------------------------------------------------------------
-  //! Prepare one or more files for access - sync
+  // Prepare one or more files for access - sync
   //------------------------------------------------------------------------
   XRootDStatus FileSystem::Prepare( const std::vector<std::string>  &fileList,
                                     PrepareFlags::Flags              flags,
@@ -1761,6 +1912,146 @@ namespace XrdCl
     return MessageUtils::WaitForResponse( &handler, response );
   }
 
+  //------------------------------------------------------------------------
+  // Set extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::SetXAttr( const std::string           &path,
+                                     const std::vector<xattr_t>  &attrs,
+                                     ResponseHandler             *handler,
+                                     uint16_t                     timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return XAttrOperationImpl( kXR_fattrSet, 0, path, attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Set extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::SetXAttr( const std::string           &path,
+                                     const std::vector<xattr_t>  &attrs,
+                                     std::vector<XAttrStatus>    &result,
+                                     uint16_t                     timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = SetXAttr( path, attrs, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttrStatus> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+  //------------------------------------------------------------------------
+  // Get extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::GetXAttr( const std::string               &path,
+                                     const std::vector<std::string>  &attrs,
+                                     ResponseHandler                 *handler,
+                                     uint16_t                         timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return XAttrOperationImpl( kXR_fattrGet, 0, path, attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Get extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::GetXAttr( const std::string               &path,
+                                     const std::vector<std::string>  &attrs,
+                                     std::vector<XAttr>              &result,
+                                     uint16_t                         timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = GetXAttr( path, attrs, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttr> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+  //------------------------------------------------------------------------
+  // Delete extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::DelXAttr( const std::string               &path,
+                                     const std::vector<std::string>  &attrs,
+                                     ResponseHandler                *handler,
+                                     uint16_t                        timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return XAttrOperationImpl( kXR_fattrDel, 0, path, attrs, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Delete extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::DelXAttr( const std::string               &path,
+                                     const std::vector<std::string>  &attrs,
+                                     std::vector<XAttrStatus>        &result,
+                                     uint16_t                         timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = DelXAttr( path, attrs, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttrStatus> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
+  //------------------------------------------------------------------------
+  // List extended attributes - async
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::ListXAttr( const std::string         &path,
+                                      ResponseHandler           *handler,
+                                      uint16_t                   timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    static const std::vector<std::string> nothing;
+    return XAttrOperationImpl( kXR_fattrList,  ClientFattrRequest::aData,
+                               path, nothing, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // List extended attributes - sync
+  //------------------------------------------------------------------------
+  XRootDStatus FileSystem::ListXAttr( const std::string    &path,
+                                      std::vector<XAttr>   &result,
+                                      uint16_t              timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = ListXAttr( path, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    std::vector<XAttr> *resp = 0;
+    st = MessageUtils::WaitForResponse( &handler, resp );
+    if( resp ) result.swap( *resp );
+    delete resp;
+
+    return st;
+  }
+
   //----------------------------------------------------------------------------
   // Set file property
   //----------------------------------------------------------------------------
@@ -1772,8 +2063,8 @@ namespace XrdCl
 
     if( name == "FollowRedirects" )
     {
-      if( value == "true" ) pFollowRedirects = true;
-      else pFollowRedirects = false;
+      if( value == "true" ) pImpl->pFollowRedirects = true;
+      else pImpl->pFollowRedirects = false;
       return true;
     }
     return false;
@@ -1790,50 +2081,67 @@ namespace XrdCl
 
     if( name == "FollowRedirects" )
     {
-      if( pFollowRedirects ) value = "true";
+      if( pImpl->pFollowRedirects ) value = "true";
       else value = "false";
       return true;
     }
+    else if( name == "LastURL" )
+    {
+      if( pImpl->pLastUrl )
+      {
+        value = pImpl->pLastUrl->GetURL();
+        return true;
+      }
+      else return false;
+    }
+
     return false;
   }
 
-  //----------------------------------------------------------------------------
-  // Assign a load balancer if it has not already been assigned
-  //----------------------------------------------------------------------------
-  void FileSystem::AssignLoadBalancer( const URL &url )
+  //------------------------------------------------------------------------
+  // Generic implementation of xattr operation
+  //------------------------------------------------------------------------
+  template<typename T>
+  Status FileSystem::XAttrOperationImpl( kXR_char               subcode,
+                                         kXR_char               options,
+                                         const std::string     &path,
+                                         const std::vector<T>  &attrs,
+                                         ResponseHandler       *handler,
+                                         uint16_t               timeout )
   {
-    Log *log = DefaultEnv::GetLog();
-    XrdSysMutexHelper scopedLock( pMutex );
+    Message            *msg;
+    ClientFattrRequest *req;
+    MessageUtils::CreateRequest( msg, req );
 
-    if( pLoadBalancerLookupDone )
-      return;
+    req->requestid = kXR_fattr;
+    req->subcode   = subcode;
+    req->options   = options;
+    req->numattr   = attrs.size();
+    memset( req->fhandle, 0, 4 );
+    XRootDStatus st = MessageUtils::CreateXAttrBody( msg, attrs, path );
+    if( !st.IsOK() ) return st;
 
-    log->Dump( FileSystemMsg, "[0x%x@%s] Assigning %s as load balancer", this,
-               pUrl->GetHostId().c_str(), url.GetHostId().c_str() );
+    MessageSendParams params; params.timeout = timeout;
+    MessageUtils::ProcessSendParams( params );
 
-    delete pUrl;
-    pUrl = new URL( url );
-    pLoadBalancerLookupDone = true;
+    XRootDTransport::SetDescription( msg );
+
+    return pImpl->Send( msg, handler, params );
   }
 
-  //----------------------------------------------------------------------------
-  // Send a message in a locked environment
-  //----------------------------------------------------------------------------
-  Status FileSystem::Send( Message                  *msg,
-                           ResponseHandler          *handler,
-                           MessageSendParams        &params )
+  //------------------------------------------------------------------------
+  // Lock the internal lock
+  //------------------------------------------------------------------------
+  void FileSystem::Lock()
   {
-    Log *log = DefaultEnv::GetLog();
-    XrdSysMutexHelper scopedLock( pMutex );
+    pImpl->pMutex.Lock();
+  }
 
-    log->Dump( FileSystemMsg, "[0x%x@%s] Sending %s", this,
-               pUrl->GetHostId().c_str(), msg->GetDescription().c_str() );
-
-    if( !pLoadBalancerLookupDone && pFollowRedirects )
-      handler = new AssignLBHandler( this, handler );
-
-    params.followRedirects = pFollowRedirects;
-
-    return MessageUtils::SendMessage( *pUrl, msg, handler, params, 0 );
+  //------------------------------------------------------------------------
+  // Unlock the internal lock
+  //------------------------------------------------------------------------
+  void FileSystem::UnLock()
+  {
+    pImpl->pMutex.UnLock();
   }
 }
