@@ -68,8 +68,7 @@ namespace XrdCl
       //----------------------------------------------------------------------------
       // Handle the response
       //----------------------------------------------------------------------------
-      virtual void HandleResponseWithHosts( XRootDStatus *status,
-          AnyObject *response, HostList *hostList )
+      virtual void HandleResponse( XRootDStatus *status, AnyObject *response )
       {
         try
         {
@@ -102,9 +101,8 @@ namespace XrdCl
               delete mrh;
               throw new XRootDStatus( st );
             }
-            // clean up
-            DeallocArgs( 0, response, hostList );
-          } else // we have the whole metalink file
+          }
+          else // we have the whole metalink file
           {
             // we don't need the File object anymore
             delete pRedirector->pFile;
@@ -115,20 +113,20 @@ namespace XrdCl
             pRedirector->FinalizeInitialization();
             // we are done, pass the status to the user (whatever it is)
             if( pUserHandler )
-              pUserHandler->HandleResponseWithHosts( new XRootDStatus( st ),
-                  response, hostList );
-            else
-              DeallocArgs( 0, response, hostList );
+              pUserHandler->HandleResponse( new XRootDStatus( st ), 0 );
           }
-        } catch( XRootDStatus *status )
+          // clean up
+          delete response;
+        }
+        catch( XRootDStatus *status )
         {
           pRedirector->FinalizeInitialization( *status );
           // if we were not able to read from the metalink,
           // propagate the error to the user handler
           if( pUserHandler )
-            pUserHandler->HandleResponseWithHosts( status, response, hostList );
+            pUserHandler->HandleResponse( status, 0 );
           else
-            DeallocArgs( status, response, hostList );
+            DeallocArgs( status, response, 0 );
         }
 
         delete this;
@@ -324,7 +322,7 @@ namespace XrdCl
     // get the redirect location
     std::string replica;
     if( !GetReplica( msg, replica ).IsOK() )
-      return GetErrorMsg( msg, "No more replicas to try.", kXR_NotFound );
+      return GetErrorMsg( msg, "Metalink: no more replicas to try.", kXR_noReplicas );
     Message *resp = new Message( sizeof(ServerResponse) );
     ServerResponse* response =
         reinterpret_cast<ServerResponse*>( resp->GetBuffer() );
@@ -395,6 +393,15 @@ namespace XrdCl
     return HandleRequestImpl( msg, handler );
   }
 
+  //----------------------------------------------------------------------------
+  //! Count how many replicas do we have left to try for given request
+  //----------------------------------------------------------------------------
+  int MetalinkRedirector::Count( Message *req ) const
+  {
+    ReplicaList::const_iterator itr = GetReplica( req );
+    return pReplicas.end() - itr;
+  }
+
   //----------------------------------------------------------------------------
   // Gets the file checksum if specified in the metalink
   //----------------------------------------------------------------------------
@@ -417,7 +424,7 @@ namespace XrdCl
     while( ( url = fileInfos[0]->GetUrl() ) )
     {
       replica = URL( url );
-      if( replica.GetURL().size() > 4096 )
+      if( !replica.IsValid() || replica.GetURL().size() > 4096 )
         continue; // this is the internal limit (defined in the protocol)
       pReplicas.push_back( replica.GetURL() );
     }
@@ -429,30 +436,46 @@ namespace XrdCl
   XRootDStatus MetalinkRedirector::GetReplica( const Message *msg,
       std::string &replica ) const
   {
-    if( pReplicas.empty() )
+    ReplicaList::const_iterator itr = GetReplica( msg );
+    if( itr == pReplicas.end() )
       return XRootDStatus( stError, errNotFound );
 
+    XrdCl::Env *env = XrdCl::DefaultEnv::GetEnv();
+    int tlsmtl = DefaultTlsMetalink;
+    env->GetInt( "TlsMetalink", tlsmtl );
+    URL url( *itr );
+    if( tlsmtl && ( url.GetProtocol() == "root" || url.GetProtocol() == "xroot" ) )
+      url.SetProtocol( "roots" );
+    replica = url.GetURL();
+
+    return XRootDStatus();
+  }
+
+  //----------------------------------------------------------------------------
+  //! Get the next replica for the given message
+  //----------------------------------------------------------------------------
+  MetalinkRedirector::ReplicaList::const_iterator
+  MetalinkRedirector::GetReplica( const Message *msg ) const
+  {
+    if( pReplicas.empty() )
+      return pReplicas.cend();
+
     std::string tried;
     if( !GetCgiInfo( msg, "tried", tried ).IsOK() )
-    {
-      replica = pReplicas.front();
-      return XRootDStatus();
-    }
+      return pReplicas.cbegin();
+
     ReplicaList triedList;
     Utils::splitString( triedList, tried, "," );
-    ReplicaList::const_iterator tItr = triedList.begin(), rItr =
-        pReplicas.begin();
-    while( tItr != triedList.end() && rItr != pReplicas.end() )
+    std::set<std::string> triedSet( triedList.begin(), triedList.end() );
+
+    ReplicaList::const_iterator itr = pReplicas.begin();
+    for( ; itr != pReplicas.end(); ++itr )
     {
-      URL rUrl( *rItr );
-      if( rUrl.GetHostName() == *tItr )
-        ++rItr;
-      ++tItr;
+      URL url( *itr );
+      if( !triedSet.count( url.GetHostName() ) ) break;
     }
-    if( rItr == pReplicas.end() )
-      return XRootDStatus( stError, errNotFound ); // there are no more replicas to try
-    replica = *rItr;
-    return XRootDStatus();
+
+    return itr;
   }
 
   //----------------------------------------------------------------------------
