@@ -31,20 +31,49 @@
 #include "XrdCl/XrdClLog.hh"
 #include "XrdCl/XrdClRedirectorRegistry.hh"
 
+#include "XrdSys/XrdSysPthread.hh"
+
 namespace XrdCl
 {
+  struct PostMasterImpl
+  {
+    PostMasterImpl() : pPoller( 0 ), pInitialized( false )
+    {
+      Env *env = DefaultEnv::GetEnv();
+      int workerThreads = DefaultWorkerThreads;
+      env->GetInt( "WorkerThreads", workerThreads );
+
+      pTaskManager = new TaskManager();
+      pJobManager  = new JobManager(workerThreads);
+    }
+
+    ~PostMasterImpl()
+    {
+      delete pPoller;
+      delete pTaskManager;
+      delete pJobManager;
+    }
+
+    typedef std::map<std::string, Channel*> ChannelMap;
+
+    Poller               *pPoller;
+    TaskManager          *pTaskManager;
+    ChannelMap            pChannelMap;
+    XrdSysMutex           pChannelMapMutex;
+    bool                  pInitialized;
+    JobManager           *pJobManager;
+
+    XrdSysMutex           pMtx;
+    std::unique_ptr<Job>  pOnConnJob;
+
+    XrdSysRWLock          pDisconnectLock;
+  };
+
   //----------------------------------------------------------------------------
   // Constructor
   //----------------------------------------------------------------------------
-  PostMaster::PostMaster():
-    pPoller( 0 ), pInitialized( false )
+  PostMaster::PostMaster(): pImpl( new PostMasterImpl() )
   {
-    Env *env = DefaultEnv::GetEnv();
-    int workerThreads = DefaultWorkerThreads;
-    env->GetInt( "WorkerThreads", workerThreads );
-
-    pTaskManager = new TaskManager();
-    pJobManager  = new JobManager(workerThreads);
   }
 
   //----------------------------------------------------------------------------
@@ -52,9 +81,6 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   PostMaster::~PostMaster()
   {
-    delete pPoller;
-    delete pTaskManager;
-    delete pJobManager;
   }
 
   //----------------------------------------------------------------------------
@@ -66,21 +92,21 @@ namespace XrdCl
     std::string pollerPref = DefaultPollerPreference;
     env->GetString( "PollerPreference", pollerPref );
 
-    pPoller = PollerFactory::CreatePoller( pollerPref );
+    pImpl->pPoller = PollerFactory::CreatePoller( pollerPref );
 
-    if( !pPoller )
+    if( !pImpl->pPoller )
       return false;
 
-    bool st = pPoller->Initialize();
+    bool st = pImpl->pPoller->Initialize();
 
     if( !st )
     {
-      delete pPoller;
+      delete pImpl->pPoller;
       return false;
     }
 
-    pJobManager->Initialize();
-    pInitialized = true;
+    pImpl->pJobManager->Initialize();
+    pImpl->pInitialized = true;
     return true;
   }
 
@@ -92,18 +118,18 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Clean up the channels
     //--------------------------------------------------------------------------
-    if( !pInitialized )
+    if( !pImpl->pInitialized )
       return true;
 
-    pInitialized = false;
-    pJobManager->Finalize();
-    ChannelMap::iterator it;
+    pImpl->pInitialized = false;
+    pImpl->pJobManager->Finalize();
+    PostMasterImpl::ChannelMap::iterator it;
 
-    for( it = pChannelMap.begin(); it != pChannelMap.end(); ++it )
+    for( it = pImpl->pChannelMap.begin(); it != pImpl->pChannelMap.end(); ++it )
       delete it->second;
 
-    pChannelMap.clear();
-    return pPoller->Finalize();
+    pImpl->pChannelMap.clear();
+    return pImpl->pPoller->Finalize();
   }
 
   //----------------------------------------------------------------------------
@@ -111,22 +137,22 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   bool PostMaster::Start()
   {
-    if( !pInitialized )
+    if( !pImpl->pInitialized )
       return false;
 
-    if( !pPoller->Start() )
+    if( !pImpl->pPoller->Start() )
       return false;
 
-    if( !pTaskManager->Start() )
+    if( !pImpl->pTaskManager->Start() )
     {
-      pPoller->Stop();
+      pImpl->pPoller->Stop();
       return false;
     }
 
-    if( !pJobManager->Start() )
+    if( !pImpl->pJobManager->Start() )
     {
-      pPoller->Stop();
-      pTaskManager->Stop();
+      pImpl->pPoller->Stop();
+      pImpl->pTaskManager->Stop();
       return false;
     }
 
@@ -138,14 +164,14 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   bool PostMaster::Stop()
   {
-    if( !pInitialized )
+    if( !pImpl->pInitialized )
       return true;
 
-    if( !pJobManager->Stop() )
+    if( !pImpl->pJobManager->Stop() )
       return false;
-    if( !pTaskManager->Stop() )
+    if( !pImpl->pTaskManager->Stop() )
       return false;
-    if( !pPoller->Stop() )
+    if( !pImpl->pPoller->Stop() )
       return false;
     return true;
   }
@@ -161,15 +187,16 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Send a message synchronously
   //----------------------------------------------------------------------------
-  Status PostMaster::Send( const URL &url,
-                           Message   *msg,
-                           bool       stateful,
-                           time_t     expires )
+  XRootDStatus PostMaster::Send( const URL &url,
+                                 Message   *msg,
+                                 bool       stateful,
+                                 time_t     expires )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
-      return Status( stError, errNotSupported );
+      return XRootDStatus( stError, errNotSupported );
 
     return channel->Send( msg, stateful, expires );
   }
@@ -177,30 +204,29 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Send the message asynchronously
   //----------------------------------------------------------------------------
-  Status PostMaster::Send( const URL            &url,
-                           Message              *msg,
-                           OutgoingMsgHandler   *handler,
-                           bool                  stateful,
-                           time_t                expires )
+  XRootDStatus PostMaster::Send( const URL            &url,
+                                 Message              *msg,
+                                 OutgoingMsgHandler   *handler,
+                                 bool                  stateful,
+                                 time_t                expires )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
-      return Status( stError, errNotSupported );
+      return XRootDStatus( stError, errNotSupported );
 
     return channel->Send( msg, handler, stateful, expires );
   }
 
   Status PostMaster::Redirect( const URL          &url,
                                Message            *msg,
-                               OutgoingMsgHandler *outHandler,
                                IncomingMsgHandler *inHandler )
   {
     RedirectorRegistry &registry  = RedirectorRegistry::Instance();
     VirtualRedirector *redirector = registry.Get( url );
     if( !redirector )
       return Status( stError, errInvalidOp );
-    outHandler->OnStatusReady( msg, Status() );
     return redirector->HandleRequest( msg, inHandler );
   }
 
@@ -212,6 +238,7 @@ namespace XrdCl
                               MessageFilter  *filter,
                               time_t          expires )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
@@ -227,6 +254,7 @@ namespace XrdCl
                               IncomingMsgHandler *handler,
                               time_t              expires )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
@@ -242,6 +270,7 @@ namespace XrdCl
                                      uint16_t   query,
                                      AnyObject &result )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
@@ -256,6 +285,7 @@ namespace XrdCl
   Status PostMaster::RegisterEventHandler( const URL           &url,
                                            ChannelEventHandler *handler )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
@@ -271,6 +301,7 @@ namespace XrdCl
   Status PostMaster::RemoveEventHandler( const URL           &url,
                                        ChannelEventHandler *handler )
   {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
 
     if( !channel )
@@ -280,16 +311,96 @@ namespace XrdCl
     return Status();
   }
 
+  //------------------------------------------------------------------------
+  // Get the task manager object user by the post master
+  //------------------------------------------------------------------------
+  TaskManager* PostMaster::GetTaskManager()
+  {
+    return pImpl->pTaskManager;
+  }
+
+  //------------------------------------------------------------------------
+  // Get the job manager object user by the post master
+  //------------------------------------------------------------------------
+  JobManager* PostMaster::GetJobManager()
+  {
+    return pImpl->pJobManager;
+  }
+
+  //------------------------------------------------------------------------
+  // Shut down a channel
+  //------------------------------------------------------------------------
+  Status PostMaster::ForceDisconnect( const URL &url )
+  {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock, false );
+    PostMasterImpl::ChannelMap::iterator it =
+        pImpl->pChannelMap.find( url.GetChannelId() );
+
+    if( it == pImpl->pChannelMap.end() )
+      return Status( stError, errInvalidOp );
+
+    it->second->ForceDisconnect();
+    delete it->second;
+    pImpl->pChannelMap.erase( it );
+
+    return Status();
+  }
+
+  //------------------------------------------------------------------------
+  // Get the number of connected data streams
+  //------------------------------------------------------------------------
+  uint16_t PostMaster::NbConnectedStrm( const URL &url )
+  {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
+    Channel *channel = GetChannel( url );
+    if( !channel ) return 0;
+    return channel->NbConnectedStrm();
+  }
+
+  //------------------------------------------------------------------------
+  //! Set the on-connect handler for data streams
+  //------------------------------------------------------------------------
+  void PostMaster::SetOnDataConnectHandler( const URL            &url,
+                                            std::shared_ptr<Job>  onConnJob )
+  {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
+    Channel *channel = GetChannel( url );
+    if( !channel ) return;
+    channel->SetOnDataConnectHandler( onConnJob );
+  }
+
+  //------------------------------------------------------------------------
+  //! Set the global on-connect handler for control streams
+  //------------------------------------------------------------------------
+  void PostMaster::SetOnConnectHandler( std::unique_ptr<Job> onConnJob )
+  {
+    XrdSysMutexHelper lck( pImpl->pMtx );
+    pImpl->pOnConnJob = std::move( onConnJob );
+  }
+
+  //------------------------------------------------------------------------
+  //! Notify the global on-connect handler
+  //------------------------------------------------------------------------
+  void PostMaster::NotifyConnectHandler( const URL &url )
+  {
+    XrdSysMutexHelper lck( pImpl->pMtx );
+    if( pImpl->pOnConnJob )
+    {
+      URL *ptr = new URL( url );
+      pImpl->pJobManager->QueueJob( pImpl->pOnConnJob.get(), ptr );
+    }
+  }
+
   //----------------------------------------------------------------------------
   // Get the channel
   //----------------------------------------------------------------------------
   Channel *PostMaster::GetChannel( const URL &url )
   {
-    XrdSysMutexHelper scopedLock( pChannelMapMutex );
+    XrdSysMutexHelper scopedLock( pImpl->pChannelMapMutex );
     Channel *channel = 0;
-    ChannelMap::iterator it = pChannelMap.find( url.GetHostId() );
+    PostMasterImpl::ChannelMap::iterator it = pImpl->pChannelMap.find( url.GetChannelId() );
 
-    if( it == pChannelMap.end() )
+    if( it == pImpl->pChannelMap.end() )
     {
       TransportManager *trManager = DefaultEnv::GetTransportManager();
       TransportHandler *trHandler = trManager->GetHandler( url.GetProtocol() );
@@ -302,8 +413,9 @@ namespace XrdCl
         return 0;
       }
 
-      channel = new Channel( url, pPoller, trHandler, pTaskManager, pJobManager );
-      pChannelMap[url.GetHostId()] = channel;
+      channel = new Channel( url, pImpl->pPoller, trHandler, pImpl->pTaskManager,
+                             pImpl->pJobManager );
+      pImpl->pChannelMap[url.GetChannelId()] = channel;
     }
     else
       channel = it->second;
