@@ -22,6 +22,7 @@
 #include "XrdCl/XrdClConstants.hh"
 #include "XrdCl/XrdClMessage.hh"
 #include "XrdCl/XrdClDefaultEnv.hh"
+#include "XrdCl/XrdClTls.hh"
 
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -33,22 +34,41 @@
 #include <signal.h>
 #include <cstdlib>
 #include <cstring>
+#include <sys/uio.h>
+#include <netinet/tcp.h>
+#include <stdexcept>
 
 namespace XrdCl
 {
+  Socket::Socket( int socket, SocketStatus status ):
+    pSocket(socket), pStatus( status ), pServerAddr( 0 ),
+    pProtocolFamily( AF_INET ),
+    pChannelID( 0 ),
+    pCorked( false )
+  {
+  };
+
+  //------------------------------------------------------------------------
+  // Desctuctor
+  //------------------------------------------------------------------------
+  Socket::~Socket()
+  {
+    Close();
+  };
+
   //----------------------------------------------------------------------------
   // Initialize
   //----------------------------------------------------------------------------
-  Status Socket::Initialize( int family )
+  XRootDStatus Socket::Initialize( int family )
   {
     if( pSocket != -1 )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     pSocket = ::socket( family, SOCK_STREAM, 0 );
     if( pSocket < 0 )
     {
       pSocket = -1;
-      return Status( stError, errSocketError );
+      return XRootDStatus( stError, errSocketError );
     }
 
     pProtocolFamily = family;
@@ -63,7 +83,16 @@ namespace XrdCl
     if( ::fcntl( pSocket, F_SETFL, flags | O_NONBLOCK | O_NDELAY ) == -1 )
     {
       Close();
-      return Status( stError, errFcntl, errno );
+      return XRootDStatus( stError, errFcntl, errno );
+    }
+
+    XrdCl::Env *env = XrdCl::DefaultEnv::GetEnv();
+    flags = DefaultNoDelay;
+    env->GetInt( "NoDelay", flags );
+    if( setsockopt( pSocket, IPPROTO_TCP, TCP_NODELAY, &flags, sizeof( int ) ) < 0 )
+    {
+      Close();
+      return XRootDStatus( stError, errFcntl, errno );
     }
 
     //--------------------------------------------------------------------------
@@ -72,7 +101,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
 #ifdef __APPLE__
     int set = 1;
-    Status st = SetSockOpt( SOL_SOCKET, SO_NOSIGPIPE, &set, sizeof(int) );
+    XRootDStatus st = SetSockOpt( SOL_SOCKET, SO_NOSIGPIPE, &set, sizeof(int) );
     if( !st.IsOK() )
     {
       Close();
@@ -84,82 +113,82 @@ namespace XrdCl
     sigaction( SIGPIPE, &act, NULL );
 #endif
 
-    return Status();
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
   // Set the socket flags
   //----------------------------------------------------------------------------
-  Status Socket::SetFlags( int flags )
+  XRootDStatus Socket::SetFlags( int flags )
   {
     if( pSocket == -1 )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     int st = ::fcntl( pSocket, F_SETFL, flags );
     if( st == -1 )
-      return Status( stError, errSocketError, errno );
-    return Status();
+      return XRootDStatus( stError, errSocketError, errno );
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
   // Get the socket flags
   //----------------------------------------------------------------------------
-  Status Socket::GetFlags( int &flags )
+  XRootDStatus Socket::GetFlags( int &flags )
   {
     if( pSocket == -1 )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     int st = ::fcntl( pSocket, F_GETFL, 0 );
     if( st == -1 )
-      return Status( stError, errSocketError, errno );
+      return XRootDStatus( stError, errSocketError, errno );
     flags = st;
-    return Status();
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
   // Get socket options
   //----------------------------------------------------------------------------
-  Status Socket::GetSockOpt( int level, int optname, void *optval,
-                             socklen_t *optlen )
+  XRootDStatus Socket::GetSockOpt( int level, int optname, void *optval,
+                                   socklen_t *optlen )
   {
     if( pSocket == -1 )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     if( ::getsockopt( pSocket, level, optname, optval, optlen ) != 0 )
-      return Status( stError, errSocketOptError, errno );
+      return XRootDStatus( stError, errSocketOptError, errno );
 
-    return Status();
+    return XRootDStatus();
   }
 
   //------------------------------------------------------------------------
   // Set socket options
   //------------------------------------------------------------------------
-  Status Socket::SetSockOpt( int level, int optname, const void *optval,
-                             socklen_t optlen )
+  XRootDStatus Socket::SetSockOpt( int level, int optname, const void *optval,
+                                   socklen_t optlen )
   {
     if( pSocket == -1 )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     if( ::setsockopt( pSocket, level, optname, optval, optlen ) != 0 )
-      return Status( stError, errSocketOptError, errno );
+      return XRootDStatus( stError, errSocketOptError, errno );
 
-    return Status();
+    return XRootDStatus();
   }
 
 
   //----------------------------------------------------------------------------
   // Connect to the given host name
   //----------------------------------------------------------------------------
-  Status Socket::Connect( const std::string &host,
-                          uint16_t           port,
-                          uint16_t           timeout )
+  XRootDStatus Socket::Connect( const std::string &host,
+                                uint16_t           port,
+                                uint16_t           timeout )
   {
     if( pSocket == -1 || pStatus == Connected || pStatus == Connecting )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     std::vector<XrdNetAddr> addrs;
     std::ostringstream o; o << host << ":" << port;
-    Status st;
+    XRootDStatus st;
 
     if( pProtocolFamily == AF_INET6 )
       st = Utils::GetHostAddresses( addrs, URL( o.str() ), Utils::IPAll );
@@ -179,14 +208,19 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Connect to the given host
   //----------------------------------------------------------------------------
-  Status Socket::ConnectToAddress( const XrdNetAddr &addr,
-                                   uint16_t          timeout )
+  XRootDStatus Socket::ConnectToAddress( const XrdNetAddr &addr,
+                                         uint16_t          timeout )
   {
     if( pSocket == -1 || pStatus == Connected || pStatus == Connecting )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     pServerAddr = addr;
 
+    //--------------------------------------------------------------------------
+    // Make sure TLS is off when the physical connection is newly established
+    //--------------------------------------------------------------------------
+    pTls.reset();
+
     //--------------------------------------------------------------------------
     // Connect
     //--------------------------------------------------------------------------
@@ -194,7 +228,7 @@ namespace XrdCl
                                          pServerAddr.SockSize(), timeout );
     if( status != 0 )
     {
-      Status st( stError );
+      XRootDStatus st( stError );
 
       //------------------------------------------------------------------------
       // If we connect asynchronously this is not really an error
@@ -202,7 +236,7 @@ namespace XrdCl
       if( !timeout && status == EINPROGRESS )
       {
         pStatus = Connecting;
-        return Status();
+        return XRootDStatus();
       }
 
       //------------------------------------------------------------------------
@@ -218,7 +252,7 @@ namespace XrdCl
       return st;
     }
     pStatus = Connected;
-    return Status();
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
@@ -240,25 +274,25 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   //! Read raw bytes from the socket
   //----------------------------------------------------------------------------
-  Status Socket::ReadRaw( void *buffer, uint32_t size, int32_t timeout,
-                          uint32_t &bytesRead )
+  XRootDStatus Socket::ReadRaw( void *buffer, uint32_t size, int32_t timeout,
+                                uint32_t &bytesRead )
   {
     //--------------------------------------------------------------------------
     // Check if we're connected
     //--------------------------------------------------------------------------
     if( pStatus != Connected )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     //--------------------------------------------------------------------------
     // Some useful variables
     //--------------------------------------------------------------------------
     bytesRead = 0;
 
-    char    *current    = (char *)buffer;
-    bool     useTimeout = (timeout!=-1);
-    time_t   now        = 0;
-    time_t   newNow     = 0;
-    Status   sc;
+    char          *current    = (char *)buffer;
+    bool           useTimeout = (timeout!=-1);
+    time_t         now        = 0;
+    time_t         newNow     = 0;
+    XRootDStatus   sc;
 
     if( useTimeout )
       now = ::time(0);
@@ -293,7 +327,7 @@ namespace XrdCl
         if( n == 0 )
         {
           Close();
-          return Status( stError, errSocketDisconnected );
+          return XRootDStatus( stError, errSocketDisconnected );
         }
 
         //----------------------------------------------------------------------
@@ -302,7 +336,7 @@ namespace XrdCl
         if( (n < 0) && (errno != EAGAIN) && (errno != EWOULDBLOCK) )
         {
           Close();
-          return Status( stError, errSocketError, errno );
+          return XRootDStatus( stError, errSocketError, errno );
         }
       }
       else
@@ -328,32 +362,32 @@ namespace XrdCl
     // Have we managed to read everything?
     //--------------------------------------------------------------------------
     if( bytesRead < size )
-      return Status( stError, errSocketTimeout );
-    return Status( stOK );
+      return XRootDStatus( stError, errSocketTimeout );
+    return XRootDStatus( stOK );
   }
 
   //----------------------------------------------------------------------------
   // Write raw bytes to the socket
   //----------------------------------------------------------------------------
-  Status Socket::WriteRaw( void *buffer, uint32_t size, int32_t timeout,
-                           uint32_t &bytesWritten )
+  XRootDStatus Socket::WriteRaw( void *buffer, uint32_t size, int32_t timeout,
+                                 uint32_t &bytesWritten )
   {
     //--------------------------------------------------------------------------
     // Check if we're connected
     //--------------------------------------------------------------------------
     if( pStatus != Connected )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     //--------------------------------------------------------------------------
     // Some useful variables
     //--------------------------------------------------------------------------
     bytesWritten = 0;
 
-    char    *current    = (char *)buffer;
-    bool     useTimeout = (timeout!=-1);
-    time_t   now        = 0;
-    time_t   newNow     = 0;
-    Status   sc;
+    char          *current    = (char *)buffer;
+    bool           useTimeout = (timeout!=-1);
+    time_t         now        = 0;
+    time_t         newNow     = 0;
+    XRootDStatus   sc;
 
     if( useTimeout )
       now = ::time(0);
@@ -387,7 +421,7 @@ namespace XrdCl
         if( (n <= 0) && (errno != EAGAIN) && (errno != EWOULDBLOCK) )
         {
           Close();
-          return Status( stError, errSocketError, errno );
+          return XRootDStatus( stError, errSocketError, errno );
         }
       }
       else
@@ -413,37 +447,45 @@ namespace XrdCl
     // Have we managed to read everything?
     //--------------------------------------------------------------------------
     if( bytesWritten < size )
-      return Status( stError, errSocketTimeout );
+      return XRootDStatus( stError, errSocketTimeout );
 
-    return Status( stOK );
+    return XRootDStatus( stOK );
   }
 
   //------------------------------------------------------------------------
   // Portable wrapper around SIGPIPE free send
   //----------------------------------------------------------------------------
-  ssize_t Socket::Send( void *buffer, uint32_t size )
+  XRootDStatus Socket::Send( const char *buffer, size_t size, int &bytesWritten )
   {
+    if( pTls ) return pTls->Send( buffer, size, bytesWritten );
+
     //--------------------------------------------------------------------------
     // We use send with MSG_NOSIGNAL to avoid SIGPIPEs on Linux
     //--------------------------------------------------------------------------
 #ifdef __linux__
-    return ::send( pSocket, buffer, size, MSG_NOSIGNAL );
+    int status = ::send( pSocket, buffer, size, MSG_NOSIGNAL );
 #else
-    return ::write( pSocket, buffer, size );
+    int status = ::write( pSocket, buffer, size );
 #endif
+
+    if( status <= 0 )
+      return ClassifyErrno( errno );
+
+    bytesWritten = status;
+    return XRootDStatus();
   }
 
   //----------------------------------------------------------------------------
   // Poll the descriptor
   //----------------------------------------------------------------------------
-  Status Socket::Poll( bool readyForReading, bool readyForWriting,
-                       int32_t timeout )
+  XRootDStatus Socket::Poll( bool readyForReading, bool readyForWriting,
+                             int32_t timeout )
   {
     //--------------------------------------------------------------------------
     // Check if we're connected
     //--------------------------------------------------------------------------
     if( pStatus != Connected )
-      return Status( stError, errInvalidOp );
+      return XRootDStatus( stError, errInvalidOp );
 
     //--------------------------------------------------------------------------
     // Prepare the stuff
@@ -482,7 +524,7 @@ namespace XrdCl
     {
       pollRet = poll( &pollDesc, 1, (useTimeout ? timeout*1000 : -1) );
       if( (pollRet < 0) && (errno != EINTR) && (errno != EAGAIN) )
-        return Status( stError, errPoll, errno );
+        return XRootDStatus( stError, errPoll, errno );
 
       //------------------------------------------------------------------------
       // Check if we did not time out in the case where we are not supposed
@@ -494,7 +536,7 @@ namespace XrdCl
         timeout -= (newNow-now);
         now      = newNow;
         if( timeout < 0 )
-          return Status( stError, errSocketTimeout );
+          return XRootDStatus( stError, errSocketTimeout );
       }
     }
     while( pollRet == -1 );
@@ -503,25 +545,25 @@ namespace XrdCl
     // Check if we have timed out
     //--------------------------------------------------------------------------
     if( pollRet == 0 )
-      return Status( stError, errSocketTimeout );
+      return XRootDStatus( stError, errSocketTimeout );
 
     //--------------------------------------------------------------------------
     // We have some events
     //--------------------------------------------------------------------------
     if( pollDesc.revents & (POLLIN | POLLPRI | POLLOUT) )
-      return Status( stOK );
+      return XRootDStatus( stOK );
 
     //--------------------------------------------------------------------------
     // We've been hang up on
     //--------------------------------------------------------------------------
     if( pollDesc.revents & hupEvents )
-      return Status( stError, errSocketDisconnected );
+      return XRootDStatus( stError, errSocketDisconnected );
 
     //--------------------------------------------------------------------------
     // We're messed up, either because we messed up ourselves (POLLNVAL) or
     // got messed up by the network (POLLERR)
     //--------------------------------------------------------------------------
-    return Status( stError, errSocketError );
+    return XRootDStatus( stError, errSocketError );
   }
 
   //----------------------------------------------------------------------------
@@ -582,4 +624,167 @@ namespace XrdCl
     pName += ">";
     return pName;
   }
+
+
+  //------------------------------------------------------------------------
+  // Classify errno while reading/writing
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::ClassifyErrno( int error )
+  {
+    switch( errno )
+    {
+
+      case EAGAIN:
+#if EAGAIN != EWOULDBLOCK
+      case EWOULDBLOCK:
+#endif
+      {
+        //------------------------------------------------------------------
+        // Reading/writing operation would block! So we are done for now,
+        // but we will be back ;-)
+        //------------------------------------------------------------------
+        return XRootDStatus( stOK, suRetry );
+      }
+      case ECONNRESET:
+      case EDESTADDRREQ:
+      case EMSGSIZE:
+      case ENOTCONN:
+      case ENOTSOCK:
+      {
+        //------------------------------------------------------------------
+        // Actual socket error error!
+        //------------------------------------------------------------------
+        return XRootDStatus( stError, errSocketError, errno );
+      }
+      case EFAULT:
+      {
+        //------------------------------------------------------------------
+        // The buffer provided by the user for reading/writing is invalid
+        //------------------------------------------------------------------
+        return XRootDStatus( stError, errInvalidArgs );
+      }
+      default:
+      {
+        //------------------------------------------------------------------
+        // Not a socket error
+        //------------------------------------------------------------------
+        return XRootDStatus( stError, errInternal, errno );
+      }
+    }
+  }
+
+
+  //----------------------------------------------------------------------------
+  // Read helper from raw socket helper
+  //----------------------------------------------------------------------------
+  XRootDStatus Socket::Read( char *buffer, size_t size, int &bytesRead )
+  {
+    if( pTls ) return pTls->Read( buffer, size, bytesRead );
+
+    int status = ::read( pSocket, buffer, size );
+
+    // if the server shut down the socket declare a socket error (it
+    // will trigger a re-connect)
+    if( status == 0 )
+      return XRootDStatus( stError, errSocketError, errno );
+
+    if( status < 0 )
+      return ClassifyErrno( errno );
+
+    bytesRead = status;
+    return XRootDStatus();
+  }
+
+  //------------------------------------------------------------------------
+  // Cork the underlying socket
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::Cork()
+  {
+#if defined(TCP_CORK) // it's not defined on mac, we might want explore the possibility of using TCP_NOPUSH
+    if( pCorked ) return XRootDStatus();
+
+    int state = 1;
+    int rc = setsockopt( pSocket, IPPROTO_TCP, TCP_CORK, &state, sizeof( state ) );
+    if( rc != 0 )
+      return XRootDStatus( stFatal, errSocketOptError, errno );
+#endif
+    pCorked = true;
+    return XRootDStatus();
+  }
+
+  //------------------------------------------------------------------------
+  // Uncork the underlying socket
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::Uncork()
+  {
+#if defined(TCP_CORK) // it's not defined on mac, we might want explore the possibility of using TCP_NOPUSH
+    if( !pCorked ) return XRootDStatus();
+
+    int state = 0;
+    int rc = setsockopt( pSocket, IPPROTO_TCP, TCP_CORK, &state, sizeof( state ) );
+    if( rc != 0 )
+      return XRootDStatus( stFatal, errSocketOptError, errno );
+#endif
+    pCorked = false;
+    return XRootDStatus();
+  }
+
+  //------------------------------------------------------------------------
+  // Flash the underlying socket
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::Flash()
+  {
+    //----------------------------------------------------------------------
+    // Uncork the socket in order to flash the socket
+    //----------------------------------------------------------------------
+    XRootDStatus st = Uncork();
+    if( !st.IsOK() ) return st;
+
+    //----------------------------------------------------------------------
+    // Once the data has been flashed we can cork the socket back
+    //----------------------------------------------------------------------
+    return Cork();
+  }
+
+  //------------------------------------------------------------------------
+  // Do special event mapping if applicable
+  //------------------------------------------------------------------------
+  uint8_t Socket::MapEvent( uint8_t event )
+  {
+    if( pTls ) return pTls->MapEvent( event );
+    return event;
+  }
+
+  //------------------------------------------------------------------------
+  // Enable encryption
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::TlsHandShake( AsyncSocketHandler *socketHandler,
+                                   const std::string  &thehost )
+  {
+    try
+    {
+      if( !pTls ) pTls.reset( new Tls( this, socketHandler ) );
+      return pTls->Connect( thehost, &pServerAddr );
+    }
+    catch( std::exception& ex )
+    {
+      // the exception has been thrown when we tried to create
+      // the TLS context
+      return XRootDStatus( stFatal, errTlsError, 0, ex.what() );
+    }
+
+    return XRootDStatus();
+  }
+
+  //------------------------------------------------------------------------
+  // @return : true if socket is using TLS layer for encryption,
+  //           false otherwise
+  //------------------------------------------------------------------------
+  bool Socket::IsEncrypted()
+  {
+    return bool( pTls.get() );
+  }
+
 }
+
+
