@@ -17,6 +17,7 @@
 //------------------------------------------------------------------------------
 
 #include "XrdCl/XrdClStatus.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XProtocol/XProtocol.hh"
 #include <cstring>
 
@@ -30,45 +31,47 @@ namespace
   };
 
   ErrorMap errors[] = {
-    { errUnknown,            "Unknown error"        },
-    { errInvalidOp,          "Invalid operation"    },
-    { errFcntl,              "Fcntl error"          },
-    { errPoll,               "Poll error"           },
-    { errConfig,             "Configuration error"  },
-    { errInternal,           "Internal error"       },
-    { errUnknownCommand,     "Command not found"    },
-    { errInvalidArgs,        "Invalid arguments"    },
-    { errInProgress,         "Operation in progress" },
-    { errUninitialized,      "Initialization error" },
-    { errOSError,            "OS Error"             },
-    { errNotSupported,       "Operation not supported" },
-    { errDataError,          "Received corrupted data" },
-    { errNotImplemented,     "Operation is not implemented" },
-    { errNoMoreReplicas,     "No more replicas to try" },
-    { errInvalidAddr,        "Invalid address"      },
-    { errSocketError,        "Socket error"         },
-    { errSocketTimeout,      "Socket timeout"       },
-    { errSocketDisconnected, "Socket disconnected"  },
-    { errPollerError,        "Poller error"         },
-    { errSocketOptError,     "Socket opt error"     },
-    { errStreamDisconnect,   "Stream disconnect"    },
-    { errConnectionError,    "Connection error"     },
-    { errInvalidSession,     "Invalid session"      },
-    { errInvalidMessage,     "Invalid message"      },
-    { errNotFound,           "Resource not found"   },
-    { errCheckSumError,      "CheckSum error"       },
-    { errRedirectLimit,      "Redirect limit has been reached" },
-    { errHandShakeFailed,    "Hand shake failed"    },
-    { errLoginFailed,        "Login failed"         },
-    { errAuthFailed,         "Auth failed"          },
-    { errQueryNotSupported,  "Query not supported"  },
-    { errOperationExpired,   "Operation expired"    },
-    { errNoMoreFreeSIDs,     "No more free SIDs"    },
-    { errInvalidRedirectURL, "Invalid redirect URL" },
-    { errInvalidResponse,    "Invalid response"     },
-    { errRedirect,           "Unhandled redirect"   },
-    { errErrorResponse,      "Error response"       },
-    { errResponseNegative,   "Query response negative" },
+    { errUnknown,              "Unknown error"        },
+    { errInvalidOp,            "Invalid operation"    },
+    { errFcntl,                "Fcntl error"          },
+    { errPoll,                 "Poll error"           },
+    { errConfig,               "Configuration error"  },
+    { errInternal,             "Internal error"       },
+    { errUnknownCommand,       "Command not found"    },
+    { errInvalidArgs,          "Invalid arguments"    },
+    { errInProgress,           "Operation in progress" },
+    { errUninitialized,        "Initialization error" },
+    { errOSError,              "OS Error"             },
+    { errNotSupported,         "Operation not supported" },
+    { errDataError,            "Received corrupted data" },
+    { errNotImplemented,       "Operation is not implemented" },
+    { errNoMoreReplicas,       "No more replicas to try" },
+    { errInvalidAddr,          "Invalid address"      },
+    { errSocketError,          "Socket error"         },
+    { errSocketTimeout,        "Socket timeout"       },
+    { errSocketDisconnected,   "Socket disconnected"  },
+    { errPollerError,          "Poller error"         },
+    { errSocketOptError,       "Socket opt error"     },
+    { errStreamDisconnect,     "Stream disconnect"    },
+    { errConnectionError,      "Connection error"     },
+    { errInvalidSession,       "Invalid session"      },
+    { errTlsError,             "TLS error"            },
+    { errInvalidMessage,       "Invalid message"      },
+    { errNotFound,             "Resource not found"   },
+    { errCheckSumError,        "CheckSum error"       },
+    { errRedirectLimit,        "Redirect limit has been reached" },
+    { errHandShakeFailed,      "Hand shake failed"    },
+    { errLoginFailed,          "Login failed"         },
+    { errAuthFailed,           "Auth failed"          },
+    { errQueryNotSupported,    "Query not supported"  },
+    { errOperationExpired,     "Operation expired"    },
+    { errOperationInterrupted, "Operation interrupted" },
+    { errNoMoreFreeSIDs,       "No more free SIDs"    },
+    { errInvalidRedirectURL,   "Invalid redirect URL" },
+    { errInvalidResponse,      "Invalid response"     },
+    { errRedirect,             "Unhandled redirect"   },
+    { errErrorResponse,        "Error response"       },
+    { errResponseNegative,     "Query response negative" },
     { 0, 0 } };
 
   //----------------------------------------------------------------------------
@@ -123,9 +126,9 @@ namespace XrdCl
     if( errNo >= kXR_ArgInvalid ) // kXR_ArgInvalid is the first (lowest) xrootd error code
       // it is used in an inconsistent way sometimes it is
       // xrootd error code and sometimes it is a plain errno
-      o << ": " << strerror( XProtocol::toErrno( errNo ) );
+      o << ": " << XrdSysE2T( XProtocol::toErrno( errNo ) );
     else if ( errNo )
-      o << ": " << strerror( errNo );
+      o << ": " << XrdSysE2T( errNo );
 
     return o.str();
   }
