@@ -32,9 +32,10 @@
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClOutQueue.hh"
 #include "XrdCl/XrdClMonitor.hh"
-#include "XrdCl/XrdClAsyncSocketHandler.hh"
 #include "XrdCl/XrdClMessageUtils.hh"
 #include "XrdCl/XrdClXRootDTransport.hh"
+#include "XrdCl/XrdClXRootDMsgHandler.hh"
+#include "XrdClAsyncSocketHandler.hh"
 
 #include <sys/types.h>
 #include <algorithm>
@@ -107,9 +108,8 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Constructor
   //----------------------------------------------------------------------------
-  Stream::Stream( const URL *url, uint16_t streamNum ):
+  Stream::Stream( const URL *url ):
     pUrl( url ),
-    pStreamNum( streamNum ),
     pTransport( 0 ),
     pPoller( 0 ),
     pTaskManager( 0 ),
@@ -129,7 +129,7 @@ namespace XrdCl
     pConnectionDone.tv_sec = 0;    pConnectionDone.tv_usec = 0;
 
     std::ostringstream o;
-    o << pUrl->GetHostId() << " #" << pStreamNum;
+    o << pUrl->GetHostId();
     pStreamName = o.str();
 
     pConnectionWindow  = Utils::GetIntParameter( *url, "ConnectionWindow",
@@ -147,7 +147,7 @@ namespace XrdCl
     Log *log = DefaultEnv::GetLog();
     log->Debug( PostMasterMsg, "[%s] Stream parameters: Network Stack: %s, "
                 "Connection Window: %d, ConnectionRetry: %d, Stream Error "
-                "Widnow: %d", pStreamName.c_str(), netStack.c_str(),
+                "Window: %d", pStreamName.c_str(), netStack.c_str(),
                 pConnectionWindow, pConnectionRetry, pStreamErrorWindow );
   }
 
@@ -162,7 +162,7 @@ namespace XrdCl
     log->Debug( PostMasterMsg, "[%s] Destroying stream",
                 pStreamName.c_str() );
 
-    MonitorDisconnection( Status() );
+    MonitorDisconnection( XRootDStatus() );
 
     SubStreamList::iterator it;
     for( it = pSubStreams.begin(); it != pSubStreams.end(); ++it )
@@ -174,27 +174,25 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Initializer
   //----------------------------------------------------------------------------
-  Status Stream::Initialize()
+  XRootDStatus Stream::Initialize()
   {
     if( !pTransport || !pPoller || !pChannelData )
-      return Status( stError, errUninitialized );
+      return XRootDStatus( stError, errUninitialized );
 
-    AsyncSocketHandler *s = new AsyncSocketHandler( pPoller,
-                                                    pTransport,
-                                                    pChannelData,
-                                                    0 );
+    AsyncSocketHandler *s = new AsyncSocketHandler( *pUrl, pPoller, pTransport,
+                                                    pChannelData, 0 );
     s->SetStream( this );
 
     pSubStreams.push_back( new SubStreamData() );
     pSubStreams[0]->socket = s;
-    return Status();
+    return XRootDStatus();
   }
 
   //------------------------------------------------------------------------
   // Make sure that the underlying socket handler gets write readiness
   // events
   //------------------------------------------------------------------------
-  Status Stream::EnableLink( PathID &path )
+  XRootDStatus Stream::EnableLink( PathID &path )
   {
     XrdSysMutexHelper scopedLock( pMutex );
 
@@ -204,7 +202,7 @@ namespace XrdCl
     // all the other streams
     //--------------------------------------------------------------------------
     if( pSubStreams[0]->status == Socket::Connecting )
-      return Status();
+      return XRootDStatus();
 
     //--------------------------------------------------------------------------
     // The main stream is connected, so we can verify whether we have
@@ -225,7 +223,7 @@ namespace XrdCl
       if( pSubStreams[path.up]->status == Socket::Connected )
         return pSubStreams[path.up]->socket->EnableUplink();
 
-      return Status();
+      return XRootDStatus();
     }
 
     //--------------------------------------------------------------------------
@@ -245,7 +243,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Resolve all the addresses of the host we're supposed to connect to
     //--------------------------------------------------------------------------
-    Status st = Utils::GetHostAddresses( pAddresses, *pUrl, pAddressType );
+    XRootDStatus st = Utils::GetHostAddresses( pAddresses, *pUrl, pAddressType );
     if( !st.IsOK() )
     {
       log->Error( PostMasterMsg, "[%s] Unable to resolve IP address for "
@@ -277,10 +275,10 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Queue the message for sending
   //----------------------------------------------------------------------------
-  Status Stream::Send( Message              *msg,
-                       OutgoingMsgHandler   *handler,
-                       bool                  stateful,
-                       time_t                expires )
+  XRootDStatus Stream::Send( Message              *msg,
+                             OutgoingMsgHandler   *handler,
+                             bool                  stateful,
+                             time_t                expires )
   {
     XrdSysMutexHelper scopedLock( pMutex );
     Log *log = DefaultEnv::GetLog();
@@ -291,17 +289,16 @@ namespace XrdCl
     if( msg->GetSessionId() &&
         (pSubStreams[0]->status != Socket::Connected ||
         pSessionId != msg->GetSessionId()) )
-      return Status( stError, errInvalidSession );
+      return XRootDStatus( stError, errInvalidSession );
 
     //--------------------------------------------------------------------------
     // Decide on the path to send the message
     //--------------------------------------------------------------------------
-    PathID path = pTransport->MultiplexSubStream( msg, pStreamNum,
-                                                  *pChannelData );
+    PathID path = pTransport->MultiplexSubStream( msg, *pChannelData );
     if( pSubStreams.size() <= path.up )
     {
       log->Warning( PostMasterMsg, "[%s] Unable to send message %s through "
-                    "substream %d using 0 instead", pStreamName.c_str(),
+                    "substream %d, using 0 instead", pStreamName.c_str(),
                     msg->GetDescription().c_str(), path.up );
       path.up = 0;
     }
@@ -313,10 +310,10 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Enable *a* path and insert the message to the right queue
     //--------------------------------------------------------------------------
-    Status st = EnableLink( path );
+    XRootDStatus st = EnableLink( path );
     if( st.IsOK() )
     {
-      pTransport->MultiplexSubStream( msg, pStreamNum, *pChannelData, &path );
+      pTransport->MultiplexSubStream( msg, *pChannelData, &path );
       pSubStreams[path.up]->outQueue->PushBack( msg, handler,
                                                 expires, stateful );
     }
@@ -333,7 +330,7 @@ namespace XrdCl
     XrdSysMutexHelper scopedLock( pMutex );
     pSubStreams[0]->status = Socket::Disconnected;
     XrdCl::PathID path( 0, 0 );
-    XrdCl::Status st = EnableLink( path );
+    XrdCl::XRootDStatus st = EnableLink( path );
     if( !st.IsOK() )
       OnConnectError( 0, st );
   }
@@ -367,9 +364,8 @@ namespace XrdCl
       q.GrabExpired( *(*it)->outQueue, now );
     pMutex.UnLock();
 
-    q.Report( Status( stError, errOperationExpired ) );
-    if( pStreamNum == 0 )
-      pIncomingQueue->ReportTimeout( now );
+    q.Report( XRootDStatus( stError, errOperationExpired ) );
+    pIncomingQueue->ReportTimeout( now );
   }
 }
 
@@ -408,10 +404,10 @@ namespace
 
 namespace XrdCl
 {
-  Status Stream::RequestClose( Message *response )
+  XRootDStatus Stream::RequestClose( Message *response )
   {
     ServerResponse *rsp = reinterpret_cast<ServerResponse*>( response->GetBuffer() );
-    if( rsp->hdr.dlen < 4 ) return Status( stError );
+    if( rsp->hdr.dlen < 4 ) return XRootDStatus( stError );
     Message            *msg;
     ClientCloseRequest *req;
     MessageUtils::CreateRequest( msg, req );
@@ -438,8 +434,7 @@ namespace XrdCl
     msg->SetSessionId( pSessionId );
     pBytesReceived += bytesReceived;
 
-    uint32_t streamAction = pTransport->MessageReceived( msg, pStreamNum,
-                                                         subStream,
+    uint32_t streamAction = pTransport->MessageReceived( msg, subStream,
                                                          *pChannelData );
     if( streamAction & TransportHandler::DigestMsg )
       return;
@@ -479,9 +474,19 @@ namespace XrdCl
     {
       log->Dump( PostMasterMsg, "[%s] Ignoring the processing handler for: 0x%x.",
                  pStreamName.c_str(), msg->GetDescription().c_str() );
+
+      // if we are handling oksofar we have to take down the timeout fence
+      ServerResponse *rsp = (ServerResponse *)msg->GetBuffer();
+      if( rsp->hdr.status == kXR_oksofar )
+      {
+        XRootDMsgHandler *xrdHandler = dynamic_cast<XRootDMsgHandler*>( mh.handler );
+        if( xrdHandler ) xrdHandler->TakeDownTimeoutFence();
+      }
+
       bool delit = ( mh.action & IncomingMsgHandler::Ignore );
       mh.Reset();
       if (delit) delete msg;
+
       return;
     }
 
@@ -513,7 +518,7 @@ namespace XrdCl
                                                           h.stateful );
     scopedLock.UnLock();
     if( h.handler )
-      h.handler->OnReadyToSend( h.msg, pStreamNum );
+      h.handler->OnReadyToSend( h.msg );
     return std::make_pair( h.msg, h.handler );
   }
 
@@ -537,12 +542,12 @@ namespace XrdCl
                               Message  *msg,
                               uint32_t  bytesSent )
   {
-    pTransport->MessageSent( msg, pStreamNum, subStream, bytesSent,
+    pTransport->MessageSent( msg, subStream, bytesSent,
                              *pChannelData );
     OutMessageHelper &h = pSubStreams[subStream]->outMsgHelper;
     pBytesSent += bytesSent;
     if( h.handler )
-      h.handler->OnStatusReady( msg, Status() );
+      h.handler->OnStatusReady( msg, XRootDStatus() );
     pSubStreams[subStream]->outMsgHelper.Reset();
   }
 
@@ -560,7 +565,7 @@ namespace XrdCl
     if( subStream == 0 )
     {
       pLastStreamError = 0;
-      pLastFatalError  = Status();
+      pLastFatalError  = XRootDStatus();
       pConnectionCount = 0;
       uint16_t numSub = pTransport->SubStreamNumber( *pChannelData );
       ++pSessionId;
@@ -572,8 +577,8 @@ namespace XrdCl
       {
         for( uint16_t i = 1; i < numSub; ++i )
         {
-          AsyncSocketHandler *s = new AsyncSocketHandler( pPoller, pTransport,
-                                                          pChannelData, i );
+          AsyncSocketHandler *s = new AsyncSocketHandler( *pUrl, pPoller,
+                                                   pTransport, pChannelData, i );
           s->SetStream( this );
           pSubStreams.push_back( new SubStreamData() );
           pSubStreams[i]->socket = s;
@@ -592,7 +597,7 @@ namespace XrdCl
         for( size_t i = 1; i < pSubStreams.size(); ++i )
         {
           pSubStreams[i]->socket->SetAddress( pSubStreams[0]->socket->GetAddress() );
-          Status st = pSubStreams[i]->socket->Connect( pConnectionWindow );
+          XRootDStatus st = pSubStreams[i]->socket->Connect( pConnectionWindow );
           if( !st.IsOK() )
           {
             pSubStreams[0]->outQueue->GrabItems( *pSubStreams[i]->outQueue );
@@ -628,13 +633,25 @@ namespace XrdCl
         delete qryResponse;
         mon->Event( Monitor::EvConnect, &i );
       }
+
+      //------------------------------------------------------------------------
+      // For every connected control-stream call the global on-connect handler
+      //------------------------------------------------------------------------
+      XrdCl::DefaultEnv::GetPostMaster()->NotifyConnectHandler( *pUrl );
+    }
+    else if( pOnDataConnJob )
+    {
+      //------------------------------------------------------------------------
+      // For every connected data-stream call the on-connect handler
+      //------------------------------------------------------------------------
+      pJobManager->QueueJob( pOnDataConnJob.get(), 0 );
     }
   }
 
   //----------------------------------------------------------------------------
   // On connect error
   //----------------------------------------------------------------------------
-  void Stream::OnConnectError( uint16_t subStream, Status status )
+  void Stream::OnConnectError( uint16_t subStream, XRootDStatus status )
   {
     XrdSysMutexHelper scopedLock( pMutex );
     Log *log = DefaultEnv::GetLog();
@@ -651,7 +668,7 @@ namespace XrdCl
       pSubStreams[0]->outQueue->GrabItems( *pSubStreams[subStream]->outQueue );
       if( pSubStreams[0]->status == Socket::Connected )
       {
-        Status st = pSubStreams[0]->socket->EnableUplink();
+        XRootDStatus st = pSubStreams[0]->socket->EnableUplink();
         if( !st.IsOK() )
           OnFatalError( 0, st, scopedLock );
         return;
@@ -676,7 +693,7 @@ namespace XrdCl
     //------------------------------------------------------------------------
     if( !pAddresses.empty() )
     {
-      Status st;
+      XRootDStatus st;
       do
       {
         pSubStreams[0]->socket->SetAddress( pAddresses.back() );
@@ -714,7 +731,7 @@ namespace XrdCl
       pAddresses.clear();
       pSubStreams[0]->status = Socket::Disconnected;
       PathID path( 0, 0 );
-      Status st = EnableLink( path );
+      XRootDStatus st = EnableLink( path );
       if( !st.IsOK() )
         OnFatalError( subStream, st, scopedLock );
       return;
@@ -729,7 +746,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Call back when an error has occurred
   //----------------------------------------------------------------------------
-  void Stream::OnError( uint16_t subStream, Status status )
+  void Stream::OnError( uint16_t subStream, XRootDStatus status )
   {
     XrdSysMutexHelper scopedLock( pMutex );
     Log *log = DefaultEnv::GetLog();
@@ -775,7 +792,7 @@ namespace XrdCl
         pSubStreams[0]->outQueue->GrabItems( *pSubStreams[subStream]->outQueue );
         if( pSubStreams[0]->status == Socket::Connected )
         {
-          Status st = pSubStreams[0]->socket->EnableUplink();
+          XRootDStatus st = pSubStreams[0]->socket->EnableUplink();
           if( !st.IsOK() )
             OnFatalError( 0, st, scopedLock );
           return;
@@ -802,7 +819,7 @@ namespace XrdCl
       if( outstanding )
       {
         PathID path( 0, 0 );
-        Status st = EnableLink( path );
+        XRootDStatus st = EnableLink( path );
         if( !st.IsOK() )
         {
           OnFatalError( 0, st, scopedLock );
@@ -822,19 +839,75 @@ namespace XrdCl
       scopedLock.UnLock();
 
       q.Report( status );
-      pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::Broken,
-                                         pStreamNum, status );
-      pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status,
-                                      pStreamNum );
+      pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::Broken, status );
+      pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status );
       return;
     }
   }
 
+  //------------------------------------------------------------------------
+  // Force error
+  //------------------------------------------------------------------------
+  void Stream::ForceError( XRootDStatus status )
+  {
+    XrdSysMutexHelper scopedLock( pMutex );
+    for( size_t substream = 0; substream < pSubStreams.size(); ++substream )
+    {
+      Log    *log = DefaultEnv::GetLog();
+      if( pSubStreams[substream]->status != Socket::Connected ) continue;
+      pSubStreams[substream]->socket->Close();
+      pSubStreams[substream]->status = Socket::Disconnected;
+      log->Error( PostMasterMsg, "[%s] Forcing error on disconnect: %s.",
+                  pStreamName.c_str(), status.ToString().c_str() );
+
+      //--------------------------------------------------------------------
+      // Reinsert the stuff that we have failed to sent
+      //--------------------------------------------------------------------
+      if( pSubStreams[substream]->outMsgHelper.msg )
+      {
+        OutMessageHelper &h = pSubStreams[substream]->outMsgHelper;
+        pSubStreams[substream]->outQueue->PushFront( h.msg, h.handler, h.expires,
+                                                     h.stateful );
+        pSubStreams[substream]->outMsgHelper.Reset();
+      }
+
+      //--------------------------------------------------------------------
+      // Reinsert the receiving handler
+      //--------------------------------------------------------------------
+      if( pSubStreams[substream]->inMsgHelper.handler )
+      {
+        InMessageHelper &h = pSubStreams[substream]->inMsgHelper;
+        pIncomingQueue->ReAddMessageHandler( h.handler, h.expires );
+        h.Reset();
+      }
+
+      pConnectionCount = 0;
+
+      //------------------------------------------------------------------------
+      // We're done here, unlock the stream mutex to avoid deadlocks and
+      // report the disconnection event to the handlers
+      //------------------------------------------------------------------------
+      log->Debug( PostMasterMsg, "[%s] Reporting disconnection to queued "
+                  "message handlers.", pStreamName.c_str() );
+
+      SubStreamList::iterator it;
+      OutQueue q;
+      for( it = pSubStreams.begin(); it != pSubStreams.end(); ++it )
+        q.GrabItems( *(*it)->outQueue );
+      scopedLock.UnLock();
+
+      q.Report( status );
+
+      pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::Broken, status );
+      pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status );
+    }
+  }
+
   //----------------------------------------------------------------------------
   // On fatal error
   //----------------------------------------------------------------------------
   void Stream::OnFatalError( uint16_t           subStream,
-                             Status             status,
+                             XRootDStatus       status,
                              XrdSysMutexHelper &lock )
   {
     Log    *log = DefaultEnv::GetLog();
@@ -842,9 +915,16 @@ namespace XrdCl
     log->Error( PostMasterMsg, "[%s] Unable to recover: %s.",
                 pStreamName.c_str(), status.ToString().c_str() );
 
-    pConnectionCount = 0;
-    pLastStreamError = ::time(0);
-    pLastFatalError  = status;
+    //--------------------------------------------------------------------------
+    // Don't set the stream error windows for authentication errors as the user
+    // may refresh his credential at any time
+    //--------------------------------------------------------------------------
+    if( status.code != errAuthFailed )
+    {
+      pConnectionCount = 0;
+      pLastStreamError = ::time(0);
+      pLastFatalError  = status;
+    }
 
     SubStreamList::iterator it;
     OutQueue q;
@@ -854,17 +934,15 @@ namespace XrdCl
 
     status.status = stFatal;
     q.Report( status );
-    pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::FatalError,
-                                       pStreamNum, status );
-    pChannelEvHandlers.ReportEvent( ChannelEventHandler::FatalError, status,
-                                    pStreamNum );
+    pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::FatalError, status );
+    pChannelEvHandlers.ReportEvent( ChannelEventHandler::FatalError, status );
 
   }
 
   //----------------------------------------------------------------------------
   // Inform monitoring about disconnection
   //----------------------------------------------------------------------------
-  void Stream::MonitorDisconnection( Status status )
+  void Stream::MonitorDisconnection( XRootDStatus status )
   {
     Monitor *mon = DefaultEnv::GetMonitor();
     if( mon )
@@ -916,21 +994,28 @@ namespace XrdCl
     if( !outgoingMessages )
     {
       bool disconnect = pTransport->IsStreamTTLElapsed( now-lastActivity,
-                                                        pStreamNum,
                                                         *pChannelData );
       if( disconnect )
       {
         log->Debug( PostMasterMsg, "[%s] Stream TTL elapsed, disconnecting...",
                     pStreamName.c_str() );
-        Disconnect();
+        scopedLock.UnLock();
+        //----------------------------------------------------------------------
+        // Important note!
+        //
+        // This destroys the Stream object itself, the underlined
+        // AsyncSocketHandler object (that called this method) and the Channel
+        // object that aggregates this Stream.
+        //----------------------------------------------------------------------
+        DefaultEnv::GetPostMaster()->ForceDisconnect( *pUrl );
+        return;
       }
     }
 
     //--------------------------------------------------------------------------
     // Check if the stream is broken
     //--------------------------------------------------------------------------
-    Status st = pTransport->IsStreamBroken( now-lastActivity,
-                                            pStreamNum,
+    XRootDStatus st = pTransport->IsStreamBroken( now-lastActivity,
                                             *pChannelData );
     if( !st.IsOK() )
     {
