@@ -308,9 +308,6 @@ void XrdClientPhyConnection::Disconnect()
 
 //____________________________________________________________________________
 bool XrdClientPhyConnection::CheckAutoTerm() {
-   bool doexit = FALSE;
-
-  {
    XrdSysMutexHelper l(fMutex);
 
    // Parametric asynchronous stuff
@@ -320,25 +317,9 @@ bool XrdClientPhyConnection::CheckAutoTerm() {
          Info(XrdClientDebug::kHIDEBUG,
               "CheckAutoTerm", "Self-Cancelling reader thread.");
 
-         {
-            XrdSysMutexHelper l(fMutex);
-            fReaderthreadrunning--;
-         }
-
-         //delete fSocket;
-         //fSocket = 0;
-
-         doexit = TRUE;
-      }
-
-  }
-
-
-  if (doexit) {
-	UnlockChannel();
-        return true;
+         fReaderthreadrunning--;
+         return true;
    }
-
   return false;
 }
 
@@ -708,7 +689,7 @@ ERemoteServerType XrdClientPhyConnection::DoHandShake(ServerInitHandShake &xbody
    ERemoteServerType typeres = kSTNone;
    int isPS = (substreamid < 0);
 
-   int writeres, readres, len;
+   int writeres, readres, len = 0;
 
    // Set field in network byte order
    memset(&initHS, 0, sizeof(initHS));
