@@ -118,8 +118,6 @@ XrdCmsCluster::XrdCmsCluster()
      SelRcnt = 0;
      SelRtot = 0;
      SelTcnt = 0;
-     doReset = 0;
-     resetMask = 0;
      peerHost  = 0;
      peerMask  = ~peerHost;
 }
@@ -268,8 +266,9 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
                       nP->isBad & XrdCmsNode::isSuspend ? 0 : 1,
                       nP->isNoStage ? 0 : 1);
 
-// All done
+// All done. Return the node locked.
 //
+   nP->Lock(false);
    return nP;
 }
   
@@ -348,8 +347,7 @@ void XrdCmsCluster::BlackList(XrdOucTList *blP)
            {inBL = (blP && (blRD = XrdCmsBlackList::Present(nP->Name(), blP)));
             if ((!inBL && !(nP->isBad & XrdCmsNode::isBlisted))
             ||  ( inBL &&  (nP->isBad & XrdCmsNode::isBlisted))) continue;
-            nP->Lock(true);
-            STMutex.UnLock();
+            nP->g2nLock(STMutex); // Downgrade to only node lock
             if (inBL)
                {nP->isBad |= XrdCmsNode::isBlisted | XrdCmsNode::isDoomed;
                 if (blRD < -1)
@@ -364,8 +362,7 @@ void XrdCmsCluster::BlackList(XrdOucTList *blP)
                 nP->isBad &= ~(XrdCmsNode::isBlisted | XrdCmsNode::isDoomed);
                 Say.Emsg("Manager", nP->Name(), "removed from blacklist.");
                }
-            nP->UnLock();
-            STMutex.Lock();
+            nP->n2gLock(STMutex);
            }
        }
    STMutex.UnLock();
@@ -388,18 +385,20 @@ SMask_t XrdCmsCluster::Broadcast(SMask_t smask, const struct iovec *iod,
    STMutex.Lock();
    bmask = smask & peerMask;
 
-// Run through the table looking for nodes to send messages to
+// Run through the table looking for nodes to send messages to. We don't need
+// the node lock for this but we do need to up the reference count to keep the
+// node pointer valid for the duration of the send() (may or may not block).
 //
    for (i = 0; i <= STHi; i++)
        {if ((nP = NodeTab[i]) && nP->isNode(bmask))
-           {nP->Lock(true);
-            STMutex.UnLock();
-            if (nP->Send(iod, iovcnt, iotot) < 0) 
-               {unQueried |= nP->Mask();
-                DEBUG(nP->Ident <<" is unreachable");
-               }
-            nP->UnLock();
-            STMutex.Lock();
+           {if (nP->isOffline) unQueried |= nP->Mask();
+               else {nP->g2Ref(STMutex);
+                     if (nP->Send(iod, iovcnt, iotot) < 0)
+                        {unQueried |= nP->Mask();
+                         DEBUG(nP->Ident <<" is unreachable");
+                        }
+                     nP->Ref2g(STMutex);
+                    }
            }
        }
    STMutex.UnLock();
@@ -466,16 +465,17 @@ int XrdCmsCluster::Broadsend(SMask_t Who, XrdCms::CmsRRHdr &Hdr,
    Beg = Start = (Start <= STHi ? Start+1 : 0);
    Fin = STHi;
 
-// Run through the table looking for a node to send the message to
+// Run through the table looking for nodes to send messages to. We don't need
+// the node lock for this but we do need to up the reference count to keep the
+// node pointer valid for the duration of the send() (may or may not block).
 //
 do{for (i = Beg; i <= Fin; i++)
        {if ((nP = NodeTab[i]) && nP->isNode(Who))
-           {nP->Lock(true);
-            STMutex.UnLock();
+           {if (nP->isOffline) continue;
+            nP->g2Ref(STMutex);
             if (nP->Send(ioV, 2, ioTot) >= 0) {nP->UnLock(); return 1;}
             DEBUG(nP->Ident <<" is unreachable");
-            nP->UnLock();
-            STMutex.Lock();
+            nP->Ref2g(STMutex);
            }
        }
     if (!Beg) break;
@@ -610,7 +610,7 @@ int XrdCmsCluster::Locate(XrdCmsSelect &Sel)
 //
    if (!Cache.Paths.Find(Path, pinfo) || !pinfo.rovec)
       {Sel.Vec.hf = Sel.Vec.pf = Sel.Vec.wf = 0;
-       return -1;
+       return NotFound;
       } else Sel.Vec.wf = pinfo.rwvec;
 
 // Check if this was a non-lookup request
@@ -639,7 +639,7 @@ int XrdCmsCluster::Locate(XrdCmsSelect &Sel)
        if (!(retc = SelDFS(Sel, amask, pmask, smask, 1)))
           return (Sel.Opts & XrdCmsSelect::Asap && Sel.InfoP
                 ? Cache.WT4File(Sel,Sel.Vec.hf) : Config.LUPDelay);
-       if (retc < 0) return -1;
+       if (retc < 0) return NotFound;
        return 0;
       }
 
@@ -657,7 +657,7 @@ int XrdCmsCluster::Locate(XrdCmsSelect &Sel)
 // Compute the delay, if any
 //
    if ((!qfVec && retc >= 0) || (Sel.Vec.hf && Sel.InfoP)) retc =  0;
-      else if (!(retc = Cache.WT4File(Sel, Sel.Vec.hf)))   retc = -2;
+      else if (!(retc = Cache.WT4File(Sel, Sel.Vec.hf)))   retc = Wait4CBk;
 
 // Check if we have to ask any nodes if they have the file
 //
@@ -701,8 +701,7 @@ void *XrdCmsCluster::MonPerf()
   
 void *XrdCmsCluster::MonRefs()
 {
-   XrdCmsNode *nP;
-   int  i, snooze_interval = 10*60, loopmax, loopcnt = 0;
+   int  snooze_interval = 10*60, loopmax, loopcnt = 0;
    int resetW, resetR, resetWR;
 
 // Compute snooze interval
@@ -722,23 +721,12 @@ void *XrdCmsCluster::MonRefs()
        resetW  = (SelWcnt >= Config.RefTurn);
        resetR  = (SelRcnt >= Config.RefTurn);
        resetWR = (loopmax && loopcnt >= loopmax && (resetW || resetR));
-       if (doReset || resetWR)
-           {for (i = 0; i <= STHi; i++)
-                if ((nP = NodeTab[i])
-                &&  (resetWR || (doReset && nP->isNode(resetMask))) )
-                    {nP->Lock(true);
-                     if (resetW || doReset) nP->RefW=0;
-                     if (resetR || doReset) nP->RefR=0;
-                     nP->Shrem = nP->Share;
-                     nP->UnLock();
-                    }
-            if (resetWR)
-               {if (resetW) {SelWtot += SelWcnt; SelWcnt = 0;}
-                if (resetR) {SelRtot += SelRcnt; SelRcnt = 0;}
-                loopcnt = 0;
-               }
-            if (doReset) {doReset = 0; resetMask = 0;}
-           }
+       if (resetWR)
+          {ResetRef((SMask_t)0, true);
+           if (resetW) {SelWtot += SelWcnt; SelWcnt = 0;}
+           if (resetR) {SelRtot += SelRcnt; SelRcnt = 0;}
+           loopcnt = 0;
+          }
        STMutex.UnLock();
       } while(1);
    return (void *)0;
@@ -814,23 +802,26 @@ void XrdCmsCluster::Remove(const char *reason, XrdCmsNode *theNode, int immed)
 
 // Mark node as being offline and remove any drop job from it
 //
-   theNode->isOffline = 1;
+   theNode->isOffline = 1; // STMutex is held here
 
-// If the node is connected the simply close the connection. This will cause
-// the connection handler to re-initiate the node removal. The LockHandler
-// destructor will release the node table and node object locks as needed.
-// This condition exists only if one node is being displaced by another node.
+// If the node is connected we simply close the connection. This will cause
+// the connection handler to re-initiate the node removal. This condition
+// exists only if one node is being displaced by another node. The Disc()
+// may take a long time, but it's done async by default on the WAN and sync
+// on the LAN (local connections are fast enough and error-free for this).
 //
    if (theNode->isConn)
       {theNode->Disc(reason, 0);
-       theNode->isGone = 1;
+       theNode->isGone = 1; // Disc() sets the isOffline flag
        return;
       }
 
 // If we are not the primary node, then get rid of this node post-haste
 //
    if (!(NodeTab[NodeID] == theNode))
-      {Say.Emsg("Remove_Node", theNode->Ident, "dropped as alternate.");
+      {const char *why = (theNode->isMan ? "dropped as alternate."
+                                         : "dropped and redirected.");
+       Say.Emsg("Remove_Node", theNode->Ident, why);
        LockHandler.doDrop = true;
        return;
       }
@@ -893,21 +884,28 @@ void XrdCmsCluster::Remove(const char *reason, XrdCmsNode *theNode, int immed)
 /*                              R e s e t R e f                               */
 /******************************************************************************/
   
-void XrdCmsCluster::ResetRef(SMask_t smask)
+void XrdCmsCluster::ResetRef(SMask_t nMask, bool isLocked)
 {
+   XrdCmsNode *nP;
+   bool doAll (nMask == 0);
 
-// Obtain a lock on the table
+// Obtain a lock on the table if not already locked
 //
-   STMutex.Lock();
+   if (!isLocked) STMutex.Lock();
 
-// Inform the reset thread that we need a reset
+// Reset reference counts as needed
 //
-   doReset = 1;
-   resetMask |= smask;
+   for (int i = 0; i <= STHi; i++)
+       {if ((nP = NodeTab[i]) && (doAll || nP->isNode(nMask)))
+           {nP->RefW  = 0;
+            nP->RefR  = 0;
+            nP->Shrem = nP->Share;
+           }
+       }
 
 // Unlock table and exit
 //
-   STMutex.UnLock();
+   if (!isLocked) STMutex.UnLock();
 }
 
 /******************************************************************************/
@@ -939,7 +937,8 @@ int XrdCmsCluster::Select(XrdCmsSelect &Sel)
    || (amask = ((isRW ? pinfo.rwvec : pinfo.rovec) & ~Sel.nmask)) == 0)
       {Sel.Resp.DLen = snprintf(Sel.Resp.Data, sizeof(Sel.Resp.Data)-1,
                        "No servers have %s access to the file", Amode)+1;
-       return -1;
+       Sel.Resp.Port = kYR_ENOENT;
+       return EReplete;
       }
 
 // If we are running a shared file system preform an optional restricted
@@ -963,7 +962,7 @@ int XrdCmsCluster::Select(XrdCmsSelect &Sel)
           {Sel.Resp.DLen = 0;
            if (!(retc = SelDFS(Sel, amask, pmask, smask, isRW)))
               return (fRD ? Cache.WT4File(Sel,Sel.Vec.hf) : Config.LUPDelay);
-           if (retc < 0) return -1;
+           if (retc < 0) return retc;
           } else if (noSel) return 0;
        return SelNode(Sel, pmask, smask);
       }
@@ -1086,21 +1085,25 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
 //
    selR.needNet = XrdNetIF::Mask(nType);
 
+// Initialize
+//
+   selR.needSpace = 0;
+
 // Packed selection can never occur in this code path so we turn it off
 //
-   selR.selPack = false;
+   selR.selPack = 0;
 
 // If we are exporting a shared-everything system then the incomming mask
 // may have more than one server indicated. So, we need to do a full select.
-// This is forced when isMulti is true, indicating a choice may exist.
+// This is forced when isMulti is true, indicating a choice may exist. Note
+// that the node, if any, is returned unlocked but we have the global mutex.
 //
    if (isMulti || baseFS.isDFS())
       {STMutex.Lock();
        nP = (Config.sched_RR ? SelbyRef(pmask,selR) : SelbyLoad(pmask,selR));
+       if (nP) hlen = nP->netIF.GetName(hbuff, port, nType) + 1;
+          else hlen = 0;
        STMutex.UnLock();
-       if (!nP) return 0;
-       hlen = nP->netIF.GetName(hbuff, port, nType) + 1;
-       nP->UnLock();
        return hlen != 1;
       }
 
@@ -1121,20 +1124,20 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
        if (nP)
           {if (isrw)
               if (nP->isNoStage || nP->DiskFree < nP->DiskMinF)    nP = 0;
-                 else {SelWcnt++; nP->RefTotW++; nP->RefW++; nP->Lock(true);}
-              else    {SelRcnt++; nP->RefTotR++; nP->RefR++; nP->Lock(true);}
+                 else {SelWcnt++; nP->RefTotW++; nP->RefW++;}
+              else    {SelRcnt++; nP->RefTotR++; nP->RefR++;}
           }
       }
-   STMutex.UnLock();
 
 // At this point either we have a node or we do not
 //
    if (nP)
       {hlen = nP->netIF.GetName(hbuff, port, nType) + 1;
        nP->RefR++;
-       nP->UnLock();
+       STMutex.UnLock();
        return hlen != 1;
       }
+   STMutex.UnLock();
    return 0;
 }
 
@@ -1145,27 +1148,45 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
 int XrdCmsCluster::SelFail(XrdCmsSelect &Sel, int rc)
 {
 //
-    const char *etext;
+    const char *etext, *Item = "file";
 
     switch(rc)
-   {case eExists: etext = "Unable to create new file; file already exists.";
+   {case eExists: if (Sel.Opts & XrdCmsSelect::isMeta) Item = "directory";
+                  etext = "Unable to create %s; it already exists.";
+                  Sel.Resp.Port = kYR_RWConflict;
                   break;
-    case eROfs:   etext = "Unable to write file; r/o file already exists.";
+    case eROfs:   etext = "Unable to modify %s; r/o copy already exists.";
+                  Sel.Resp.Port = kYR_RWConflict;
                   break;
-    case eDups:   etext = "Unable to write file; multiple files exist.";
+    case eDups:   etext = "Unable to modify %s; multiple copies exist.";
+                  Sel.Resp.Port = kYR_RWConflict;
                   break;
-    case eNoRep:  etext = "Unable to replicate file; no new sites available.";
+    case eNoRep:  etext = "Unable to replicate %s; no new sites available.";
+                  Sel.Resp.Port = kYR_noReplicas;
                   break;
-    case eNoSel:  etext = (Sel.Vec.hf & Sel.nmask
-                        ? "Unable to write file; eligible servers shunned."
-                        : "Unable to write file; r/w exports not found.");
+    case eNoSel:  if (Sel.Vec.hf & Sel.nmask)
+                     {etext = "Unable to access %s; eligible servers shunned.";
+                      if (Sel.Opts & XrdCmsSelect::isDir) Item = "directory";
+                     } else {
+                      if (Sel.Opts & XrdCmsSelect::Write)
+                         {etext = "Unable to write %s; r/w exports not found.";
+                         } else {
+                          etext = "Unable to access %s; it does not exist.";
+                          if (Sel.Opts & XrdCmsSelect::isDir) Item = "directory";
+                         }
+                     }
+                  Sel.Resp.Port = kYR_ENOENT;
                   break;
-    default:      etext = "Unable to access file; file does not exist.";
+    default:      etext = "Unable to access %s; it does not exist.";
+                  Sel.Resp.Port = kYR_ENOENT;
                   break;
    };
 
-    Sel.Resp.DLen = strlcpy(Sel.Resp.Data, etext, sizeof(Sel.Resp.Data))+1;
-    return -1;
+    int n = snprintf(Sel.Resp.Data, sizeof(Sel.Resp.Data), etext, Item);
+    if (n < (int)sizeof(Sel.Resp.Data)) Sel.Resp.DLen = n+1;
+       else Sel.Resp.DLen = sizeof(Sel.Resp.Data);
+
+    return EReplete;
 }
   
 /******************************************************************************/
@@ -1416,7 +1437,7 @@ int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
 // Cleanup status
 //
    NodeTab[sent] = 0;
-   nP->isOffline = 1;
+   nP->isOffline = 1; // STMutex is locked
    nP->DropTime  = 0;
    nP->DropJob   = 0;
    nP->isBound   = 0;
@@ -1539,7 +1560,7 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 {
     EPNAME("SelNode")
     const char *act=0;
-    int isalt = 0, pass = 2;
+    int affsel = 1, count = 0, isalt = 0, pass = 2;
     SMask_t mask;
     XrdCmsNode *nP = 0;
     XrdCmsSelector selR;
@@ -1551,7 +1572,12 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 
 // Indicate whether or not stable selection is required
 //
-   selR.selPack = (Sel.Opts & XrdCmsSelect::Pack) != 0;
+   if (!(Sel.Opts & XrdCmsSelect::Pack)) selR.selPack = 0;
+      else {SMask_t sVec = pmask;
+            for (count = 0; sVec; count++) sVec &= (sVec - 1);
+            if (count > 1) selR.selPack = affsel = (Sel.Path.Hash % count) + 1;
+               else        selR.selPack = 0;
+           }
 
 // There is a difference bwteen needing space and needing r/w access. The former
 // is needed when we will be writing data the latter for inode modifications.
@@ -1561,7 +1587,8 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
                           ?  XrdCmsNode::allowsRW : 0);
 
 // Scan for a primary and alternate node (alternates do staging). At this
-// point we omit all peer nodes as they are our last resort.
+// point we omit all peer nodes as they are our last resort. Note that Selbyxxx
+// returns the node unlocked but we have he global mutex so that is OK.
 //
    STMutex.Lock();
    mask = pmask & peerMask;
@@ -1575,25 +1602,45 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
          mask = amask & peerMask; isalt = XrdCmsNode::allowsSS;
          if (!(Sel.Opts & XrdCmsSelect::isMeta)) selR.needSpace |= isalt;
         }
-   STMutex.UnLock();
 
-// If we found an eligible node then dispatch the client to it.
+// Produce affinity result trace
+//
+   if (Sel.Opts & XrdCmsSelect::Pack && nP)
+      {TRACE(Redirect, "affinity " <<affsel <<'/' <<count <<'/'
+                       <<(int)selR.selPack <<(selR.selPack ? " go " : " ng ")
+                       <<nP->Name() <<' ' <<Sel.Path.Val);
+      }
+
+// If we found an eligible node then dispatch the client to it. We will
+// swap the global mutex for the node mutex to minimize interefrence.
 //
    if (nP)
-      {Sel.Resp.DLen = nP->netIF.GetName(Sel.Resp.Data, Sel.Resp.Port, nType);
+      {nP->g2nLock(STMutex);
+       Sel.Resp.DLen = nP->netIF.GetName(Sel.Resp.Data, Sel.Resp.Port, nType);
        if (!Sel.Resp.DLen) {nP->UnLock(); return Unreachable(Sel, false);}
        Sel.Resp.DLen++; Sel.smask = nP->NodeMask;
-       if (isalt || (Sel.Opts & XrdCmsSelect::Create) || Sel.iovN)
-          {if (isalt || (Sel.Opts & XrdCmsSelect::Create))
-              {Sel.Opts |= (XrdCmsSelect::Pending | XrdCmsSelect::Advisory);
-               if (Sel.Opts & XrdCmsSelect::noBind) act = " handling ";
-                  else Cache.AddFile(Sel, nP->NodeMask);
-              }
-           if (Sel.iovN && Sel.iovP) 
-              {nP->Send(Sel.iovP, Sel.iovN); act = " staging ";}
-              else if (!act)                 act = " assigned ";
-          } else                             act = " serving ";
+
+       // If a message is to be sent to the selected server, send it.
+       //
+       if (Sel.iovN && Sel.iovP) nP->Send(Sel.iovP, Sel.iovN);
+
+       // Do special post proccessing when any of:
+       // a) isalt true: Secondary selection occured
+       // b) Create set: File creation will occur
+       //
+       if (isalt || (Sel.Opts & XrdCmsSelect::Create))
+          {Sel.Opts |= (XrdCmsSelect::Pending | XrdCmsSelect::Advisory);
+           if (Sel.Opts & XrdCmsSelect::noBind) act = " handling ";
+              else Cache.AddFile(Sel, nP->NodeMask);
+          }
+
+       // Determine what we are actually doing here
+       //
        nP->UnLock();
+       if (!act)
+          {if (isalt) act = (Sel.iovN ? " staging " : " assigned ");
+              else    act = " serving ";
+          }
        TRACE(Stage, Sel.Resp.Data <<act <<Sel.Path.Val);
        return 0;
       }
@@ -1602,28 +1649,31 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 // do not forward to a peer unless we have a suffficient number of local nodes.
 //
    if (!selR.delay && NodeCnt < Config.SUPCount)
-      {Record(Sel.Path.Val, "insufficient number of nodes", true);
+      {STMutex.UnLock();
+       Record(Sel.Path.Val, "insufficient number of nodes", true);
        return Config.SUPDelay;
       }
 
 // Return delay if we should avoid selecting a peer manager
 //
    if (selR.delay && selR.delay < Config.PSDelay)
-      {Record(Sel.Path.Val, selR.reason);
+      {STMutex.UnLock();
+       Record(Sel.Path.Val, selR.reason);
        return selR.delay;
       }
 
-// At this point, we attempt a peer node selection (choice of last resort)
+// At this point, we attempt a peer node selection (choice of last resort). Note
+// that we are still holding the global lock! If we find a peer node we will
+// swap it with the node lock.
 //
    if (Sel.Opts & XrdCmsSelect::Peers)
       {const char *reason1 = selR.reason;
        int delay1 = selR.delay;
        bool noNet = selR.xNoNet;
-       STMutex.Lock();
        if ((mask = (pmask | amask) & peerHost)) nP = SelbyCost(mask, selR);
-       STMutex.UnLock();
        if (nP)
-          {Sel.Resp.DLen = nP->netIF.GetName(Sel.Resp.Data,Sel.Resp.Port,nType);
+          {nP->g2nLock(STMutex);
+           Sel.Resp.DLen = nP->netIF.GetName(Sel.Resp.Data,Sel.Resp.Port,nType);
            if (!Sel.Resp.DLen) {nP->UnLock(); return Unreachable(Sel, false);}
            Sel.Resp.DLen++; Sel.smask = nP->NodeMask;
            if (Sel.iovN && Sel.iovP) nP->Send(Sel.iovP, Sel.iovN);
@@ -1635,6 +1685,10 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
           {selR.delay = delay1; selR.reason = reason1; selR.xNoNet = noNet;}
       }
 
+// At this point we don't need the global lock so let it go.
+//
+   STMutex.UnLock();
+
 // At this point we either don't have enough nodes or simply can't handle this
 //
    if (selR.delay)
@@ -1670,6 +1724,8 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 // Cost selection is used only for peer node selection as peers do not
 // report a load and handle their own scheduling.
 
+// Caller must have the STMutex locked. The returned node. if any, is unlocked.
+
 XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
 {
     XrdCmsNode *np, *sp = 0;
@@ -1688,7 +1744,9 @@ XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
            if (!sp) sp = np;
               else{if (abs(sp->myCost - np->myCost) <= Config.P_fuzz)
                       {     if (selR.selPack)
-                               {if (sp->Inst() > np->Inst()) sp=np;}
+                               {if (--selR.selPack) sp=np;
+                                   else break;
+                               }
                        else if (selR.needSpace)
                                {if (sp->RefW > (np->RefW+Config.DiskLinger))
                                    sp=np;
@@ -1703,7 +1761,6 @@ XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
 // Check for overloaded node and return result
 //
    if (!sp) return calcDelay(selR);
-   sp->Lock(true);
    RefCount(sp, Multi, selR.needSpace);
    return sp;
 }
@@ -1711,6 +1768,8 @@ XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
 /******************************************************************************/
 /*                             S e l b y L o a d                              */
 /******************************************************************************/
+
+// Caller must have the STMutex locked. The returned node. if any, is unlocked.
   
 XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
 {
@@ -1733,16 +1792,14 @@ XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
            if (!sp) sp = np;
               else{if (selR.needSpace)
                       {if (abs(sp->myMass - np->myMass) <= Config.P_fuzz)
-                          {if (selR.selPack)
-                              {if (sp->Inst() > np->Inst())             sp=np;}
-                           else
-                           if (sp->RefW > (np->RefW+Config.DiskLinger)) sp=np;
-                          }
+                          {if (sp->RefW > (np->RefW+Config.DiskLinger)) sp=np;}
                           else if (sp->myMass > np->myMass)             sp=np;
                       } else {
                        if (abs(sp->myLoad - np->myLoad) <= Config.P_fuzz)
                           {if (selR.selPack)
-                              {if (sp->Inst() > np->Inst())             sp=np;}
+                              {if (--selR.selPack)                      sp=np;
+                                  else break;
+                              }
                               else if (sp->RefR > np->RefR)             sp=np;
                           }
                           else if (sp->myLoad > np->myLoad)             sp=np;
@@ -1754,7 +1811,6 @@ XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
 // Check for overloaded node and return result
 //
    if (!sp) return calcDelay(selR);
-   sp->Lock(true);
    RefCount(sp, Multi, selR.needSpace);
    return sp;
 }
@@ -1763,6 +1819,8 @@ XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
 /*                              S e l b y R e f                               */
 /******************************************************************************/
 
+// Caller must have the STMutex locked. The returned node. if any, is unlocked.
+
 XrdCmsNode *XrdCmsCluster::SelbyRef(SMask_t mask, XrdCmsSelector &selR)
 {
     XrdCmsNode *np, *sp = 0;
@@ -1782,7 +1840,10 @@ XrdCmsNode *XrdCmsCluster::SelbyRef(SMask_t mask, XrdCmsSelector &selR)
               {selR.xFull = true; continue;}
            if (!sp) sp = np;
               else {Multi = true;
-                         if (selR.selPack) {if (sp->Inst() > np->Inst())sp=np;}
+                         if (selR.selPack)
+                            {if (--selR.selPack) sp=np;
+                                else break;
+                            }
                     else if (selR.needSpace)
                             {if (sp->RefW > (np->RefW+Config.DiskLinger)) sp=np;}
                     else if (sp->RefR > np->RefR)                         sp=np;
@@ -1792,7 +1853,6 @@ XrdCmsNode *XrdCmsCluster::SelbyRef(SMask_t mask, XrdCmsSelector &selR)
 // Check for overloaded node and return result
 //
    if (!sp) return calcDelay(selR);
-   sp->Lock(true);
    RefCount(sp, Multi, selR.needSpace);
    return sp;
 }
@@ -1950,8 +2010,8 @@ int XrdCmsCluster::Unreachable(XrdCmsSelect &Sel, bool none)
                "Eligible server is unreachable via %s network to %s%s the file.",
                XrdNetIF::Name(nType), Xmode, Amode) + 1;
       }
-
-   return -1;
+   Sel.Resp.Port = kYR_ENETUNREACH;
+   return EReplete;
 }
   
 /******************************************************************************/
@@ -1962,8 +2022,14 @@ int XrdCmsCluster::Unuseable(XrdCmsSelect &Sel)
 {
    const char *Amode = (Sel.Opts & XrdCmsSelect::Write  ? "write" : "read");
    const char *Xmode = (Sel.Opts & XrdCmsSelect::Online ? "immediately " : "");
+   const char *EType = (Sel.Opts & XrdCmsSelect::isDir  ? "directory" : "file");
 
-   Sel.Resp.DLen = snprintf(Sel.Resp.Data, sizeof(Sel.Resp.Data)-1,
-                   "No servers are available to %s%s the file.",Xmode,Amode)+1;
-   return -1;
+   int n = snprintf(Sel.Resp.Data, sizeof(Sel.Resp.Data),
+                   "No servers are available to %s%s the %s.",
+                   Xmode, Amode, EType)+1;
+    if (n < (int)sizeof(Sel.Resp.Data)) Sel.Resp.DLen = n+1;
+       else Sel.Resp.DLen = sizeof(Sel.Resp.Data);
+
+   Sel.Resp.Port = kYR_ENOENT;
+   return EReplete;
 }
