@@ -194,7 +194,7 @@ int XrdCmsFinderRMT::Configure(const char *cfn, char *Args, XrdOucEnv *envP)
 //
    if (!isMeta && !isTarget && config.haveMeta)
       {XrdCmsFinderTRG *Rsp = new XrdCmsFinderTRG(Say.logger(),Topts,myPort);
-       return Rsp->RunAdmin(CMSPath);
+       return Rsp->RunAdmin(CMSPath, config.myVNID);
       }
 
 // All done
@@ -422,6 +422,8 @@ int XrdCmsFinderRMT::Locate(XrdOucErrInfo &Resp, const char *path, int flags,
                Data.Opts |= CmsSelectRequest::kYR_trySVER;
        else if (!strcmp(triedRC, "resel"))
                Data.Opts |= CmsSelectRequest::kYR_tryRSEL;
+       else if (!strcmp(triedRC, "reseg"))
+               Data.Opts |= CmsSelectRequest::kYR_tryRSEG;
       }
   }
 
@@ -887,20 +889,17 @@ XrdCmsFinderTRG::XrdCmsFinderTRG(XrdSysLogger *lp, int whoami, int port,
                                  XrdOss *theSS)
                : XrdCmsClient(XrdCmsClient::amTarget)
 {
-   char buff [256];
    isRedir = whoami & IsRedir;
    isProxy = whoami & IsProxy;
    SS      = theSS;
    CMSPath = 0;
+   Login   = 0;
    myManList = 0;
    CMSp    = new XrdOucStream(&Say);
    Active  = 0;
    myPort  = port;
    resMax  = -1;
    resCur  = 0;
-   sprintf(buff, "login %c %d port %d\n",(isProxy ? 'P' : 'p'),
-                 static_cast<int>(getpid()), port);
-   Login = strdup(buff);
    Say.logger(lp);
 }
  
@@ -944,19 +943,27 @@ void XrdCmsFinderTRG::Added(const char *path, int Pend)
       {CMSp->Close(); Active = 0;}
    myData.UnLock();
 }
-
+  
 /******************************************************************************/
 /*                             C o n f i g u r e                              */
 /******************************************************************************/
 
-void *XrdCmsStartRsp(void *carg)
+namespace
+{
+void *StartPM(void *carg)
+      {XrdCmsFinderTRG *mp = (XrdCmsFinderTRG *)carg;
+       return mp->RunPM();
+      }
+
+void *StartRsp(void *carg)
       {XrdCmsFinderTRG *mp = (XrdCmsFinderTRG *)carg;
        return mp->Start();
       }
+}
   
 int XrdCmsFinderTRG::Configure(const char *cfn, char *Ags, XrdOucEnv *envP)
 {
-   XrdCmsClientConfig             config;
+   XrdCmsClientConfig             config(this);
    XrdCmsClientConfig::configWhat What;
 
 // Establish what we will be configuring
@@ -970,12 +977,25 @@ int XrdCmsFinderTRG::Configure(const char *cfn, char *Ags, XrdOucEnv *envP)
       else      {myManList = config.ManList; config.ManList = 0;}
 
 // Set the error dest and simply call the configration object and if
-// successful, run the Admin thread. Note that unlike FinderRMT, we do not
-// extract the security function pointer or the network object pointer from
-// the environment as we don't need these at all.
 //
    if (config.Configure(cfn, What, XrdCmsClientConfig::configNorm)) return 0;
-   return RunAdmin(config.CMSPath);
+
+// Run the Admin thread. Note that unlike FinderRMT, we do not extract the
+// security function pointer or the network object pointer from the
+// environment as we don't need these at all.
+//
+   if (RunAdmin(config.CMSPath, config.myVNID)
+   &&  config.perfMon && config.perfInt)
+      {pthread_t tid;
+       perfMon = config.perfMon;
+       perfInt = config.perfInt;
+       if (XrdSysThread::Run(&tid, StartPM, (void *)this, 0, "perfmon"))
+//     if (XrdSysThread::Run(&tid, StartRsp, (void *)this, 0, "cms i/f"))
+          {Say.Emsg("Config", errno, "start performance monitor."); return 0;}
+      }
+// All done
+//
+   return 1;
 }
   
 /******************************************************************************/
@@ -1006,6 +1026,37 @@ int XrdCmsFinderTRG::Locate(XrdOucErrInfo &Resp, const char *path, int flags,
    return SFS_DATA;
 }
   
+
+/******************************************************************************/
+/*                               P u t I n f o                                */
+/******************************************************************************/
+  
+void XrdCmsFinderTRG::PutInfo(XrdCmsPerfMon::PerfInfo &perfInfo, bool alert)
+{
+   char  buff[256];
+   char *data[2] = {buff, 0};
+   int   dlen[2];
+   uint32_t cpu_load, mem_load, net_load, pag_load, xeq_load;
+
+   cpu_load = (perfInfo.cpu_load <= 100 ? perfInfo.cpu_load : 100);
+   mem_load = (perfInfo.mem_load <= 100 ? perfInfo.mem_load : 100);
+   net_load = (perfInfo.net_load <= 100 ? perfInfo.net_load : 100);
+   pag_load = (perfInfo.pag_load <= 100 ? perfInfo.pag_load : 100);
+   xeq_load = (perfInfo.xeq_load <= 100 ? perfInfo.xeq_load : 100);
+
+   dlen[0] = snprintf(buff, sizeof(buff), "%s %u %u %u %u %u\n",
+                      (alert ? "PERF" : "perf"),
+                      xeq_load, cpu_load, mem_load, pag_load, net_load);
+   dlen[1] = 0;
+
+// Now send the notification
+//
+   myData.Lock();
+   if (Active && CMSp->Put((const char **)data, (const int *)dlen))
+      {CMSp->Close(); Active = 0;}
+   myData.UnLock();
+}
+
 /******************************************************************************/
 /*                               R e l e a s e                                */
 /******************************************************************************/
@@ -1160,22 +1211,50 @@ void XrdCmsFinderTRG::Suspend(int Perm)
 /*                              R u n A d m i n                               */
 /******************************************************************************/
   
-int XrdCmsFinderTRG::RunAdmin(char *Path)
+int XrdCmsFinderTRG::RunAdmin(char *Path, const char *vnid)
 {
+   const char *lFmt;
    pthread_t tid;
+   char buff [512];
 
 // Make sure we have a path to the cmsd
 //
    if (!(CMSPath = Path))
       {Say.Emsg("Config", "Unable to determine cms admin path"); return 0;}
 
+// Construct the login line
+//
+   lFmt = (vnid ? "login %c %d port %d vnid %s\n" : "login %c %d port %d\n");
+   snprintf(buff, sizeof(buff), lFmt, (isProxy ? 'P' : 'p'),
+                  static_cast<int>(getpid()), myPort, vnid);
+   Login = strdup(buff);
+
 // Start a thread to connect with the local cmsd
 //
-   if (XrdSysThread::Run(&tid, XrdCmsStartRsp, (void *)this, 0, "cms i/f"))
+   if (XrdSysThread::Run(&tid, StartRsp, (void *)this, 0, "cms i/f"))
       {Say.Emsg("Config", errno, "start cmsd interface"); return 0;}
 
    return 1;
 }
+  
+/******************************************************************************/
+/*                                 R u n P M                                  */
+/******************************************************************************/
+
+void *XrdCmsFinderTRG::RunPM()
+{
+   XrdCmsPerfMon::PerfInfo perfInfo;
+
+// Keep asking the plugin for statistics.
+//
+   while(1)
+        {perfMon->GetInfo(perfInfo);
+         PutInfo(perfInfo);
+         perfInfo.Clear();
+         XrdSysTimer::Snooze(perfInt);
+        }
+   return (void *)0;
+}
 
 /******************************************************************************/
 /*                                 S t a r t                                  */
@@ -1225,6 +1304,28 @@ void *XrdCmsFinderTRG::Start()
    return (void *)0;
 }
   
+/******************************************************************************/
+/*                           U t i l i z a t i o n                            */
+/******************************************************************************/
+  
+void XrdCmsFinderTRG::Utilization(unsigned int util, bool alert)
+{
+   XrdCmsPerfMon::PerfInfo perfInfo;
+
+// Make sure value is in range
+//
+   if (util > 100) util = 100;
+  
+// Send this out as a performance figure
+//
+   perfInfo.cpu_load = util;
+   perfInfo.mem_load = util;
+   perfInfo.net_load = util;
+   perfInfo.pag_load = util;
+   perfInfo.xeq_load = util;
+   PutInfo(perfInfo, alert);
+}
+
 /******************************************************************************/
 /*                                V C h e c k                                 */
 /******************************************************************************/
