@@ -28,7 +28,6 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
   
-#include <errno.h>
 #include <limits.h>
 #include <stdio.h>
 #include <time.h>
@@ -66,6 +65,7 @@
 
 #include "XrdNet/XrdNetUtils.hh"
 
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 
 using namespace XrdCms;
@@ -82,6 +82,11 @@ namespace
 {
 XrdNetIF::ifType ifVec[4] = {XrdNetIF::PublicV4, XrdNetIF::Public46,
                              XrdNetIF::PublicV6, XrdNetIF::Public64};
+
+const char *msrcmsg = "Cluster does not support multi-source access.";
+int msrclen = strlen(msrcmsg)+1;
+const char *mtrymsg = "Cluster retry limit exceeded.";
+int mtrylen = strlen(mtrymsg)+1;
 };
   
 /******************************************************************************/
@@ -505,16 +510,17 @@ const char *XrdCmsNode::do_Have(XrdCmsRRData &Arg)
 const char *XrdCmsNode::do_Load(XrdCmsRRData &Arg)
 {
    EPNAME("do_Load")
-   int temp, pcpu, pnet, pxeq, pmem, ppag, pdsk;
+   uint32_t pcpu, pnet, pxeq, pmem, ppag, pdsk;
+   int temp;
 
 // Process: load <cpu> <io> <load> <mem> <pag> <util> <rsvd> <dskFree>
 //               0     1    2      3     4     5      6
-   pcpu = static_cast<int>(Arg.Opaque[CmsLoadRequest::cpuLoad]);
-   pnet = static_cast<int>(Arg.Opaque[CmsLoadRequest::netLoad]);
-   pxeq = static_cast<int>(Arg.Opaque[CmsLoadRequest::xeqLoad]);
-   pmem = static_cast<int>(Arg.Opaque[CmsLoadRequest::memLoad]);
-   ppag = static_cast<int>(Arg.Opaque[CmsLoadRequest::pagLoad]);
-   pdsk = static_cast<int>(Arg.Opaque[CmsLoadRequest::dskLoad]);
+   pcpu = static_cast<uint32_t>(Arg.Opaque[CmsLoadRequest::cpuLoad]);
+   pnet = static_cast<uint32_t>(Arg.Opaque[CmsLoadRequest::netLoad]);
+   pxeq = static_cast<uint32_t>(Arg.Opaque[CmsLoadRequest::xeqLoad]);
+   pmem = static_cast<uint32_t>(Arg.Opaque[CmsLoadRequest::memLoad]);
+   ppag = static_cast<uint32_t>(Arg.Opaque[CmsLoadRequest::pagLoad]);
+   pdsk = static_cast<uint32_t>(Arg.Opaque[CmsLoadRequest::dskLoad]);
 
 // Compute actual load value
 //
@@ -555,9 +561,9 @@ const char *XrdCmsNode::do_Load(XrdCmsRRData &Arg)
           else nRefs = sRefs = 0;
        snprintf(buff, sizeof(buff)-1,
                "load=%d; cpu=%d net=%d inq=%d mem=%d pag=%d dsk=%d utl=%d "
-               "shr=[%d %lld %lld]",
+               "shr=[%d %lld %lld] ref=[%d %d]",
                myLoad, pcpu, pnet, pxeq, pmem, ppag, Arg.dskFree, pdsk,
-               myShr, nRefs, sRefs);
+               myShr, nRefs, sRefs, RefTotR+RefR, RefTotW+RefW);
        Say.Emsg("Node", Name(), buff);
        logload = Config.LogPerf;
       } else logload--;
@@ -646,7 +652,7 @@ const char *XrdCmsNode::do_Locate(XrdCmsRRData &Arg)
           {Arg.Request.rrCode = kYR_wait;
            bytes = sizeof(Resp.Val); Why = "delay ";
           } else {
-           if (rc == -2) return 0;
+           if (rc == XrdCmsCluster::Wait4CBk) return 0;
            Arg.Request.rrCode = kYR_error;
            rc = kYR_ENOENT; Why = "miss ";
            bytes = strlcpy(Resp.outbuff, "No servers have access to the file",
@@ -1021,6 +1027,74 @@ const char *XrdCmsNode::do_Rmdir(XrdCmsRRData &Arg)
    return (rc ? fsFail(Arg.Ident, "rmdir", Arg.Path, rc) : 0);
 }
   
+/******************************************************************************/
+/*                           d o _ S e l A v o i d                            */
+/******************************************************************************/
+
+int XrdCmsNode::do_SelAvoid(XrdCmsRRData &Arg, XrdCmsSelect &Sel,
+                            char *Avoid, bool &doRedir)
+{
+   XrdNetAddr avoidAddr;
+   char *Comma;
+   int avNum = 0;
+
+// Process the avoid list
+//
+   Sel.InfoP = 0;
+   do {if ((Comma = index(Avoid,','))) *Comma = '\0';
+       if (*Avoid == '+') Sel.nmask |= Cluster.getMask(Avoid+1);
+          else if (!avoidAddr.Set(Avoid,0))
+                          Sel.nmask |= Cluster.getMask(&avoidAddr);
+       Avoid = Comma+1; avNum++;
+      } while(Comma && *Avoid);
+
+// Check why we have an avoid list. For dfs style clusters, the limits on
+// selections are handled by the basefs object.
+//
+   if (baseFS.isDFS())
+      {if (Arg.Opts & CmsSelectRequest::kYR_tryRSEL)
+          Sel.Opts |= XrdCmsSelect::NoTryLim;
+       return 0;
+      }
+
+// This is a standard cluster, check if client is expanding the server base
+// and whether or not this is allowed in this cluster.
+//
+   if ((Arg.Opts & CmsSelectRequest::kYR_tryMSRC) && !Config.MultiSrc)
+      {if (Config.msRdrHost)
+          {strcpy(Sel.Resp.Data, Config.msRdrHost); // Gauranteed to fit!
+           Sel.Resp.DLen = Config.msRdrHLen;
+           Sel.Resp.Port = Config.msRdrPort;
+           doRedir = true;
+           return -1;
+          }
+       strncpy(Sel.Resp.Data, msrcmsg, sizeof(Sel.Resp.Data));
+       Sel.Resp.DLen = msrclen;
+       if (Arg.Opts & CmsSelectRequest::kYR_tryRSEL) Sel.Resp.Port = kYR_EPERM;
+          else Sel.Resp.Port = kYR_ENOENT;
+       return XrdCmsCluster::EReplete;
+      }
+
+// Check if we exceeded the retry count
+//
+   if (avNum > Config.MaxRetries)
+      {if (Config.mrRdrHost)
+          {strcpy(Sel.Resp.Data, Config.mrRdrHost); // Gauranteed to fit!
+           Sel.Resp.DLen = Config.mrRdrHLen;
+           Sel.Resp.Port = Config.mrRdrPort;
+           doRedir = true;
+          } else {
+           strncpy(Sel.Resp.Data, mtrymsg, sizeof(Sel.Resp.Data));
+           Sel.Resp.DLen = mtrylen;
+          }
+       return -1;
+      }
+
+// We suceeded, indicate selection can proceed.
+//
+   return 0;
+}
+  
 /******************************************************************************/
 /*                             d o _ S e l e c t                              */
 /******************************************************************************/
@@ -1036,7 +1110,7 @@ const char *XrdCmsNode::do_Select(XrdCmsRRData &Arg)
    XrdCmsRRQInfo reqInfo(Instance,RSlot,Arg.Request.streamid,Config.QryMinum);
    XrdCmsSelect Sel(XrdCmsSelect::Peers, Arg.Path, Arg.PathLen-1);
    struct iovec ioV[2];
-   char theopts[16], *Avoid, *toP = theopts;
+   char theopts[16], *toP = theopts;
    XrdNetIF::ifType ifType;
    int rc, bytes;
 
@@ -1066,9 +1140,12 @@ const char *XrdCmsNode::do_Select(XrdCmsRRData &Arg)
            {Sel.Opts |= XrdCmsSelect::Write;                       *toP++='w';
             if (Arg.Opts & CmsSelectRequest::kYR_mwfiles || !Config.DoMWChk)
                {Sel.Opts |= XrdCmsSelect::MWFiles;                *(toP-1)='W';}
-            }
+           }
          if (Arg.Opts & CmsSelectRequest::kYR_metaop)
-           {Sel.Opts |= XrdCmsSelect::Write|XrdCmsSelect::isMeta;  *toP++='m';}
+           {Sel.Opts |= XrdCmsSelect::isMeta;                      *toP++='m';
+            if (!(Arg.Opts & CmsSelectRequest::kYR_write))
+               {Sel.Opts |= XrdCmsSelect::isDir;                   *toP++='D';}
+           }
          if (Arg.Opts & CmsSelectRequest::kYR_create)  
            {Sel.Opts |= XrdCmsSelect::Write|XrdCmsSelect::NewFile; *toP++='c';
             if (Arg.Opts & CmsSelectRequest::kYR_replica)
@@ -1098,25 +1175,14 @@ const char *XrdCmsNode::do_Select(XrdCmsRRData &Arg)
 
 // Check if an avoid node present. If so, this is ineligible for fast redirect.
 //
+   bool doRedir = false;
    Sel.nmask = SMask_t(0);
-   if ((Avoid = Arg.Avoid))
-      {XrdNetAddr avoidAddr;
-       char *Comma;
-       DEBUGR(theopts <<' ' <<Arg.Path <<" avoiding " <<Avoid);
-       if (Arg.Opts & CmsSelectRequest::kYR_tryRSEL)
-          Sel.Opts |= XrdCmsSelect::NoTryLim;
-       Sel.InfoP = 0;
-       do {if ((Comma = index(Avoid,','))) *Comma = '\0';
-           if (*Avoid == '+') Sel.nmask |= Cluster.getMask(Avoid+1);
-              else if (!avoidAddr.Set(Avoid,0))
-                              Sel.nmask |= Cluster.getMask(&avoidAddr);
-           Avoid = Comma+1;
-          } while(Comma && *Avoid);
-      } else DEBUGR(theopts <<' ' <<Arg.Path);
+   if (Arg.Avoid) rc = do_SelAvoid(Arg, Sel, Arg.Avoid, doRedir);
+      else rc = 0;
 
 // Perform selection
 //
-   if ((rc = Cluster.Select(Sel)))
+   if (!doRedir && (rc || (rc = Cluster.Select(Sel))))
       {if (rc > 0)
           {Arg.Request.rrCode = kYR_wait;
            Sel.Resp.Port      = rc;
@@ -1124,11 +1190,16 @@ const char *XrdCmsNode::do_Select(XrdCmsRRData &Arg)
            DEBUGR("delay " <<rc <<' ' <<Arg.Path);
           } else {
            Arg.Request.rrCode = kYR_error;
-           if (rc != XrdCmsCluster::RetryErr) Sel.Resp.Port = kYR_ENOENT;
-              else {int rtRC = (Arg.Opts & CmsSelectRequest::kYR_tryMASK)
-                               >> CmsSelectRequest::kYR_trySHFT;
-                    Sel.Resp.Port = rtEC[rtRC];
-                   }
+           if (rc != XrdCmsCluster::EReplete) // if error info has not been set
+              {if (rc == XrdCmsCluster::RetryErr)
+                  {int rtRC = (Arg.Opts & CmsSelectRequest::kYR_tryMASK)
+                                       >> CmsSelectRequest::kYR_trySHFT;
+                   Sel.Resp.Port = rtEC[rtRC];
+                  } else {
+                   Sel.Resp.Port = kYR_ENOENT; // This should never happen!
+                   Sel.Resp.DLen = sprintf(Sel.Resp.Data,"%s","Item not found.")+1;
+                  }
+              }
            DEBUGR("failed; " <<Sel.Resp.Data << ' ' <<Arg.Path);
           }
       } else if (!Sel.Resp.DLen) return 0;
@@ -1828,7 +1899,7 @@ const char *XrdCmsNode::fsFail(const char *Who,  const char *What,
       else {struct {const char *Ident;} Arg = {Who};
             DEBUGR("rc=" <<rc <<' ' <<What <<' ' <<Path);
            }
-   return rc ? strerror(rc) : 0;
+   return rc ? XrdSysE2T(rc) : 0;
 }
 
 /******************************************************************************/
