@@ -162,20 +162,10 @@ int XrdgetProtocolPort(const char *pname, char *parms,
        return thePort;
       }
 
-// Initialize the error message handler and some default values
+// Call the level 0 configurator
 //
-   Say.logger(pi->eDest->logger(0));
-   Config.myName    = strdup(pi->myName);
-   Config.PortTCP   = (pi->Port < 0 ? 0 : pi->Port);
-   Config.myInsName = strdup(pi->myInst);
-   Config.myProg    = strdup(pi->myProg);
-   Sched            = pi->Sched;
-   if (pi->DebugON) Trace.What = TRACE_ALL;
-
-// Create an xrootd compatabile environment
-//
-   XrdCms::theEnv.PutPtr("XrdScheduler*", Sched);
-   if (pi->theEnv) XrdCms::theEnv.PutPtr("xrdEnv*", pi->theEnv);
+   if (Config.Configure0(pi))
+      {Config.doWait = -1; return 0;}
 
 // The only parameter we accept is the name of an alternate config file
 //
@@ -190,7 +180,7 @@ int XrdgetProtocolPort(const char *pname, char *parms,
 
 // Put up the banner
 //
-   Say.Say("Copr.  2007 Stanford University/SLAC cmsd.");
+   Say.Say("Copr.  2003-2020 Stanford University/SLAC cmsd.");
 
 // Indicate failure if static init fails
 //
@@ -277,13 +267,14 @@ void XrdCmsProtocol::Pander(const char *manager, int mport)
    EPNAME("Pander");
 
    CmsLoginData Data, loginData;
+   time_t ddmsg = time(0);
    unsigned int Mode, Role = 0;
    int myShare = Config.P_gshr << CmsLoginData::kYR_shift;
    int myTimeZ = Config.TimeZone<< CmsLoginData::kYR_shifttz;
    int Lvl=0, Netopts=0, waits=6, tries=6, fails=0, xport=mport;
    int rc, fsUtil, KickedOut, blRedir, myNID = Manager->ManTree->Register();
    int chk4Suspend = XrdCmsState::All_Suspend, TimeOut = Config.AskPing*1000;
-   char envBuff[128], manbuff[264];
+   char manbuff[264];
    const char *Reason = 0, *manp = manager;
    const int manblen = sizeof(manbuff);
    bool terminate;
@@ -306,13 +297,7 @@ void XrdCmsProtocol::Pander(const char *manager, int mport)
    loginData.Mode    = 0;
    loginData.Size    = 0;
    loginData.ifList  = (kXR_char *)Config.ifList;
-
-// Fill our the environment
-//
-   if (!Config.mySite) loginData.envCGI  = 0;
-      else {snprintf(envBuff, sizeof(envBuff), "site=%s", Config.mySite);
-            loginData.envCGI = (kXR_char *)envBuff;
-           }
+   loginData.envCGI  = (kXR_char *)Config.envCGI;
 
 // Establish request routing based on who we are
 //
@@ -354,7 +339,12 @@ void XrdCmsProtocol::Pander(const char *manager, int mport)
        DEBUG("trying to connect to lvl " <<Lvl <<' ' <<manp <<':' <<xport);
 
        if (!(Link = Config.NetTCP->Connect(manp, xport, Netopts)))
-          {if (tries--) Netopts = XRDNET_NOEMSG;
+          {if (!Netopts && XrdNetAddr::DynDNS() && (time(0) - ddmsg) >= 90)
+              {Say.Emsg("Pander", "Is hostname", manp, "spelled correctly "
+                                  "or just not running?");
+               ddmsg = time(0);
+              }
+           if (tries--) Netopts = XRDNET_NOEMSG;
               else {tries = 6; Netopts = 0;}
            if ((Lvl = Manager->myMans->Next(xport,manbuff,manblen)))
                    {XrdSysTimer::Snooze(3); manp = manbuff;}
@@ -365,6 +355,23 @@ void XrdCmsProtocol::Pander(const char *manager, int mport)
           }
        Netopts = 0; tries = waits = 6;
 
+       // Verify that this node has the real DNS name if it's IPv6
+       //
+       if (!(Link->AddrInfo()->isRegistered())
+       &&    Link->AddrInfo()->isIPType(XrdNetAddrInfo::IPv6))
+          {char *oldName = strdup(Link->Host());
+           Say.Emsg("Protocol", oldName, "is missing an IPv6 ptr record; "
+                               "attempting local registration as", manp);
+           if (!(Link->Register(manp)))
+              {Say.Emsg("Protocol", oldName,
+                        "registration failed; address mismatch.");
+              } else {
+               Say.Emsg("Protocol", oldName,
+                        "is now locally registered as", manp);
+              }
+           free(oldName);
+          }
+
        // Obtain a new node object for this connection
        //
        if (!(myNode = Manager->Add(Link, Lvl+1, terminate)))
@@ -557,7 +564,30 @@ int XrdCmsProtocol::Stats(char *buff, int blen, int do_sync)
 /******************************************************************************/
 /*                                 A d m i t                                  */
 /******************************************************************************/
-  
+
+namespace
+{
+char *getAltName(char *sid, char *buff, int blen)
+{
+   char *atsign, *spacec, *retval = 0;
+   int  n;
+   if (sid)
+   if ((atsign = index(sid, '@')))
+      {atsign++;
+       if ((spacec = index(atsign, ' ')))
+          {*spacec = 0;
+           n = strlen(atsign);
+           if (n > 3 && n < blen)
+              {strcpy(buff, atsign);
+               retval = buff;
+              }
+           *spacec = ' ';
+          }
+      }
+   return retval;
+}
+}
+
 XrdCmsRouting *XrdCmsProtocol::Admit()
 {
    EPNAME("Admit");
@@ -590,6 +620,41 @@ XrdCmsRouting *XrdCmsProtocol::Admit()
 //
    if (!Source.Admit(Link, Data, Config.mySID, envP)) return 0;
 
+// Construct environment for incomming node
+//
+   XrdOucEnv cgiEnv((const char *)Data.envCGI);
+
+// We have this problem hat many times the IPv6 address is missing the ptr
+// record in DNS. If this node is IPv6 unregistered and the incomming node
+// supplied it's host name then we can attempt to register it locally.
+//
+   if (!(Link->AddrInfo()->isRegistered())
+   &&    Link->AddrInfo()->isIPType(XrdNetAddrInfo::IPv6))
+      {const char *altName = cgiEnv.Get("myHN");
+       const char *altType = "stated mapping";
+       char hBF[256], *oldName = strdup(Link->Host());
+       if (!altName) {altName = getAltName((char *)Data.SID, hBF, sizeof(hBF));
+                      altType = "inferred mapping";
+                     }
+       Say.Emsg("Protocol", "DNS lookup for", oldName, "failed; "
+                            "IPv6 ptr record missing!");
+       if (!altName)
+          {Say.Emsg("Protocol", oldName, "did not supply a fallback "
+                                         "mapping; using IPv6 address.");
+          } else {
+           char buff[512];
+           snprintf(buff, sizeof(buff), "%s -> %s", oldName, altName);
+           Say.Emsg("Protocol", "Attempting to use", altType, buff);
+           if (!(Link->Register(altName)))
+              {Say.Emsg("Protocol", buff, altType,"failed; address mismatch.");
+              } else {
+               Say.Emsg("Protocol", oldName,
+                        "is now locally registered as", altName);
+              }
+          }
+       free(oldName);
+      }
+
 // Handle Redirectors here (minimal stuff to do)
 //
    if (Data.Mode & CmsLoginData::kYR_director) 
@@ -729,15 +794,21 @@ XrdCmsRouting *XrdCmsProtocol::Admit()
    if (Config.asManager()) {Manager->Reset(); myNode->SyncSpace();}
    myNode->isBad &= ~XrdCmsNode::isDisabled;
 
+// At this point we can switch to nonblocking sendq for this node
+//
+   if (Config.nbSQ && (Config.nbSQ > 1 || !myNode->inDomain()))
+      isNBSQ = Link->setNB();
+
 // Document the login
 //
-   XrdOucEnv cgiEnv((const char *)Data.envCGI);
    const char *sname = cgiEnv.Get("site");
    const char *lfmt  = (myNode->isMan > 1 ? "Standby%s%s" : "Primary%s%s");
    snprintf(envBuff,sizeof(envBuff),lfmt,(sname ? " ":""),(sname ? sname : ""));
    Say.Emsg("Protocol", envBuff, myNode->Ident,
             (myNode->isBad & XrdCmsNode::isSuspend ? "logged in suspended."
                                                    : "logged in."));
+   if (Data.SID)
+      Say.Emsg("Protocol", myNode->Ident, "system ID:", (const char *)Data.SID);
    myNode->ShowIF();
 
 // All done
@@ -764,7 +835,10 @@ XrdCmsRouting *XrdCmsProtocol::Admit_Redirector(int wasSuspended)
 //
    myNode = new XrdCmsNode(Link); myNode->Lock(false);
    if (!(RSlot = RTable.Add(myNode)))
-      {Say.Emsg("Protocol",myNode->Ident,"login failed; too many redirectors.");
+      {myNode->UnLock();
+       delete myNode;
+       myNode = 0;
+       Say.Emsg("Protocol",myNode->Ident,"login failed; too many redirectors.");
        return 0;
       } else myNode->setSlot(RSlot);
 
@@ -875,7 +949,6 @@ const char *XrdCmsProtocol::Dispatch(Bearing cDir, int maxWait, int maxTries)
 {
    EPNAME("Dispatch");
    static const int ReqSize = sizeof(CmsRRHdr);
-   static CmsPingRequest Ping = {{0, kYR_ping,  0, 0}};
    XrdCmsRRData *Data = XrdCmsRRData::Objectify();
    XrdCmsJob  *jp;
    const char *toRC = (cDir == isUp ? "manager not active"
@@ -897,8 +970,7 @@ do{if ((rc = Link->RecvAll((char *)&Data->Request, ReqSize, maxWait)) < 0)
        if (cDir == isDown)
           {if (myNode->isBad & XrdCmsNode::isDoomed)
               return "server blacklisted w/ redirect";
-           if (Link->Send((char *)&Ping, sizeof(Ping)) < 0)
-              return "server unreachable";
+           if (!SendPing()) return "server unreachable";
            lastPing = Config.PingTick;
           }
        continue;
@@ -909,8 +981,7 @@ do{if ((rc = Link->RecvAll((char *)&Data->Request, ReqSize, maxWait)) < 0)
    if (cDir == isDown && lastPing != Config.PingTick)
       {if (myNode->isBad & XrdCmsNode::isDoomed)
           return "server blacklisted w/ redirect";
-       if (Link->Send((char *)&Ping, sizeof(Ping)) < 0)
-          return "server unreachable";
+       if (!SendPing()) return "server unreachable";
        lastPing = Config.PingTick;
       }
 
@@ -1010,6 +1081,7 @@ void XrdCmsProtocol::Init(const char *iRole, XrdCmsManager *uMan,
    ProtLink  = 0;
    refCount  = 0;
    refWait   = 0;
+   isNBSQ    = false;
 }
   
 /******************************************************************************/
@@ -1142,6 +1214,26 @@ void XrdCmsProtocol::Reply_Error(XrdCmsRRData &Data, int ecode, const char *etex
      DEBUG(myNode->Ident <<act <<" err " <<ecode  <<' ' <<etext);
 }
 
+/******************************************************************************/
+/* Private:                     S e n d P i n g                               */
+/******************************************************************************/
+  
+bool XrdCmsProtocol::SendPing()
+{
+   static CmsPingRequest Ping = {{0, kYR_ping,  0, 0}};
+
+// We would like not send ping requests to servers that are backlogged but if
+// we don't (currently) it will look to one of the parties that the other party
+// if not functional. We should try to fix this, sigh.
+//
+// if (isNBSQ && Link->Backlog()) return true;
+
+// Send the ping
+//
+   if (Link->Send((char *)&Ping, sizeof(Ping)) < 0) return false;
+   return true;
+}
+  
 /******************************************************************************/
 /* Private:                         S y n c                                   */
 /******************************************************************************/
