@@ -50,10 +50,11 @@
 #include "XrdCns/XrdCnsLogRec.hh"
 #include "XrdCns/XrdCnsXref.hh"
 
+#include "XrdNet/XrdNetUtils.hh"
+
 #include "XrdOuc/XrdOucNSWalk.hh"
 #include "XrdOuc/XrdOucTList.hh"
 #include "XrdOuc/XrdOucUtils.hh"
-#include "XrdSys/XrdSysDNS.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysTimer.hh"
 
@@ -81,9 +82,13 @@ XrdCnsLogClient::XrdCnsLogClient(XrdOucTList     *rP,
 {
    static int cNum = 0;
    static int bSfx = static_cast<int>(time(0)) - 1248126834;
-   static char *myName = XrdSysDNS::getHostName();
+   static char *myName = 0;
    char destBuff[512];
 
+// Establish out FQN
+//
+   if (!myName) myName = XrdNetUtils::MyHostName();
+
 // Save our index into the commit array
 //
    pfxNF    = cNum++;
@@ -600,7 +605,7 @@ int XrdCnsLogClient::mapError(int rc)
         case kXR_noserver:      return EHOSTUNREACH;
         case kXR_NotFile:       return ENOTBLK;
         case kXR_isDirectory:   return EISDIR;
-        case kXR_FSError:       return ENOSYS;
+        case kXR_FSError:       return ENOSUP;
         default:                return ECANCELED;
        }
 }
