@@ -39,7 +39,7 @@ static XrdSysError eDest(0,"crypto_");
 XrdOucTrace *cryptoTrace = 0;
 //
 // Time Zone correction (wrt UTC)
-static int TZCorr = 0;
+static time_t TZCorr = 0;
 static bool TZInitialized = 0;
 
 /******************************************************************************/
@@ -74,7 +74,7 @@ void XrdCryptoSetTrace(kXR_int32 trace)
 /*  X r d C r y p t o T i m e G m                                             */
 /******************************************************************************/
 //______________________________________________________________________________
-int XrdCryptoTZCorr()
+time_t XrdCryptoTZCorr()
 {
    // Time Zone correction (wrt UTC)
    
@@ -82,7 +82,7 @@ int XrdCryptoTZCorr()
       time_t now = time(0);
       struct tm ltn, gtn;
       if (localtime_r(&now, &ltn) != 0 && gmtime_r(&now, &gtn) != 0) {
-         TZCorr = int(difftime(mktime(&ltn), mktime(&gtn)));
+         TZCorr = time_t(difftime(mktime(&ltn), mktime(&gtn)));
          TZInitialized = 1;
       }
    }
