@@ -58,7 +58,7 @@ XrdCryptoFactory::XrdCryptoFactory(const char *n, int id)
    if (n) {
       int l = strlen(n);
       l = (l > (MAXFACTORYNAMELEN - 1)) ? (MAXFACTORYNAMELEN - 1) : l;  
-      strncpy(name,n,l);
+      memcpy(name,n,l);
       name[l] = 0;  // null terminated
    }
    fID = id;
@@ -109,6 +109,15 @@ bool XrdCryptoFactory::SupportedCipher(const char *)
    return 0;
 }
 
+//______________________________________________________________________________
+bool XrdCryptoFactory::HasPaddingSupport()
+{
+   // Returns true id specified cipher is supported by the implementation
+
+   ABSTRACTMETHOD("XrdCryptoFactory::PaddingSupport");
+   return 0;
+}
+
 //______________________________________________________________________________
 XrdCryptoCipher *XrdCryptoFactory::Cipher(const char *, int)
 {
@@ -137,6 +146,15 @@ XrdCryptoCipher *XrdCryptoFactory::Cipher(XrdSutBucket *)
    return 0;
 }
 
+//______________________________________________________________________________
+XrdCryptoCipher *XrdCryptoFactory::Cipher(bool, int, char *, int, const char *)
+{
+   // Return an instance of an implementation of XrdCryptoCipher.
+
+   ABSTRACTMETHOD("XrdCryptoFactory::Cipher");
+   return 0;
+}
+
 //______________________________________________________________________________
 XrdCryptoCipher *XrdCryptoFactory::Cipher(int, char *, int, const char *)
 {
@@ -296,6 +314,16 @@ XrdCryptoX509ParseFile_t XrdCryptoFactory::X509ParseFile()
    return 0;
 }
 
+//______________________________________________________________________________
+XrdCryptoX509ParseStack_t XrdCryptoFactory::X509ParseStack()
+{
+   // Return an instance of an implementation of a function
+   // to parse a stack supposed to contain for X509 certificates.
+
+   ABSTRACTMETHOD("XrdCryptoFactory::X509ParseStack");
+   return 0;
+}
+
 //______________________________________________________________________________
 XrdCryptoX509ParseBucket_t XrdCryptoFactory::X509ParseBucket()
 {
