@@ -308,6 +308,8 @@ void XrdCryptoX509Chain::PushBack(XrdCryptoX509 *c)
          end->SetNext(nc);
       end = nc;
       size++;
+   } else if (c) {
+      delete c;
    }
 
    // Search for the effective CA (the last one, in case of subCAs)
@@ -445,19 +447,21 @@ XrdCryptoX509ChainNode *XrdCryptoX509Chain::FindIssuer(const char *issuer,
    while (cn) {
       n = cn->Next();
       c = cn->Cert();
-      const char *pi = c->Issuer();
-      if (c && pi) {
-         if (mode == kExact) {
-            if (!strcmp(pi, issuer))
-               break;
-         } else if (mode == kBegin) {
-            if (strstr(pi, issuer) == c->Issuer())
-               break;
-         } else if (mode == kEnd) {
-            int ibeg = strlen(pi) - strlen(issuer);
-            if (!strcmp(pi + ibeg, issuer))
-               break;
-         }
+      if(c) {
+        const char *pi = c->Issuer();
+        if (pi) {
+           if (mode == kExact) {
+              if (!strcmp(pi, issuer))
+                 break;
+           } else if (mode == kBegin) {
+              if (strstr(pi, issuer) == c->Issuer())
+                 break;
+           } else if (mode == kEnd) {
+              int ibeg = strlen(pi) - strlen(issuer);
+              if (!strcmp(pi + ibeg, issuer))
+                 break;
+           }
+        }
       }
       c = 0;
       cp = cn;  // previous
@@ -584,7 +588,7 @@ int XrdCryptoX509Chain::Reorder()
    }
 
    // Move it in first position if not yet there
-   if (nr != begin) { 
+   if (nr && nr != begin) {
       np->SetNext(nr->Next()); // short cut old position
       nr->SetNext(begin);      // set our next to present begin
       if (end == nr)           // Update end
