@@ -42,8 +42,8 @@
 #include "XrdCrypto/XrdCryptosslRSA.hh"
 #include "XrdCrypto/XrdCryptosslX509.hh"
 #include "XrdCrypto/XrdCryptosslTrace.hh"
+#include "XrdTls/XrdTlsPeerCerts.hh"
 #include <openssl/pem.h>
-#include <openssl/ssl.h>
 
 // Error code from verification set by verify callback function
 static int gErrVerifyChain = 0;
@@ -378,10 +378,9 @@ int XrdCryptosslX509ChainToFile(XrdCryptoX509Chain *ch, const char *fn)
 }
 
 //______________________________________________________________________________
-int XrdCryptosslX509ParseStack(void* ssl_conn, XrdCryptoX509Chain *chain)
+int XrdCryptosslX509ParseStack(XrdTlsPeerCerts* pc, XrdCryptoX509Chain *chain)
 {
    EPNAME("X509ParseStack");
-   SSL* ssl = (SSL*) ssl_conn;
    int nci = 0;
    // Make sure we got a chain where to add the certificates
    if (!chain) {
@@ -389,28 +388,23 @@ int XrdCryptosslX509ParseStack(void* ssl_conn, XrdCryptoX509Chain *chain)
       return nci;
    }
 
-   STACK_OF(X509) *st_x509 = SSL_get_peer_cert_chain(ssl);
-   // NOTE: SSL_get_peer_certificate increments the refcount;
-   // we must free it or pass along ownership.
-   X509 *peer_cert = SSL_get_peer_certificate(ssl);
-
-   if (peer_cert) {
-     XrdCryptoX509 *c = new XrdCryptosslX509(peer_cert);
+   if (pc->hasCert()) {
+     XrdCryptoX509 *c = new XrdCryptosslX509(pc->getCert());
 
      if (c) {
        chain->PushBack(c);
        nci ++;
-     } else {
-       X509_free(peer_cert);
      }
    }
 
-   if (!st_x509) {
+   if (!pc->hasChain()) {
      return nci;
    }
 
-   for (int i=0; i < sk_X509_num(st_x509); i++) {
-      X509 *cert = sk_X509_value(st_x509, i);
+   STACK_OF(X509) *pChain = pc->getChain();
+
+   for (int i=0; i < sk_X509_num(pChain); i++) {
+      X509 *cert = sk_X509_value(pChain, i);
       XrdCryptoX509 *c = new XrdCryptosslX509(cert);
 
       if (c) {
@@ -464,7 +458,7 @@ int XrdCryptosslX509ParseFile(const char *fname,
    // it should contain blocks on information in PEM form
    FILE *fcer = fopen(fname, "r");
    if (!fcer) {
-      DEBUG("unable to open file '"<<fname <<"' (errno: "<<errno<<")");
+      DEBUG("unable to open file (errno: "<<errno<<")");
       return nci;
    }
 
