@@ -30,7 +30,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <dirent.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <memory.h>
 #include <string.h>
@@ -43,6 +42,7 @@
 
 #include "XrdOuc/XrdOucStream.hh"
 
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysLogger.hh"
@@ -295,7 +295,7 @@ do{if ((retc = readdir_r(dh, d_pnt, &rp)))
            n = sizeof(dirent_full.nbf) - (n + 8);
            if ((n = readlinkat(dirFD,rp->d_name,dP,n)) < 0) strcpy(dP,"?");
               else *(dP+n) = 0;
-           strncpy(dP-4, " -> ", 4);
+           memcpy(dP-4, " -> ", 4);
           }
       }
 #endif
@@ -461,7 +461,7 @@ int      XrdDigFile::fctl(const int      cmd,
 
 // We don't support this
 //
-   out_error.setErrInfo(EEXIST, "fctl operation not supported");
+   out_error.setErrInfo(ENOTSUP, "fctl operation not supported");
    return SFS_ERROR;
 }
   
@@ -608,13 +608,13 @@ int XrdDigFS::Emsg(const char    *pfx,    // Message prefix value
                    const char    *op,     // Operation being performed
                    const char    *target) // The target (e.g., fname)
 {
-    char *etext, buffer[MAXPATHLEN+80], unkbuff[64];
+   const char *etext;
+   char buffer[MAXPATHLEN+80];
 
 // Get the reason for the error
 //
    if (ecode < 0) ecode = -ecode;
-   if (!(etext = strerror(ecode)))
-      {sprintf(unkbuff, "reason unknown (%d)", ecode); etext = unkbuff;}
+   etext = XrdSysE2T(ecode);
 
 // Format the error message
 //
