@@ -33,7 +33,7 @@
 #include <string.h>
 #include <sys/types.h>
 
-#if !defined(__solaris__)
+#if !defined(__solaris__) && !defined(__FreeBSD__)
 #include <sys/xattr.h> 
 #endif
 
@@ -56,6 +56,8 @@
   extern "C" {
 #endif
 
+#define MAXROOTURLLEN 1024 // this is also defined in other files
+
 int XrdFfsPosix_stat(const char *path, struct stat *buf)
 {
     int rc; 
@@ -94,7 +96,6 @@ int XrdFfsPosix_mkdir(const char *path, mode_t mode)
 
 int XrdFfsPosix_rmdir(const char *path)
 {
-/* Note: Xrootd returns ENOSYS rather than ENOTEMPTY when a directory is not empty */
     return XrdPosixXrootd::Rmdir(path);
 }
 
@@ -246,7 +247,7 @@ void* XrdFfsPosix_x_deleteall(void *x)
         
 int XrdFfsPosix_deleteall(const char *rdrurl, const char *path, uid_t user_uid, mode_t st_mode)
 {
-    int i, nurls, res, rval;
+    int i, nurls, res;
     char *newurls[XrdFfs_MAX_NUM_NODES];
     int res_i[XrdFfs_MAX_NUM_NODES];
     int errno_i[XrdFfs_MAX_NUM_NODES];
@@ -254,13 +255,12 @@ int XrdFfsPosix_deleteall(const char *rdrurl, const char *path, uid_t user_uid,
     struct XrdFfsQueueTasks *jobs[XrdFfs_MAX_NUM_NODES];
 
     nurls = XrdFfsMisc_get_all_urls(rdrurl, newurls, XrdFfs_MAX_NUM_NODES);
-    if (nurls < 0) rval = -1;
 
     for (i = 0; i < nurls; i++)
     {
         errno_i[i] = 0;
-        strcat(newurls[i],path);
-        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid);
+        strncat(newurls[i],path, MAXROOTURLLEN - strlen(newurls[i]) -1);
+        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid, 0);
         args[i].url = newurls[i];
         args[i].err = &errno_i[i];
         args[i].res = &res_i[i];
@@ -331,13 +331,13 @@ int XrdFfsPosix_renameall(const char *rdrurl, const char *from, const char *to,
 
         fromurl[0]='\0';
         strcat(fromurl, newurls[i]);
-        strcat(fromurl, from);
+        strncat(fromurl, from,  MAXROOTURLLEN - strlen(fromurl) -1);
         tourl[0]='\0';
         strcat(tourl, newurls[i]);
-        strcat(tourl, to);
+        strncat(tourl, to,  MAXROOTURLLEN - strlen(tourl) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(fromurl, user_uid);
-        XrdFfsMisc_xrd_secsss_editurl(tourl, user_uid);
+        XrdFfsMisc_xrd_secsss_editurl(fromurl, user_uid, 0);
+        XrdFfsMisc_xrd_secsss_editurl(tourl, user_uid, 0);
         res = (XrdFfsPosix_stat(fromurl, &stbuf));
         if (res == 0)
         {
@@ -379,8 +379,8 @@ int XrdFfsPosix_truncateall(const char *rdrurl, const char *path, off_t size, ui
     for (i = 0; i < nurls; i++)
     {
         errno = 0;
-        strcat(newurls[i],path);
-        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid);
+        strncat(newurls[i],path,  MAXROOTURLLEN - strlen(newurls[i]) -1);
+        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid, 0);
         res = (XrdFfsPosix_stat(newurls[i], &stbuf));
         if (res == 0)
         {
@@ -487,8 +487,8 @@ int XrdFfsPosix_readdirall(const char *rdrurl, const char *path, char*** direnta
     for (i = 0; i < nurls; i++)
     {
         errno_i[i] = 0;
-        strcat(newurls[i], path);
-        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid);
+        strncat(newurls[i], path,  MAXROOTURLLEN - strlen(newurls[i]) -1);
+        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid, 0);
         args[i].url = newurls[i];
         args[i].err = &errno_i[i];
         args[i].res = &res_i[i];
@@ -679,7 +679,7 @@ int XrdFfsPosix_statvfsall(const char *rdrurl, const char *path, struct statvfs
         osscgroup = 0;
     for (i = 0; i < nurls; i++)
     {
-        strcat(newurls[i], path);
+        strncat(newurls[i], path,  MAXROOTURLLEN - strlen(newurls[i]) -1);
 //        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid);
         args[i].url = newurls[i];
         args[i].res = &res_i[i];
@@ -747,11 +747,11 @@ int XrdFfsPosix_statall(const char *rdrurl, const char *path, struct stat *stbuf
     struct XrdFfsPosixX_statall_args args[XrdFfs_MAX_NUM_NODES];
     struct XrdFfsQueueTasks *jobs[XrdFfs_MAX_NUM_NODES];
 
-    char *p1, *p2, *dir, *file, rootpath[1024];
+    char *p1, *p2, *dir, *file, rootpath[MAXROOTURLLEN];
 
     rootpath[0] = '\0';
-    strcat(rootpath,rdrurl);
-    strcat(rootpath,path);
+    strncat(rootpath,rdrurl, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path,  MAXROOTURLLEN - strlen(rootpath) -1);
     p1 = strdup(path);
     p2 = strdup(path);
     dir = dirname(p1);
@@ -760,7 +760,7 @@ int XrdFfsPosix_statall(const char *rdrurl, const char *path, struct stat *stbuf
 // if task queue is too long, or if the stat() is from an ls -l command, the stat() against redirector
     if ( XrdFfsQueue_count_tasks()/XrdFfsMisc_get_number_of_data_servers() > 20 || XrdFfsDent_cache_search(dir, file))
     {
-         XrdFfsMisc_xrd_secsss_editurl(rootpath, user_uid);
+         XrdFfsMisc_xrd_secsss_editurl(rootpath, user_uid, 0);
          res = XrdFfsPosix_stat(rootpath, stbuf);
 // maybe a data server is down since the last _readdir()? in that case, continue 
 // we also saw a case where redirectors report the file exist but meta redirector report
@@ -779,8 +779,8 @@ int XrdFfsPosix_statall(const char *rdrurl, const char *path, struct stat *stbuf
 
     for (i = 0; i < nurls; i++)
     {
-        strcat(newurls[i], path);
-        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid);
+        strncat(newurls[i], path, MAXROOTURLLEN - strlen(path) -1);
+        XrdFfsMisc_xrd_secsss_editurl(newurls[i], user_uid, 0);
         args[i].url = newurls[i];
         args[i].res = &res_i[i];
         args[i].err = &errno_i[i];
