@@ -150,6 +150,8 @@ XrdFrmConfig::XrdFrmConfig(SubSys ss, const char *vopts, const char *uinfo)
    QPath    = 0;
    AdminMode= 0740;
    xfrMax   = 2;
+   xfrMaxIn = 0;
+   xfrMaxOt = 0;
    FailHold = 3*60*60;
    IdleHold = 10*60;
    WaitMigr = 60*60;
@@ -231,7 +233,6 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 {
    extern XrdOss *XrdOssGetSS(XrdSysLogger *, const char *, const char *,
                               const char   *, XrdOucEnv *,  XrdVersionInfo &);
-   extern int *XrdOssRunMode;
    static XrdNetAddr myAddr(0);
    XrdFrmConfigSE theSE;
    int retc, isMum = 0, myXfrMax = -1, NoGo = 0, optBG = 0;
@@ -382,7 +383,7 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 
 // Put out the herald
 //
-   sprintf(buff, "Scalla %s is starting. . .", myProg);
+   sprintf(buff, "File Residency Manager %s is starting. . .", myProg);
    Say.Say(0, buff);
    Say.Say(XrdBANNER);
 
@@ -411,7 +412,6 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
                     OfsCfg->Plugin(ossFS);
                     OfsCfg->Plugin(CksMan);
                     doStatPF = ossFS->StatPF("/", &Stat) != -ENOTSUP;
-                    runNew = !(runOld = XrdOssRunMode ? *XrdOssRunMode : 0);
                    }
           }
       }
@@ -513,7 +513,7 @@ int XrdFrmConfig::NeedsCTA(const char *Lfn)
 {
    extern XrdOucPListAnchor *XrdOssRPList;
 
-   return (XrdOssRPList->Find(Lfn) & XRDEXP_MAKELF) != 0;
+   return (XrdOssRPList->Find(Lfn) & XRDEXP_MIGPRG) != 0;
 }
 
 /******************************************************************************/
@@ -672,7 +672,7 @@ int XrdFrmConfig::ConfigMP(const char *pType)
         if (*psVal == '/')
            {pOpts = XrdOssRPList->Find(psVal);
             if (pOpts & xOpt) mypList = InsertPL(mypList, psVal, psLen,
-                                                (pOpts & XRDEXP_MAKELF ? 1:0));
+                                                (pOpts & XRDEXP_MIGPRG ? 1:0));
                else {Say.Say("Config", psVal, "not marked", pType); NoGo = 1;}
            } else {
             VPInfo *vP = VPList;
@@ -694,7 +694,7 @@ int XrdFrmConfig::ConfigMP(const char *pType)
       {XrdOucPList *fP = XrdOssRPList->First();
        short sval[4];    // Last two elements are unused
        while(fP)
-            {sval[0] = (fP->Flag() & XRDEXP_MAKELF ? 1 : 0);
+            {sval[0] = (fP->Flag() & XRDEXP_MIGPRG ? 1 : 0);
              sval[1] = fP->Plen();
              if (fP->Flag() & xOpt)
                  mypList = new XrdOucTList(fP->Path(), sval, mypList);
@@ -972,6 +972,8 @@ int XrdFrmConfig::ConfigProc()
        return 1;
       }
    cfgFile.Attach(cfgFD); cFile = &cfgFile;
+   static const char *cvec[] = { "*** frm server config:", 0 };
+   cfgFile.Capture(cvec);
 
 // Now start reading records until eof.
 //
@@ -1016,9 +1018,9 @@ int XrdFrmConfig::ConfigXeq(char *var, int mbok)
        if (!strcmp(var, "ofs.ckslib"    )) PARSEPI(theCksLib);
        if (!strcmp(var, "ofs.osslib"    )) PARSEPI(theOssLib);
        if (!strcmp(var, "ofs.xattrlib"  )) PARSEPI(theAtrLib);
-       if (!strcmp(var, "oss.cache"     )){hasCache = 1; // runOld
-                                           return xspace(0,0);
-                                          }
+//     if (!strcmp(var, "oss.cache"     )){hasCache = 1; // runOld
+//                                         return xspace(0,0);
+//                                        }
        if (!strcmp(var, "oss.localroot" )) return Grab(var, &LocalRoot, 0);
        if (!strcmp(var, "oss.namelib"   )) return xnml();
        if (!strcmp(var, "oss.remoteroot")) return Grab(var, &RemoteRoot, 0);
@@ -1499,20 +1501,32 @@ int XrdFrmConfig::xcopy(int &TLim)
 
 /* Function: copymax
 
-   Purpose:  To parse the directive: copymax  <num>
+   Purpose:  To parse the directive: copymax  <num> | split <inmax> <outmax>
 
              <num>     maximum number of simultaneous transfers
+             <inmax>   maximum number of simultaneous transfers in  (get|stage)
+             <outmax>  maximum number of simultaneous transfers out (put|migr)
 
    Output: 0 upon success or !0 upon failure.
 */
 int XrdFrmConfig::xcmax()
-{   int xmax = 1;
-    char *val;
+{   char *val;
 
     if (!(val = cFile->GetWord()))
-       {Say.Emsg("Config", "maxio value not specified"); return 1;}
-    if (XrdOuca2x::a2i(Say, "maxio", val, &xmax, 1)) return 1;
-    xfrMax = xmax;
+       {Say.Emsg("Config", "copymax value not specified"); return 1;}
+
+    if (!strcmp(val, "split"))
+       {if (!(val = cFile->GetWord()))
+           {Say.Emsg("Config", "copymax in value not specified"); return 1;}
+        if (XrdOuca2x::a2i(Say,"copymax in",  val, &xfrMaxIn, 1)) return 1;
+        if (!(val = cFile->GetWord()))
+           {Say.Emsg("Config", "copymax out value not specified"); return 1;}
+        if (XrdOuca2x::a2i(Say,"copymax out", val, &xfrMaxOt, 1))  return 1;
+        xfrMax = xfrMaxIn + xfrMaxOt;
+       } else {
+        if (XrdOuca2x::a2i(Say, "copymax", val, &xfrMax, 1)) return 1;
+        xfrMaxIn = xfrMaxOt = 0;
+       }
     return 0;
 }
 
@@ -2018,6 +2032,14 @@ int XrdFrmConfig::xspace(int isPrg, int isXA)
           else isXA = 1;
       }
 
+// We only support xa caches now
+//
+    if (!isXA)
+       {Say.Emsg("Config","old-style spaces using oss.cache are no longer "
+                 "supported!");
+        return 1;
+       }
+
    if (fn[k-1] != '*')
       {for (i = k-1; i; i--) if (fn[i] != '/') break;
        fn[i+1] = '/'; fn[i+2] = '\0';
