@@ -25,20 +25,34 @@
 #include "XrdHttpExtHandler.hh"
 #include "XrdHttpReq.hh"
 #include "XrdHttpProtocol.hh"
+#include "XrdOuc/XrdOucEnv.hh"
 
 /// Sends a basic response. If the length is < 0 then it is calculated internally
 int XrdHttpExtReq::SendSimpleResp(int code, const char* desc, const char* header_to_add, const char* body, long long int bodylen)
 {
   if (!prot) return -1;
-  
-  return prot->SendSimpleResp(code, desc, header_to_add, body, bodylen);
+
+  // @FIXME
+  // - need this to circumvent missing API calls and keep ABI compatibility
+  // - when large files are returned we cannot return them in a single buffer
+  // @TODO: for XRootD 5.0 this two hidden calls should just be added to the external handler API and the code here can be removed
+
+  if ( code == 0 ) {
+    return prot->StartSimpleResp(200, desc, header_to_add, bodylen, true);
+  }
+
+  if ( code == 1 ) {
+    return prot->SendData(body, bodylen);
+  }
+
+  return prot->SendSimpleResp(code, desc, header_to_add, body, bodylen, true);
 }
 
 int XrdHttpExtReq::StartChunkedResp(int code, const char *desc, const char *header_to_add)
 {
   if (!prot) return -1;
 
-  return prot->StartChunkedResp(code, desc, header_to_add);
+  return prot->StartChunkedResp(code, desc, header_to_add, true);
 }
 
 int XrdHttpExtReq::ChunkResp(const char *body, long long bodylen)
@@ -67,6 +81,11 @@ XrdHttpExtReq::XrdHttpExtReq(XrdHttpReq *req, XrdHttpProtocol *pr): prot(pr),
 verb(req->requestverb), headers(req->allheaders) {
   // Here we fill the request summary with all the fields we can
   resource = req->resource.c_str();
+  int envlen = 0;
+  
+  headers["xrd-http-query"] = req->opaque?req->opaque->Env(envlen):"";
+  headers["xrd-http-fullresource"] = req->resourceplusopaque.c_str();
+  headers["xrd-http-prot"] = prot->isHTTPS()?"https":"http";
   
   // These fields usually identify the client that connected
 
