@@ -661,6 +661,15 @@ bool XrdHttpReq::Redir(XrdXrootd::Bridge::Context &info, //!< the result context
   else
     redirdest = "Location: http://";
   
+  // port < 0 signals switch to full URL
+  if (port < 0)
+  {
+    if (strncmp(hname, "file://", 7) == 0)
+    {
+      TRACE(REQ, " XrdHttpReq::Redir Switching to file:// ");
+      redirdest = "Location: "; // "file://" already contained in hname
+    }
+  }
   // Beware, certain Ofs implementations (e.g. EOS) add opaque data directly to the host name
   // This must be correctly treated here and appended to the opaque info
   // that we may already have
@@ -681,7 +690,7 @@ bool XrdHttpReq::Redir(XrdXrootd::Bridge::Context &info, //!< the result context
   else
     redirdest += hname;
 
-  if (port) {
+  if (port > 0) {
     sprintf(buf, ":%d", port);
     redirdest += buf;
   }
@@ -1118,6 +1127,11 @@ int XrdHttpReq::ProcessHTTPReq() {
         case 1: // Open() or dirlist
         {
 
+          if (!prot->Bridge) {
+              prot->SendSimpleResp(500, NULL, NULL, (char *) "prot->Bridge is NULL.", 0, false);
+              return -1;
+            }
+
           if (fileflags & kXR_isDir) {
 
             if (prot->listdeny) {
@@ -1876,10 +1890,14 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
           stringresp += "</head>\n"
                   "<body>\n";
 
+          char *estr = escapeXML(resource.c_str());
+          
           stringresp += "<h1>Listing of: ";
-          stringresp += resource.c_str();
+          stringresp += estr;
           stringresp += "</h1>\n";
 
+          free(estr);
+          
           stringresp += "<div id=\"header\">";
 
 
@@ -1952,13 +1970,22 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                       "<a href=\"";
 
               if (resource != "/") {
-                  p += resource.c_str();
+                
+                char *estr = escapeXML(resource.c_str());
+                
+                  p += estr;
                   p += "/";
+                  
+                free(estr);
               }
+              
+              char *estr = escapeXML(e.path.c_str());
+              
               p += e.path + "\">";
-
               p += e.path;
-
+              
+              free(estr);
+              
               p += "</a></td></tr>";
 
               stringresp += p;
@@ -2440,7 +2467,15 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
 
               string p;
               stringresp += "<D:response xmlns:lp1=\"DAV:\" xmlns:lp2=\"http://apache.org/dav/props/\" xmlns:lp3=\"LCGDM:\">\n";
-              stringresp += "<D:href>" + e.path + "</D:href>\n";
+              
+              char *estr = escapeXML(e.path.c_str());
+              
+              stringresp += "<D:href>";
+              stringresp += estr;
+              stringresp += "</D:href>\n";
+              
+              free(estr);
+              
               stringresp += "<D:propstat>\n<D:prop>\n";
 
               // Now add the properties that we have to add
@@ -2547,9 +2582,17 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
 
                 string p = resource.c_str();
                 if (*p.rbegin() != '/') p += "/";
+                
                 p += e.path;
+                
                 stringresp += "<D:response xmlns:lp1=\"DAV:\" xmlns:lp2=\"http://apache.org/dav/props/\" xmlns:lp3=\"LCGDM:\">\n";
-                stringresp += "<D:href>" + p + "</D:href>\n";
+                
+                char *estr = escapeXML(p.c_str());
+                stringresp += "<D:href>";
+                stringresp += estr;
+                stringresp += "</D:href>\n";
+                free(estr);
+                
                 stringresp += "<D:propstat>\n<D:prop>\n";
 
 
