@@ -29,7 +29,6 @@
 /******************************************************************************/
   
 #include <ctype.h>
-#include <errno.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -39,6 +38,7 @@
 #include "XrdNet/XrdNetAddr.hh"
 #include "XrdNet/XrdNetCache.hh"
 #include "XrdNet/XrdNetUtils.hh"
+#include "XrdSys/XrdSysE2T.hh"
 
 /******************************************************************************/
 /*                 P l a t f o r m   D e p e n d e n c i e s                  */
@@ -164,6 +164,42 @@ int XrdNetAddr::Port(int pNum)
    return pNum;
 }
   
+/******************************************************************************/
+/*                              R e g i s t e r                               */
+/******************************************************************************/
+  
+bool XrdNetAddr::Register(const char *hName)
+{
+   XrdNetAddr *aListVec = 0;
+   int i, aListNum;
+
+// Step one is to make sure the incomming name is not an address
+//
+   if (!isHostName(hName)) return false;
+
+// The next step is to get all of the IP addresses registered for this name
+//
+   if (XrdNetUtils::GetAddrs(hName, &aListVec, aListNum,
+                      XrdNetUtils::allIPMap, XrdNetUtils::NoPortRaw))
+      return false;
+
+// In order to use the given name, one of the IP addresses in the list must
+// match our address. This is about as secure we can get.
+//
+   for (i = 0; i < aListNum; i++) {if (Same(&aListVec[i])) break;}
+   delete [] aListVec;
+
+// If we didn't find a match, report it
+//
+   if (i >= aListNum) return false;
+
+// Replace current hostname with the wanted one
+//
+   if (hostName) free(hostName);
+   hostName = strdup(hName);
+   return true;
+}
+
 /******************************************************************************/
 /*                                   S e t                                    */
 /******************************************************************************/
@@ -360,15 +396,11 @@ const char *XrdNetAddr::Set(const char *hSpec, int &numIP, int maxIP,
 
 const char *XrdNetAddr::Set(const struct sockaddr *sockP, int sockFD)
 {
-// Make sure we won't loose any bits of sockFD (we should use an int)
-//
-   if (sockFD >=0 && (sockFD & 0xffff0000) != 0) return "FD is out of range";
-
 // Clear translation if set
 //
    if (hostName)             {free(hostName);  hostName = 0;}
    if (sockAddr != &IP.Addr) {delete unixPipe; sockAddr = &IP.Addr;}
-   sockNum = static_cast<unsigned short>(sockFD);
+   sockNum = sockFD;
 
 // Copy the address based on address family
 //
@@ -400,35 +432,28 @@ const char *XrdNetAddr::Set(const struct sockaddr *sockP, int sockFD)
   
 const char *XrdNetAddr::Set(int sockFD, bool peer)
 {
+   SOCKLEN_t aSize = static_cast<SOCKLEN_t>(sizeof(IP));
    int rc;
 
-// Make sure we won't loose any bits of sockFD (we should use an int)
-//
-   if ((sockFD & 0xffff0000) != 0) return "FD is out of range";
-
 // Clear translation if set
 //
    if (hostName)             {free(hostName);  hostName = 0;}
    if (sockAddr != &IP.Addr) {delete unixPipe; sockAddr = &IP.Addr;}
-   addrSize = sizeof(sockaddr_in6);
-   sockNum = static_cast<unsigned short>(sockFD);
+   sockNum = sockFD;
 
 // Get the address on the appropriate side of this socket
 //
-   if (peer) rc = getpeername(sockFD, &IP.Addr, &addrSize);
-      else   rc = getsockname(sockFD, &IP.Addr, &addrSize);
+   if (peer) rc = getpeername(sockFD, &IP.Addr, &aSize);
+      else   rc = getsockname(sockFD, &IP.Addr, &aSize);
    if (rc < 0)
       {addrSize = 0;
-       return strerror(errno);
+       return XrdSysE2T(errno);
       }
 
-// Set the correct address size
+// Set the correct address size and protocol family
 //
-   if (IP.Addr.sa_family == AF_INET)
-      {addrSize = sizeof(sockaddr_in);  protType = PF_INET;
-      } else {
-       addrSize = sizeof(sockaddr_in6); protType = PF_INET6;
-      }
+   addrSize = aSize;
+   protType = (IP.Addr.sa_family == AF_INET ? PF_INET : PF_INET6);
 
 // All done
 //
@@ -537,3 +562,24 @@ void XrdNetAddr::SetIPV6()
 //
    XrdNetUtils::SetAuto(XrdNetUtils::allIPMap);
 }
+
+/******************************************************************************/
+/*                           S e t L o c a t i o n                            */
+/******************************************************************************/
+  
+void XrdNetAddr::SetLocation(XrdNetAddrInfo::LocInfo &loc)
+{
+// Copy in the new location information but preserve the flags
+//
+   addrLoc = loc;
+}
+
+/******************************************************************************/
+/*                                S e t T L S                                 */
+/******************************************************************************/
+  
+void XrdNetAddr::SetTLS(bool val)
+{
+   if (val) protFlgs |=  isTLS;
+      else  protFlgs &= ~isTLS;
+}
