@@ -29,7 +29,6 @@
 /******************************************************************************/
 
 #include <ctype.h>
-#include <errno.h>
 #include <netdb.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -43,6 +42,7 @@
 #include "XrdNet/XrdNetIF.hh"
 #include "XrdNet/XrdNetUtils.hh"
 #include "XrdOuc/XrdOucTList.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
 
 #include <iostream>
@@ -133,6 +133,8 @@ int               XrdNetIF::dfPort    = 1094;
 
 XrdNetIF::ifData  XrdNetIF::ifNull;
 
+bool              XrdNetIF::rPIPA = false;
+
 /******************************************************************************/
 /*                               D i s p l a y                                */
 /******************************************************************************/
@@ -264,7 +266,7 @@ bool XrdNetIF::GenAddrs(ifAddrs &ifTab, const char *hName, bool wantV6)
 
 #define RLOSLOT(xdst) xdst = (ifData *)(ifBuff+((char *)xdst-buff))
 
-bool XrdNetIF::GenIF(XrdNetAddrInfo **src, int srcnum)
+bool XrdNetIF::GenIF(XrdNetAddrInfo **src, int srcnum, const char *xName)
 {
    ifAddrs ifTab;
    const char *hName;
@@ -316,12 +318,14 @@ for (i = 0; i < srcnum; i++)
 // If this is a private interface, then set private pointers to actual addresses
 // since, technically, private addresses should not be registered. Otherwise,
 // fill in the public interface information. We also set unregistered public
-// addresses (what a pain).
+// addresses (what a pain). Of course, that is a configurable detail.
 //
-   if (isPrivate)
+   if (!rPIPA && isPrivate)
       {ADDSLOT(ifName[ifT], ifTab.hAddr, ifTab.hALen);
       } else {
-       if ((hName = src[i]->Name()) && src[i]->isRegistered())
+       hName = src[i]->Name();
+       if (!hName || !(src[i]->isRegistered())) hName = xName;
+       if (hName)
           {ADDSLOT(ifName[ifT], hName, strlen(hName));
            ifxDNS[ifT] = true;
           } else  ifName[ifT]  = ifDest[ifT];
@@ -339,9 +343,11 @@ for (i = 0; i < srcnum; i++)
       {if (!ifTab.prvt)
           {ADDSLOT(ifDest[PublicV4], ifTab.hDest, ifTab.hDLen);
            ifName[PublicV4]  = ifName[PublicV6];
+           ifxDNS[PublicV4]  = ifxDNS[PublicV6];
           } else if (ifDest[PrivateV4] == &ifNull)
           {ADDSLOT(ifDest[PrivateV4], ifTab.hDest, ifTab.hDLen);
            ifName[PrivateV4] = ifName[PublicV6];
+           ifxDNS[PrivateV4] = ifxDNS[PublicV6];
           }
       }
 
@@ -350,9 +356,11 @@ for (i = 0; i < srcnum; i++)
       {if (!ifTab.prvt)
           {ADDSLOT(ifDest[PublicV6], ifTab.hDest, ifTab.hDLen);
            ifName[PublicV6]  = ifName[PublicV4];
+           ifxDNS[PublicV6]  = ifxDNS[PublicV4];
           } else if (ifDest[PrivateV6] == &ifNull)
           {ADDSLOT(ifDest[PrivateV6], ifTab.hDest, ifTab.hDLen);
            ifName[PrivateV6] = ifName[PublicV4];
+           ifxDNS[PrivateV6] = ifxDNS[PublicV4];
           }
       }
 
@@ -422,7 +430,7 @@ int XrdNetIF::GetIF(XrdOucTList **ifList, const char **eText)
    int             n = 0;
 
    if (getifaddrs(&ifBase) < 0)
-      {if (eText) *eText = strerror(errno);
+      {if (eText) *eText = XrdSysE2T(errno);
        if (ifList) *ifList = 0;
        if (eDest) eDest->Emsg("GetIF", errno, "get interface addresses.");
        return 0;
@@ -634,6 +642,12 @@ int XrdNetIF::Port(int pnum)
    return prevport;
 }
   
+/******************************************************************************/
+/*                           P o r t D e f a u l t                            */
+/******************************************************************************/
+
+void XrdNetIF::PortDefault(int pnum) {dfPort = pnum;}
+  
 /******************************************************************************/
 /*                               R o u t i n g                                */
 /******************************************************************************/
@@ -676,7 +690,7 @@ char *XrdNetIF::SetDomain()
 /******************************************************************************/
   
 bool XrdNetIF::SetIF(XrdNetAddrInfo *src, const char *ifList, int port,
-                     netType nettype)
+                     netType nettype, const char *xName)
 {
    XrdNetAddrInfo *netIF[4] = {0,0,0,0}; //pub 0:v4, prv 1:v4 pub 2:v6 prv 3:v6
    XrdNetAddr      netAdr[4];
@@ -698,13 +712,14 @@ bool XrdNetIF::SetIF(XrdNetAddrInfo *src, const char *ifList, int port,
       {XrdNetAddrInfo *ifVec[8];
        XrdNetAddr *iP;
        const char *hName = src->Name();
+       if (!(src->isRegistered()) && xName) hName = xName;
        ifCnt = 0;
        if (!hName
        ||  XrdNetUtils::GetAddrs(hName,&iP,ifCnt,XrdNetUtils::allIPv64,ifPort)
        || !ifCnt) return SetIF64(GenIF(&src, 1));
        if (ifCnt > 8) ifCnt = 8;
        for (i = 0; i < ifCnt; i++) ifVec[i] = &iP[i];
-       bool aOK = GenIF(ifVec, ifCnt);
+       bool aOK = GenIF(ifVec, ifCnt, hName);
        delete [] iP;
        return SetIF64(aOK);
       }
@@ -794,8 +809,8 @@ void XrdNetIF::SetIFPP()
 // Now set all undefined private interfaces for common and local network routing
 //
    i = (int)PrivateV4; j = PublicV4;
-   do {if (ifName[i] == &ifNull) ifName[i]=ifName[j];
-       if (ifDest[i] == &ifNull) ifDest[i]=ifDest[j];
+   do {if (ifName[i] == &ifNull) {ifName[i]=ifName[j]; ifxDNS[i]=ifxDNS[j];}
+       if (ifDest[i] == &ifNull)  ifDest[i]=ifDest[j];
        if (i == (int)PrivateV6) break;
        i = (int)PrivateV6; j = (int)PublicV6;
       } while(true);
@@ -807,8 +822,8 @@ void XrdNetIF::SetIFPP()
 // Now set all undefined public  interfaces for local network routing
 //
    i = (int)PublicV4; j = PrivateV4;
-   do {if (ifName[i] == &ifNull) ifName[i]=ifName[j];
-       if (ifDest[i] == &ifNull) ifDest[i]=ifDest[j];
+   do {if (ifName[i] == &ifNull) {ifName[i]=ifName[j]; ifxDNS[i]=ifxDNS[j];}
+       if (ifDest[i] == &ifNull)  ifDest[i]=ifDest[j];
        if (i == (int)PublicV6) break;
        i = (int)PublicV6; j = (int)PrivateV6;
       } while(true);
@@ -853,6 +868,18 @@ bool XrdNetIF::SetIF64(bool retVal)
    return retVal;
 }
 
+/******************************************************************************/
+/*                               S e t M s g s                                */
+/******************************************************************************/
+
+void XrdNetIF::SetMsgs(XrdSysError *erp) {eDest = erp;}
+  
+/******************************************************************************/
+/*                              S e t R P I P A                               */
+/******************************************************************************/
+
+void XrdNetIF::SetRPIPA(bool rval) {rPIPA = rval;}
+  
 /******************************************************************************/
 /* Private:                  V 4 L i n k L o c a l                            */
 /******************************************************************************/
