@@ -30,6 +30,7 @@
 
 #ifndef WIN32
 #include <unistd.h>
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <poll.h>
@@ -76,9 +77,6 @@ namespace XrdNetSocketCFG
 /******************************************************************************/
 /*                         l o c a l   d e f i n e s                          */
 /******************************************************************************/
-  
-#define XRDNETSOCKET_MAXBKLG 255
-#define XRDNETSOCKET_LINGER    3
 
 #define Err(p,a,b,c) (ErrCode = (eroute ? eroute->Emsg(#p, a, b, c) : ErrCode),-1)
 #define ErrM(p,a,b,c) (ErrCode = (eroute ? eroute->Emsg(#p, a, b, c) : ErrCode),-1)
@@ -242,9 +240,9 @@ int XrdNetSocket::Open(const char *inpath, int port, int flags, int windowsz)
    if ((eText = SockInfo.Set(inpath,(port < 0 ? XrdNetAddr::PortInSpec:port))))
       {ErrCode = EHOSTUNREACH;
        if (eroute)
-          {char buff[256];
-           snprintf(buff, sizeof(buff), "'; %s", eText);
-           eroute->Emsg("Open", "Unable to create socket for '", epath, buff);
+          {char buff[512];
+           snprintf(buff,sizeof(buff),"'%s'; %c%s",epath,tolower(*eText),eText+1);
+           eroute->Emsg("Open", "Unable to create socket for", buff);
           }
        return -1;
       }
@@ -479,7 +477,7 @@ char *XrdNetSocket::socketPath(XrdSysError *Say, char *fnbuff,
            return 0;
           }
        if (access(fnbuff, W_OK))
-          {Say->Emsg("cratePath", errno, "access path", fnbuff);
+          {Say->Emsg("createPath", errno, "access path", fnbuff);
            return 0;
           }
       } else chmod(fnbuff, mode); // This may fail on some platforms
