@@ -29,7 +29,6 @@
 /******************************************************************************/
 
 #include <ctype.h>
-#include <errno.h>
 #include <inttypes.h>
 #include <netdb.h>
 #include <string.h>
@@ -43,6 +42,7 @@
 #include "XrdNet/XrdNetIF.hh"
 #include "XrdNet/XrdNetUtils.hh"
 #include "XrdOuc/XrdOucTList.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 #ifndef HAVE_PROTOR
 #include "XrdSys/XrdSysPthread.hh"
@@ -230,7 +230,7 @@ const char  *XrdNetUtils::GetAddrs(const char            *hSpec,
 //
    if (hints.ai_family == AF_INET6 && aBuff.ipAdr[0] != '['
    && !XrdNetAddrInfo::isHostName(aBuff.ipAdr))
-      {strncpy(aBuff.ipMap, "::ffff:", 7);
+      {memcpy(aBuff.ipMap, "::ffff:", 7);
        ipAddr = aBuff.ipMap;
       } else ipAddr = hnBeg;
 
@@ -380,7 +380,7 @@ bool XrdNetUtils::Match(const char *HostName, const char *HostPat)
        {XrdNetAddr InetAddr[maxIP];
         char hBuff[264];
         if (i >= (int)sizeof(hBuff)) return false;
-        strncpy(hBuff, HostPat, i-1);
+        memcpy(hBuff, HostPat, i-1);
         hBuff[i-1] = 0;
         if (InetAddr[0].Set(hBuff, i, maxIP, 0)) return false;
         while(i--) if ((mval = InetAddr[i].Name()) && !strcmp(mval, HostName))
@@ -516,7 +516,7 @@ bool XrdNetUtils::Parse(const char *hSpec,
 /*                                  P o r t                                   */
 /******************************************************************************/
 
-int XrdNetUtils::Port(int fd, char **eText)
+int XrdNetUtils::Port(int fd, const char **eText)
 {
    XrdNetSockAddr Inet;
    SOCKLEN_t slen = (socklen_t)sizeof(Inet);
@@ -646,9 +646,9 @@ int XrdNetUtils::SetAuto(XrdNetUtils::AddrOpts aOpts)
 /* Private:                        s e t E T                                  */
 /******************************************************************************/
   
-int XrdNetUtils::setET(char **errtxt, int rc)
+int XrdNetUtils::setET(const char **errtxt, int rc)
 {
-    if (rc) *errtxt = strerror(rc);
-       else *errtxt = (char *)"unexpected error";
+    if (rc) *errtxt = XrdSysE2T(rc);
+       else *errtxt = "unexpected error";
     return 0;
 }
