@@ -42,6 +42,8 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
+#include "XProtocol/XProtocol.hh"
+
 #include "XrdCks/XrdCks.hh"
 #include "XrdCks/XrdCksConfig.hh"
 #include "XrdCks/XrdCksData.hh"
@@ -54,6 +56,7 @@
 #include "XrdOfs/XrdOfsEvs.hh"
 #include "XrdOfs/XrdOfsHandle.hh"
 #include "XrdOfs/XrdOfsPoscq.hh"
+#include "XrdOfs/XrdOfsPrepare.hh"
 #include "XrdOfs/XrdOfsTrace.hh"
 #include "XrdOfs/XrdOfsSecurity.hh"
 #include "XrdOfs/XrdOfsStats.hh"
@@ -86,6 +89,10 @@
 #include <sys/mode.h>
 #endif
 
+#ifndef O_DIRECT
+#define O_DIRECT 0
+#endif
+
 /******************************************************************************/
 /*                  E r r o r   R o u t i n g   O b j e c t                   */
 /******************************************************************************/
@@ -125,20 +132,23 @@ XrdOss *XrdOfsOss;
 /*                    X r d O f s   C o n s t r u c t o r                     */
 /******************************************************************************/
 
-XrdOfs::XrdOfs()
+XrdOfs::XrdOfs() : tpcRdrHost{}, tpcRdrPort{}
 {
    const char *bp;
 
 // Establish defaults
 //
    ofsConfig     = 0;
+   FSctl_PI      = 0;
    Authorization = 0;
    Finder        = 0;
    Balancer      = 0;
    evsObject     = 0;
+   ossRPList     = 0;
    myRole        = strdup("server");
    OssIsProxy    = 0;
    ossRW         =' ';
+   ossFeatures   = 0;
 
 // Obtain port number we will be using. Note that the constructor must occur
 // after the port number is known (i.e., this cannot be a global static).
@@ -151,6 +161,7 @@ XrdOfs::XrdOfs()
    poscLog = 0;
    poscHold= 10*60;
    poscAuto= 0;
+   poscSync= 1;
 
 // Set the configuration file name and dummy handle
 //
@@ -161,13 +172,29 @@ XrdOfs::XrdOfs()
 //
    Cks       = 0;
    CksPfn    = true;
+   CksRdr    = true;
+
+// Prepare handling
+//
+   prepHandler = 0;
+   prepAuth = true;
+
+// Eextended attribute limits
+//
+   usxMaxNsz = kXR_faMaxNlen;
+   usxMaxVsz = kXR_faMaxVlen;
+
+// Other options
+//
+   DirRdr    = false;
 }
   
 /******************************************************************************/
 /*                X r d O f s F i l e   C o n s t r u c t o r                 */
 /******************************************************************************/
 
-XrdOfsFile::XrdOfsFile(const char *user, int monid) : XrdSfsFile(user, monid)
+XrdOfsFile::XrdOfsFile(XrdOucErrInfo &eInfo, const char *user)
+                      : XrdSfsFile(eInfo)
 {
    oh = XrdOfs::dummyHandle; 
    dorawio = 0;
@@ -204,6 +231,7 @@ int XrdOfsDirectory::open(const char              *dir_path, // In
 */
 {
    EPNAME("opendir");
+   static const int od_mode = SFS_O_RDONLY|SFS_O_META;
    XrdOucEnv Open_Env(info,0,client);
    int retc;
 
@@ -220,6 +248,12 @@ int XrdOfsDirectory::open(const char              *dir_path, // In
 //
    AUTHORIZE(client,&Open_Env,AOP_Readdir,"open directory",dir_path,error);
 
+// Find out where we should open this directory
+//
+   if (XrdOfsFS->DirRdr && XrdOfsFS->Finder && XrdOfsFS->Finder->isRemote()
+   &&  (retc = XrdOfsFS->Finder->Locate(error, dir_path, od_mode, &Open_Env)))
+      return XrdOfsFS->fsError(error, retc);
+
 // Open the directory and allocate a handle for it
 //
    if (!(dp = XrdOfsOss->newDir(tident))) retc = -ENOMEM;
@@ -366,15 +400,10 @@ int XrdOfsDirectory::autoStat(struct stat *buf)
              return SFS_ERROR;
             }
 
-// Set the stat buffer in the storage system directory.
+// Set the stat buffer in the storage system directory but don't complain.
 //
-    if ((retc = dp->StatRet(buf)))
-       retc = XrdOfsFS->Emsg(epname, error, retc, "autostat", fname);
-       else retc = SFS_OK;
-
-// All done
-//
-   return retc;
+    if ((retc = dp->StatRet(buf))) return retc;
+    return SFS_OK;
 }
   
 /******************************************************************************/
@@ -399,6 +428,7 @@ int XrdOfsFile::open(const char          *path,      // In
                         SFS_O_RDONLY - Open file for reading.
                         SFS_O_WRONLY - Open file for writing.
                         SFS_O_RDWR   - Open file for update
+                        SFS_O_NOTPC  - Disallow TPC opens
                         SFS_O_REPLICA- Open file for replication
                         SFS_O_CREAT  - Create the file open in RW mode
                         SFS_O_TRUNC  - Trunc  the file open in RW mode
@@ -492,6 +522,20 @@ int XrdOfsFile::open(const char          *path,      // In
                       break;
   }
 
+// Preset TPC handling
+//
+   tpcKey = Open_Env.Get(XrdOucTPC::tpcKey);
+
+// Check if we will be redirecting the tpc request
+//
+   if (tpcKey && isRW && (XrdOfsFS->Options & XrdOfs::RdrTPC))
+      {const char *dOn = Open_Env.Get(XrdOucTPC::tpcDlgOn);
+       int k = ((dOn && *dOn == '1') || strcmp(tpcKey, "delegate") ? 1 : 0);
+       if (XrdOfsFS->tpcRdrHost[k])
+          {error.setErrInfo(XrdOfsFS->tpcRdrPort[k], XrdOfsFS->tpcRdrHost[k]);
+           return SFS_REDIRECT;
+          }
+      }
 
 // If we have a finder object, use it to direct the client. The final
 // destination will apply the security that is needed
@@ -500,9 +544,10 @@ int XrdOfsFile::open(const char          *path,      // In
                                                    find_flag, &Open_Env)))
       return XrdOfsFS->fsError(error, retc);
 
-// Preset TPC handling
+// Preset TPC handling and if not allowed, complain
 //
-   tpcKey = Open_Env.Get(XrdOucTPC::tpcKey);
+   if (tpcKey && (open_mode & SFS_O_NOTPC))
+      return XrdOfsFS->Emsg(epname, error, EPROTOTYPE, "tpc", path);
 
 // Create the file if so requested o/w try to attach the file
 //
@@ -562,7 +607,8 @@ int XrdOfsFile::open(const char          *path,      // In
       }
 
 // If this is a third party copy and we are the destination, then validate
-// specification at this point and setup to transfer.
+// specification at this point and setup to transfer. Note that if the
+// call fails and auto removal is enabled, the file we created will be deleted.
 //
    if (tpcKey && isRW)
       {char pfnbuff[MAXPATHLEN+8]; const char *pfnP;
@@ -608,9 +654,13 @@ int XrdOfsFile::open(const char          *path,      // In
       return XrdOfsFS->Emsg(epname, error, ENOMEM, "open", path);
 
 // We need to make special provisions for proxy servers in the presence of
-// the TPC option as it's handled differently in this case.
+// the TPC option and possibly cache as it's handled differently in this case.
 //
-   if (myTPC && XrdOfsFS->OssIsProxy) open_flag |= O_NOFOLLOW;
+   if (XrdOfsFS->OssIsProxy)
+      {if (myTPC) open_flag |= O_NOFOLLOW;
+       if (error.getUCap() & XrdOucEI::uUrlOK &&
+           error.getUCap() & XrdOucEI::uLclF) open_flag |= O_DIRECT;
+      }
 
 // Open the file
 //
@@ -621,7 +671,11 @@ int XrdOfsFile::open(const char          *path,      // In
            return XrdOfsFS->fsError(error, SFS_STARTED);
           }
        if (retc == -ETXTBSY) return XrdOfsFS->Stall(error, -1, path);
-       if (XrdOfsFS->Balancer && retc != -ECANCELED)
+       if (retc == -EDESTADDRREQ)
+          {char *url = Open_Env.Get("FileURL");
+           if (url) {error.setErrInfo(-1, url); return SFS_REDIRECT;}
+          }
+       if (XrdOfsFS->Balancer && retc == -ENOENT)
           XrdOfsFS->Balancer->Removed(path);
        return XrdOfsFS->Emsg(epname, error, retc, "open", path);
       }
@@ -787,10 +841,8 @@ int            XrdOfsFile::fctl(const int               cmd,
 
 /******************************************************************************/
 
-int            XrdOfsFile::fctl(const int               cmd,
-                                      int               alen,
-                                const char             *args,
-                                const XrdSecEntity     *client)
+int XrdOfsFile::fctl(const int cmd, int alen, const char *args,
+                     const XrdSecEntity *client)
 {                             // 12345678901234
    static const char *fctlArg = "ofs.tpc cancel";
    static const int   fctlAsz = 15;
@@ -798,9 +850,7 @@ int            XrdOfsFile::fctl(const int               cmd,
 // See if the is a tpc cancellation (the only thing we support here)
 //
    if (cmd != SFS_FCTL_SPEC1 || !args || alen < fctlAsz || strcmp(fctlArg,args))
-      {error.setErrInfo(ENOTSUP, "fctl operation not supported");
-       return SFS_ERROR;
-      }
+      return XrdOfsFS->FSctl(*this, cmd, alen, args, client);
 
 // Check if we have a tpc operation in progress
 //
@@ -1359,7 +1409,7 @@ int XrdOfs::chksum(      csFunc            Func,   // In
 
 // If we are a menager then we need to redirect the client to where the file is
 //
-   if (Finder && Finder->isRemote() 
+   if (CksRdr && Finder && Finder->isRemote() 
    &&  (rc = Finder->Locate(einfo, Path, SFS_O_RDONLY, &cksEnv)))
       return fsError(einfo, rc);
 
@@ -1368,6 +1418,12 @@ int XrdOfs::chksum(      csFunc            Func,   // In
    if (CksPfn && !(Path = XrdOfsOss->Lfn2Pfn(Path, buff, MAXPATHLEN, rc)))
       return Emsg(epname, einfo, rc, "checksum", Path);
 
+// Originally we only passed he env pointer for proxy servers. Due to popular
+// demand, we always pass the env as it points to the SecEntity object.
+//
+   if (Func == XrdSfsFileSystem::csGet || Func == XrdSfsFileSystem::csCalc)
+      cksData.envP = &cksEnv;
+
 // Now determine what to do
 //
         if (Func == XrdSfsFileSystem::csCalc) rc = Cks->Calc(Path, cksData);
@@ -1384,7 +1440,7 @@ int XrdOfs::chksum(      csFunc            Func,   // In
    if (rc >= 0 || rc == -ENODATA || rc == -ESTALE || rc == -ESRCH)
 #endif
       {if (rc >= 0) {cksData.Get(buff, MAXPATHLEN); rc = 0;}
-          else {*buff = 0; rc = ENOENT;}
+          else {*buff = 0; rc = -rc;}
        einfo.setErrInfo(rc, buff);
        return SFS_OK;
       }
@@ -1454,6 +1510,32 @@ int XrdOfs::chmod(const char             *path,    // In
    return XrdOfsFS->Emsg(epname, einfo, retc, "change", path);
 }
 
+/******************************************************************************/
+/*                               C o n n e c t                                */
+/******************************************************************************/
+
+void XrdOfs::Connect(const XrdSecEntity *client)
+{
+   XrdOucEnv myEnv(0, 0, client);
+
+// Pass this call along
+//
+   XrdOfsOss->Connect(myEnv);
+}
+
+/******************************************************************************/
+/*                                  D i s c                                   */
+/******************************************************************************/
+  
+void XrdOfs::Disc(const XrdSecEntity *client)
+{
+   XrdOucEnv myEnv(0, 0, client);
+
+// Pass this call along
+//
+   XrdOfsOss->Disc(myEnv);
+}
+
 /******************************************************************************/
 /*                                e x i s t s                                 */
 /******************************************************************************/
@@ -1517,172 +1599,6 @@ int XrdOfs::exists(const char                *path,        // In
    return XrdOfsFS->Emsg(epname, einfo, retc, "locate", path);
 }
 
-/******************************************************************************/
-/*                                 f s c t l                                  */
-/******************************************************************************/
-
-int XrdOfs::fsctl(const int               cmd,
-                  const char             *args,
-                  XrdOucErrInfo          &einfo,
-                  const XrdSecEntity     *client)
-/*
-  Function: Perform filesystem operations:
-
-  Input:    cmd       - Operation command (currently supported):
-                        SFS_FSCTL_LOCATE - locate file
-                        SFS_FSCTL_STATCC - return cluster config status
-                        SFS_FSCTL_STATFS - return file system info (physical)
-                        SFS_FSCTL_STATLS - return file system info (logical)
-                        SFS_FSCTL_STATXA - return file extended attributes
-            arg       - Command dependent argument:
-                      - Locate: The path whose location is wanted
-            buf       - The stat structure to hold the results
-            einfo     - Error/Response information structure.
-            client    - Authentication credentials, if any.
-
-  Output:   Returns SFS_OK upon success and SFS_ERROR upon failure.
-*/
-{
-   EPNAME("fsctl");
-   static int PrivTab[]     = {XrdAccPriv_Delete, XrdAccPriv_Insert,
-                               XrdAccPriv_Lock,   XrdAccPriv_Lookup,
-                               XrdAccPriv_Rename, XrdAccPriv_Read,
-                               XrdAccPriv_Write};
-   static char PrivLet[]    = {'d',               'i',
-                               'k',               'l',
-                               'n',               'r',
-                               'w'};
-   static const int PrivNum = sizeof(PrivLet);
-
-   int retc, i, blen, privs, opcode = cmd & SFS_FSCTL_CMD;
-   const char *tident = einfo.getErrUser();
-   char *bP, *cP;
-   XTRACE(fsctl, args, "");
-
-// Process the LOCATE request
-//
-   if (opcode == SFS_FSCTL_LOCATE)
-      {static const int locMask = (SFS_O_FORCE|SFS_O_NOWAIT|SFS_O_RESET|
-                                   SFS_O_HNAME|SFS_O_RAWIO);
-       struct stat fstat;
-       char pbuff[1024], rType[3];
-       const char *Resp[2] = {rType, pbuff};
-       const char *locArg, *opq, *Path = Split(args,&opq,pbuff,sizeof(pbuff));
-       XrdNetIF::ifType ifType;
-       int Resp1Len;
-       int find_flag = SFS_O_LOCATE | (cmd & locMask);
-       XrdOucEnv loc_Env(opq ? opq+1 : 0,0,client);
-
-       if (cmd & SFS_O_TRUNC)           locArg = (char *)"*";
-          else {     if (*Path == '*') {locArg = Path; Path++;}
-                        else            locArg = Path;
-                AUTHORIZE(client,0,AOP_Stat,"locate",Path,einfo);
-               }
-       if (Finder && Finder->isRemote()
-       &&  (retc = Finder->Locate(einfo, locArg, find_flag, &loc_Env)))
-          return fsError(einfo, retc);
-
-       if (cmd & SFS_O_TRUNC) {rType[0] = 'S'; rType[1] = ossRW;}
-          else {if ((retc = XrdOfsOss->Stat(Path, &fstat, 0, &loc_Env)))
-                   return XrdOfsFS->Emsg(epname, einfo, retc, "locate", Path);
-                rType[0] = ((fstat.st_mode & S_IFBLK) == S_IFBLK ? 's' : 'S');
-                rType[1] =  (fstat.st_mode & S_IWUSR             ? 'w' : 'r');
-               }
-       rType[2] = '\0';
-
-       ifType = XrdNetIF::GetIFType((einfo.getUCap() & XrdOucEI::uIPv4)  != 0,
-                                    (einfo.getUCap() & XrdOucEI::uIPv64) != 0,
-                                    (einfo.getUCap() & XrdOucEI::uPrip)  != 0);
-       bool retHN = (cmd & SFS_O_HNAME) != 0;
-       if ((Resp1Len = myIF->GetDest(pbuff, sizeof(pbuff), ifType, retHN)))
-           {einfo.setErrInfo(Resp1Len+3, (const char **)Resp, 2);
-            return SFS_DATA;
-           }
-       return Emsg(epname, einfo, ENETUNREACH, "locate", Path);
-      }
-
-// Process the STATFS request
-//
-   if (opcode == SFS_FSCTL_STATFS)
-      {char pbuff[1024];
-       const char *opq, *Path = Split(args, &opq, pbuff, sizeof(pbuff));
-       XrdOucEnv fs_Env(opq ? opq+1 : 0,0,client);
-       AUTHORIZE(client,0,AOP_Stat,"statfs",Path,einfo);
-       if (Finder && Finder->isRemote()
-       &&  (retc = Finder->Space(einfo, Path, &fs_Env)))
-          return fsError(einfo, retc);
-       bP = einfo.getMsgBuff(blen);
-       if ((retc = XrdOfsOss->StatFS(Path, bP, blen, &fs_Env)))
-          return XrdOfsFS->Emsg(epname, einfo, retc, "statfs", args);
-       einfo.setErrCode(blen+1);
-       return SFS_DATA;
-      }
-
-// Process the STATLS request
-//
-   if (opcode == SFS_FSCTL_STATLS)
-      {char pbuff[1024];
-       const char *opq, *Path = Split(args, &opq, pbuff, sizeof(pbuff));
-       XrdOucEnv statls_Env(opq ? opq+1 : 0,0,client);
-       AUTHORIZE(client,0,AOP_Stat,"statfs",Path,einfo);
-       if (Finder && Finder->isRemote())
-          {statls_Env.Put("cms.qvfs", "1");
-           if ((retc = Finder->Space(einfo, Path, &statls_Env)))
-              {if (retc == SFS_DATA) retc = Reformat(einfo);
-               return fsError(einfo, retc);
-              }
-          }
-       bP = einfo.getMsgBuff(blen);
-       if ((retc = XrdOfsOss->StatLS(statls_Env, Path, bP, blen)))
-          return XrdOfsFS->Emsg(epname, einfo, retc, "statls", Path);
-       einfo.setErrCode(blen+1);
-       return SFS_DATA;
-      }
-
-// Process the STATXA request
-//
-   if (opcode == SFS_FSCTL_STATXA)
-      {char pbuff[1024];
-       const char *opq, *Path = Split(args, &opq, pbuff, sizeof(pbuff));
-       XrdOucEnv xa_Env(opq ? opq+1 : 0,0,client);
-       AUTHORIZE(client,0,AOP_Stat,"statxa",Path,einfo);
-       if (Finder && Finder->isRemote()
-       && (retc = Finder->Locate(einfo,Path,SFS_O_RDONLY|SFS_O_STAT,&xa_Env)))
-          return fsError(einfo, retc);
-       bP = einfo.getMsgBuff(blen);
-       if ((retc = XrdOfsOss->StatXA(Path, bP, blen, &xa_Env)))
-          return XrdOfsFS->Emsg(epname, einfo, retc, "statxa", Path);
-       if (!client || !XrdOfsFS->Authorization) privs = XrdAccPriv_All;
-          else privs = XrdOfsFS->Authorization->Access(client, Path, AOP_Any);
-       cP = bP + blen; strcpy(cP, "&ofs.ap="); cP += 8;
-       if (privs == XrdAccPriv_All) *cP++ = 'a';
-          else {for (i = 0; i < PrivNum; i++)
-                    if (PrivTab[i] & privs) *cP++ = PrivLet[i];
-                if (cP == (bP + blen + 1)) *cP++ = '?';
-               }
-       *cP++ = '\0';
-       einfo.setErrCode(cP-bP+1);
-       return SFS_DATA;
-      }
-
-// Process the STATCC request (this should always succeed)
-//
-   if (opcode == SFS_FSCTL_STATCC)
-      {static const int lcc_flag = SFS_O_LOCATE | SFS_O_LOCAL;
-       XrdOucEnv lcc_Env(0,0,client);
-            if (Finder)   retc = Finder  ->Locate(einfo,".",lcc_flag,&lcc_Env);
-       else if (Balancer) retc = Balancer->Locate(einfo,".",lcc_flag,&lcc_Env);
-       else retc = SFS_ERROR;
-       if (retc != SFS_DATA) einfo.setErrInfo(5, "none|");
-       return fsError(einfo, SFS_DATA);
-      }
-
-// Operation is not supported
-//
-   return XrdOfsFS->Emsg(epname, einfo, ENOTSUP, "fsctl", args);
-
-}
-
 /******************************************************************************/
 /*                              g e t S t a t s                               */
 /******************************************************************************/
@@ -1794,12 +1710,25 @@ int XrdOfs::prepare(      XrdSfsPrep       &pargs,      // In
    XrdOucTList *tp = pargs.paths;
    int retc;
 
-// Run through the paths to make sure client can read each one
+// Run through the paths to make sure client can read each one unless we aren't
+// supposed to apply authorization.
 //
-   while(tp)
-        {AUTHORIZE(client,0,AOP_Read,"prepare",tp->text,out_error);
-         tp = tp->next;
-        }
+   if (prepAuth)
+      while(tp)
+           {AUTHORIZE(client,0,AOP_Read,"prepare",tp->text,out_error);
+            tp = tp->next;
+           }
+
+// If there is a prepare plugin, invoke it and return the result.
+//
+   if (prepHandler)
+      {if (pargs.opts & Prep_QUERY)
+          return prepHandler->query(pargs, out_error, client);
+       if (pargs.opts & Prep_CANCEL)
+          return prepHandler->cancel(pargs, out_error, client);
+
+       return prepHandler->begin(pargs, out_error, client);
+      }
 
 // If we have a finder object, use it to prepare the paths. Otherwise,
 // ignore this prepare request (we may change this in the future).
