@@ -38,11 +38,17 @@
 #include <stdio.h>
 #include <netinet/in.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 
 #include "XrdVersion.hh"
+#include "XProtocol/XProtocol.hh"
 
 #include "XrdCks/XrdCks.hh"
 
+#include "XrdNet/XrdNetUtils.hh"
+
+#include "XrdSfs/XrdSfsFlags.hh"
+
 #include "XrdOfs/XrdOfs.hh"
 #include "XrdOfs/XrdOfsConfigPI.hh"
 #include "XrdOfs/XrdOfsEvs.hh"
@@ -57,6 +63,7 @@
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
+#include "XrdOuc/XrdOucNSWalk.hh"
 #include "XrdOuc/XrdOucStream.hh"
 #include "XrdOuc/XrdOucTrace.hh"
 #include "XrdOuc/XrdOucUtils.hh"
@@ -80,8 +87,18 @@ extern XrdOucTrace OfsTrace;
 class  XrdOss;
 extern XrdOss     *XrdOfsOss;
 
+class  XrdScheduler;
+       XrdScheduler *ofsSchedP;
+
 XrdVERSIONINFO(XrdOfs,XrdOfs);
 
+namespace
+{
+XrdOfsTPC::iParm  Parms;          // TPC parameters
+
+int SetMode(const char *path, mode_t mode) {return chmod(path, mode);}
+}
+
 /******************************************************************************/
 /*                               d e f i n e s                                */
 /******************************************************************************/
@@ -122,11 +139,9 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 
   Output:   0 upon success or !0 otherwise.
 */
-//?extern XrdOss *XrdOssGetSS(XrdSysLogger *, const char *, const char *,
-//?                           const char   *, XrdOucEnv  *, XrdVersionInfo &);
    char *var;
    const char *tmp;
-   int  i, j, cfgFD, retc, NoGo = 0;
+   int   cfgFD, retc, NoGo = 0;
    XrdOucEnv myEnv;
    XrdOucStream Config(&Eroute, getenv("XRDINSTANCE"), &myEnv, "=====> ");
 
@@ -145,6 +160,7 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
       {Eroute.Emsg("Finder", "Network i/f undefined; unable to self-locate.");
        NoGo = 1;
       }
+   ofsSchedP = (XrdScheduler *)EnvInfo->GetPtr("XrdScheduler*");
 
 // Preset all variables with common defaults
 //
@@ -153,7 +169,7 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 
 // Allocate a our plugin configurator
 //
-   ofsConfig = XrdOfsConfigPI::New(ConfigFN, &Config, &Eroute);
+   ofsConfig = XrdOfsConfigPI::New(ConfigFN, &Config, &Eroute, 0, this);
 
 // If there is no config file, return with the defaults sets.
 //
@@ -166,6 +182,8 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
               return Eroute.Emsg("Config", errno, "open config file",
                                  ConfigFN);
            Config.Attach(cfgFD);
+           static const char *cvec[] = {"*** ofs plugin config:",0};
+           Config.Capture(cvec);
 
            // Now start reading records until eof.
            //
@@ -193,22 +211,6 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 //
    if (ossRW == ' ') ossRW = 'w';
 
-// Check if redirection wanted
-//
-   if (getenv("XRDREDIRECT")) i  = isManager;
-      else i = 0;
-   if (getenv("XRDRETARGET")) i |= isServer;
-   if (getenv("XRDREDPROXY")) i |= isProxy;
-   if (i)
-      {if ((j = Options & haveRole) && (i ^ j))
-          {free(myRole); myRole = strdup(theRole(i));
-           Eroute.Say("Config warning: command line role options override "
-                       "config file; 'all.role", myRole, "' in effect.");
-          }
-       Options &= ~(haveRole);
-       Options |= i;
-      }
-
 // Export our role if we actually have one
 //
    if (myRole) XrdOucEnv::Export("XRDROLE", myRole);
@@ -235,7 +237,7 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 // Configure third party copy but only if we are not a manager
 //
    if ((Options & ThirdPC) && !(Options & isManager))
-      if (!XrdOfsTPC::Start()) NoGo = 1;
+      NoGo |= ConfigTPC(Eroute);
 
 // We need to do pre-initialization for event recording as the oss needs some
 // environmental information from that initialization to initialize the frm,
@@ -249,18 +251,38 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
    int piOpts = XrdOfsConfigPI::allXXXLib;
    if (!(Options & Authorize)) piOpts &= ~XrdOfsConfigPI::theAutLib;
 
+// We need to export plugins to other protocols which means we need to
+// record them in the outmost environment. So get it.
+//
+   XrdOucEnv *xrdEnv = 0;
+   if (EnvInfo) xrdEnv = (XrdOucEnv*)EnvInfo->GetPtr("xrdEnv*");
+
 // Now load all of the required plugins
 //
    if (!ofsConfig->Load(piOpts, EnvInfo)) NoGo = 1;
       else {ofsConfig->Plugin(XrdOfsOss);
+            ossFeatures = XrdOfsOss->Features();
+            if (ossFeatures & XRDOSS_HASNOSF)  FeatureSet |= XrdSfs::hasNOSF;
+            if (xrdEnv) xrdEnv->PutPtr("XrdOss*", XrdOfsOss);
             ofsConfig->Plugin(Cks);
             CksPfn = !ofsConfig->OssCks();
+            CksRdr = !ofsConfig->LclCks();
+            if (ofsConfig->Plugin(prepHandler))
+               {prepAuth = ofsConfig->PrepAuth();
+                FeatureSet |= XrdSfs::hasPRP2;
+               }
             if (Options & Authorize)
                {ofsConfig->Plugin(Authorization);
                 XrdOfsTPC::Init(Authorization);
+                if (xrdEnv) xrdEnv->PutPtr("XrdAccAuthorize*",Authorization);
+                FeatureSet |= XrdSfs::hasAUTZ;
                }
            }
 
+// Extract out the export list should it have been supplied by the oss plugin
+//
+   ossRPList = (XrdOucPListAnchor *)EnvInfo->GetPtr("XrdOssRPList*");
+
 // Initialize redirection.  We type te herald here to minimize confusion
 //
    if (Options & haveRole)
@@ -271,6 +293,14 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
           }
       }
 
+// Initialize the FSctl plugin if we have one. Note that we needed to defer
+// until now because we needed to configure the cms plugin first (see above).
+//
+   if (ofsConfig->Plugin(FSctl_PI) && !ofsConfig->ConfigCtl(Finder, EnvInfo))
+      {Eroute.Emsg("Config", "Unable to configure FSctl plugin.");
+       NoGo = 1;
+      }
+
 // Initialize th Evr object if we are an actual server
 //
    if (!(Options & isManager) && !evrObject.Init(Balancer)) NoGo = 1;
@@ -278,32 +308,42 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 // Turn off forwarding if we are not a pure remote redirector or a peer
 //
    if (Options & Forwarding)
-      if (!(Options & isPeer)
-      && (Options & (isServer | isProxy)))
-         {Eroute.Say("Config warning: forwarding turned off; not a pure manager");
+      {const char *why = 0;
+       if (!(Options & Authorize)) why = "authorization not enabled";
+           else if (!(Options & isPeer) && (Options & (isServer | isProxy)))
+                   why = "not a pure manager";
+       if (why)
+         {Eroute.Say("Config warning: forwarding turned off; ", why);
           Options &= ~(Forwarding);
           fwdCHMOD.Reset(); fwdMKDIR.Reset(); fwdMKPATH.Reset();
           fwdMV.Reset();    fwdRM.Reset();    fwdRMDIR.Reset();
           fwdTRUNC.Reset();
          }
+      }
 
 // If we need to send notifications, initialize the interface
 //
    if (!NoGo && evsObject) NoGo = evsObject->Start(&Eroute);
 
+// If the OSS plugin is really a proxy. If it is, it will export its origin.
+// We also suppress translating lfn to pfn (usually done via osslib +cksio).
+// Note: consulting the ENVAR below is historic and remains for compatibility
+//
+   if (ossFeatures & XRDOSS_HASPRXY || getenv("XRDXROOTD_PROXY"))
+      {OssIsProxy = 1;
+       CksPfn = false;
+       FeatureSet |= XrdSfs::hasPRXY;
+      }
+
 // If POSC processing is enabled (as by default) do it. Warning! This must be
 // the last item in the configuration list as we need a working filesystem.
 // Note that in proxy mode we always disable posc!
 //
-   if (getenv("XRDXROOTD_NOPOSC"))
+   if (OssIsProxy || getenv("XRDXROOTD_NOPOSC"))
       {if (poscAuto != -1 && !NoGo)
           Eroute.Say("Config POSC has been disabled by the osslib plugin.");
       } else if (poscAuto != -1 && !NoGo) NoGo |= ConfigPosc(Eroute);
 
-// If the OSS plugin is really a proxy. If it is, it will export its origin.
-//
-   if (getenv("XRDXROOTD_PROXY")) OssIsProxy = 1;
-
 // Setup statistical monitoring
 //
    OfsStats.setRole(myRole);
@@ -470,7 +510,7 @@ int XrdOfs::ConfigPosc(XrdSysError &Eroute)
 
 // Create object then initialize it
 //
-   poscQ = new XrdOfsPoscq(&Eroute, XrdOfsOss, poscLog);
+   poscQ = new XrdOfsPoscq(&Eroute, XrdOfsOss, poscLog, int(poscSync));
    rP = poscQ->Init(rc);
    if (!rc) return 1;
 
@@ -500,6 +540,7 @@ int XrdOfs::ConfigPosc(XrdSysError &Eroute)
 
 // All done
 //
+   if (!NoGo) FeatureSet |= XrdSfs::hasPOSC;
    return NoGo;
 }
 
@@ -572,6 +613,101 @@ int XrdOfs::ConfigRedir(XrdSysError &Eroute, XrdOucEnv *EnvInfo)
    return 0;
 }
   
+/******************************************************************************/
+/*                             C o n f i g T P C                              */
+/******************************************************************************/
+  
+int XrdOfs::ConfigTPC(XrdSysError &Eroute)
+{
+
+// Check if we need to configure rge credentials directory
+//
+   if (Parms.fCreds)
+      {char *cpath = Parms.cpath;
+       if (!(Parms.cpath = ConfigTPCDir(Eroute, cpath))) return 1;
+       free(cpath);
+      }
+
+// Initialize the TPC object
+//
+   XrdOfsTPC::Init(Parms);
+
+// Start TPC operations
+//
+   return (XrdOfsTPC::Start() ? 0 : 1);
+}
+
+/******************************************************************************/
+/*                          C o n f i g T P C D i r                           */
+/******************************************************************************/
+
+char *XrdOfs::ConfigTPCDir(XrdSysError &Eroute, const char *xPath)
+{
+  
+   const int AMode = S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH; // 775
+   const int BMode = S_IRWXU|        S_IRGRP|S_IXGRP; // 750
+   const char *iName;
+   char pBuff[MAXPATHLEN], *aPath;
+   int rc;
+
+// Construct the proper path to stored credentials
+//
+   iName = XrdOucUtils::InstName(-1);
+   if (xPath) aPath = XrdOucUtils::genPath(xPath, iName, ".ofs/.tpccreds/");
+      else {if (!(aPath = getenv("XRDADMINPATH")))
+               {XrdOucUtils::genPath(pBuff, MAXPATHLEN, "/tmp", iName);
+                aPath = pBuff;
+               }
+            aPath = XrdOucUtils::genPath(aPath, (char *)0, ".ofs/.tpccreds/");
+           }
+
+// Make sure directory path exists
+//
+   if ((rc = XrdOucUtils::makePath(aPath, AMode)))
+      {Eroute.Emsg("Config", rc, "create TPC path", aPath);
+       free(aPath);
+       return 0;
+      }
+
+// Protect the last component
+//
+   if (SetMode(aPath, BMode))
+      {Eroute.Emsg("Config", errno, "protect TPC path", aPath);
+       free(aPath);
+       return 0;
+      }
+
+// list the contents of teh directory
+//
+   XrdOucNSWalk nsWalk(&Eroute, aPath, 0, XrdOucNSWalk::retFile);
+   XrdOucNSWalk::NSEnt *nsX, *nsP = nsWalk.Index(rc);
+   if (rc)
+      {Eroute.Emsg("Config", rc, "list TPC path", aPath);
+       free(aPath);
+       return 0;
+      }
+
+// Remove directory contents of all files
+//
+   bool isBad = false;
+   while((nsX = nsP))
+        {nsP = nsP->Next;
+         if (unlink(nsX->Path))
+            {Eroute.Emsg("Config", errno, "remove TPC creds", nsX->Path);
+             isBad = true;
+            }
+         delete nsX;
+        }
+
+// Check if all went well
+//
+   if (isBad) {free(aPath); return 0;}
+
+// All done
+//
+   return aPath;
+}   
+  
 /******************************************************************************/
 /*                             C o n f i g X e q                              */
 /******************************************************************************/
@@ -588,15 +724,19 @@ int XrdOfs::ConfigXeq(char *var, XrdOucStream &Config,
     TS_XPI("ckslib",        theCksLib);
     TS_Xeq("cksrdsz",       xcrds);
     TS_XPI("cmslib",        theCmsLib);
+    TS_XPI("ctllib",        theCtlLib);
+    TS_Xeq("dirlist",       xdirl);
     TS_Xeq("forward",       xforward);
     TS_Xeq("maxdelay",      xmaxd);
     TS_Xeq("notify",        xnot);
     TS_Xeq("notifymsg",     xnmsg);
     TS_XPI("osslib",        theOssLib);
     TS_Xeq("persist",       xpers);
+    TS_XPI("preplib",       thePrpLib);
     TS_Xeq("role",          xrole);
     TS_Xeq("tpc",           xtpc);
     TS_Xeq("trace",         xtrace);
+    TS_Xeq("xattr",         xatr);
     TS_XPI("xattrlib",      theAtrLib);
 
     // Screen out the subcluster directive (we need to track that)
@@ -650,6 +790,40 @@ int XrdOfs::xcrds(XrdOucStream &Config, XrdSysError &Eroute)
    return 0;
 }
   
+/******************************************************************************/
+/*                                 x d i r l                                  */
+/******************************************************************************/
+  
+/* Function: xdirl
+
+   Purpose:  To parse the directive: dirlist {local | remote}
+
+             local   processes directory listings locally. The oss plugin
+                     must be capable of doing this. This is the default.
+             remote  if clustering is enabled, directory listings are
+                     processed as directed by the cmsd.
+
+  Output: 0 upon success or !0 upon failure.
+*/
+
+int XrdOfs::xdirl(XrdOucStream &Config, XrdSysError &Eroute)
+{
+   char *val;
+
+// Get the parameter
+//
+   if (!(val = Config.GetWord()) || !val[0])
+      {Eroute.Emsg("Config", "dirlist parameter not specified"); return 1;}
+
+// Set appropriate option
+//
+        if (!strcmp(val, "local"))  DirRdr = false;
+   else if (!strcmp(val, "remote")) DirRdr = true;
+   else {Eroute.Emsg("Config", "Invalid dirlist parameter -", val); return 1;}
+
+   return 0;
+}
+  
 /******************************************************************************/
 /*                                  x e x p                                   */
 /******************************************************************************/
@@ -682,7 +856,8 @@ int XrdOfs::xexp(XrdOucStream &Config, XrdSysError &Eroute, bool isExport)
 //
    while((val = Config.GetWord()))
         {for (int i = 0; i < numopts; i++)
-             if (!strcmp(val, rwtab[i].opname)) {isrw = rwtab[i].isRW; break;}
+             if (!strcmp(val, rwtab[i].opname))  isrw = rwtab[i].isRW;
+                else if (!strcmp(val, "cache")) {isrw = 0; break;}
         }
 
 // Handle result depending if this is an export or a defaults
@@ -1021,12 +1196,14 @@ int XrdOfs::xnot(XrdOucStream &Config, XrdSysError &Eroute)
 
    Purpose:  To parse the directive: persist [auto | manual | off]
                                              [hold <sec>] [logdir <dirp>]
+                                             [sync <snum>]
 
              auto      POSC processing always on for creation requests
              manual    POSC processing must be requested (default)
              off       POSC processing is disabled
              <sec>     Seconds inclomplete files held (default 10m)
              <dirp>    Directory to hold POSC recovery log (default adminpath)
+             <snum>    Number of outstanding equests before syncing to disk.
 
    Output: 0 upon success or !0 upon failure.
 */
@@ -1034,7 +1211,7 @@ int XrdOfs::xnot(XrdOucStream &Config, XrdSysError &Eroute)
 int XrdOfs::xpers(XrdOucStream &Config, XrdSysError &Eroute)
 {
    char *val;
-   int htime = -1, popt = -2;
+   int snum = -1, htime = -1, popt = -2;
 
    if (!(val = Config.GetWord()))
       {Eroute.Emsg("Config","persist option not specified");return 1;}
@@ -1068,6 +1245,14 @@ int XrdOfs::xpers(XrdOucStream &Config, XrdSysError &Eroute)
                   if (poscLog) free(poscLog);
                   poscLog = strdup(val);
                  }
+         else if (!strcmp(val, "sync"))
+                 {if (!(val = Config.GetWord()))
+                     {Eroute.Emsg("Config","sync value not specified");
+                      return 1;
+                     }
+                  if (XrdOuca2x::a2i(Eroute,"sync value",val,&snum,0,32767))
+                      return 1;
+                 }
          else Eroute.Say("Config warning: ignoring invalid persist option '",val,"'.");
          val = Config.GetWord();
         }
@@ -1076,6 +1261,7 @@ int XrdOfs::xpers(XrdOucStream &Config, XrdSysError &Eroute)
 //
    if (htime >= 0) poscHold = htime;
    if (popt  > -2) poscAuto = popt;
+   if (snum  > -1) poscSync = snum;
    return 0;
 }
 
@@ -1195,9 +1381,16 @@ int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
    Purpose:  To parse the directive: tpc [cksum <type>] [ttl <dflt> [<max>]]
                                          [logok] [xfr <n>] [allow <parms>]
                                          [require {all|client|dest} <auth>[+]]
-                                         [restrict <path>] [streams <num>]
+                                         [restrict <path>]
+                                         [streams <num>[,<max>]]
                                          [echo] [scan {stderr | stdout}]
                                          [autorm] [pgm <path> [parms]]
+                                         [fcreds  [?]<auth> =<evar>]
+                                         [fcpath <path>] [oids]
+
+                                     tpc redirect [xdlg] <host>:<port> [<cgi>]
+
+             xdlg:  delegated | undelegated
 
              parms: [dn <name>] [group <grp>] [host <hn>] [vo <vo>]
 
@@ -1208,7 +1401,8 @@ int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
              allow   only allow destinations that match the specified
                      authentication specification.
              <n>     maximum number of simultaneous transfers.
-             <num>   the number of TCP streams to use for the copy.
+             <num>   the default number of TCP streams to use for the copy.
+             <max>   The maximum number of TCP streams to use for the copy/
              <auth>  require that the client, destination, or both (i.e. all)
                      use the specified authentication protocol. Additional
                      require statements may be specified to add additional
@@ -1221,19 +1415,37 @@ int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
                      default is to scan both.
              pgm     specifies the transfer command with optional paramaters.
                      It must be the last parameter on the line.
+             fcreds  Forward destination credentials for protocol <auth>. The
+                     request fails if thee are no credentials for <auth>. If a
+                     question mark preceeds <auth> then if the client has not
+                     forwarded its credentials, the server's credentials are
+                     used. Otherwise, the copy fails.
+             =<evar> the name of the envar to be set with the path to the
+                     credentials to be forwarded.
+             fcpath  where creds are stored (default <adminpath>/.ofs/.tpccreds).
+             oids    Object ID's are acceptable for the source lfn.
+             <host>  The redirection target host which may be localhost.
+             <port>  The redirection target port.
+             <cgi>   Optional cgi information.
 
    Output: 0 upon success or !0 upon failure.
 */
 
 int XrdOfs::xtpc(XrdOucStream &Config, XrdSysError &Eroute)
 {
-   XrdOfsTPC::iParm Parms;
    char *val, pgm[1024];
-   int  reqType;
    *pgm = 0;
+   int  reqType;
+   bool rdrok = true;
 
    while((val =  Config.GetWord()))
-        {if (!strcmp(val, "allow"))
+        {if (!strcmp(val, "redirect"))
+            {if (rdrok) return xtpcr(Config, Eroute);
+             Eroute.Emsg("Config", "tpc redirect must be seprately specified.");
+             return 1;
+            }
+         rdrok = false;
+         if (!strcmp(val, "allow"))
             {if (!xtpcal(Config, Eroute)) return 1;
              continue;
             }
@@ -1256,12 +1468,13 @@ int XrdOfs::xtpc(XrdOucStream &Config, XrdSysError &Eroute)
          if (!strcmp(val, "echo"))  {Parms.xEcho = 1; continue;}
          if (!strcmp(val, "logok")) {Parms.Logok = 1; continue;}
          if (!strcmp(val, "autorm")){Parms.autoRM = 1; continue;}
+         if (!strcmp(val, "oids"))  {Parms.oidsOK = 1; continue;}
          if (!strcmp(val, "pgm"))
             {if (!Config.GetRest(pgm, sizeof(pgm)))
                 {Eroute.Emsg("Config", "tpc command line too long"); return 1;}
              if (!*pgm)
                 {Eroute.Emsg("Config", "tpc program not specified"); return 1;}
-             Parms.Pgm = pgm;
+             Parms.Pgm = strdup( pgm );
              break;
             }
          if (!strcmp(val, "require"))
@@ -1305,13 +1518,41 @@ int XrdOfs::xtpc(XrdOucStream &Config, XrdSysError &Eroute)
          if (!strcmp(val, "streams"))
             {if (!(val = Config.GetWord()))
                 {Eroute.Emsg("Config","tpc streams value not specified"); return 1;}
-             if (XrdOuca2x::a2i(Eroute,"tpc streams",val,&Parms.Strm,1)) return 1;
+             char *comma = index(val,',');
+             if (comma)
+                {*comma++ = 0;
+                 if (!(*comma))
+                    {Eroute.Emsg("Config","tpc streams max value missing"); return 1;}
+                 if (XrdOuca2x::a2i(Eroute,"tpc max streams",comma,&Parms.SMax,0,15))
+                    return 1;
+                }
+             if (XrdOuca2x::a2i(Eroute,"tpc streams",val,&Parms.Strm,0,15)) return 1;
+             continue;
+            }
+         if (!strcmp(val, "fcreds"))
+            {char aBuff[64];
+             Parms.fCreds = 1;
+             if (!(val = Config.GetWord()) || (*val == '?' && *(val+1) == '\0'))
+                {Eroute.Emsg("Config","tpc fcreds auth not specified"); return 1;}
+             if (strlen(val) >= sizeof(aBuff))
+                {Eroute.Emsg("Config","invalid fcreds auth -", val); return 1;}
+             strcpy(aBuff, val);
+             if (!(val = Config.GetWord()) || *val != '=' || *(val+1) == 0)
+                {Eroute.Emsg("Config","tpc fcreds envar not specified"); return 1;}
+             const char *emsg = XrdOfsTPC::AddAuth(aBuff,val+1);
+             if (emsg) {Eroute.Emsg("Config",emsg,"-", val); return 1;}
+             continue;
+            }
+         if (!strcmp(val, "fcpath"))
+            {if (Parms.cpath) {free(Parms.cpath); Parms.cpath = 0;}
+             if (!(val = Config.GetWord()))
+                {Eroute.Emsg("Config","tpc fcpath arg not specified"); return 1;}
+             Parms.cpath = strdup(val);
              continue;
             }
          Eroute.Say("Config warning: ignoring invalid tpc option '",val,"'.");
         }
 
-   XrdOfsTPC::Init(Parms);
    Options |= ThirdPC;
    return 0;
 }
@@ -1349,6 +1590,96 @@ int XrdOfs::xtpcal(XrdOucStream &Config, XrdSysError &Eroute)
    return 1;
 }
   
+/******************************************************************************/
+/*                                 x t p c r                                  */
+/******************************************************************************/
+
+int XrdOfs::xtpcr(XrdOucStream &Config, XrdSysError &Eroute)
+{
+   char hname[256];
+   const char *cgi, *cgisep, *hBeg, *hEnd, *pBeg, *pEnd, *eText;
+   char *val;
+   int  n, port, dlgI;
+
+// Get the next token
+//
+   if (!(val = Config.GetWord()))
+      {Eroute.Emsg("Config", "tpc redirect host not specified"); return 1;}
+
+// See if this is for delegated or undelegated (all is the default)
+//
+   if (!strcmp(val, "delegated")) dlgI = 0;
+      else if (!strcmp(val, "undelegated")) dlgI = 1;
+           else dlgI = -1;
+
+// Get host and port
+//
+   if (dlgI >= 0 && !(val = Config.GetWord()))
+      {Eroute.Emsg("Config", "tpc redirect host not specified"); return 1;}
+
+// Parse this as it may be complicated.
+//
+   if (!XrdNetUtils::Parse(val, &hBeg, &hEnd, &pBeg, &pEnd))
+      {Eroute.Emsg("Config", "Invalid tpc redirect target -", val); return 1;}
+
+// Copy out the host target (make sure it's not too long)
+//
+   n = hEnd - hBeg;
+   if (*val == '[') n += 2;
+   if (n >= (int)sizeof(hname))
+      {Eroute.Emsg("Config", "Invalid tpc redirect target -", val); return 1;}
+   strncpy(hname, val, n);
+   hname[n] = 0;
+
+// Substitute our hostname for localhost if present
+//
+   if (!strcmp(hname, "localhost"))
+      {char *myHost = XrdNetUtils::MyHostName(0, &eText);
+       if (!myHost)
+          {Eroute.Emsg("Config", "Unable to determine tpc localhost;",eText);
+           return 1;
+          }
+       n = snprintf(hname, sizeof(hname), "%s", myHost);
+       free(myHost);
+       if (n >= (int)sizeof(hname))
+          {Eroute.Emsg("Config", "Invalid tpc localhost resolution -", hname);
+           return 1;
+          }
+      }
+
+// Make sure a port was specified
+//
+   if (pBeg == hEnd)
+      {Eroute.Emsg("Config", "tpc redirect port not specified"); return 1;}
+
+// Get the numeric version of the port number
+//
+   if (!(port = XrdNetUtils::ServPort(pBeg, false, &eText)))
+      {Eroute.Emsg("Config", "Invalid tpc redirect port;",eText); return 1;}
+
+// Check if there is cgi that must be included
+//
+   if (!(cgi = Config.GetWord())) cgisep =  cgi = (char *)"";
+      else cgisep = (*cgi != '?' ? "?" : "");
+
+// Copy out the hostname to be used
+//
+   int k = (dlgI < 0 ? 0 : dlgI);
+do{if (tpcRdrHost[k]) {free(tpcRdrHost[k]); tpcRdrHost[k] = 0;}
+
+   n = strlen(hname) + strlen(cgisep) + strlen(cgi) + 1;
+   tpcRdrHost[k] = (char *)malloc(n);
+   snprintf(tpcRdrHost[k], n, "%s%s%s", hname, cgisep, cgi);
+   tpcRdrPort[k] = port;
+   k++;
+  } while(dlgI < 0 && k < 2);
+
+// All done
+//
+   Options |= RdrTPC;
+   return 0;
+}
+  
 /******************************************************************************/
 /*                                x t r a c e                                 */
 /******************************************************************************/
@@ -1366,8 +1697,7 @@ int XrdOfs::xtpcal(XrdOucStream &Config, XrdSysError &Eroute)
 int XrdOfs::xtrace(XrdOucStream &Config, XrdSysError &Eroute)
 {
     static struct traceopts {const char *opname; int opval;} tropts[] =
-       {
-        {"aio",      TRACE_aio},
+       {{"aio",      TRACE_aio},
         {"all",      TRACE_ALL},
         {"chmod",    TRACE_chmod},
         {"close",    TRACE_close},
@@ -1420,6 +1750,78 @@ int XrdOfs::xtrace(XrdOucStream &Config, XrdSysError &Eroute)
    return 0;
 }
   
+/******************************************************************************/
+/*                                  x a t r                                   */
+/******************************************************************************/
+
+/* Function: xatr
+
+   Purpose:  To parse the directive: xattr [maxnsz <nsz>] [maxvsz <vsz>]
+
+                                           [uset {on|off}]
+
+             on       enables  user settable extended attributes.
+
+             off      disaables user settable extended attributes.
+
+             <nsz>    maximum length of an attribute name. The user
+                      specifiable limit will be 8 less.
+
+             <vsz>    maximum length of an attribute value.
+
+   Notes:    1. This directive is not cummalative.
+
+   Output: 0 upon success or !0 upon failure.
+*/
+
+int XrdOfs::xatr(XrdOucStream &Config, XrdSysError &Eroute)
+{
+   char *val;
+   static const int xanRsv = 7;
+   long long vtmp;
+   int maxN = kXR_faMaxNlen, maxV = kXR_faMaxVlen;
+   bool isOn = true;
+
+   while((val =  Config.GetWord()))
+        {     if (!strcmp("maxnsz", val))
+                 {if (!(val = Config.GetWord()))
+                     {Eroute.Emsg("Config","xattr maxnsz value not specified");
+                      return 1;
+                     }
+                  if (XrdOuca2x::a2sz(Eroute,"maxnsz",val,&vtmp,
+                                     xanRsv+1,kXR_faMaxNlen+xanRsv)) return 1;
+                  maxN = static_cast<int>(vtmp);
+                 }
+         else if (!strcmp("maxvsz", val))
+                 {if (!(val = Config.GetWord()))
+                     {Eroute.Emsg("Config","xattr maxvsz value not specified");
+                      return 1;
+                     }
+                  if (XrdOuca2x::a2sz(Eroute,"maxvsz",val,&vtmp,0,kXR_faMaxVlen))
+                     return 1;
+                  maxV = static_cast<int>(vtmp);
+                 }
+         else if (!strcmp("uset",   val))
+                 {if (!(val = Config.GetWord()))
+                     {Eroute.Emsg("Config","xattr uset value not specified");
+                      return 1;
+                     }
+                       if (!strcmp("on",     val)) isOn = true;
+                  else if (!strcmp("off",    val)) isOn = false;
+                  else {Eroute.Emsg("Config", "invalid xattr uset value -", val);
+                        return 1;
+                       }
+                 }
+         else {Eroute.Emsg("Config", "invalid xattr option -", val);
+               return 1;
+              }
+        }
+
+   usxMaxNsz = (isOn ? maxN-xanRsv : 0);
+   usxMaxVsz = maxV;
+   return 0;
+}
+  
 /******************************************************************************/
 /*                               t h e R o l e                                */
 /******************************************************************************/
