@@ -1,6 +1,6 @@
 /******************************************************************************/
 /*                                                                            */
-/*                     X r d O f s C o n f i g P I . h h                      */
+/*                     X r d O f s C o n f i g P I . c c                      */
 /*                                                                            */
 /* (c) 2014 by the Board of Trustees of the Leland Stanford, Jr., University  */
 /*                            All Rights Reserved                             */
@@ -46,6 +46,7 @@
 
 #include "XrdOfs/XrdOfs.hh"
 #include "XrdOfs/XrdOfsConfigPI.hh"
+#include "XrdOfs/XrdOfsFSctl_PI.hh"
 #include "XrdOfs/XrdOfsPrepare.hh"
 
 #include "XrdOss/XrdOss.hh"
@@ -67,7 +68,7 @@
 namespace
 {
 const char *drctv[] = {"xattrlib", "authlib", "ckslib", "cmslib",
-                         "osslib", "preplib"};
+                         "ctllib", "osslib", "preplib"};
 
 const char *nullParms = 0;
 }
@@ -86,11 +87,13 @@ XrdVERSIONINFOREF(XrdOfs);
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
   
-XrdOfsConfigPI::XrdOfsConfigPI(const char  *cfn,  XrdOucStream   *cfgP,
-                               XrdSysError *errP, XrdVersionInfo *verP)
-                 : autPI(0), cksPI(0), cmsPI(0), prpPI(0), ossPI(0), urVer(verP),
+XrdOfsConfigPI::XrdOfsConfigPI(const char  *cfn,  XrdOucStream     *cfgP,
+                               XrdSysError *errP, XrdSfsFileSystem *sfsP,
+                               XrdVersionInfo *verP)
+                 : autPI(0), cksPI(0), cmsPI(0), ctlPI(0), prpPI(0), ossPI(0),
+                   sfsPI(sfsP), urVer(verP),
                    Config(cfgP),  Eroute(errP), CksConfig(0), ConfigFN(cfn),
-                   CksAlg(0), CksRdsz(0), ossXAttr(false), ossCksio(false),
+                   CksAlg(0), CksRdsz(0), ossXAttr(false), ossCksio(true),
                    prpAuth(true), Loaded(false), LoadOK(false), cksLcl(false)
 {
    int rc;
@@ -107,6 +110,16 @@ XrdOfsConfigPI::XrdOfsConfigPI(const char  *cfn,  XrdOucStream   *cfgP,
 //
    CksConfig = new XrdCksConfig(ConfigFN, Eroute, rc, *urVer);
    if (!rc) {delete CksConfig; CksConfig = 0;}
+
+// Set Pushable attributes
+//
+   pushOK[PIX(theAtrLib)] = true;
+   pushOK[PIX(theAutLib)] = true;
+   pushOK[PIX(theCksLib)] = false;
+   pushOK[PIX(theCmsLib)] = false;
+   pushOK[PIX(theCtlLib)] = true;
+   pushOK[PIX(theOssLib)] = true;
+   pushOK[PIX(thePrpLib)] = true;
 }
 
 /******************************************************************************/
@@ -119,6 +132,155 @@ XrdOfsConfigPI::~XrdOfsConfigPI()
    if (CksAlg)    free(CksAlg);
 }
   
+/******************************************************************************/
+/* Private:                       A d d L i b                                 */
+/******************************************************************************/
+
+bool XrdOfsConfigPI::AddLib(XrdOfsConfigPI::TheLib what)
+{
+   struct xxxLP newALP;
+   char *val, *path;
+   char  parms[2048];
+   int  i, xLib = PIX(what);
+
+// Get the path
+//
+   if (!(val = Config->GetWord()) || !val[0])
+      {Eroute->Emsg("Config", drctv[PIX(what)],"not specified"); return false;}
+   path = strdup(val);
+
+// Get the parameters
+//
+   if (!Config->GetRest(parms, sizeof(parms)))
+      {Eroute->Emsg("Config", drctv[xLib], "parameters too long");
+       free(path);
+       return false;
+      }
+
+// Add this library
+//
+   i = ALP[xLib].size();
+   ALP[xLib].push_back(newALP);
+   ALP[xLib][i].lib   = path;
+   if (*parms) ALP[xLib][i].parms = strdup(parms);
+   return true;
+}
+  
+/******************************************************************************/
+/* Private:                    A d d L i b A t r                              */
+/******************************************************************************/
+
+bool   XrdOfsConfigPI::AddLibAtr(XrdOucEnv *envP, XrdSysXAttr *&atrPI)
+{
+   const char *epName = "XrdSysAddXAttrObject";
+   XrdSysAddXAttrObject_t ep;
+   int n = ALP[PIX(theAtrLib)].size();
+
+   for (int i = 0; i < n; i++)
+       {const char *path  = ALP[PIX(theAtrLib)][i].lib;
+        const char *parms = ALP[PIX(theAtrLib)][i].parms;
+        XrdOucPinLoader myLib(Eroute, urVer, "xattrlib", path);
+        ep = (XrdSysAddXAttrObject_t)(myLib.Resolve(epName));
+        if (!ep) return false;
+        atrPI = ep(Eroute, ConfigFN, parms, envP, atrPI);
+        if (!atrPI) return false;
+       }
+   return true;
+}
+
+/******************************************************************************/
+/* Private:                    A d d L i b A u t                              */
+/******************************************************************************/
+
+bool   XrdOfsConfigPI::AddLibAut(XrdOucEnv *envP)
+{
+   const char *epName = "XrdAccAuthorizeObjAdd";
+   int n = ALP[PIX(theAutLib)].size();
+
+   for (int i = 0; i < n; i++)
+       {const char *path  = ALP[PIX(theAutLib)][i].lib;
+        const char *parms = ALP[PIX(theAutLib)][i].parms;
+        XrdAccAuthorizeObjAdd_t addAut;
+        XrdOucPinLoader myLib(Eroute, urVer, "authlib", path);
+        addAut = (XrdAccAuthorizeObjAdd_t)myLib.Resolve(epName);
+        if (!addAut) return false;
+        autPI = addAut(Eroute->logger(), ConfigFN, parms, envP, autPI);
+        if (!autPI) return false;
+       }
+   return true;
+}
+  
+
+/******************************************************************************/
+/* Private:                    A d d L i b C t l                              */
+/******************************************************************************/
+
+bool   XrdOfsConfigPI::AddLibCtl(XrdOucEnv *envP)
+{
+   const char *objName = "XrdOfsFSctl";
+   XrdOfsFSctl_PI *ctlObj;
+   int n = ALP[PIX(theCtlLib)].size();
+
+   for (int i = 0; i < n; i++)
+       {const char *path  = ALP[PIX(theCtlLib)][i].lib;
+        const char *parms = ALP[PIX(theCtlLib)][i].parms;
+        XrdOucPinLoader myLib(Eroute, urVer, "ctllib", path);
+        ctlObj = (XrdOfsFSctl_PI *)myLib.Resolve(objName);
+        if (!ctlObj) return false;
+        ctlObj->eDest = Eroute;
+        ctlObj->prvPI = ctlPI;
+        ctlPI = ctlObj;
+        ctlLP theCTL = {ctlObj, parms};
+        ctlVec.push_back(theCTL);
+       }
+   return true;
+}
+  
+/******************************************************************************/
+/* Private:                    A d d L i b O s s                              */
+/******************************************************************************/
+
+bool   XrdOfsConfigPI::AddLibOss(XrdOucEnv *envP)
+{
+   const char *epName = "XrdOssAddStorageSystem2";
+   int n = ALP[PIX(theOssLib)].size();
+
+   for (int i = 0; i < n; i++)
+       {const char *path  = ALP[PIX(theOssLib)][i].lib;
+        const char *parms = ALP[PIX(theOssLib)][i].parms;
+        XrdOssAddStorageSystem2_t addOss2;
+        XrdOucPinLoader myLib(Eroute, urVer, "osslib", path);
+
+        addOss2 = (XrdOssGetStorageSystem2_t)myLib.Resolve(epName);
+        if (!addOss2) return false;
+        ossPI = addOss2(ossPI, Eroute->logger(), ConfigFN, parms, envP);
+        if (!ossPI) return false;
+       }
+   return true;
+}
+
+/******************************************************************************/
+/* Private:                    A d d L i b P r p                              */
+/******************************************************************************/
+
+bool   XrdOfsConfigPI::AddLibPrp(XrdOucEnv *envP)
+{
+   const char *epName = "XrdOfsAddPrepare";
+   int n = ALP[PIX(thePrpLib)].size();
+
+   for (int i = 0; i < n; i++)
+       {const char *path  = ALP[PIX(thePrpLib)][i].lib;
+        const char *parms = ALP[PIX(thePrpLib)][i].parms;
+        XrdOfsAddPrepare_t addPrp;
+        XrdOucPinLoader myLib(Eroute, urVer, "preplib", path);
+        addPrp = (XrdOfsAddPrepare_t)myLib.Resolve(epName);
+        if (!addPrp) return false;
+        prpPI = addPrp(Eroute, ConfigFN, parms, sfsPI, ossPI, envP, prpPI);
+        if (!prpPI) return false;
+       }
+   return true;
+}
+  
 /******************************************************************************/
 /*                             C o n f i g u r e                              */
 /******************************************************************************/
@@ -128,6 +290,32 @@ bool   XrdOfsConfigPI::Configure(XrdCmsClient *cmscP, XrdOucEnv *envP)
    return 0 != cmscP->Configure(ConfigFN, LP[PIX(theCmsLib)].parms, envP);
 }
   
+/******************************************************************************/
+/*                             C o n f i g C t l                              */
+/******************************************************************************/
+
+bool   XrdOfsConfigPI::ConfigCtl(XrdCmsClient *cmsP, XrdOucEnv *envP)
+{
+   struct XrdOfsFSctl_PI::Plugins thePI = {autPI, cmsP, ossPI, sfsPI};
+
+// If there is no fsctl plugin, we are done.
+//
+   if (!ctlPI) return true;
+
+// Initialize all of the plugin in FIFO order.
+//
+   if (!ctlPI->Configure(ConfigFN, LP[PIX(theCtlLib)].parms, envP, thePI))
+      return false;
+
+   int n = ctlVec.size();
+
+   for (int i = 0; i < n; i++)
+       {if (!ctlVec[i].ctlPI->Configure(ConfigFN,ctlVec[i].parms,envP,thePI))
+           return false;
+       }
+   return true;
+}
+  
 /******************************************************************************/
 /*                               D e f a u l t                                */
 /******************************************************************************/
@@ -159,8 +347,9 @@ void   XrdOfsConfigPI::DefaultCS(const char *alg)
 void   XrdOfsConfigPI::Display()
 {
    xxxLP *lP;
-   char *oP, buff[2046];
-   int aI = PIX(theAtrLib), oI = PIX(theOssLib);
+   char *oP, buff[4096];
+   const char *theLib;
+   int n, aI = PIX(theAtrLib), oI = PIX(theOssLib);
 
 // Display what we have
 //
@@ -169,11 +358,20 @@ void   XrdOfsConfigPI::Display()
              if (i != aI)   lP = &LP[i];
         else if (ossXAttr) {lP = &LP[oI]; oP = 0;}
         else                lP = &LP[i];
-        if (lP->lib)
-           {snprintf(buff, sizeof(buff), "ofs.%s %s%s %s", drctv[i],
-                     (oP ? oP : ""), lP->lib, (lP->parms ? lP->parms : ""));
+        n = ALP[i].size();
+        if (n || lP->lib)
+           {theLib = (lP->lib ? lP->lib : "default");
+            snprintf(buff, sizeof(buff), "ofs.%s %s%s %s", drctv[i],
+                     (oP ? oP : ""), theLib,  (lP->parms ? lP->parms : ""));
             Eroute->Say("       ", buff);
+            for (int k = 0; k < n; k++)
+                {lP = &(ALP[i][k]);
+                 snprintf(buff, sizeof(buff), "ofs.%s ++ %s %s", drctv[i],
+                         lP->lib, (lP->parms ? lP->parms : ""));
+                 Eroute->Say("       ", buff);
+                }
            }
+
        }
 }
   
@@ -181,8 +379,9 @@ void   XrdOfsConfigPI::Display()
 /*                                  L o a d                                   */
 /******************************************************************************/
   
-bool XrdOfsConfigPI::Load(int loadLib, XrdOfs *ofsP, XrdOucEnv *envP)
+bool XrdOfsConfigPI::Load(int loadLib, XrdOucEnv *envP)
 {
+   extern XrdSysXAttr *XrdSysXAttrActive;
    extern XrdOss *XrdOssGetSS(XrdSysLogger *, const char *, const char *,
                               const char   *, XrdOucEnv  *, XrdVersionInfo &);
    bool aOK;
@@ -202,21 +401,30 @@ bool XrdOfsConfigPI::Load(int loadLib, XrdOfs *ofsP, XrdOucEnv *envP)
           {free(LP[PIX(theOssLib)].lib);
            LP[PIX(theOssLib)].lib = strdup(ossLib);
           }
+       if (!AddLibOss(envP)) return false;
       }
 
 // Now setup the extended attribute plugin if so desired
 //
    if (DO_LOAD(theAtrLib))
-      {     if (ossXAttr && LP[PIX(theOssLib)].lib) aOK = SetupAttr(theOssLib);
-       else if             (LP[PIX(theAtrLib)].lib) aOK = SetupAttr(theAtrLib);
-       else aOK = true;
+      {     if (ossXAttr && LP[PIX(theOssLib)].lib)
+               aOK = SetupAttr(theOssLib, envP);
+       else if             (LP[PIX(theAtrLib)].lib)
+               aOK = SetupAttr(theAtrLib, envP);
+       else {XrdSysXAttr *theObj = XrdSysXAttrActive;
+             if (!AddLibAtr(envP, theObj)) aOK = false;
+                else {if (theObj != XrdSysXAttrActive)
+                          XrdSysFAttr::SetPlugin(theObj, true);
+                      aOK = true;
+                     }
+           }
        if (!aOK) return false;
       }
    XrdSysFAttr::Xat->SetMsgRoute(Eroute);
 
 // Setup authorization if we need to
 //
-   if (DO_LOAD(theAutLib) && !SetupAuth()) return false;
+   if (DO_LOAD(theAutLib) && !SetupAuth(envP)) return false;
 
 // Setup checksumming if we need to
 //
@@ -234,9 +442,13 @@ bool XrdOfsConfigPI::Load(int loadLib, XrdOfs *ofsP, XrdOucEnv *envP)
 //
    if (DO_LOAD(theCmsLib) && !SetupCms()) return false;
 
+// Setup the fsctl plugin if need be
+//
+   if (DO_LOAD(theCtlLib) && !SetupCtl(envP)) return false;
+
 // Setup the prepare plugin if need be
 //
-   if (DO_LOAD(thePrpLib) && !SetupPrp(ofsP, envP)) return false;
+   if (DO_LOAD(thePrpLib) && !SetupPrp(envP)) return false;
 
 // All done
 //
@@ -249,7 +461,8 @@ bool XrdOfsConfigPI::Load(int loadLib, XrdOfs *ofsP, XrdOucEnv *envP)
 /******************************************************************************/
   
 XrdOfsConfigPI *XrdOfsConfigPI::New(const char  *cfn,  XrdOucStream   *cfgP,
-                                    XrdSysError *errP, XrdVersionInfo *verP)
+                                    XrdSysError *errP, XrdVersionInfo *verP,
+                                    XrdSfsFileSystem *sfsP)
 {
 // Handle caller's version if so indicated
 //
@@ -258,7 +471,7 @@ XrdOfsConfigPI *XrdOfsConfigPI::New(const char  *cfn,  XrdOucStream   *cfgP,
 
 // Return an actual instance
 //
-   return new XrdOfsConfigPI(cfn, cfgP, errP, verP);
+   return new XrdOfsConfigPI(cfn, cfgP, errP, sfsP, verP);
 }
 
 /******************************************************************************/
@@ -294,6 +507,8 @@ bool XrdOfsConfigPI::Parse(TheLib what)
                           break;
           case theCmsLib: break;
                           break;
+          case theCtlLib: break;
+                          break;
           case theOssLib: return ParseOssLib();
                           break;
           case thePrpLib: return ParsePrpLib();
@@ -308,7 +523,16 @@ bool XrdOfsConfigPI::Parse(TheLib what)
    if (!(val = Config->GetWord()) || !val[0])
       {Eroute->Emsg("Config", drctv[PIX(what)],"not specified"); return false;}
 
-// Set the lib and parameterss
+// If this may be a pushable library, then see if the pushable tag is present
+//
+   if (!strcmp("++", val))
+      {if (pushOK[PIX(what)]) return AddLib(what);
+       Eroute->Emsg("Config", "'++' option not supported for",
+                              drctv[PIX(what)], "directive.");
+       return false;
+      }
+
+// Set the lib and parameters
 //
    return RepLib(what, val);
 }
@@ -319,8 +543,10 @@ bool XrdOfsConfigPI::Parse(TheLib what)
   
 /* Function: ParseAtrLib
 
-   Purpose:  To parse the directive: xattrlib {osslib | <path>} [<parms>]
+   Purpose:  To parse the directive: xattrlib {osslib | [++] <path>} [<parms>]
 
+             ++        stack this plugin.
+             osslib    The plugin resides in the osslib plugin.
              <path>    the path of the xattr library to be used.
              <parms>   optional parms to be passed
 
@@ -336,6 +562,10 @@ bool XrdOfsConfigPI::ParseAtrLib()
    if (!(val = Config->GetWord()) || !val[0])
       {Eroute->Emsg("Config", "xattrlib not specified"); return false;}
 
+// Check for a push wrapper
+//
+   if (!strcmp("++", val)) return AddLib(theAtrLib);
+
 // Record the path and parms
 //
    ossXAttr = !strcmp("osslib", val);
@@ -348,10 +578,14 @@ bool XrdOfsConfigPI::ParseAtrLib()
   
 /* Function: ParseOssLib
 
-   Purpose:  To parse the directive: osslib [<opts>] <path> [<parms>]
+   Purpose:  To parse the directive: osslib [++ | <opts>] <path> [<parms>]
              <opts>: [+cksio] [+xattr] [<opts>]
 
-             +cksio    use the oss plugin for checkum I/O.
+             +cksio    use the oss plugin for checkum I/O. This is now the
+                       default and is kept (not documented) for backward
+                       compatability.
+             +mmapio   use memory mapping i/o (previously the default). This
+                       is also not documented but here just in case.
              +xattr    the library contains the xattr plugin.
              <path>    the path of the oss library to be used.
              <parms>   optional parms to be passed
@@ -364,20 +598,28 @@ bool XrdOfsConfigPI::ParseOssLib()
    char *val, oBuff[80];
    int   oI = PIX(theOssLib);
 
+// Check if we are pushing another library here
+//
+   if ((val = Config->GetWord()) && !strcmp("++",val)) return AddLib(theOssLib);
+
 // Reset to defaults
 //
-    ossCksio = false;
+    ossCksio = true;
+    ossXAttr = false;
     if (LP[oI].opts) {free(LP[oI].opts); LP[oI].opts = 0;}
     *oBuff = 0;
 
 // Get the path and parms, and process keywords
 //
-   while((val = Config->GetWord()))
+   while(val)
         {     if (!strcmp("+cksio",  val))
-                 {if (!ossCksio) strcat(oBuff, "+cksio "); ossCksio = true;}
+                 {if (!ossCksio) strcat(oBuff, "+cksio  "); ossCksio = true;}
+         else if (!strcmp("+mmapio", val))
+                 {if ( ossCksio) strcat(oBuff, "+mmapio "); ossCksio = false;}
          else if (!strcmp("+xattr",  val))
-                 {if (!ossXAttr) strcat(oBuff, "+xattr "); ossXAttr = true;}
+                 {if (!ossXAttr) strcat(oBuff, "+xattr ");  ossXAttr = true;}
          else break;
+         val = Config->GetWord();
         }
 
 // Check if we an osslib
@@ -397,9 +639,10 @@ bool XrdOfsConfigPI::ParseOssLib()
   
 /* Function: ParsePrpLib
 
-   Purpose:  To parse the directive: preplib [<opts>] <path> [<parms>]
+   Purpose:  To parse the directive: preplib [++ | [<opts>]} <path> [<parms>]
              <opts>: [+noauth]
 
+             ++        Stack this plugin.
              +noauth   do not apply authorization to path list.
              <path>    the path of the prepare library to be used.
              <parms>   optional parms to be passed
@@ -412,6 +655,10 @@ bool XrdOfsConfigPI::ParsePrpLib()
    char *val, oBuff[80];
    int   oI = PIX(thePrpLib);
 
+// Check if we are pushing another library here
+//
+   if ((val = Config->GetWord()) && !strcmp("++",val)) return AddLib(thePrpLib);
+
 // Reset to defaults
 //
     prpAuth = true;
@@ -420,10 +667,11 @@ bool XrdOfsConfigPI::ParsePrpLib()
 
 // Get the path and parms, and process keywords
 //
-   while((val = Config->GetWord()))
+   while(val)
         {     if (!strcmp("+noauth",  val))
                  {if (prpAuth) strcat(oBuff, "+noauth "); prpAuth = false;}
          else break;
+         val = Config->GetWord();
         }
 
 // Check if we a library path
@@ -450,6 +698,9 @@ bool   XrdOfsConfigPI::Plugin(XrdCks          *&piP)
 bool   XrdOfsConfigPI::Plugin(XrdCmsClient_t   &piP)
 {      return (piP = cmsPI) != 0;}
 
+bool   XrdOfsConfigPI::Plugin(XrdOfsFSctl_PI  *&piP)
+{      return (piP = ctlPI) != 0;}
+
 bool   XrdOfsConfigPI::Plugin(XrdOfsPrepare   *&piP)
 {      return (piP = prpPI) != 0;}
 
@@ -513,9 +764,9 @@ void   XrdOfsConfigPI::SetCksRdSz(int rdsz) {CksRdsz = rdsz;}
 /* Private:                    S e t u p A t t r                              */
 /******************************************************************************/
 
-bool XrdOfsConfigPI::SetupAttr(XrdOfsConfigPI::TheLib what)
+bool XrdOfsConfigPI::SetupAttr(XrdOfsConfigPI::TheLib what, XrdOucEnv *envP)
 {
-   XrdSysXAttr *(*ep)(XrdSysError *, const char *, const char *);
+   XrdSysGetXAttrObject_t ep;
    XrdSysXAttr *theObj;
    char *AtrLib   = LP[PIX(what)].lib;
    char *AtrParms = LP[PIX(what)].parms;
@@ -523,8 +774,7 @@ bool XrdOfsConfigPI::SetupAttr(XrdOfsConfigPI::TheLib what)
 // Create a plugin object
 //
   {XrdOucPinLoader myLib(Eroute, urVer, "xattrlib", AtrLib);
-   ep = (XrdSysXAttr *(*)(XrdSysError *, const char *, const char *))
-                         (myLib.Resolve("XrdSysGetXAttrObject"));
+   ep = (XrdSysGetXAttrObject_t)(myLib.Resolve("XrdSysGetXAttrObject"));
    if (!ep) return false;
    if (strcmp(AtrLib, myLib.Path()))
       {free(AtrLib); AtrLib = LP[PIX(what)].lib = strdup(myLib.Path());}
@@ -534,6 +784,10 @@ bool XrdOfsConfigPI::SetupAttr(XrdOfsConfigPI::TheLib what)
 //
    if (!(theObj = ep(Eroute, ConfigFN, AtrParms))) return false;
 
+// Push any additional objects
+//
+   if (!AddLibAtr(envP, theObj)) return false;
+
 // Tell the interface to use this object instead of the default implementation
 //
    XrdSysFAttr::SetPlugin(theObj);
@@ -544,26 +798,29 @@ bool XrdOfsConfigPI::SetupAttr(XrdOfsConfigPI::TheLib what)
 /* Private:                    S e t u p A u t h                              */
 /******************************************************************************/
 
-bool XrdOfsConfigPI::SetupAuth()
+bool XrdOfsConfigPI::SetupAuth(XrdOucEnv *envP)
 {
    extern XrdAccAuthorize *XrdAccDefaultAuthorizeObject
                           (XrdSysLogger   *lp,    const char   *cfn,
                            const char     *parm,  XrdVersionInfo &vInfo);
 
-   XrdAccAuthorize *(*ep)(XrdSysLogger *, const char *, const char *);
+   XrdAccAuthorizeObject_t ep;
    char *AuthLib   = LP[PIX(theAutLib)].lib;
    char *AuthParms = LP[PIX(theAutLib)].parms;
 
 // Authorization comes from the library or we use the default
 //
-   if (!AuthLib) return 0 != (autPI = XrdAccDefaultAuthorizeObject
-                             (Eroute->logger(), ConfigFN, AuthParms, *urVer));
+   if (!AuthLib)
+      {if (!(autPI = XrdAccDefaultAuthorizeObject
+                        (Eroute->logger(), ConfigFN, AuthParms, *urVer)))
+          return false;
+       return AddLibAut(envP);
+      }
 
 // Create a plugin object
 //
   {XrdOucPinLoader myLib(Eroute, urVer, "authlib", AuthLib);
-   ep = (XrdAccAuthorize *(*)(XrdSysLogger *, const char *, const char *))
-                             (myLib.Resolve("XrdAccAuthorizeObject"));
+   ep = (XrdAccAuthorizeObject_t)(myLib.Resolve("XrdAccAuthorizeObject"));
    if (!ep) return false;
    if (strcmp(AuthLib, myLib.Path()))
       {free(AuthLib); AuthLib = LP[PIX(theAutLib)].lib = strdup(myLib.Path());}
@@ -571,7 +828,8 @@ bool XrdOfsConfigPI::SetupAuth()
 
 // Get the Object now
 //
-   return 0 != (autPI = ep(Eroute->logger(), ConfigFN, AuthParms));
+   if (!(autPI = ep(Eroute->logger(), ConfigFN, AuthParms))) return false;
+   return AddLibAut(envP);
 }
 
 /******************************************************************************/
@@ -597,11 +855,40 @@ bool XrdOfsConfigPI::SetupCms()
    return true;
 }
 
+/******************************************************************************/
+/*                              S e t u p C t l                               */
+/******************************************************************************/
+  
+bool XrdOfsConfigPI::SetupCtl(XrdOucEnv *envP)
+{
+   XrdOfsFSctl_PI *obj = 0;
+   char *CtlLib   = LP[PIX(theCtlLib)].lib;
+
+// Load the plugin if we have to
+//
+   if (LP[PIX(theCtlLib)].lib)
+      {XrdOucPinLoader myLib(Eroute, urVer, "ctllib", CtlLib);
+       obj = (XrdOfsFSctl_PI *)(myLib.Resolve("XrdOfsFSctl"));
+       if (!obj) return false;
+       if (strcmp(CtlLib, myLib.Path()))
+          {free(CtlLib);
+           CtlLib = LP[PIX(theCtlLib)].lib = strdup(myLib.Path());
+          }
+      } else return true;
+
+// Record the object (it will be fully initialized later_
+//
+   obj->eDest = Eroute;
+   obj->prvPI = 0;
+   ctlPI = obj;
+   return AddLibCtl(envP);
+}
+
 /******************************************************************************/
 /*                              S e t u p P r p                               */
 /******************************************************************************/
   
-bool XrdOfsConfigPI::SetupPrp(XrdOfs *ofsP, XrdOucEnv *envP)
+bool XrdOfsConfigPI::SetupPrp(XrdOucEnv *envP)
 {
    XrdOfsgetPrepare_t ep = 0;
    char *PrpLib   = LP[PIX(thePrpLib)].lib;
@@ -617,11 +904,11 @@ bool XrdOfsConfigPI::SetupPrp(XrdOfs *ofsP, XrdOucEnv *envP)
           {free(PrpLib);
            PrpLib = LP[PIX(thePrpLib)].lib = strdup(myLib.Path());
           }
-      }
+      } else return true;
 
 // Get the Object now
 //
-   if (ep)
-      return 0 != (prpPI = ep(Eroute, ConfigFN, PrpParms, ofsP, ossPI, envP));
-   return true;
+   if (!(prpPI = ep(Eroute, ConfigFN, PrpParms, sfsPI, ossPI, envP)))
+      return false;
+   return AddLibPrp(envP);
 }
