@@ -260,9 +260,10 @@ void XrdOfsEvs::Notify(Event eID, XrdOfsEvsInfo &Info)
 //
    if (!(tp = getMsg(isBig)))
       {if ((++warnings & 0xff) == 1)
-          eDest->Emsg("Notify", "Ran out of message objects;", eName(eNum),
+         {eDest->Emsg("Notify", "Ran out of message objects;", eName(eNum),
                                 "event notification not sent.");
-          return;
+         }
+       return;
       }
 
 // Format the message
