@@ -84,7 +84,6 @@ XrdOucTrace OssTrace(&OssEroute);
 /*           S t o r a g e   S y s t e m   I n s t a n t i a t o r            */
 /******************************************************************************/
 
-int       XrdOssSys::runOld  = 0;
 char      XrdOssSys::tryMmap = 0;
 char      XrdOssSys::chkMmap = 0;
 
@@ -104,7 +103,6 @@ XrdOss *XrdOssGetSS(XrdSysLogger *Logger, const char *config_fn,
    extern XrdSysError OssEroute;
    XrdOucPinLoader *myLib;
    XrdOss          *ossP;
-   XrdOss          *(*ep)(XrdOss *, XrdSysLogger *, const char *, const char *);
 
 // Verify that versions are compatible.
 //
@@ -123,16 +121,28 @@ XrdOss *XrdOssGetSS(XrdSysLogger *Logger, const char *config_fn,
    OssEroute.logger(Logger);
    if (!(myLib = new XrdOucPinLoader(&OssEroute, myOssSys.myVersion,
                                      "osslib",   OssLib))) return 0;
-
-// Now get the entry point of the object creator
+// Declare the interface versions
 //
-   ep = (XrdOss *(*)(XrdOss *, XrdSysLogger *, const char *, const char *))
-                    (myLib->Resolve("XrdOssGetStorageSystem"));
-   if (!ep) return 0;
+   XrdOssGetStorageSystem_t  getOSS1;
+   const char               *epName1 = "XrdOssGetStorageSystem";
+   XrdOssGetStorageSystem2_t getOSS2;
+   const char               *epName2 ="?XrdOssGetStorageSystem2";
 
-// Get the Object now
+// First try finding version 2 of the initializer. If that fails try version 1.
+// In the process, we will get an oss object if we succeed at all.
 //
-   if ((ossP = ep((XrdOss *)&myOssSys, Logger, config_fn, OssParms)) && envP)
+   getOSS2 = (XrdOssGetStorageSystem2_t)myLib->Resolve(epName2);
+   if (getOSS2) ossP = getOSS2((XrdOss *)&myOssSys, Logger,   config_fn,
+                                                    OssParms, envP);
+      else {getOSS1 = (XrdOssGetStorageSystem_t)myLib->Resolve(epName1);
+            if (!getOSS1) return 0;
+            ossP = getOSS1((XrdOss *)&myOssSys, Logger, config_fn, OssParms);
+           }
+
+// Call the legacy EnvInfo() method and set what library we are using if it
+// differs from what we wre passed.
+//
+   if (ossP && envP)
       {ossP->EnvInfo(envP);
        if (envP && strcmp(OssLib, myLib->Path()))
           envP->Put("oss.lib", myLib->Path());
@@ -483,8 +493,11 @@ int XrdOssDir::Opendir(const char *dir_path, XrdOucEnv &Env)
 
 // Get the processing flags for this directory
 //
-   pflags = XrdOssSS->PathOpts(dir_path);
-   ateof = 0;
+   unsigned long long pflags = XrdOssSS->PathOpts(dir_path);
+   if (pflags & XRDEXP_STAGE)   dOpts |= isStage;
+   if (pflags & XRDEXP_NODREAD) dOpts |= noDread;
+   if (pflags & XRDEXP_NOCHECK) dOpts |= noCheck;
+   ateof = false;
 
 // Generate local path
 //
@@ -494,12 +507,15 @@ int XrdOssDir::Opendir(const char *dir_path, XrdOucEnv &Env)
          else local_path = actual_path;
       else local_path = (char *)dir_path;
 
-// If this is a local filesystem request, open locally.
+// If this is a local filesystem request, open locally. We also obtian the
+// underlying file descriptor.
 //
-   if (!(pflags & XRDEXP_STAGE) || (pflags & XRDEXP_NODREAD))
+   if (!(dOpts & isStage) || (dOpts & noDread))
       {TRACE(Opendir, "lcl path " <<local_path <<" (" <<dir_path <<")");
-       if ((lclfd = opendir((char *)local_path))) {isopen = 1; return XrdOssOK;}
-       return -errno;
+       if (!(lclfd = XrdSysFD_OpenDir(local_path))) return -errno;
+       fd = dirfd(lclfd);
+       isopen = true;
+       return XrdOssOK;
       }
 
 // Generate remote path
@@ -517,18 +533,18 @@ int XrdOssDir::Opendir(const char *dir_path, XrdOucEnv &Env)
 // by making NODREAD mean to read the local directory only (which is not always
 // ideal). So, we keep the code below but comment it out for now.
 //
-// if ((pflags & XRDEXP_NODREAD) && !(pflags & XRDEXP_NOCHECK))
+// if ((dOpts & noDread) && !(dOpts & noCheck))
 //    {struct stat fstat;
 //     if ((retc = XrdOssSS->MSS_Stat(remote_path,&fstat))) return retc;
 //     if (!(S_ISDIR(fstat.st_mode))) return -ENOTDIR;
-//     isopen = 1;
+//     isopen = true;
 //     return XrdOssOK;
 //    }
 
 // Open the directory at the remote location.
 //
    if (!(mssfd = XrdOssSS->MSS_Opendir(remote_path, retc))) return retc;
-   isopen = 1;
+   isopen = true;
    return XrdOssOK;
 }
 
@@ -566,19 +582,19 @@ int XrdOssDir::Readdir(char *buff, int blen)
        if ((rp = readdir(lclfd)))
           {strlcpy(buff, rp->d_name, blen);
 #ifdef HAVE_FSTATAT
-           if (Stat && fstatat(dirFD, rp->d_name, Stat, 0)) return -errno;
+           if (Stat && fstatat(fd, rp->d_name, Stat, 0)) return -errno;
 #endif
            return XrdOssOK;
           }
-       *buff = '\0'; ateof = 1;
+       *buff = '\0'; ateof = true;
        return -errno;
       }
 
 // Simulate the read operation, if need be.
 //
-   if (pflags & XRDEXP_NODREAD)
+   if (noDread)
       {if (ateof) *buff = '\0';
-          else   {*buff = '.'; ateof = 1;}
+          else   {*buff = '.'; ateof = true;}
        return XrdOssOK;
       }
 
@@ -618,14 +634,6 @@ int XrdOssDir::StatRet(struct stat *buff)
    return -ENOTSUP;
 #endif
 
-// Now obtain the correct file descriptor which is special in Solaris
-//
-#ifdef __solaris__
-   dirFD = lclfd->dd_fd;
-#else
-   dirFD = dirfd(lclfd);
-#endif
-
 // All is well
 //
    Stat = buff;
@@ -647,15 +655,25 @@ int XrdOssDir::Close(long long *retsz)
 {
     int retc;
 
+// We do not support returing a size
+//
+   if (retsz) *retsz = 0;
+
 // Make sure this object is open
 //
     if (!isopen) return -XRDOSS_E8002;
 
 // Close whichever handle is open
 //
-    if (lclfd) {if (!(retc = closedir(lclfd))) lclfd = 0;}
-       else if (mssfd) { if (!(retc = XrdOssSS->MSS_Closedir(mssfd))) mssfd = 0;}
-               else retc = 0;
+    if (lclfd)
+       {if (!(retc = closedir(lclfd)))
+           {lclfd = 0;
+            isopen = false;
+           }
+       } else {
+        if (mssfd) { if (!(retc = XrdOssSS->MSS_Closedir(mssfd))) mssfd = 0;}
+           else retc = 0;
+       }
 
 // Indicate whether or not we really closed this object
 //
@@ -1017,6 +1035,28 @@ int XrdOssFile::Fchmod(mode_t Mode)
     return (fchmod(fd, Mode) ? -errno : XrdOssOK);
 }
   
+/******************************************************************************/
+/*                                 F l u s h                                  */
+/******************************************************************************/
+
+/*
+  Function: Flush file pages from the filesyste cache.
+
+  Output:   Returns XrdOssOK upon success and -errno upon failure.
+*/
+
+void XrdOssFile::Flush()
+{
+// This actually only works in Linux so we punt otherwise
+//
+#if defined(__linux__)
+   if (fd>= 0)
+      {fdatasync(fd);
+       posix_fadvise(fd, 0, 0, POSIX_FADV_DONTNEED);
+      }
+#endif
+}
+
 /******************************************************************************/
 /*                                 F s t a t                                  */
 /******************************************************************************/
