@@ -83,10 +83,11 @@ class XrdOssCreateInfo
      {public:
       unsigned long long pOpts;
       const char        *Path;
+      const char        *LFN;
       mode_t             Amode;
       int                cOpts;
-      XrdOssCreateInfo(const char *path, mode_t amode, int opts)
-                      : Path(path), Amode(amode), cOpts(opts) {}
+      XrdOssCreateInfo(const char *path, const char *lfn, mode_t amode, int opts)
+                      : Path(path), LFN(lfn), Amode(amode), cOpts(opts) {}
      ~XrdOssCreateInfo() {}
      };
 
@@ -117,7 +118,7 @@ int XrdOssSys::Create(const char *tident, const char *path, mode_t access_mode,
     char  local_path[MAXPATHLEN+1], *p, pc;
     unsigned long long remotefs;
     int isLink = 0, Missing = 1, retc = 0, datfd;
-    XrdOssCreateInfo crInfo(local_path, access_mode, Opts);
+    XrdOssCreateInfo crInfo(local_path, path, access_mode, Opts);
     struct stat buf;
 
 // Get options associated with this path and check if it's r/w
@@ -228,6 +229,7 @@ int XrdOssSys::Alloc_Cache(XrdOssCreateInfo &crInfo, XrdOucEnv &env)
 {
    EPNAME("Alloc_Cache")
    int datfd, rc;
+   const char *spName;
    char pbuff[MAXPATHLEN+1], cgbuff[XrdOssSpace::minSNbsz], *tmp;
    XrdOssCache::allocInfo aInfo(crInfo.Path, pbuff, sizeof(pbuff));
 
@@ -237,9 +239,17 @@ int XrdOssSys::Alloc_Cache(XrdOssCreateInfo &crInfo, XrdOucEnv &env)
    &&  XrdOuca2x::a2sz(OssEroute,"invalid asize",tmp,&aInfo.cgSize,0))
       return -XRDOSS_E8018;
 
+// Determine the space we should use for this allocation
+//
+   spName = env.Get(OSS_CGROUP);
+   if (!spName || (SPList.NotEmpty() && SPList.Default() == spAssign))
+      {XrdOucPList *pl = SPList.About(crInfo.LFN);
+       if (pl && (!spName || pl->Attr() == spAssign)) spName = pl->Name();
+      }
+
 // Get the correct cache group and partition path
 //
-   if ((aInfo.cgPath=XrdOssCache::Parse(env.Get(OSS_CGROUP),cgbuff,sizeof(cgbuff))))
+   if ((aInfo.cgPath=XrdOssCache::Parse(spName,cgbuff,sizeof(cgbuff))))
       aInfo.cgPlen = strlen(aInfo.cgPath);
 
 // Allocate space in the cache.
@@ -250,7 +260,7 @@ int XrdOssSys::Alloc_Cache(XrdOssCreateInfo &crInfo, XrdOucEnv &env)
 
 // Set the pfn as the extended attribute if we are in new mode
 //
-   if (!runOld && !(crInfo.pOpts & XRDEXP_NOXATTR)
+   if (!(crInfo.pOpts & XRDEXP_NOXATTR)
    &&  (rc = XrdSysFAttr::Xat->Set(XrdFrcXAttrPfn::Name(), crInfo.Path,
                                    strlen(crInfo.Path)+1, pbuff, datfd)))
       {close(datfd); return rc;}
@@ -266,16 +276,6 @@ int XrdOssSys::Alloc_Cache(XrdOssCreateInfo &crInfo, XrdOucEnv &env)
    ||  unlink(crInfo.Path) || symlink(pbuff, crInfo.Path))
       {rc = -errno; unlink(pbuff);}
 
-// Now create a symlink from the cache pfn to the actual path (xa runOld only)
-//
-   if (runOld && aInfo.cgPsfx)
-      {strcpy(aInfo.cgPsfx, ".pfn");
-       if ((symlink(crInfo.Path, pbuff) && errno != EEXIST)
-       ||  unlink(pbuff) || symlink(crInfo.Path, pbuff)) rc = -errno;
-       *(aInfo.cgPsfx) = '\0';
-       if (rc) {unlink(pbuff); unlink(crInfo.Path);}
-      }
-
 // All done
 //
    DEBUG(aInfo.cgName <<" cache for " <<pbuff);
@@ -312,9 +312,6 @@ int XrdOssSys::Alloc_Local(XrdOssCreateInfo &crInfo, XrdOucEnv &env)
   
 int XrdOssSys::SetFattr(XrdOssCreateInfo &crInfo, int fd, time_t mtime)
 {
-   static const char *lkSuffix = ".lock";
-   static const int   lkSuffsz = 5;
-
    class  fdCloser
          {public:
           const char *Path;
@@ -327,34 +324,10 @@ int XrdOssSys::SetFattr(XrdOssCreateInfo &crInfo, int fd, time_t mtime)
    XrdOucXAttr<XrdFrcXAttrCpy> crX;
    int rc;
 
-// Skip all of this if we do not need to create a lock file
+// Check if we need or can record the create time
 //
-   if (!(XRDEXP_MAKELF & crInfo.pOpts)) return Act.Done(0);
-
-// If we are running in backward compatability mode, then we need to create
-// an old-style lock file.
-//
-   if (runOld)
-      {struct utimbuf times;
-       char lkBuff[MAXPATHLEN+lkSuffsz+1];
-       int  lkfd, n = strlen(crInfo.Path);
-       if (n+lkSuffsz >= (int)sizeof(lkBuff))
-          return Act.Done(OssEroute.Emsg("Create", -ENAMETOOLONG,
-                                         "generate lkfname for", crInfo.Path));
-       strcpy(lkBuff, crInfo.Path); strcpy(lkBuff+n, lkSuffix);
-       do {lkfd = open(lkBuff, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);}
-          while( lkfd < 0 && errno == EINTR);
-       if ( lkfd < 0)
-          return Act.Done(OssEroute.Emsg("Create", -errno, "create", lkBuff));
-       close(lkfd); times.actime = time(0); times.modtime = mtime;
-       if (utime(lkBuff, (const struct utimbuf *)&times))
-          return Act.Done(OssEroute.Emsg("Create",-errno,"set mtime for",lkBuff));
-       return Act.Done(0);
-      }
-
-// Check if we should really create any extended attribute
-//
-   if ((crInfo.pOpts & XRDEXP_NOXATTR)) return Act.Done(0);
+   if (!(XRDEXP_MIGPRG & crInfo.pOpts)
+   ||  (crInfo.pOpts & XRDEXP_NOXATTR)) return Act.Done(0);
 
 // Set copy time
 //
