@@ -149,10 +149,11 @@ XrdOucCacheIO *XrdOucCacheData::Detach()
    if (RetVal)
       {Cache->Stats.Add(Statistics);
        if (Cache->Lgs)
-          {char sBuff[2048];
-           sprintf(sBuff, "Cache: Stats: %lld Read; %lld Get; %lld Pass; "
-                          "%lld Write; %lld Put; %d Hits; %d Miss; "
-                          "%lld pead; %d HitsPR; %d MissPR; Path %s\n",
+          {char sBuff[4096];
+           snprintf(sBuff, sizeof(sBuff),
+                          "Cache: Stats: %lld Read; %lld Get; %lld Pass; "
+                          "%lld Write; %lld Put; %lld Hits; %lld Miss; "
+                          "%lld pead; %lld HitsPR; %lld MissPR; Path %s\n",
                           Statistics.BytesRead, Statistics.BytesGet,
                           Statistics.BytesPass, Statistics.BytesWrite,
                           Statistics.BytesPut,
