@@ -37,7 +37,7 @@
 #include <fcntl.h>
 
 #include "XrdOuc/XrdOuca2x.hh"
-#include "XrdOuc/XrdOucCache2.hh"
+#include "XrdOuc/XrdOucCache.hh"
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucN2NLoader.hh"
 #include "XrdOuc/XrdOucName2Name.hh"
@@ -86,6 +86,8 @@ XrdOucPsx::~XrdOucPsx()
    if (N2NParms)  free(N2NParms);
    if (cPath)     free(cPath);
    if (cParm)     free(cParm);
+   if (mPath)     free(mPath);
+   if (mParm)     free(mParm);
    if (configFN)  free(configFN);
 
    while((tP = setFirst)) {setFirst = tP->next; delete tP;}
@@ -169,41 +171,15 @@ bool XrdOucPsx::ClientConfig(const char *pfx, bool hush)
 bool XrdOucPsx::ConfigCache(XrdSysError &eDest)
 {
    XrdOucPinLoader  myLib(&eDest,myVersion,"cachelib",cPath);
-   const char *cName;
-   bool isCache2;
 
-// First find out if this is a cache2 or old cache library
+// Get the cache Object now
 //
-   if (myLib.Resolve("?XrdOucGetCache2") == 0)
-      {cName = "XrdOucGetCache";  isCache2 = false;
-      } else {
-       cName = "XrdOucGetCache2"; isCache2 = true;
-      } 
-
-// Get the Object now
-//
-   if (isCache2) {
-      XrdOucCache2     *(*ep)(XrdSysLogger *, const char *, const char *);
-      ep = (XrdOucCache2 *(*)(XrdSysLogger *, const char *, const char *))
-         (myLib.Resolve(cName));
-
-      if (!ep) return false;
-
-      theCache2 = (XrdOucCache2*)ep(eDest.logger(), configFN, cParm);
-      return theCache2 != 0;
-   }
-   else {
-      XrdOucCache     *(*ep)(XrdSysLogger *, const char *, const char *);
-      ep = (XrdOucCache *(*)(XrdSysLogger *, const char *, const char *))
-         (myLib.Resolve(cName));
-
-      if (!ep) return false;
-
-      theCache = (XrdOucCache*)ep(eDest.logger(), configFN, cParm);
-      return theCache != 0;
-   }
+   XrdOucCache_t ep = (XrdOucCache_t)myLib.Resolve("XrdOucGetCache");
+   if (!ep) return false;
+   theCache = (XrdOucCache*)ep(eDest.logger(), configFN, cParm, theEnv);
+   return theCache != 0;
 }
-
+  
 /******************************************************************************/
 /*                             C o n f i g N 2 N                              */
 /******************************************************************************/
@@ -214,7 +190,11 @@ bool XrdOucPsx::ConfigN2N(XrdSysError &eDest)
 
 // Skip all of this we are not doing name mapping.
 //
-  if (!N2NLib && !LocalRoot) {xLfn2Pfn = xPfn2Lfn = false; return true;}
+  if (!N2NLib && !LocalRoot)
+     {xLfn2Pfn = false;
+      xPfn2Lfn = xP2Loff;
+      return true;
+     }
 
 // Check if the n2n is applicable
 //
@@ -243,7 +223,7 @@ bool XrdOucPsx::ConfigSetup(XrdSysError &eDest, bool hush)
 //
    if (hush) eDest.logger()->Capture(&tFifo);
 
-// Initialize an alternate cache if one is present
+// Initialize an alternate cache if one is present and load a CCM if need be
 //
    if (cPath && !ConfigCache(eDest))
       {aOK = false;
@@ -253,6 +233,16 @@ bool XrdOucPsx::ConfigSetup(XrdSysError &eDest, bool hush)
            tFifo.Clear();
            eDest.logger()->Capture(&tFifo);
           }
+      } else {
+       if (mPath && theCache && !LoadCCM(eDest))
+          {aOK = false;
+           if (hush)
+              {eDest.logger()->Capture(0);
+               WarnPlugin(eDest, tFifo.first, "ccmlib", mPath);
+               tFifo.Clear();
+               eDest.logger()->Capture(&tFifo);
+              }
+          }
       }
 
 // Configure the N2N library:
@@ -272,6 +262,20 @@ bool XrdOucPsx::ConfigSetup(XrdSysError &eDest, bool hush)
    if (hush) eDest.logger()->Capture(0);
    return aOK;
 }
+
+/******************************************************************************/
+/* Private:                      L o a d C C M                                */
+/******************************************************************************/
+
+bool XrdOucPsx::LoadCCM(XrdSysError &eDest)
+{
+   XrdOucPinLoader  myLib(&eDest,myVersion,"ccmlib",mPath);
+
+// Resolve the context manager entry point
+//
+   initCCM = (XrdOucCacheCMInit_t)myLib.Resolve("XrdOucCacheCMInit");
+   return initCCM != 0;
+}
   
 /******************************************************************************/
 /*                                 P a r s e                                  */
@@ -285,6 +289,7 @@ bool XrdOucPsx::Parse(char *var, XrdOucStream &Config, XrdSysError &eDest)
    TS_Xeq("memcache",      ParseCache);  // Backward compatibility
    TS_Xeq("cache",         ParseCache);
    TS_Xeq("cachelib",      ParseCLib);
+   TS_Xeq("ccmlib",        ParseMLib);
    TS_Xeq("ciosync",       ParseCio);
    TS_Xeq("inetmode",      ParseINet);
    TS_Xeq("namelib",       ParseNLib);
@@ -529,7 +534,13 @@ bool XrdOucPsx::ParseCLib(XrdSysError *Eroute, XrdOucStream &Config)
 // Save the path
 //
    if (cPath) free(cPath);
-   cPath = (strcmp(val,"default") ? strdup(val) : strdup("libXrdFileCache.so"));
+   if (!strcmp(val,"libXrdFileCache.so") || !strcmp(val,"libXrdFileCache-4.so"))
+      {Eroute->Say("Config warning: 'libXrdFileCache' has been replaced by "
+            "'libXrdPfc'; for future compatability specify 'default' instead!");
+       cPath = strdup("libXrdPfc.so");
+      } else {
+       cPath = (strcmp(val,"default") ? strdup(val) : strdup("libXrdPfc.so"));
+      }
 
 // Get the parameters
 //
@@ -543,6 +554,46 @@ bool XrdOucPsx::ParseCLib(XrdSysError *Eroute, XrdOucStream &Config)
    return true;
 }
   
+/******************************************************************************/
+/*                             P a r s e M L i b                              */
+/******************************************************************************/
+
+/* Function: ParseCLib
+
+   Purpose:  To parse the directive: ccmlib <path> [<parms>]
+
+             <path>    the path of the cache context mgmt library to be used.
+             <parms>   optional parms to be passed
+
+  Output: true upon success or false upon failure.
+*/
+
+bool XrdOucPsx::ParseMLib(XrdSysError *Eroute, XrdOucStream &Config)
+{
+    char *val, parms[2048];
+
+// Get the path and parms
+//
+   if (!(val = Config.GetWord()) || !val[0])
+      {Eroute->Emsg("Config", "ccmlib not specified"); return false;}
+
+// Save the path
+//
+   if (mPath) free(mPath);
+   mPath = strdup(val);
+
+// Get the parameters
+//
+   if (!Config.GetRest(parms, sizeof(parms)))
+      {Eroute->Emsg("Config", "ccmlib parameters too long"); return false;}
+   if (mParm) free(mParm);
+   mParm = (*parms ? strdup(parms) : 0);
+
+// All done
+//
+   return true;
+}
+  
 /******************************************************************************/
 /*                             P a r s e I N e t                              */
 /******************************************************************************/
@@ -585,7 +636,7 @@ bool XrdOucPsx::ParseINet(XrdSysError *Eroute, XrdOucStream &Config)
 
    Purpose:  To parse the directive: namelib [<opts>] pfn<path> [<parms>]
 
-             <opts>    one or more: [-lfn2pfn] [-lfncache]
+             <opts>    one or more: [-lfn2pfn] [-lfncache[src[+]]]
              <path>    the path of the filesystem library to be used.
              <parms>   optional parms to be passed
 
@@ -595,18 +646,24 @@ bool XrdOucPsx::ParseINet(XrdSysError *Eroute, XrdOucStream &Config)
 bool XrdOucPsx::ParseNLib(XrdSysError *Eroute, XrdOucStream &Config)
 {
     char *val, parms[1024];
-    bool l2p = false, p2l = false;
+    bool l2p = false, p2l = false, p2lsrc = false, p2lsgi = false;
 
 // Parse options, if any
 //
    while((val = Config.GetWord()) && val[0])
-        {     if (!strcmp(val, "-lfn2pfn"))  l2p = true;
-         else if (!strcmp(val, "-lfncache")) p2l = true;
+        {     if (!strcmp(val, "-lfn2pfn"))      l2p = true;
+         else if (!strcmp(val, "-lfncache"))     p2l = true;
+         else if (!strcmp(val, "-lfncachesrc"))  p2l = p2lsrc = true;
+         else if (!strcmp(val, "-lfncachesrc+")) p2l = p2lsgi = true;
          else break;
         }
+
    if (!l2p && !p2l) l2p = true;
    xLfn2Pfn = l2p;
-   xPfn2Lfn = p2l;
+         if (!p2l)   xPfn2Lfn = xP2Loff;
+    else if (p2lsrc) xPfn2Lfn = xP2Lsrc;
+    else if (p2lsgi) xPfn2Lfn = xP2Lsgi;
+    else             xPfn2Lfn = xP2Lon;
 
 // Get the path
 //
