@@ -28,7 +28,6 @@
 /******************************************************************************/
 
 #include <ctype.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,7 +43,7 @@
 #include <sys/conf.h>
 #endif
 #include <sys/stat.h>
-#include <sys/termios.h>
+#include <termios.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #else // WIN32
@@ -52,9 +51,17 @@
 #include <process.h>
 #endif // WIN32
 
+#include <set>
+#include <string>
+
 #include "XrdOuc/XrdOucEnv.hh"
+#include "XrdOuc/XrdOucNSWalk.hh"
+#include "XrdOuc/XrdOucString.hh"
 #include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucTList.hh"
 #include "XrdOuc/XrdOucUtils.hh"
+#include "XrdSys/XrdSysE2T.hh"
+#include "XrdSys/XrdSysFD.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysLogger.hh"
 #include "XrdSys/XrdSysPlatform.hh"
@@ -69,19 +76,113 @@
 #define XrdOucStream_BUSY 0x02
 #define XrdOucStream_ELIF 0x80
 
+#define XrdOucStream_CADD 0x010000
+#define XrdOucStream_CONT 0xff0000
+#define XrdOucStream_CMAX 0x0f0000
+
 #define Erq(p, a, b) Err(p, a, b, (char *)0)
 #define Err(p, a, b, c) (ecode=(Eroute ? Eroute->Emsg(#p, a, b, c) : a), -1)
 #define Erp(p, a, b, c)  ecode=(Eroute ? Eroute->Emsg(#p, a, b, c) : a)
 
 // The following is used by child processes prior to exec() to avoid deadlocks
 //
-#define Erx(p, a, b) cerr <<#p <<": " <<strerror(a) <<' ' <<b <<endl;
+#define Erx(p, a, b) cerr <<#p <<": " <<XrdSysE2T(a) <<' ' <<b <<endl;
 
+/******************************************************************************/
+/*              S t a t i c   M e m b e r s   &   O b j e c t s               */
+/******************************************************************************/
+  
 // The following mutex is used to allow only one fork at a time so that
 // we do not leak file descriptors. It is a short-lived lock.
 //
 namespace {XrdSysMutex forkMutex;}
 
+XrdOucString *XrdOucStream::theCFG = 0;
+
+/******************************************************************************/
+/*                         L o c a l   C l a s s e s                          */
+/******************************************************************************/
+
+struct StreamInfo
+      {char *myHost;
+       char *myName;
+       char *myExec;
+
+       std::set<std::string> *fcList;
+       std::set<std::string>::iterator itFC;
+
+       StreamInfo() : myHost(0), myName(0), myExec(0),
+                      fcList(0) {}
+      ~StreamInfo() {if (fcList) delete fcList;}
+      };
+
+namespace
+{
+class contHandler
+{
+public:
+
+char        *path;
+XrdOucTList *tlP;
+
+void  Add(const char *sfx) {tlP = new XrdOucTList(sfx,(int)strlen(sfx),tlP);}
+
+      contHandler() : path(0), tlP(0) {}
+     ~contHandler() {XrdOucTList *tlN;
+                     while(tlP) {tlN = tlP; tlP = tlP->next; delete tlN;}
+                     if (path) free(path);
+                    }
+};
+}
+  
+/******************************************************************************/
+/*                       L o c a l   F u n c t i o n s                        */
+/******************************************************************************/
+
+namespace
+{
+bool KeepFile(const char *fname, XrdOucTList *tlP)
+{
+   struct sfxList {const char *txt; int len;};
+   static sfxList sfx[] = {{".cfsaved",    8},
+                           {".rpmsave",    8},
+                           {".rpmnew",     7},
+                           {".dpkg-old",   9},
+                           {".dpkg-dist", 10},
+                           {"~",           1}
+                          };
+   static int sfxLNum = sizeof(sfx)/sizeof(struct sfxList);
+   int n;
+
+// We don't keep file that start with a dot
+//
+   if (*fname == '.') return false;
+   n = strlen(fname);
+
+// Process white list first, otherwise use the black list
+//
+   if (tlP)
+      {while(tlP)
+            {if (tlP->ival[0] < n && !strcmp(tlP->text, fname+n-tlP->ival[0]))
+                return true;
+             tlP = tlP->next;
+            }
+       return false;
+      }
+
+// Check all other suffixes we wish to avoid
+//
+   for (int i = 0; i < sfxLNum; i++)
+       {if (sfx[i].len < n && !strcmp(sfx[i].txt, fname+n-sfx[i].len))
+           return false;
+       }
+
+// This file can be kept
+//
+   return true;
+}
+}
+
 /******************************************************************************/
 /*               o o u c _ S t r e a m   C o n s t r u c t o r                */
 /******************************************************************************/
@@ -90,18 +191,22 @@ XrdOucStream::XrdOucStream(XrdSysError *erobj, const char *ifname,
                            XrdOucEnv   *anEnv, const char *Pfx)
 {
  char *cp;
+
+
      if (ifname)
         {myInst = strdup(ifname);
-         if (!(cp = index(myInst, ' '))) {cp = myInst; myExec = 0;}
+         myInfo = new StreamInfo;
+         if (!(cp = index(myInst, ' '))) {cp = myInst; myInfo->myExec = 0;}
             else {*cp = '\0'; cp++;
-                  myExec = (*myInst ? myInst : 0);
+                  myInfo->myExec = (*myInst ? myInst : 0);
                  }
-         if ((myHost = index(cp, '@')))
-            {*myHost = '\0';
-             myHost++;
-             myName = (*cp ? cp : 0);
-            } else {myHost = cp; myName = 0;}
-        } else myInst = myHost = myName = myExec = 0;
+         if ( (myInfo->myHost = index(cp, '@')))
+            {*(myInfo->myHost) = '\0';
+             myInfo->myHost++;
+             myInfo->myName = (*cp ? cp : 0);
+            } else {myInfo->myHost = cp; myInfo->myName = 0;}
+        } else {myInst = 0; myInfo = 0;}
+     myRsv1 = myRsv2 = 0;
 
      FD     = -1;
      FE     = -1;
@@ -152,7 +257,9 @@ int XrdOucStream::Attach(int FileDescriptor, int bsz)
 
     // Close the current stream. Close will handle unopened streams.
     //
+    StreamInfo *saveInfo = myInfo; myInfo = 0;
     Close();
+    myInfo = saveInfo;
 
     // Allocate a new buffer for this stream
     //
@@ -179,6 +286,39 @@ int XrdOucStream::Attach(int FileDescriptor, int bsz)
     return  0;
 }
   
+/******************************************************************************/
+/*                               C a p t u r e                                */
+/******************************************************************************/
+
+void XrdOucStream::Capture(const char **cVec, bool linefeed)
+{
+// Make sure we can handle this
+//
+   if (theCFG && cVec && cVec[0])
+      {if (linefeed) theCFG->append("\n# ");
+          else theCFG->append("# ");
+       int i = 0;
+       while(cVec[i]) theCFG->append(cVec[i++]);
+       theCFG->append('\n');
+      }
+}
+
+/******************************************************************************/
+
+XrdOucString *XrdOucStream::Capture(XrdOucString *newCFG)
+{
+   XrdOucString *oldCFG = theCFG;
+   theCFG = newCFG;
+   return oldCFG;
+}
+
+/******************************************************************************/
+
+XrdOucString *XrdOucStream::Capture()
+{
+   return theCFG;
+}
+
 /******************************************************************************/
 /*                                 C l o s e                                  */
 /******************************************************************************/
@@ -207,10 +347,20 @@ void XrdOucStream::Close(int hold)
 
     // Check if we should echo the last line
     //
-    if (llBuff && Verbose && Eroute)
-       {if (*llBuff && llBok > 1) Eroute->Say(llPrefix, llBuff);
+    if (llBuff)
+       {if (Verbose && *llBuff && llBok > 1)
+           {if (Eroute) Eroute->Say(llPrefix, llBuff);
+            if (theCFG) add2CFG(llBuff);
+           }
         llBok = 0;
        }
+
+    // Delete any info object we have allocated
+    //
+    if (myInfo)
+       {delete myInfo;
+        myInfo = 0;
+       }
 }
 
 /******************************************************************************/
@@ -242,13 +392,29 @@ int XrdOucStream::Drain()
 /******************************************************************************/
 /*                                  E c h o                                   */
 /******************************************************************************/
-  
+
 void XrdOucStream::Echo()
 {
-   if (llBok && Verbose && *llBuff && Eroute) Eroute->Say(llPrefix, llBuff);
+   if (llBok > 1 && Verbose && llBuff)
+      {if (Eroute) Eroute->Say(llPrefix,llBuff);
+       if (theCFG) add2CFG(llBuff);
+      }
    llBok = 0;
 }
 
+/******************************************************************************/
+/*                              E c h o O n l y                               */
+/******************************************************************************/
+
+void XrdOucStream::Echo(bool capture)
+{
+   if (llBok && Verbose && llBuff)
+      {if (Eroute) Eroute->Say(llPrefix,llBuff);
+       if (capture && theCFG) add2CFG(llBuff);
+      }
+   llBok = 0;
+}
+  
 /******************************************************************************/
 /*                               E   x   e   c                                */
 /******************************************************************************/
@@ -363,6 +529,15 @@ int XrdOucStream::Exec(char **parm, int inrd, int efd)
            } else close(Child_log);
        }
 
+    // Check if we need to set any envornment variables
+    //
+    if (myEnv)
+       {char **envP;
+        int i = 0;
+        if ((envP = (char **)myEnv->GetPtr("XrdEnvars**")))
+           while(envP[i]) {putenv(envP[i]); i++;}
+       }
+
     // Set our process group (the parent should have done this by now) then
     // invoke the command never to return
     //
@@ -507,7 +682,7 @@ char *XrdOucStream::GetFirstWord(int lowcase)
       // If in the middle of a line, flush to the end of the line. Suppress
       // variable substitution when doing this to avoid errors.
       //
-      if (xline) 
+      if (xline)
          {XrdOucEnv *oldEnv = SetEnv(0);
           while(GetWord(lowcase)) {}
           SetEnv(oldEnv);
@@ -524,9 +699,7 @@ char *XrdOucStream::GetMyFirstWord(int lowcase)
    char *var;
    int   skip2fi = 0;
 
-
-   if (llBok > 1 && Verbose && *llBuff && Eroute) Eroute->Say(llPrefix,llBuff);
-   llBok = 0;
+   Echo();
 
    if (!myInst)
       {if (!myEnv) return add2llB(GetFirstWord(lowcase), 1);
@@ -536,13 +709,20 @@ char *XrdOucStream::GetMyFirstWord(int lowcase)
       }
 
    do {if (!(var = GetFirstWord(lowcase)))
-          {if (sawif)
+          {if (sawif && !ecode)
               {ecode = EINVAL;
                if (Eroute) Eroute->Emsg("Stream", "Missing 'fi' for last 'if'.");
               }
            return add2llB(var, 1);
           }
 
+        add2llB(var, 1);
+
+        if (!strcmp("continue", var))
+           {if (!docont()) return 0;
+            continue;
+           }
+
         if (       !strcmp("if",   var)) var = doif();
         if (var && !strcmp("else", var)) var = doelse();
         if (var && !strcmp("fi",   var))
@@ -581,7 +761,7 @@ char *XrdOucStream::GetWord(int lowcase)
 
      // Find the next non-blank non-comment line
      //
-     while(GetLine())
+do  {while(GetLine())
         {// Get the first token (none if it is a blank line)
          //
          if (!(wp = GetToken(lowcase))) continue;
@@ -599,6 +779,21 @@ char *XrdOucStream::GetWord(int lowcase)
             else xcont = 0;
          return add2llB((myEnv ? vSubs(wp) : wp));
          }
+
+     if (myInfo && myInfo->fcList)
+        {if (myInfo->itFC == myInfo->fcList->end())
+            {bleft = 0;
+             flags |= XrdOucStream_EOM;
+             break;
+            }
+         const char *path = (*(myInfo->itFC)).c_str();
+         myInfo->itFC++;
+         if (!docontF(path)) break;
+         bleft = 0;
+         flags &= ~XrdOucStream_EOM;
+        } else break;
+    } while(true);
+
       xline = 0;
       return (char *)0;
 }
@@ -759,6 +954,17 @@ int XrdOucStream::Wait4Data(int msMax)
 /******************************************************************************/
 /*                       P r i v a t e   M e t h o d s                        */
 /******************************************************************************/
+/******************************************************************************/
+/*                               a d d 2 C F G                                */
+/******************************************************************************/
+
+void XrdOucStream::add2CFG(const char *data, bool isCMT)
+{
+   if (isCMT) theCFG->append("# ");
+   theCFG->append(data);
+   theCFG->append('\n');
+}
+  
 /******************************************************************************/
 /*                               a d d 2 l l B                                */
 /******************************************************************************/
@@ -788,11 +994,211 @@ char *XrdOucStream::add2llB(char *tok, int reset)
 //
    if (tok)
       {tlen = strlen(tok);
-       if (tlen < llBsz)
+       if (tlen < llBleft)
           {strcpy(llBcur, tok); llBcur += tlen; llBleft -= tlen;}
       }
    return tok;
 }
+  
+/******************************************************************************/
+/*                                  E c h o                                   */
+/******************************************************************************/
+  
+bool XrdOucStream::Echo(int ec, const char *t1, const char *t2, const char *t3)
+{
+   if (Eroute)
+      {if (t1) Eroute->Emsg("Stream", t1, t2, t3);
+       if (llBok > 1 && Verbose && llBuff) Eroute->Say(llPrefix,llBuff);
+      }
+   ecode = ec;
+   llBok = 0;
+   return false;
+}
+
+/******************************************************************************/
+/*                                d o c o n t                                 */
+/******************************************************************************/
+  
+bool XrdOucStream::docont()
+{
+   char *theWord;
+
+// A continue is not valid within the scope of an if
+//
+   if (sawif) return Echo(EINVAL, "'continue' invalid within 'if-fi'.");
+
+// Get the path (keep case), if none then ignore this continue
+//
+   theWord = GetWord();
+   if (!theWord)
+      {Echo();
+       return true;
+      }
+
+// Prepare to handle the directive
+//
+   contHandler cH;
+   cH.path = strdup(theWord);
+
+// Grab additioal tokens which may be suffixes
+//
+   theWord = GetWord();
+   while(theWord && *theWord == '*')
+        {if (!*(theWord+1)) return Echo(EINVAL, "suffix missing after '*'.");
+         cH.Add(theWord+1);
+         theWord = GetWord();
+        }
+
+// If we have a token, it better be an if
+//
+   if (theWord && strcmp(theWord, "if"))
+      return Echo(EINVAL, "expecting 'if' but found", theWord);
+
+// Process the 'if'
+//
+   if (theWord && !XrdOucUtils::doIf(Eroute, *this, "continue directive",
+                                myInfo->myHost,myInfo->myName,myInfo->myExec))
+      return true;
+   Echo();
+// if (Eroute) Eroute->Say(llPrefix, "continue ", path, " if true");
+// if (Eroute) Eroute->Say(llPrefix, "continue ", bnext);
+   return docont(cH.path, cH.tlP);
+}
+  
+/******************************************************************************/
+
+bool XrdOucStream::docont(const char *path, XrdOucTList *tlP)
+{
+   struct stat Stat;
+   bool noentok;
+
+// A continue directive in the context of a continuation is illegal
+//
+   if ((myInfo && myInfo->fcList) || (flags & XrdOucStream_CONT) != 0)
+      return Echo(EINVAL, "'continue' is a continuation is not allowed.");
+
+// Check if this file must exist (we also take care of empty paths)
+//
+   if ((noentok = (*path == '?')))
+      {path++;
+       if (!(*path)) return true;
+      }
+
+// Check if this is a file or directory
+//
+   if (stat(path, &Stat))
+      {if (errno == ENOENT && noentok) return true;
+       if (Eroute)
+          {Eroute->Emsg("Stream", errno, "open", path);
+           ecode = ECANCELED;
+          } else ecode = errno;
+       return false;
+      }
+
+// For directory continuation, there is much more to do (this can only happen
+// once). Note that we used to allow a limited number of chained fle
+// continuations. No more, but we are still setup to easily do so.
+//
+   if ((Stat.st_mode & S_IFMT) == S_IFDIR)
+      {if (!docontD(path, tlP)) return false;
+       path = (*(myInfo->itFC)).c_str();
+       myInfo->itFC++;
+      } else flags |= XrdOucStream_CADD;
+
+//     if ((flags & XrdOucStream_CONT) > XrdOucStream_CMAX)
+//        {if (Eroute)
+//            {Eroute->Emsg("Stream", EMLINK, "continue to", path);
+//             ecode = ECANCELED;
+//            } else ecode = EMLINK;
+//         return false;
+//        }
+//    }
+
+// Continue with the next file
+//
+   return docontF(path, noentok);
+}
+
+/******************************************************************************/
+/*                               d o c o n t D                                */
+/******************************************************************************/
+  
+bool XrdOucStream::docontD(const char *path, XrdOucTList *tlP)
+{
+   static const mode_t isXeq = S_IXUSR | S_IXGRP | S_IXOTH;
+   XrdOucNSWalk nsWalk(Eroute, path, 0, XrdOucNSWalk::retFile);
+   int rc;
+
+// Get all of the file entries in this directory
+//
+   XrdOucNSWalk::NSEnt *nsX, *nsP = nsWalk.Index(rc);
+   if (rc)
+      {if (Eroute) Eroute->Emsg("Stream", rc, "index config files in", path);
+       ecode = ECANCELED;
+       return false;
+      }
+
+// Keep only files of interest
+//
+   myInfo->fcList = new std::set<std::string>;
+   while((nsX = nsP))
+        {nsP = nsP->Next;
+         if ((nsX->Stat.st_mode & isXeq) == 0 && KeepFile(nsX->File, tlP))
+            myInfo->fcList->insert(std::string(nsX->Path));
+         delete nsX;
+        }
+
+// Check if we have anything in the map
+//
+   if (myInfo->fcList->size() == 0)
+      {delete myInfo->fcList;
+       myInfo->fcList = 0;
+       return false;
+      }
+
+// All done
+//
+   myInfo->itFC = myInfo->fcList->begin();
+   return true;
+}
+  
+/******************************************************************************/
+/*                                 c o n t F                                  */
+/******************************************************************************/
+
+bool XrdOucStream::docontF(const char *path, bool noentok)
+{
+   int cFD;
+
+// Open the file and handle any errors
+//
+   if ((cFD = XrdSysFD_Open(path, O_RDONLY)) < 0)
+      {if (errno == ENOENT && noentok) return true;
+       if (Eroute)
+          {Eroute->Emsg("Stream", errno, "open", path);
+           ecode = ECANCELED;
+          } else ecode = errno;
+       return false;
+      }
+
+// Continue to the next file
+//
+   if (XrdSysFD_Dup2(cFD, FD) < 0)
+      {if (Eroute)
+          {Eroute->Emsg("Stream", ecode, "switch to", path);
+           close(cFD);
+           ecode = ECANCELED;
+          } else ecode = errno;
+       return false;
+      }
+
+// Indicate we are switching to anther file
+//
+   if (Eroute) Eroute->Say("Config continuing with file ", path, " ...");
+   bleft = 0;
+   return true;
+}
+  
 /******************************************************************************/
 /*                                d o e l s e                                 */
 /******************************************************************************/
@@ -882,7 +1288,8 @@ char *XrdOucStream::doif()
 // Check if we should continue
 //
    sawif = 1; skpel = 0;
-   if ((rc = XrdOucUtils::doIf(Eroute,*this,"if directive",myHost,myName,myExec)))
+   if ((rc = XrdOucUtils::doIf(Eroute,*this,"if directive",
+                               myInfo->myHost,myInfo->myName,myInfo->myExec)))
       {if (rc >= 0) skpel = 1;
           else {ecode = EINVAL;
                 if(Eroute) Eroute->Say(llPrefix,
