@@ -407,7 +407,7 @@ bool XrdOucString::endswith(char c)
 {
    // returns 1 if the stored string ends with string s
 
-   return (rfind(c) == (int)(len-1));
+   return ((len > 0) ? (rfind(c) == (int)(len-1)) : 0);
 }
 
 //______________________________________________________________________________
@@ -415,7 +415,11 @@ bool XrdOucString::endswith(const char *s)
 {
    // returns 1 if the stored string ends with string s
 
-   return (s ? (rfind(s) == (int)(len-strlen(s))) : 0);
+   int ls = -1;
+   if (s && (ls = (int)strlen(s)) <= len)
+      return ((rfind(s) == (int)(len-strlen(s))) ? 1 : 0);
+
+   return 0;
 }
 
 //___________________________________________________________________________
@@ -804,7 +808,8 @@ int XrdOucString::replace(const char *s1, const char *s2, int from, int to)
                memcpy(pc-l2,s2,l2);
             nc--;
             atn = at;
-            at = rfind(s1,at-l1);
+            if( at - l1 < 0 ) at = -1;
+            else at = rfind(s1,at-l1);
          }
          dl = nr*dd;
       }
@@ -950,7 +955,7 @@ void XrdOucString::hardreset()
 
    if (str) {
       volatile char *buf = 0;
-      for (buf = (volatile char *)str; len; buf[--len] = 0);
+      for (buf = (volatile char *)str; len; buf[--len] = 0) {}
       len = 0;
    }
    len = 0;
