@@ -29,7 +29,6 @@
 /******************************************************************************/
 
 #include <ctype.h>
-#include <errno.h>
 #include <grp.h>
 #include <stdio.h>
 
@@ -42,12 +41,67 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #endif
+#include <map>
 #include "XrdNet/XrdNetUtils.hh"
 #include "XrdOuc/XrdOucEnv.hh"
+#include "XrdOuc/XrdOucStream.hh"
 #include "XrdOuc/XrdOucUtils.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysPlatform.hh"
-#include "XrdOuc/XrdOucStream.hh"
+#include "XrdSys/XrdSysPthread.hh"
+  
+/******************************************************************************/
+/*                         L o c a l   M e t h o d s                          */
+/******************************************************************************/
+  
+namespace
+{
+struct idInfo
+{      time_t  Expr;
+       char   *Name;
+
+       idInfo(const char *name, time_t keep)
+             : Expr(time(0)+keep), Name(strdup(name)) {}
+      ~idInfo() {free(Name);}
+};
+
+typedef std::map<unsigned int, struct idInfo*> idMap_t;
+
+idMap_t     gidMap;
+idMap_t     uidMap;
+XrdSysMutex idMutex;
+
+void AddID(idMap_t &idMap, unsigned int id, const char *name, time_t keepT)
+{
+   std::pair<idMap_t::iterator,bool> ret;
+   idInfo *infoP = new idInfo(name, keepT);
+
+   idMutex.Lock();
+   ret = idMap.insert(std::pair<unsigned int, struct idInfo*>(id, infoP));
+   if (ret.second == false) delete infoP;
+   idMutex.UnLock();
+}
+
+int LookUp(idMap_t &idMap, unsigned int id, char *buff, int blen)
+{
+   idMap_t::iterator it;
+   int luRet = 0;
+
+   idMutex.Lock();
+   it = idMap.find(id);
+   if (it != idMap.end())
+      {if (it->second->Expr <= time(0))
+          {delete it->second;
+           idMap.erase(it);
+          } else {
+           if (blen > 0) luRet = snprintf(buff, blen, "%s", it->second->Name);
+          }
+      }
+   idMutex.UnLock();
+   return luRet;
+}
+}
   
 /******************************************************************************/
 /*                              e n d s W i t h                               */
@@ -64,21 +118,20 @@ bool XrdOucUtils::endsWith(const char *text, const char *ending, int endlen)
 /*                                 e T e x t                                  */
 /******************************************************************************/
   
-// eText() returns the text associated with the error, making the first
-// character in the text lower case. The text buffer pointer is returned.
+// eText() returns the text associated with the error.
+// The text buffer pointer is returned.
 
-char *XrdOucUtils::eText(int rc, char *eBuff, int eBlen, int AsIs)
+char *XrdOucUtils::eText(int rc, char *eBuff, int eBlen)
 {
    const char *etP;
 
 // Get error text
 //
-   if (!(etP = strerror(rc)) || !(*etP)) etP = "reason unknown";
+   etP = XrdSysE2T(rc);
 
 // Copy the text and lower case the first letter
 //
    strlcpy(eBuff, etP, eBlen);
-   if (!AsIs) *eBuff = tolower(*eBuff);
 
 // All done
 //
@@ -288,7 +341,90 @@ int XrdOucUtils::genPath(char *buff, int blen, const char *path, const char *psf
 }
 
 /******************************************************************************/
-/*                                G r o u p s                                 */
+/*                                g e t G I D                                 */
+/******************************************************************************/
+  
+bool XrdOucUtils::getGID(const char *gName, gid_t &gID)
+{
+   struct group Grp, *result;
+   char buff[65536];
+
+   getgrnam_r(gName, &Grp, buff, sizeof(buff), &result);
+   if (!result) return false;
+
+   gID = Grp.gr_gid;
+   return true;
+}
+
+/******************************************************************************/
+/*                                g e t U I D                                 */
+/******************************************************************************/
+  
+bool XrdOucUtils::getUID(const char *uName, uid_t &uID, gid_t *gID)
+{
+   struct passwd pwd, *result;
+   char buff[16384];
+
+   getpwnam_r(uName, &pwd, buff, sizeof(buff), &result);
+   if (!result) return false;
+
+   uID = pwd.pw_uid;
+   if (gID) *gID = pwd.pw_gid;
+
+   return true;
+}
+  
+/******************************************************************************/
+/*                               G i d N a m e                                */
+/******************************************************************************/
+  
+int XrdOucUtils::GidName(gid_t gID, char *gName, int gNsz, time_t keepT)
+{
+   static const int maxgBsz = 256*1024;
+   static const int addGsz  = 4096;
+   struct group  *gEnt, gStruct;
+   char gBuff[1024], *gBp = gBuff;
+   int glen = 0, gBsz = sizeof(gBuff), aOK = 1;
+   int n, retVal = 0;
+
+// Get ID from cache, if allowed
+//
+   if (keepT)
+      {int n = LookUp(gidMap, static_cast<unsigned int>(gID),gName,gNsz);
+       if (n > 0) return (n < gNsz ? n : 0);
+      }
+
+// Get the the group struct. If we don't have a large enough buffer, get a
+// larger one and try again up to the maximum buffer we will tolerate.
+//
+   while(( retVal = getgrgid_r(gID, &gStruct, gBp, gBsz, &gEnt) ) == ERANGE)
+        {if (gBsz >= maxgBsz) {aOK = 0; break;}
+         if (gBsz >  addGsz) free(gBp);
+         gBsz += addGsz;
+         if (!(gBp = (char *)malloc(gBsz))) {aOK = 0; break;}
+        }
+
+// Return a group name if all went well
+//
+   if (aOK && retVal == 0 && gEnt != NULL)
+      {if (keepT)
+          AddID(gidMap, static_cast<unsigned int>(gID), gEnt->gr_name, keepT);
+       glen = strlen(gEnt->gr_name);
+       if (glen >= gNsz) glen = 0;
+          else strcpy(gName, gEnt->gr_name);
+      } else {
+       n = snprintf(gName, gNsz, "%ud", static_cast<unsigned int>(gID));
+       if (n >= gNsz) glen = 0;
+      }
+
+// Free any allocated buffer and return result
+//
+   if (gBsz >  addGsz && gBp) free(gBp);
+   return glen;
+}
+
+/******************************************************************************/
+/*                             G r o u p N a m e                              */
 /******************************************************************************/
   
 int XrdOucUtils::GroupName(gid_t gID, char *gName, int gNsz)
@@ -484,7 +620,7 @@ int XrdOucUtils::Log10(unsigned long long n)
   
 void XrdOucUtils::makeHome(XrdSysError &eDest, const char *inst)
 {
-   char buff[1024];
+   char buff[2048];
 
    if (!inst || !getcwd(buff, sizeof(buff))) return;
 
@@ -498,18 +634,66 @@ void XrdOucUtils::makeHome(XrdSysError &eDest, const char *inst)
       eDest.Emsg("Config", errno, "chdir to home directory", buff);
 }
 
+/******************************************************************************/
+  
+bool XrdOucUtils::makeHome(XrdSysError &eDest, const char *inst,
+                                               const char *path, mode_t mode)
+{
+   char cwDir[2048];
+   const char *slash = "", *slash2 = "";
+   int n, rc;
+
+// Provide backward compatability for instance name qualification
+//
+
+   if (!path || !(n = strlen(path)))
+      {if (inst) makeHome(eDest, inst);
+       return true;
+      }
+
+// Augment the path with instance name, if need be
+//
+   if (path[n-1] != '/') slash = "/";
+   if (!inst || !(n = strlen(inst))) inst = "";
+      else slash2 = "/";
+    n = snprintf(cwDir, sizeof(cwDir), "%s%s%s%s", path, slash, inst, slash2);
+    if (n >= (int)sizeof(cwDir))
+       {eDest.Emsg("Config", ENAMETOOLONG, "create home directory", cwDir);
+        return false;
+       }
+
+// Create the path if it doesn't exist
+//
+   if ((rc = makePath(cwDir, mode, true)))
+      {eDest.Emsg("Config", rc, "create home directory", cwDir);
+       return false;
+      }
+
+// Switch to this directory
+//
+   if (chdir(cwDir) < 0)
+      {eDest.Emsg("Config", errno, "chdir to home directory", cwDir);
+       return false;
+      }
+
+// All done
+//
+   return true;
+}
+
 /******************************************************************************/
 /*                              m a k e P a t h                               */
 /******************************************************************************/
   
-int XrdOucUtils::makePath(char *path, mode_t mode)
+int XrdOucUtils::makePath(char *path, mode_t mode, bool reset)
 {
     char *next_path = path+1;
     struct stat buf;
+    bool dochmod = false; // The 1st component stays as is
 
 // Typically, the path exists. So, do a quick check before launching into it
 //
-   if (!stat(path, &buf)) return 0;
+   if (!reset && !stat(path, &buf)) return 0;
 
 // Start creating directories starting with the root
 //
@@ -517,6 +701,8 @@ int XrdOucUtils::makePath(char *path, mode_t mode)
         {*next_path = '\0';
          if (MAKEDIR(path, mode))
             if (errno != EEXIST) return -errno;
+         if (dochmod) CHMOD(path, mode);
+         dochmod = reset;
          *next_path = '/';
          next_path = next_path+1;
         }
@@ -526,13 +712,82 @@ int XrdOucUtils::makePath(char *path, mode_t mode)
    return 0;
 }
  
+/******************************************************************************/
+/*                              p a r s e L i b                               */
+/******************************************************************************/
+  
+bool XrdOucUtils::parseLib(XrdSysError &eDest, XrdOucStream &Config,
+                           const char *libName, char *&libPath, char **libParm)
+{
+    char *val, parms[2048];
+
+// Get the path
+//
+   if (!(val = Config.GetWord()) || !val[0])
+      {eDest.Emsg("Config", libName, "not specified"); return false;}
+
+// Record the path
+//
+   if (libPath) free(libPath);
+   libPath = strdup(val);
+
+// Handle optional parameter
+//
+   if (!libParm) return true;
+   if (*libParm) free(*libParm);
+   *libParm = 0;
+
+// Record any parms
+//
+   if (!Config.GetRest(parms, sizeof(parms)))
+      {eDest.Emsg("Config", libName, "parameters too long"); return false;}
+   if (*parms) *libParm = strdup(parms);
+   return true;
+}
+
+/******************************************************************************/
+/*                             p a r s e H o m e                              */
+/******************************************************************************/
+  
+char *XrdOucUtils::parseHome(XrdSysError &eDest, XrdOucStream &Config, int &mode)
+{
+   char *pval, *val, *HomePath = 0;
+
+// Get the path
+//
+   pval = Config.GetWord();
+   if (!pval || !pval[0])
+      {eDest.Emsg("Config", "home path not specified"); return 0;}
+
+// Make sure it's an absolute path
+//
+   if (*pval != '/')
+      {eDest.Emsg("Config", "home path not absolute"); return 0;}
+
+// Record the path
+//
+   HomePath = strdup(pval);
+
+// Get the optional access rights
+//
+   mode = S_IRWXU;
+   if ((val = Config.GetWord()) && val[0])
+      {if (!strcmp("group", val)) mode |= (S_IRGRP | S_IXGRP);
+          else {eDest.Emsg("Config", "invalid home path modifier -", val);
+                free(HomePath);
+                return 0;
+               }
+      }
+   return HomePath;
+}
+
 /******************************************************************************/
 /*                                R e L i n k                                 */
 /******************************************************************************/
 
 int XrdOucUtils::ReLink(const char *path, const char *target, mode_t mode)
 {
-   const mode_t AMode = S_IRWXU|S_IRWXG; // 770
+   const mode_t AMode = S_IRWXU;  // Only us as a default
    char pbuff[MAXPATHLEN+64];
    int n;
 
@@ -550,6 +805,28 @@ int XrdOucUtils::ReLink(const char *path, const char *target, mode_t mode)
    return 0;
 }
 
+/******************************************************************************/
+/*                              S a n i t i z e                               */
+/******************************************************************************/
+
+void XrdOucUtils::Sanitize(char *str, char subc)
+{
+
+// Sanitize string according to POSIX.1-2008 stanadard using only the
+// Portable Filename Character Set: a-z A-Z 0-9 ._- with 1st char not being -
+//
+   if (*str)
+      {if (*str == '-') *str = subc;
+          else if (*str == ' ') *str = subc;
+       char *blank = rindex(str, ' ');
+       if (blank) while(*blank == ' ') *blank-- = 0;
+       while(*str)
+            {if (!isalnum(*str) && index("_-.", *str) == 0) *str = subc;
+             str++;
+            }
+      }
+}
+
 /******************************************************************************/
 /*                              s u b L o g f n                               */
 /******************************************************************************/
@@ -706,6 +983,45 @@ void XrdOucUtils::Undercover(XrdSysError &eDest, int noLog, int *pipeFD)
   for (myfd = 3; myfd < maxFiles; myfd++)
       if( (!pipeFD || myfd != pipeFD[1]) && myfd != logFD ) close(myfd);
 }
+  
+/******************************************************************************/
+/*                               U i d N a m e                                */
+/******************************************************************************/
+  
+int XrdOucUtils::UidName(uid_t uID, char *uName, int uNsz, time_t keepT)
+{
+   struct passwd *pEnt, pStruct;
+   char pBuff[1024];
+   int n, rc;
+
+// Get ID from cache, if allowed
+//
+   if (keepT)
+      {int n = LookUp(uidMap, static_cast<unsigned int>(uID),uName,uNsz);
+       if (n > 0) return (n < uNsz ? n : 0);
+      }
+
+// Try to obtain the username. We use this form to make sure we are using
+// the standards conforming version (compilation error otherwise).
+//
+   rc = getpwuid_r(uID, &pStruct, pBuff, sizeof(pBuff), &pEnt);
+   if (rc || !pEnt)
+      {n = snprintf(uName, uNsz, "%ud", static_cast<unsigned int>(uID));
+       return (n >= uNsz ? 0 : n);
+      }
+
+// Add entry to the cache if need be
+//
+   if (keepT)
+      AddID(uidMap, static_cast<unsigned int>(uID), pEnt->pw_name, keepT);
+
+// Return length of username or zero if it is too big
+//
+   n = strlen(pEnt->pw_name);
+   if (uNsz <= (int)strlen(pEnt->pw_name)) return 0;
+   strcpy(uName, pEnt->pw_name);
+   return n;
+}
 
 /******************************************************************************/
 /*                              U s e r N a m e                               */
@@ -731,6 +1047,39 @@ int XrdOucUtils::UserName(uid_t uID, char *uName, int uNsz)
    return 0;
 }
 
+/******************************************************************************/
+/*                               V a l P a t h                                */
+/******************************************************************************/
+
+const char *XrdOucUtils::ValPath(const char *path, mode_t allow, bool isdir)
+{
+   static const mode_t mMask = S_IRWXU | S_IRWXG | S_IRWXO;
+   struct stat buf;
+
+// Check if this really exists
+//
+   if (stat(path, &buf))
+      {if (errno == ENOENT) return "does not exist.";
+       return XrdSysE2T(errno);
+      }
+
+// Verify that this is the correct type of file
+//
+   if (isdir)
+      {if (!S_ISDIR(buf.st_mode)) return "is not a directory.";
+      } else {
+       if (!S_ISREG(buf.st_mode)) return "is not a file.";
+      }
+
+// Verify that the does not have excessive privileges
+//
+   if ((buf.st_mode & mMask) & ~allow) return "has excessive access rights.";
+
+// All went well
+//
+   return 0;
+}
+  
 /******************************************************************************/
 /*                               P i d F i l e                                */
 /******************************************************************************/
