@@ -166,7 +166,7 @@ int XrdPosix_Fclose(FILE *stream)
 
 // Close the associated file
 //
-   if (Xroot.myFD(nullfd)) Xroot.Close(nullfd, 1);
+   if (Xroot.myFD(nullfd)) Xroot.Close(nullfd);
 
 // Now close the stream
 //
@@ -313,7 +313,7 @@ size_t XrdPosix_Fread(void *ptr, size_t size, size_t nitems, FILE *stream)
 #if defined(__linux__)
    else if (bytes < 0) stream->_flags |= _IO_ERR_SEEN;
    else                stream->_flags |= _IO_EOF_SEEN;
-#elif defined(__macos__)
+#elif defined(__APPLE__) || defined(__FreeBSD__)
    else if (bytes < 0) stream->_flags |= __SEOF;
    else                stream->_flags |= __SERR;
 #else
@@ -375,7 +375,7 @@ int XrdPosix_Fstat(int fildes, struct stat *buf)
 //
    return (Xroot.myFD(fildes)
           ? Xroot.Fstat(fildes, buf)
-#ifdef __linux__
+#if defined(__linux__) and defined(_STAT_VER)
           : Xunix.Fstat64(_STAT_VER, fildes, (struct stat64 *)buf));
 #else
           : Xunix.Fstat64(           fildes, (struct stat64 *)buf));
@@ -387,7 +387,11 @@ int XrdPosix_FstatV(int ver, int fildes, struct stat *buf)
 {
    return (Xroot.myFD(fildes)
           ? Xroot.Fstat(fildes, buf)
+#ifdef _STAT_VER
           : Xunix.Fstat64(ver, fildes, (struct stat64 *)buf));
+#else
+          : Xunix.Fstat64(     fildes, (struct stat64 *)buf));
+#endif
 }
 #endif
 }
@@ -479,7 +483,7 @@ size_t XrdPosix_Fwrite(const void *ptr, size_t size, size_t nitems, FILE *stream
 #ifndef SUNX86
 #if defined(__linux__)
       else stream->_flags |= _IO_ERR_SEEN;
-#elif defined(__macos__)
+#elif defined(__APPLE__) || defined(__FreeBSD__)
       else stream->_flags |= __SERR;
 #else
       else stream->_flag  |= _IOERR;
@@ -559,7 +563,7 @@ int XrdPosix_Lstat(const char *path, struct stat *buf)
 // Return the results of an open of a Unix file
 //
    return (!(myPath = XrootPath.URL(path, buff, sizeof(buff)))
-#ifdef __linux__
+#if defined(__linux__) and defined(_STAT_VER)
           ? Xunix.Lstat64(_STAT_VER, path, (struct stat64 *)buf)
 #else
           ? Xunix.Lstat64(           path, (struct stat64 *)buf)
@@ -887,7 +891,7 @@ int XrdPosix_Stat(const char *path, struct stat *buf)
 // Return the results of an open of a Unix file
 //
    return (!(myPath = XrootPath.URL(path, buff, sizeof(buff)))
-#ifdef __linux__
+#if defined(__linux__) and defined(_STAT_VER)
           ? Xunix.Stat64(_STAT_VER, path, (struct stat64 *)buf)
 #else
           ? Xunix.Stat64(           path, (struct stat64 *)buf)
