@@ -56,7 +56,7 @@ XrdCl::URL *XrdPosixAdmin::FanOut(int &num)
 //
    xStatus = Xrd.DeepLocate(Url.GetPathWithParams(),XrdCl::OpenFlags::None,info);
    if (!xStatus.IsOK())
-      {num = XrdPosixMap::Result(xStatus);
+      {num = XrdPosixMap::Result(xStatus, false);
        return 0;
       }
 
@@ -105,12 +105,17 @@ int XrdPosixAdmin::Query(XrdCl::QueryCode::Code reqCode, void *buff, int bsz)
 //
    if (!XrdPosixMap::Result(Xrd.Query(reqCode, reqBuff, rspBuff)))
       {uint32_t rspSz = rspBuff->GetSize();
-       if (bsz >= (int)rspSz)
-          {strcpy((char *)buff, rspBuff->GetBuffer());
-           delete rspBuff;
-           return static_cast<int>(rspSz);
-          }
-       errno = ERANGE;
+       char *rspbuff = rspBuff->GetBuffer();
+       if (rspbuff && rspSz)
+          {// if the string is null-terminated decrement the size
+           if ( !(rspbuff[rspSz - 1]) ) --rspSz;
+           if (bsz >= (int)rspSz + 1)
+              {strncpy((char *)buff, rspbuff, rspSz);
+               ((char*)buff)[rspSz] = 0; // make sure it is null-terminated
+               delete rspBuff;
+               return static_cast<int>(rspSz + 1);
+              } else errno = ERANGE;
+          } else errno = EFAULT;
       }
 
 // Return error
