@@ -28,29 +28,39 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <algorithm>
 #include <iostream>
+#include <memory>
+#include <set>
 #include <stdio.h>
+#include <string>
 
 #include "XrdCl/XrdClDefaultEnv.hh"
+#include "XrdCl/XrdClJobManager.hh"
+#include "XrdCl/XrdClPostMaster.hh"
+#include "XrdCl/XrdClURL.hh"
 
-#include "XrdOuc/XrdOucCache2.hh"
-#include "XrdOuc/XrdOucCacheDram.hh"
+#include "XrdOuc/XrdOucCache.hh"
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucPsx.hh"
 #include "XrdOuc/XrdOucTList.hh"
 
 #include "XrdPosix/XrdPosixCache.hh"
-#include "XrdPosix/XrdPosixCacheBC.hh"
 #include "XrdPosix/XrdPosixConfig.hh"
 #include "XrdPosix/XrdPosixFileRH.hh"
 #include "XrdPosix/XrdPosixInfo.hh"
 #include "XrdPosix/XrdPosixMap.hh"
 #include "XrdPosix/XrdPosixPrepIO.hh"
+#include "XrdPosix/XrdPosixStats.hh"
 #include "XrdPosix/XrdPosixTrace.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
 #include "XrdPosix/XrdPosixXrootdPath.hh"
 
+#include "XrdRmc/XrdRmc.hh"
+
+#include "XrdSecsss/XrdSecsssCon.hh"
+
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysTrace.hh"
 
@@ -61,19 +71,93 @@
 namespace XrdPosixGlobals
 {
 extern XrdScheduler              *schedP;
-extern XrdOucCache2              *theCache;
-extern XrdOucCache               *myCache;
-extern XrdOucCache2              *myCache2;
+extern XrdOucCache               *theCache;
 extern XrdOucName2Name           *theN2N;
-extern XrdCl::DirListFlags::Flags dlFlag;
 extern XrdSysLogger              *theLogger;
 extern XrdSysError               *eDest;
+extern XrdPosixStats              Stats;
 extern XrdSysTrace                Trace;
 extern int                        ddInterval;
 extern int                        ddMaxTries;
+extern XrdCl::DirListFlags::Flags dlFlag;
 extern bool                       oidsOK;
+extern bool                       p2lSRC;
+extern bool                       p2lSGI;
 };
   
+/******************************************************************************/
+/*                         L o c a l   C l a s s e s                          */
+/******************************************************************************/
+
+namespace
+{
+class ConCleanup : public XrdSecsssCon
+{
+public:
+
+using XrdSecsssCon::Contact;
+
+void Cleanup(const std::set<std::string> &Contacts, const XrdSecEntity &Entity)
+{
+   std::set<std::string>::iterator it;
+
+   for (it = Contacts.begin(); it != Contacts.end(); it++)
+       {if (Blab) DMSG("Cleanup", "Disconnecting " <<(*it).c_str());
+        PostMaster->ForceDisconnect(XrdCl::URL(*it));}
+}
+
+      ConCleanup(XrdCl::PostMaster *pm, bool dbg) : PostMaster(pm), Blab(dbg) {}
+     ~ConCleanup() {}
+
+private:
+
+XrdCl::PostMaster *PostMaster;
+bool               Blab;
+};
+
+class ConTrack : public XrdCl::Job
+{
+public:
+
+void Run( void *ptr )
+        {XrdCl::URL *url = reinterpret_cast<XrdCl::URL*>( ptr );
+         const std::string &user = url->GetUserName();
+         const std::string  host = url->GetHostId();
+         if (Blab) DMSG("Tracker", "Connecting to " <<host);
+         if (user.size()) sssCon.Contact(user, host);
+         delete url;
+        }
+
+      ConTrack(ConCleanup &cm, bool dbg) : sssCon(cm), Blab(dbg) {}
+     ~ConTrack() {}
+
+private:
+
+XrdSecsssCon &sssCon;
+bool          Blab;
+};
+}
+  
+/******************************************************************************/
+/*                            C o n T r a c k e r                             */
+/******************************************************************************/
+
+XrdSecsssCon *XrdPosixConfig::conTracker(bool dbg)
+{
+   XrdCl::PostMaster *pm =  XrdCl::DefaultEnv::GetPostMaster();
+   ConCleanup *cuHandler = new ConCleanup(pm, dbg);
+   std::unique_ptr<ConTrack> ctHandler(new ConTrack(*cuHandler, dbg));
+
+// Set the callback for new connections
+//
+   pm->SetOnConnectHandler( std::move( ctHandler ) );
+
+// Return the connection cleanup handler. Note that we split the task into
+// two objects so that we don't violate the semantics of unique pointer.
+//
+   return cuHandler;
+}
+
 /******************************************************************************/
 /*                               E n v I n f o                                */
 /******************************************************************************/
@@ -86,8 +170,8 @@ void XrdPosixConfig::EnvInfo(XrdOucEnv &theEnv)
    XrdPosixGlobals::schedP = (XrdScheduler *)theEnv.GetPtr("XrdScheduler*");
 
 // If we have a new-style cache, propogate the environment to it
-//
-   if (XrdPosixGlobals::myCache2) XrdPosixGlobals::myCache2->EnvInfo(theEnv);
+//??? Get rid of EnvInfo
+// if (XrdPosixGlobals::myCache2) XrdPosixGlobals::myCache2->EnvInfo(theEnv);
 }
 
 /******************************************************************************/
@@ -141,11 +225,10 @@ bool XrdPosixConfig::initCCM(XrdOucPsx &parms)
 
 void XrdPosixConfig::initEnv(char *eData)
 {
-   static XrdOucCacheDram dramCache;
+   static XrdRmc dramCache;
+   XrdRmc::Parms myParms;
    XrdOucEnv theEnv(eData);
-   XrdOucCache::Parms myParms;
    XrdOucCacheIO::aprParms apParms;
-   XrdOucCache *v1Cache;
    long long Val;
    char * tP;
 
@@ -174,7 +257,7 @@ void XrdPosixConfig::initEnv(char *eData)
 // Get Mode
 //
    if ((tP = theEnv.Get("mode")))
-      {if (*tP == 's') myParms.Options |= XrdOucCache::isServer;
+      {if (*tP == 's') myParms.Options |= XrdRmc::isServer;
           else if (*tP != 'c') DMSG("initEnv","'XRDPOSIX_CACHE=mode=" <<tP
                                     <<"' is invalid.");
       }
@@ -182,7 +265,7 @@ void XrdPosixConfig::initEnv(char *eData)
 // Get the structured file option
 //
    if ((tP = theEnv.Get("optsf")) && *tP && *tP != '0')
-      {     if (*tP == '1') myParms.Options |= XrdOucCache::isStructured;
+      {     if (*tP == '1') myParms.Options |= XrdRmc::isStructured;
        else if (*tP == '.') {XrdPosixFile::sfSFX = strdup(tP);
                              XrdPosixFile::sfSLN = strlen(tP);
                             }
@@ -193,22 +276,17 @@ void XrdPosixConfig::initEnv(char *eData)
 // Get final options, any non-zero value will do here
 //
    if ((tP = theEnv.Get("optlg")) && *tP && *tP != '0')
-      myParms.Options |= XrdOucCache::logStats;
+      myParms.Options |= XrdRmc::logStats;
    if ((tP = theEnv.Get("optpr")) && *tP && *tP != '0')
-      myParms.Options |= XrdOucCache::canPreRead;
+      myParms.Options |= XrdRmc::canPreRead;
 // if ((tP = theEnv.Get("optwr")) && *tP && *tP != '0') isRW = 1;
 
-// Use the default cache if one was not provided
-//
-   if (!XrdPosixGlobals::myCache) XrdPosixGlobals::myCache = &dramCache;
-
 // Now allocate a cache. Indicate that we already serialize the I/O to avoid
 // additional but unnecessary locking.
 //
-   myParms.Options |= XrdOucCache::Serialized;
-   if (!(v1Cache = XrdPosixGlobals::myCache->Create(myParms, &apParms)))
-      {DMSG("initEnv", strerror(errno) <<" creating cache.");}
-      else XrdPosixGlobals::theCache = new XrdPosixCacheBC(v1Cache);
+   myParms.Options |= XrdRmc::Serialized;
+   if (!(XrdPosixGlobals::theCache = dramCache.Create(myParms, &apParms)))
+      {DMSG("initEnv", XrdSysE2T(errno) <<" creating cache.");}
 }
 
 /******************************************************************************/
@@ -288,7 +366,13 @@ bool XrdPosixConfig::SetConfig(XrdOucPsx &parms)
 
 // Handle the Name2Name for pfn2lfn translations.
 //
-   if (parms.xPfn2Lfn) XrdPosixGlobals::theN2N = parms.theN2N;
+   if (parms.xPfn2Lfn)
+      {XrdPosixGlobals::theN2N = parms.theN2N;
+       if (parms.xPfn2Lfn == parms.xP2Lsrc || parms.xPfn2Lfn == parms.xP2Lsgi)
+          {XrdPosixGlobals::p2lSRC = true;
+           XrdPosixGlobals::p2lSGI = parms.xPfn2Lfn == parms.xP2Lsgi;
+          } else XrdPosixGlobals::p2lSRC = XrdPosixGlobals::p2lSGI = false;
+      }
 
 // Handle client settings
 //
@@ -316,19 +400,14 @@ bool XrdPosixConfig::SetConfig(XrdOucPsx &parms)
        XrdPosixGlobals::ddInterval = (parms.cioWait  < 10 ? 10 : parms.cioWait);
       }
 
-// Handle the caching options
+// Handle the caching options (library or builin memory).
+// TODO: Make the memory cache a library plugin as well.
 //
-        if (parms.theCache2)
-           {XrdPosixGlobals::myCache2 = parms.theCache2;
-            XrdPosixGlobals::theCache = parms.theCache2;
+        if (parms.theCache)
+           {XrdPosixGlobals::theCache = parms.theCache;
             if (parms.initCCM) return initCCM(parms);
             return true;
            }
-   else if (parms.theCache)
-           {char ebuf[] = {0};
-            XrdPosixGlobals::myCache  = parms.theCache;
-            initEnv(ebuf);
-           }
    else if (parms.mCache && *parms.mCache) initEnv(parms.mCache);
 
    return true;
@@ -400,3 +479,90 @@ void XrdPosixConfig::setOids(bool isok)
 {
     XrdPosixGlobals::oidsOK = isok;
 }
+
+/******************************************************************************/
+/*                            S t a t s C a c h e                             */
+/******************************************************************************/
+  
+int XrdPosixConfig::Stats(const char *theID, char *buff, int blen)
+{
+   static const char stats1[] = "<stats id=\"%s\">"
+          "<open>%lld<errs>%lld</errs></open>"
+          "<close>%lld<errs>%lld</errs></close>"
+          "</stats>";
+
+   static const char stats2[] = "<stats id=\"cache\" type=\"%s\">"
+          "<prerd><in>%lld</in><hits>%lld</hits><miss>%lld</miss></prerd>"
+          "<rd><in>%lld</in><out>%lld</out>"
+              "<hits>%lld></hits><miss>%lld</miss>"
+          "</rd>"
+          "<pass>%lld<cnt>%lld</cnt></pass>"
+          "<wr><out>%lld</out><updt>%lld</updt></wr>"
+          "<saved>%lld</saved><purge>%lld</purge>"
+          "<files><opened>%lld</opened><closed>%lld</closed><new>%lld</new>"
+                  "<del>%lld</del><now>%lld</now><full>%lld</full>"
+          "</files>"
+          "<store><size>%lld</size><used>%lld</used>"
+                  "<min>%lld</min><max>%lld</max>"
+          "</store>"
+          "<mem><size>%lld</size><used>%lld</used><wq>%lld</wq></mem>"
+          "<opcl><odefer>%lld</odefer><defero>%lld</defero>"
+                "<cdefer>%lld</cdefer><clost>%lld</clost>"
+          "</opcl>"
+          "</stats>";
+
+// If the caller want the maximum length, then provide it.
+//
+   if (!blen)
+      {size_t n;
+       int len1, digitsLL = strlen("9223372036854775807");
+       std::string fmt = stats1;
+       n = std::count(fmt.begin(), fmt.end(), '%');
+       len1 = fmt.size() + (digitsLL*n) - (n*3) + strlen(theID);
+       if (!XrdPosixGlobals::theCache) return len1;
+       fmt = stats2;
+       n = std::count(fmt.begin(), fmt.end(), '%');
+       return len1 + fmt.size() + (digitsLL*n) - (n*3) + 8;
+      }
+
+// Get the standard statistics
+//
+   XrdPosixStats Y;
+   XrdPosixGlobals::Stats.Get(Y);
+
+// Format the line
+//
+   int k = snprintf(buff, blen, stats1, theID,
+                    Y.X.Opens, Y.X.OpenErrs, Y.X.Closes, Y.X.CloseErrs);
+
+// If there is no cache then there nothing to return
+//
+   if (!XrdPosixGlobals::theCache) return k;
+   buff += k; blen -= k;
+
+// Get the statistics
+//
+   XrdOucCacheStats Z;
+   XrdPosixGlobals::theCache->Statistics.Get(Z);
+
+// Format the statisics into the supplied buffer
+//
+   int n = snprintf(buff, blen, stats2, XrdPosixGlobals::theCache->CacheType,
+                    Z.X.BytesPead,   Z.X.HitsPR,       Z.X.MissPR,
+                    Z.X.BytesRead,   Z.X.BytesGet,     Z.X.Hits, Z.X.Miss,
+                    Z.X.BytesPass,   Z.X.Pass,
+                    Z.X.BytesWrite,  Z.X.BytesPut,
+                    Z.X.BytesSaved,  Z.X.BytesPurged,
+                    Z.X.FilesOpened, Z.X.FilesClosed,  Z.X.FilesCreated,
+                    Z.X.FilesPurged, Z.X.FilesInCache, Z.X.FilesAreFull,
+                    Z.X.DiskSize,    Z.X.DiskUsed,
+                    Z.X.DiskMin,     Z.X.DiskMax,
+                    Z.X.MemSize,     Z.X.MemUsed,      Z.X.MemWriteQ,
+                    Z.X.OpenDefers,  Z.X.DeferOpens,
+                    Z.X.ClosDefers,  Z.X.ClosedLost
+                   );
+
+// Return the right value
+//
+   return (n < blen ? n+k : 0);
+}
