@@ -42,6 +42,7 @@
 #include "XrdPosix/XrdPosixFile.hh"
 #include "XrdPosix/XrdPosixFileRH.hh"
 #include "XrdPosix/XrdPosixPrepIO.hh"
+#include "XrdPosix/XrdPosixStats.hh"
 #include "XrdPosix/XrdPosixTrace.hh"
 #include "XrdPosix/XrdPosixXrootdPath.hh"
 
@@ -54,12 +55,12 @@
 
 namespace XrdPosixGlobals
 {
-extern XrdOucCache2    *theCache;
+extern XrdOucCache     *theCache;
 extern XrdOucName2Name *theN2N;
 extern XrdSysError     *eDest;
+extern XrdPosixStats    Stats;
 extern int              ddInterval;
 extern int              ddMaxTries;
-       int              ddNumLost = 0;
 };
 
 namespace
@@ -90,7 +91,7 @@ int            XrdPosixFile::ddNum    =  0;
 
 XrdPosixFile::XrdPosixFile(bool &aOK, const char *path, XrdPosixCallBack *cbP,
                            int Opts)
-             : XCio((XrdOucCacheIO2 *)this), PrepIO(0),
+             : XCio((XrdOucCacheIO *)this), PrepIO(0),
                mySize(0), myMtime(0), myInode(0), myMode(0),
                theCB(cbP), fLoc(0), cOpt(0),
                isStream(Opts & isStrm ? 1 : 0)
@@ -122,13 +123,14 @@ XrdPosixFile::XrdPosixFile(bool &aOK, const char *path, XrdPosixCallBack *cbP,
   
 XrdPosixFile::~XrdPosixFile()
 {
-// Detach the cache if it is attached
-//
-   if (XCio != this) XCio->Detach();
-
 // Close the remote connection
 //
-   if (clFile.IsOpen()) {XrdCl::XRootDStatus status = clFile.Close();};
+   if (clFile.IsOpen())
+      {XrdPosixGlobals::Stats.Count(XrdPosixGlobals::Stats.X.Closes);
+       XrdCl::XRootDStatus status = clFile.Close();
+       if (!status.IsOK())
+          XrdPosixGlobals::Stats.Count(XrdPosixGlobals::Stats.X.CloseErrs);
+      }
 
 // Get rid of defered open object
 //
@@ -148,17 +150,20 @@ XrdPosixFile::~XrdPosixFile()
 void* XrdPosixFile::DelayedDestroy(void* vpf)
 {
 // Static function.
-// Called within a dedicated thread if XrdOucCacheIO is io-active or the
-// file cannot be closed in a clean fashion for some reason.
+// Called within a dedicated thread if there is a reference outstanding to the
+// file or the file cannot be closed in a clean fashion for some reason.
 //
    EPNAME("DDestroy");
 
+   XrdSysError *Say = XrdPosixGlobals::eDest;
    XrdCl::XRootDStatus Status;
    std::string statusMsg;
    const char *eTxt;
    XrdPosixFile *fCurr, *fNext;
-   int ddCount;
-   bool ioActive, doWait = false;
+   char buff[512], buff2[256];
+   static int ddNumLost = 0;
+   int ddCount, refNum;
+   bool doWait = false;
 
 // Wait for active I/O to complete
 //
@@ -179,36 +184,37 @@ do{if (doWait)
 
 // Do some debugging
 //
-   DEBUG("DLY destory of "<<ddCount<<" objects; "<<XrdPosixGlobals::ddNumLost
-         <<" already lost.");
+   DEBUG("DLY destory of "<<ddCount<<" objects; "<<ddNumLost <<" already lost.");
 
 // Try to delete all the files on the list. If we exceeded the try limit,
 // remove the file from the list and let it sit forever.
 //
+   int nowLost = ddNumLost;
    while((fCurr = fNext))
         {fNext = fCurr->nextFile;
-         if (!((ioActive = fCurr->XCio->ioActive())) && !fCurr->Refs())
-            {if (fCurr->Close(Status)) {delete fCurr; ddCount--; continue;}
+         if (!(refNum = fCurr->Refs()))
+            {if (fCurr->Close(Status) || !fCurr->clFile.IsOpen())
+                {delete fCurr; ddCount--; continue;}
                 else {statusMsg = Status.ToString();
                       eTxt = statusMsg.c_str();
                      }
-            } else    eTxt = (ioActive ? "active I/O" : "callback");
+            } else eTxt = 0;
 
          if (fCurr->numTries > XrdPosixGlobals::ddMaxTries)
-            {XrdPosixGlobals::ddNumLost++; ddCount--;
-             if (XrdPosixGlobals::eDest)
-                {char buff[256];
-                 snprintf(buff, sizeof(buff), "(%d) %s",
-                                XrdPosixGlobals:: ddNumLost, eTxt);
-                 XrdPosixGlobals::eDest->Emsg("DDestroy",
-                                         "timeout closing", fCurr->Origin());
+            {ddNumLost++; ddCount--;
+             if (!eTxt)
+                {snprintf(buff2, sizeof(buff2), "in use %d", refNum);
+                 eTxt = buff2;
+                }
+             if (Say)
+                {snprintf(buff, sizeof(buff), "%s timeout closing", eTxt);
+                 Say->Emsg("DDestroy", buff, fCurr->Origin());
                 } else {
                  DMSG("DDestroy", eTxt <<" timeout closing " <<fCurr->Origin()
-                        <<' ' <<XrdPosixGlobals::ddNumLost <<" objects lost");
+                        <<' ' <<ddNumLost <<" objects lost");
                 }
              fCurr->nextFile = ddLost;
              ddLost = fCurr;
-             fCurr->Close(Status);
             } else {
              fCurr->numTries++;
              doWait = true;
@@ -218,7 +224,17 @@ do{if (doWait)
              ddMutex.UnLock();
             }
         }
-        DEBUG("DLY destory end; "<<ddCount<<" objects deferred.");
+        if (Say && ddNumLost - nowLost >= 3)
+           {snprintf(buff, sizeof(buff), "%d objects deferred and %d lost.",
+                                         ddCount,  ddNumLost);
+            Say->Emsg("DDestroy", buff);
+           } else {
+            DEBUG("DLY destory end; "<<ddCount<<" objects deferred and "
+                             <<ddNumLost <<" lost.");
+           }
+        if (XrdPosixGlobals::theCache && ddNumLost != nowLost)
+           XrdPosixGlobals::theCache->Statistics.Set(
+          (XrdPosixGlobals::theCache->Statistics.X.ClosedLost), ddNumLost);
    } while(true);
 
    return 0;
@@ -232,6 +248,12 @@ void XrdPosixFile::DelayedDestroy(XrdPosixFile *fp)
    int  ddCount;
    bool doPost;
 
+// Count number of times this has happened (we should have a cache)
+//
+   if (XrdPosixGlobals::theCache)
+       XrdPosixGlobals::theCache->Statistics.Count(
+         (XrdPosixGlobals::theCache->Statistics.X.ClosDefers));
+
 // Place this file on the delayed delete list
 //
    ddMutex.Lock();
@@ -242,8 +264,8 @@ void XrdPosixFile::DelayedDestroy(XrdPosixFile *fp)
       else {doPost   = true;
             ddPosted = true;
            }
-   ddMutex.UnLock();
    fp->numTries = 0;
+   ddMutex.UnLock();
 
    DEBUG("DLY destory "<<(doPost ? "post " : "has ")<<ddCount
                        <<" objects; added "<<fp->Origin());
@@ -267,8 +289,11 @@ bool XrdPosixFile::Close(XrdCl::XRootDStatus &Status)
 // from the file table at this point and should be unlocked.
 //
    if (clFile.IsOpen())
-      {Status = clFile.Close();
-       return Status.IsOK();
+      {XrdPosixGlobals::Stats.Count(XrdPosixGlobals::Stats.X.Closes);
+       Status = clFile.Close();
+       if (Status.IsOK()) return true;
+       XrdPosixGlobals::Stats.Count(XrdPosixGlobals::Stats.X.CloseErrs);
+       return false;
       }
    return true;
 }
@@ -279,7 +304,7 @@ bool XrdPosixFile::Close(XrdCl::XRootDStatus &Status)
 
 bool XrdPosixFile::Finalize(XrdCl::XRootDStatus *Status)
 {
-   XrdOucCacheIO2 *ioP;
+   XrdOucCacheIO *ioP;
 
 // Indicate that we are at the start of the file
 //
@@ -288,14 +313,18 @@ bool XrdPosixFile::Finalize(XrdCl::XRootDStatus *Status)
 // Complete initialization. If the stat() fails, the caller will unwind the
 // whole open process (ick). In the process get correct I/O vector.
 
-        if (!Status)       ioP = (XrdOucCacheIO2 *)PrepIO;
-   else if (Stat(*Status)) ioP = (XrdOucCacheIO2 *)this;
+        if (!Status)       ioP = (XrdOucCacheIO *)PrepIO;
+   else if (Stat(*Status)) ioP = (XrdOucCacheIO *)this;
    else return false;
 
 // Setup the cache if it is to be used
 //
    if (XrdPosixGlobals::theCache)
-      XCio = XrdPosixGlobals::theCache->Attach(ioP, cOpt);
+      {XCio = XrdPosixGlobals::theCache->Attach(ioP, cOpt);
+       if (ioP == (XrdOucCacheIO *)PrepIO)
+       XrdPosixGlobals::theCache->Statistics.Add(
+         (XrdPosixGlobals::theCache->Statistics.X.OpenDefers), 1LL);
+      }
 
    return true;
 }
@@ -364,7 +393,7 @@ const char *XrdPosixFile::Location()
 
 // If the file is not open, then we have no location
 //
-   if (!clFile.IsOpen()) return 0;
+   if (!clFile.IsOpen()) return "";
 
 // If we have no location info, get it
 //
