@@ -31,6 +31,7 @@
 #include <sys/types.h>
 #include <stdarg.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include "XrdPosix/XrdPosixLinkage.hh"
 #include "XrdPosix/XrdPosixOsDep.hh"
@@ -46,6 +47,8 @@
 /******************************************************************************/
   
 extern XrdPosixLinkage Xunix;
+
+namespace {bool isLite = (getenv("XRD_POSIX_PRELOAD_LITE") != 0);}
   
 /******************************************************************************/
 /*                                a c c e s s                                 */
@@ -87,7 +90,7 @@ int     chdir(const char *path)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Chdir(path);
+   return (isLite ? Xunix.Chdir(path) : XrdPosix_Chdir(path));
 }
 }
 
@@ -115,7 +118,7 @@ int     closedir(DIR *dirp)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Closedir(dirp);
+   return (isLite ? Xunix.Closedir(dirp) : XrdPosix_Closedir(dirp));
 }
 }
 
@@ -258,7 +261,7 @@ int fseeko64(FILE *stream, off64_t offset, int whence)
 
 extern "C"
 {
-#if defined __linux__ && __GNUC__ && __GNUC__ >= 2
+#if defined(__linux__) and defined(_STAT_VER) and __GNUC__ and __GNUC__ >= 2
 int  __fxstat64(int ver, int fildes, struct stat64 *buf)
 #else
 int     fstat64(         int fildes, struct stat64 *buf)
@@ -266,7 +269,7 @@ int     fstat64(         int fildes, struct stat64 *buf)
 {
    static int Init = Xunix.Init(&Init);
 
-#ifdef __linux__
+#if defined(__linux__) and defined(_STAT_VER)
    return XrdPosix_FstatV(ver, fildes, (struct stat *)buf);
 #else
    return XrdPosix_Fstat (     fildes, (struct stat *)buf);
@@ -452,7 +455,7 @@ int     mkdir(const char *path, mode_t mode)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Mkdir(path, mode);
+   return (isLite ? Xunix.Mkdir(path, mode) : XrdPosix_Mkdir(path, mode));
 }
 }
 
@@ -485,7 +488,7 @@ DIR*    opendir(const char *path)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Opendir(path);
+   return (isLite ? Xunix.Opendir(path) : XrdPosix_Opendir(path));
 }
 }
   
@@ -571,7 +574,7 @@ struct dirent64* readdir64(DIR *dirp)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Readdir64(dirp);
+   return (isLite ? Xunix.Readdir64(dirp) : XrdPosix_Readdir64(dirp));
 }
 }
 
@@ -585,7 +588,8 @@ int     readdir64_r(DIR *dirp, struct dirent64 *entry, struct dirent64 **result)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Readdir64_r(dirp, entry, result);
+   return (isLite ? Xunix.Readdir64_r(dirp, entry, result) :
+                                             XrdPosix_Readdir64_r(dirp, entry, result));
 }
 }
 
@@ -599,7 +603,7 @@ int     rename(const char *oldpath, const char *newpath)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Rename(oldpath, newpath);
+   return (isLite ? Xunix.Rename(oldpath, newpath) : XrdPosix_Rename(oldpath, newpath));
 }
 }
 
@@ -614,7 +618,7 @@ void    rewinddir(DIR *dirp)
 {
    static int Init = Xunix.Init(&Init);
 
-   XrdPosix_Rewinddir(dirp);
+   (isLite ? Xunix.Rewinddir(dirp) : XrdPosix_Rewinddir(dirp));
 }
 }
 #endif
@@ -629,7 +633,7 @@ int     rmdir(const char *path)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Rmdir(path);
+   return (isLite ? Xunix.Rmdir(path) : XrdPosix_Rmdir(path));
 }
 }
 
@@ -643,7 +647,7 @@ void    seekdir(DIR *dirp, long loc)
 {
    static int Init = Xunix.Init(&Init);
 
-   XrdPosix_Seekdir(dirp, loc);
+   (isLite ? Xunix.Seekdir(dirp, loc) : XrdPosix_Seekdir(dirp, loc));
 }
 }
 
@@ -705,7 +709,7 @@ long    telldir(DIR *dirp)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Telldir(dirp);
+   return (isLite ? Xunix.Telldir(dirp) : XrdPosix_Telldir(dirp));
 }
 }
   
@@ -733,7 +737,7 @@ int     unlink(const char *path)
 {
    static int Init = Xunix.Init(&Init);
 
-   return XrdPosix_Unlink(path);
+   return (isLite ? Xunix.Unlink(path) : XrdPosix_Unlink(path));
 }
 }
 
