@@ -50,21 +50,21 @@
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 
-#include "XrdOuc/XrdOucCache2.hh"
-#include "XrdOuc/XrdOucCacheDram.hh"
+#include "XrdOuc/XrdOucCache.hh"
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucName2Name.hh"
 #include "XrdOuc/XrdOucPsx.hh"
 
 #include "XrdPosix/XrdPosixAdmin.hh"
-#include "XrdPosix/XrdPosixCacheBC.hh"
 #include "XrdPosix/XrdPosixCallBack.hh"
 #include "XrdPosix/XrdPosixConfig.hh"
 #include "XrdPosix/XrdPosixDir.hh"
 #include "XrdPosix/XrdPosixFile.hh"
 #include "XrdPosix/XrdPosixFileRH.hh"
+#include "XrdPosix/XrdPosixInfo.hh"
 #include "XrdPosix/XrdPosixMap.hh"
 #include "XrdPosix/XrdPosixPrepIO.hh"
+#include "XrdPosix/XrdPosixStats.hh"
 #include "XrdPosix/XrdPosixTrace.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
 #include "XrdPosix/XrdPosixXrootdPath.hh"
@@ -75,20 +75,24 @@
 /*                        S t a t i c   M e m b e r s                         */
 /******************************************************************************/
 
+class XrdSysError;
+
 namespace XrdPosixGlobals
 {
 XrdScheduler    *schedP    = 0;
-XrdOucCache2    *theCache  = 0;
-XrdOucCache     *myCache   = 0;
-XrdOucCache2    *myCache2  = 0;
+XrdOucCache     *theCache  = 0;
 XrdOucName2Name *theN2N    = 0;
-XrdCl::DirListFlags::Flags dlFlag = XrdCl::DirListFlags::None;
 XrdSysLogger    *theLogger = 0;
+XrdSysError     *eDest     = 0;
+XrdPosixStats    Stats;
 XrdSysTrace      Trace("Posix", 0,
                       (getenv("XRDPOSIX_DEBUG") ? TRACE_Debug : 0));
 int              ddInterval= 30;
 int              ddMaxTries= 180/30;
+XrdCl::DirListFlags::Flags dlFlag = XrdCl::DirListFlags::None;
 bool             oidsOK    = false;
+bool             p2lSRC    = false;
+bool             p2lSGI    = false;
 };
 
 int            XrdPosixXrootd::baseFD    = 0;
@@ -190,12 +194,12 @@ XrdPosixXrootd::XrdPosixXrootd(int fdnum, int dirnum, int thrnum)
           else hush = (getenv("XRDPOSIX_DEBUG") == 0);
        if (*cfn)
           {XrdOucPsx psxConfig(&XrdVERSIONINFOVAR(XrdPosix), cfn);
-           if (psxConfig.ClientConfig("posix.", hush))
-              XrdPosixConfig::SetConfig(psxConfig);
-              else {std::cerr <<"Posix: Unable to instantiate specified "
+           if (!psxConfig.ClientConfig("posix.", hush)
+           ||  !XrdPosixConfig::SetConfig(psxConfig))
+              {std::cerr <<"Posix: Unable to instantiate specified "
                            "configuration; program exiting!" <<std::endl;
-                   exit(16);
-                   }
+               exit(16);
+              }
           }
       }
 
@@ -261,13 +265,21 @@ int XrdPosixXrootd::Close(int fildes)
    if (!(fP = XrdPosixObject::ReleaseFile(fildes)))
       {errno = EBADF; return -1;}
 
-// Close the file if there is no active I/O (possible caching). Delete the
-// object if the close was successful (it might not be).
+// Detach the file from a possible cache. We need to up the reference count
+// to synchrnoize with any possible callback as we may need to place this
+// object in he delayed destroy queue if it is stil being used. Note that
+// the caller will get a zero return code should we delay the close.
 //
-   if (!fP->Refs() && !(fP->XCio->ioActive()))
+   fP->Ref();
+   if (fP->XCio->Detach((XrdOucCacheIOCD&)*fP) && fP->Refs() < 2)
       {if ((ret = fP->Close(Status))) {delete fP; fP = 0;}
-          else {DEBUG(Status.ToString().c_str() <<" closing " <<fP->Origin());}
-      } else ret = true;
+          else if (DEBUGON)
+                  {std::string eTxt = Status.ToString();
+                   DEBUG(eTxt <<" closing " <<fP->Origin());
+                  }
+      } else {
+       ret = true;
+      }
 
 // If we still have a handle then we need to do a delayed delete on this
 // object because either the close failed or there is still active I/O
@@ -380,6 +392,7 @@ int     XrdPosixXrootd::Fstat(int fildes, struct stat *buf)
 int XrdPosixXrootd::Fsync(int fildes)
 {
    XrdPosixFile *fp;
+   int rc;
 
 // Find the file object
 //
@@ -387,7 +400,7 @@ int XrdPosixXrootd::Fsync(int fildes)
 
 // Do the sync
 //
-   if (fp->XCio->Sync() < 0) return Fault(fp, errno);
+   if ((rc = fp->XCio->Sync()) < 0) return Fault(fp, -rc);
    fp->UnLock();
    return 0;
 }
@@ -414,6 +427,7 @@ void XrdPosixXrootd::Fsync(int fildes, XrdPosixCallBackIO *cbp)
 int XrdPosixXrootd::Ftruncate(int fildes, off_t offset)
 {
    XrdPosixFile *fp;
+   int rc;
 
 // Find the file object
 //
@@ -421,7 +435,7 @@ int XrdPosixXrootd::Ftruncate(int fildes, off_t offset)
 
 // Do the trunc
 //
-   if (fp->XCio->Trunc(offset) < 0) return Fault(fp, errno);
+   if ((rc = fp->XCio->Trunc(offset)) < 0) return Fault(fp, -rc);
    fp->UnLock();
    return 0;
 }
@@ -526,6 +540,12 @@ int XrdPosixXrootd::Mkdir(const char *path, mode_t mode)
   
 int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
                          XrdPosixCallBack *cbP)
+{
+    return Open(path, oflags, mode, cbP, 0);
+}
+  
+int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
+                         XrdPosixCallBack *cbP, XrdPosixInfo *infoP)
 {
    EPNAME("Open");
    XrdCl::XRootDStatus Status;
@@ -533,7 +553,7 @@ int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
    XrdCl::Access::Mode     XOmode = XrdCl::Access::None;
    XrdCl::OpenFlags::Flags XOflags;
    int Opts;
-   bool aOK;
+   bool aOK, isRO = false;
 
 // Translate R/W and R/O flags
 //
@@ -543,6 +563,7 @@ int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
       } else {
        Opts    = 0;
        XOflags = XrdCl::OpenFlags::Read;
+       isRO    = true;
       }
 
 // Pass along the stream flag
@@ -577,14 +598,21 @@ int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
 // If we have a cache, then issue a prepare as the cache may want to defer the
 // open request ans we have a lot more work to do.
 //
-   if (XrdPosixGlobals::myCache2)
-      {int rc = XrdPosixGlobals::myCache2->Prepare(path, oflags, mode);
+   if (XrdPosixGlobals::theCache)
+      {int rc;
+       if (infoP && isRO && OpenCache(*fp, *infoP))
+          {delete fp;
+           errno = 0;
+           return -3;
+          }
+       rc = XrdPosixGlobals::theCache->Prepare(fp->Path(), oflags, mode);
        if (rc > 0) return OpenDefer(fp, cbP, XOflags, XOmode, oflags&isStream);
        if (rc < 0) {delete fp; errno = -rc; return -1;}
       }
 
 // Open the file (sync or async)
 //
+   XrdPosixGlobals::Stats.Count((XrdPosixGlobals::Stats.X.Opens));
    if (!cbP) Status = fp->clFile.Open((std::string)path, XOflags, XOmode);
       else   Status = fp->clFile.Open((std::string)path, XOflags, XOmode,
                                       (XrdCl::ResponseHandler *)fp);
@@ -592,10 +620,13 @@ int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
 // If we failed, return the reason
 //
    if (!Status.IsOK())
-      {XrdPosixMap::Result(Status);
+      {XrdPosixGlobals::Stats.Count(XrdPosixGlobals::Stats.X.OpenErrs);
+       XrdPosixMap::Result(Status);
        int rc = errno;
-       if (rc != ENOENT && rc != ELOOP)
-          {DEBUG(Status.ToString().c_str() <<" open " <<fp->Origin());}
+       if (DEBUGON && rc != ENOENT && rc != ELOOP)
+          {std::string eTxt = Status.ToString();
+           DEBUG(eTxt <<" open " <<fp->Origin());
+          }
        delete fp;
        errno = rc;
        return -1;
@@ -617,6 +648,33 @@ int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
    return XrdPosixMap::Result(Status);
 }
   
+/******************************************************************************/
+/* Private:                    O p e n C a c h e                              */
+/******************************************************************************/
+  
+bool XrdPosixXrootd::OpenCache(XrdPosixFile &file,XrdPosixInfo &Info)
+{
+   EPNAME("OpenCache");
+   int rc;
+
+// Check if the full file is in the cache
+//
+   rc = XrdPosixGlobals::theCache->LocalFilePath(file.Path(), Info.cachePath,
+                                                 (int)sizeof(Info.cachePath),
+                                                 XrdOucCache::ForAccess,
+                                                 Info.ffReady);
+   if (rc == 0)
+      {Info.ffReady  = true;
+       DEBUG("File in cache url=" <<Info.cacheURL);
+       return true;
+      }
+
+// File is not fully in the cache
+//
+   Info.ffReady = false;
+   return false;
+}
+  
 /******************************************************************************/
 /*                               O p e n d i r                                */
 /******************************************************************************/
@@ -675,7 +733,7 @@ ssize_t XrdPosixXrootd::Pread(int fildes, void *buf, size_t nbyte, off_t offset)
 //
    offs = static_cast<long long>(offset);
    bytes = fp->XCio->Read((char *)buf, offs, (int)iosz);
-   if (bytes < 0) return Fault(fp,errno);
+   if (bytes < 0) return Fault(fp,-bytes);
 
 // All went well
 //
@@ -740,7 +798,7 @@ ssize_t XrdPosixXrootd::Pwrite(int fildes, const void *buf, size_t nbyte, off_t
 //
    offs = static_cast<long long>(offset);
    bytes = fp->XCio->Write((char *)buf, offs, (int)iosz);
-   if (bytes < 0) return Fault(fp, errno);
+   if (bytes < 0) return Fault(fp,-bytes);
 
 // All went well
 //
@@ -805,7 +863,7 @@ ssize_t XrdPosixXrootd::Read(int fildes, void *buf, size_t nbyte)
 // Issue the read
 //
    bytes = fp->XCio->Read((char *)buf,fp->Offset(),(int)iosz);
-   if (bytes < 0) return Fault(fp, errno);
+   if (bytes < 0) return Fault(fp,-bytes);
 
 // All went well
 //
@@ -852,7 +910,7 @@ ssize_t XrdPosixXrootd::VRead(int fildes, const XrdOucIOVec *readV, int n)
 
 // Issue the read
 //
-   if ((bytes = fp->XCio->ReadV(readV, n)) < 0) return Fault(fp, errno);
+   if ((bytes = fp->XCio->ReadV(readV, n)) < 0) return Fault(fp,-bytes);
 
 // Return bytes read
 //
@@ -1088,10 +1146,10 @@ int XrdPosixXrootd::Stat(const char *path, struct stat *buf)
 
 // Check if we can get the stat informatation from the cache
 //
-  if (XrdPosixGlobals::myCache2)
+  if (XrdPosixGlobals::theCache)
      {LfnPath statX("stat", path, false);
       if (!statX.path) return -1;
-      int rc = XrdPosixGlobals::myCache2->Stat(statX.path, *buf);
+      int rc = XrdPosixGlobals::theCache->Stat(statX.path, *buf);
       if (!rc) return 0;
       if (rc < 0) {errno = -rc; return -1;}
      }
@@ -1308,7 +1366,7 @@ ssize_t XrdPosixXrootd::Write(int fildes, const void *buf, size_t nbyte)
 // Issue the write
 //
    bytes = fp->XCio->Write((char *)buf,fp->Offset(),(int)iosz);
-   if (bytes < 0) return Fault(fp, errno);
+   if (bytes < 0) return Fault(fp,-bytes);
 
 // All went well
 //
@@ -1400,109 +1458,6 @@ long long XrdPosixXrootd::QueryOpaque(const char *path, char *value, int size)
 //
    return admin.Query(XrdCl::QueryCode::OpaqueFile, value, size);
 }
-
-/******************************************************************************/
-/* Obsolete!                    s e t C a c h e                               */
-/******************************************************************************/
-
-void XrdPosixXrootd::setCache(XrdOucCache  *cP) {XrdPosixGlobals::myCache =cP;}
-
-void XrdPosixXrootd::setCache(XrdOucCache2 *cP) {XrdPosixGlobals::myCache2=cP;}
-  
-/******************************************************************************/
-/* Obsolete!                    s e t D e b u g                               */
-/******************************************************************************/
-
-void XrdPosixXrootd::setDebug(int val, bool doDebug)
-{
-   const std::string dbgType[] = {"Info", "Warning", "Error", "Debug", "Dump"};
-
-// The default is none but once set it cannot be unset in the client
-//
-   if (val > 0)
-      {if (doDebug) val = 4;
-          else if (val > 5) val = 5;
-       XrdCl::DefaultEnv::SetLogLevel(dbgType[val-1]);
-      }
-
-// Now set the internal one which can be toggled
-//
-   XrdPosixMap::SetDebug(val > 0);
-}
-  
-/******************************************************************************/
-/* Obsolete!                      s e t E n v                                 */
-/******************************************************************************/
-
-void XrdPosixXrootd::setEnv(const char *kword, int kval)
-{
-   XrdCl::Env *env = XrdCl::DefaultEnv::GetEnv();
-   static bool dlfSet = false;
-
-// Check for internal envars before setting the external one
-//
-        if (!strcmp(kword, "DirlistAll"))
-           {XrdPosixGlobals::dlFlag = (kval ? XrdCl::DirListFlags::Locate
-                                            : XrdCl::DirListFlags::None);
-            dlfSet = true;
-           }
-   else if (!strcmp(kword, "DirlistDflt"))
-           {if (!dlfSet)
-            XrdPosixGlobals::dlFlag = (kval ? XrdCl::DirListFlags::Locate
-                                            : XrdCl::DirListFlags::None);
-           }
-   else env->PutInt((std::string)kword, kval);
-}
-  
-/******************************************************************************/
-/* Obsolete!                     s e t I P V 4                                */
-/******************************************************************************/
-
-void XrdPosixXrootd::setIPV4(bool usev4)
-{
-   const char *ipmode = (usev4 ? "IPv4" : "IPAll");
-   XrdCl::Env *env = XrdCl::DefaultEnv::GetEnv();
-
-// Set the env value
-//
-   env->PutString((std::string)"NetworkStack", (const std::string)ipmode);
-}
-  
-/******************************************************************************/
-/* Obsolete!                   s e t L o g g e r                              */
-/******************************************************************************/
-
-void XrdPosixXrootd::setLogger(XrdSysLogger *logP)
-{
-    XrdPosixGlobals::Trace.SetLogger(logP);
-}
-  
-/******************************************************************************/
-/* Obsolete!                    s e t N u m C B                               */
-/******************************************************************************/
-
-void XrdPosixXrootd::setNumCB(int numcb)
-{
-    if (numcb >= 0) XrdPosixFileRH::SetMax(numcb);
-}
-  
-/******************************************************************************/
-/* Obsolete!                      S e t N 2 N                                 */
-/******************************************************************************/
-
-void XrdPosixXrootd::setN2N(XrdOucName2Name *pN2N, int opts)
-{
-    XrdPosixGlobals::theN2N = pN2N;
-}
-  
-/******************************************************************************/
-/* Obsolete!                    s e t S c h e d                               */
-/******************************************************************************/
-
-void XrdPosixXrootd::setSched(XrdScheduler *sP)
-{
-    XrdPosixGlobals::schedP = sP;
-}
   
 /******************************************************************************/
 /*                       P r i v a t e   M e t h o d s                        */
