@@ -47,11 +47,11 @@
 
 #include "XrdVersion.hh"
 
-#include "XrdFfs/XrdFfsPosix.hh"
 #include "XrdNet/XrdNetSecurity.hh"
 #include "XrdPss/XrdPss.hh"
 #include "XrdPss/XrdPssTrace.hh"
 #include "XrdPss/XrdPssUrlInfo.hh"
+#include "XrdPss/XrdPssUtils.hh"
 #include "XrdPosix/XrdPosixConfig.hh"
 #include "XrdPosix/XrdPosixInfo.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
@@ -60,6 +60,7 @@
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucExport.hh"
 #include "XrdSec/XrdSecEntity.hh"
+#include "XrdSecsss/XrdSecsssID.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysPlatform.hh"
@@ -92,11 +93,21 @@ static XrdPssSys   XrdProxySS;
 
        XrdOucSid    *sidP   = 0;
 
-static const char *ofslclCGI = "ofs.lcl=1";
+       XrdOucEnv    *envP   = 0;
 
-static const char *osslclCGI = "oss.lcl=1";
+       XrdSecsssID  *idMapper = 0;    // -> Auth ID mapper
 
-static const int   PBsz = 4096;
+static const char   *ofslclCGI = "ofs.lcl=1";
+
+static const char   *osslclCGI = "oss.lcl=1";
+
+static const int     PBsz = 4096;
+
+       bool          idMapAll = false;
+
+       bool          outProxy = false; // True means outgoing proxy
+
+       bool          xrdProxy = false; // True means dest using xroot protocol
 
        XrdSysTrace SysTrace("Pss",0);
 }
@@ -107,22 +118,24 @@ using namespace XrdProxy;
 /*                XrdOssGetSS (a.k.a. XrdOssGetStorageSystem)                 */
 /******************************************************************************/
 
-XrdVERSIONINFO(XrdOssGetStorageSystem,XrdPss);
+XrdVERSIONINFO(XrdOssGetStorageSystem2,XrdPss);
   
 // This function is called by the OFS layer to retrieve the Storage System
 // object. We return our proxy storage system object if configuration succeeded.
 //
 extern "C"
 {
-XrdOss *XrdOssGetStorageSystem(XrdOss       *native_oss,
-                               XrdSysLogger *Logger,
-                               const char   *config_fn,
-                               const char   *parms)
+XrdOss *XrdOssGetStorageSystem2(XrdOss       *native_oss,
+                                XrdSysLogger *Logger,
+                                const char   *cFN,
+                                const char   *parms,
+                                XrdOucEnv    *envp)
 {
 
 // Ignore the parms (we accept none for now) and call the init routine
 //
-   return (XrdProxySS.Init(Logger, config_fn) ? 0 : (XrdOss *)&XrdProxySS);
+   envP = envp;
+   return (XrdProxySS.Init(Logger, cFN) ? 0 : (XrdOss *)&XrdProxySS);
 }
 }
  
@@ -134,7 +147,8 @@ XrdOss *XrdOssGetStorageSystem(XrdOss       *native_oss,
 /******************************************************************************/
   
 XrdPssSys::XrdPssSys() : LocalRoot(0), theN2N(0), DirFlags(0),
-                         myVersion(&XrdVERSIONINFOVAR(XrdOssGetStorageSystem))
+                         myVersion(&XrdVERSIONINFOVAR(XrdOssGetStorageSystem2)),
+                         myFeatures(XRDOSS_HASPRXY|XRDOSS_HASNOSF)
                          {}
 
 /******************************************************************************/
@@ -148,7 +162,7 @@ XrdPssSys::XrdPssSys() : LocalRoot(0), theN2N(0), DirFlags(0),
 
   Output:   Returns zero upon success otherwise (-errno).
 */
-int XrdPssSys::Init(XrdSysLogger *lp, const char *configfn)
+int XrdPssSys::Init(XrdSysLogger *lp, const char *cFN)
 {
    int NoGo;
    const char *tmp;
@@ -157,18 +171,18 @@ int XrdPssSys::Init(XrdSysLogger *lp, const char *configfn)
 //
    SysTrace.SetLogger(lp);
    eDest.logger(lp);
-   eDest.Say("Copr.  2018, Stanford University, Pss Version " XrdVSTRING);
+   eDest.Say("Copr.  2019, Stanford University, Pss Version " XrdVSTRING);
 
 // Initialize the subsystems
 //
-   tmp = ((NoGo=Configure(configfn)) ? "failed." : "completed.");
+   tmp = ((NoGo = Configure(cFN)) ? "failed." : "completed.");
    eDest.Say("------ Proxy storage system initialization ", tmp);
 
 // All done.
 //
    return NoGo;
 }
-
+  
 /******************************************************************************/
 /*                                 C h m o d                                  */
 /******************************************************************************/
@@ -191,6 +205,26 @@ int XrdPssSys::Chmod(const char *path, mode_t mode, XrdOucEnv *eP)
    return -ENOTSUP;
 }
 
+/******************************************************************************/
+/*                               C o n n e c t                                */
+/******************************************************************************/
+
+void XrdPssSys::Connect(XrdOucEnv &theEnv)
+{
+   EPNAME("Connect");
+   const XrdSecEntity *client = theEnv.secEnv();
+
+// If we need to personify the client, set it up
+//
+   if (idMapper && client)
+      {const char *fmt = (client->ueid & 0xf0000000 ? "%x" : "U%x");
+       char uName[32];
+       snprintf(uName, sizeof(uName), fmt, client->ueid);
+       DEBUG(client->tident,"Registering as ID "<<uName);
+       idMapper->Register(uName, client, deferID);
+      }
+}
+
 /******************************************************************************/
 /*                                c r e a t e                                 */
 /******************************************************************************/
@@ -219,6 +253,26 @@ int XrdPssSys::Create(const char *tident, const char *path, mode_t Mode,
 
    return -ENOTSUP;
 }
+  
+/******************************************************************************/
+/*                                  D i s c                                   */
+/******************************************************************************/
+
+void XrdPssSys::Disc(XrdOucEnv &theEnv)
+{
+   EPNAME("Disc");
+   const XrdSecEntity *client = theEnv.secEnv();
+
+// If we personified a client, remove that persona.
+//
+   if (idMapper && client)
+      {const char *fmt = (client->ueid & 0xf0000000 ? "%x" : "U%x");
+       char uName[32];
+       snprintf(uName, sizeof(uName), fmt, client->ueid);
+       DEBUG(client->tident,"Unregistering as ID "<<uName);
+       idMapper->Register(uName, 0);
+      }
+}
 
 /******************************************************************************/
 /*                               E n v I n f o                                */
@@ -417,9 +471,11 @@ int XrdPssSys::Stat(const char *path, struct stat *buff, int Opts, XrdOucEnv *eP
 //
    XrdPssUrlInfo uInfo(eP, path, Cgi);
 
-// Generate an ID if we need to
+// Generate an ID if we need to. We can use the server's identity unless that
+// has been prohibited because client ID mapping is taking place.
 //
-   if (sidP) uInfo.setID(sidP);
+   if (idMapAll) uInfo.setID();
+      else if (sidP) uInfo.setID(sidP);
 
 // Convert path to URL
 //
@@ -434,6 +490,24 @@ int XrdPssSys::Stat(const char *path, struct stat *buff, int Opts, XrdOucEnv *eP
    return (XrdPosixXrootd::Stat(pbuff, buff) ? -errno : XrdOssOK);
 }
 
+/******************************************************************************/
+/*                                 S t a t s                                  */
+/******************************************************************************/
+  
+/* Function: Return statistics.
+
+  Input:    buff        - Pointer to buffer for statistics data.
+            blen        - The length of the buffer.
+
+  Output:   When blen is not zero, null terminated statistics are placed
+            in buff and the length is returned. When blen is zero, the
+            maximum length needed is returned.
+*/
+int XrdPssSys::Stats(char *bp, int bl)
+{
+   return XrdPosixConfig::Stats("pss", bp, bl);
+}
+
 /******************************************************************************/
 /*                              T r u n c a t e                               */
 /******************************************************************************/
@@ -640,8 +714,6 @@ int XrdPssDir::Close(long long *retsz)
 /*                                  o p e n                                   */
 /******************************************************************************/
 
-#define IS_FWDPATH(x) (!strncmp("/xroot:/",x,8) || !strncmp("/root:/",x,7))
-
 /*
   Function: Open the file `path' in the mode indicated by `Mode'.
 
@@ -664,6 +736,10 @@ int XrdPssFile::Open(const char *path, int Oflag, mode_t Mode, XrdOucEnv &Env)
    bool ucgiOK  = true;
    bool ioCache = (Oflag & O_DIRECT);
 
+// Record the security environment
+//
+   entity = Env.secEnv();
+
 // Turn off direct flag if set (we record it separately
 //
    if (ioCache) Oflag &= ~O_DIRECT;
@@ -689,7 +765,7 @@ int XrdPssFile::Open(const char *path, int Oflag, mode_t Mode, XrdOucEnv &Env)
 //
    if (tpcMode)
       {Oflag &= ~O_NOFOLLOW;
-       if (!XrdPssSys::outProxy || !IS_FWDPATH(path))
+       if (!XrdProxy::outProxy || !IS_FWDPATH(path))
           {if (rwMode) {tpcPath = strdup(path); return XrdOssOK;}
            ucgiOK = false;
           }
@@ -698,7 +774,7 @@ int XrdPssFile::Open(const char *path, int Oflag, mode_t Mode, XrdOucEnv &Env)
 // Setup any required cgi information. Don't mess with it if it's an objectid
 // or if the we are an outgoing proxy server.
 //
-   if (!XrdPssSys::outProxy && *path == '/' && !(XRDEXP_STAGE & popts))
+   if (!XrdProxy::outProxy && *path == '/' && !(XRDEXP_STAGE & popts))
       Cgi = osslclCGI;
 
 // Construct the url info
@@ -904,7 +980,8 @@ int XrdPssFile::Fstat(struct stat *buff)
 {
     if (fd < 0)
        {if (!tpcPath) return -XRDOSS_E8004;
-        if (XrdProxySS.Stat(tpcPath, buff))
+        XrdOucEnv fstatEnv(0, 0, entity);
+        if (XrdProxySS.Stat(tpcPath, buff, 0, &fstatEnv))
            memset(buff, 0, sizeof(struct stat));
         return XrdOssOK;
        }
@@ -958,45 +1035,6 @@ int XrdPssFile::Ftruncate(unsigned long long flen)
 
     return (XrdPosixXrootd::Ftruncate(fd, flen) ?  -errno : XrdOssOK);
 }
-
-/******************************************************************************/
-/*                               g e t M m a p                                */
-/******************************************************************************/
-  
-/*
-  Function: Indicate whether or not file is memory mapped.
-
-  Input:    addr      - Points to an address which will receive the location
-                        memory where the file is mapped. If the address is
-                        null, true is returned if a mapping exist.
-
-  Output:   Returns the size of the file if it is memory mapped (see above).
-            Otherwise, zero is returned and addr is set to zero.
-*/
-off_t XrdPssFile::getMmap(void **addr)   // Not Supported for proxies
-{
-   if (addr) *addr = 0;
-   return 0;
-}
-  
-/******************************************************************************/
-/*                          i s C o m p r e s s e d                           */
-/******************************************************************************/
-  
-/*
-  Function: Indicate whether or not file is compressed.
-
-  Input:    cxidp     - Points to a four byte buffer to hold the compression
-                        algorithm used if the file is compressed or null.
-
-  Output:   Returns the region size which is 0 if the file is not compressed.
-            If cxidp is not null, the algorithm is returned only if the file
-            is compressed.
-*/
-int XrdPssFile::isCompressed(char *cxidp)  // Not supported for proxies
-{
-    return 0;
-}
   
 /******************************************************************************/
 /*                                 P 2 D S T                                  */
@@ -1042,10 +1080,10 @@ int XrdPssSys::P2OUT(char *pbuff, int pblen, XrdPssUrlInfo &uInfo)
 // Make sure the path is valid for an outgoing proxy
 //
    if (*path == '/') path++;
-   if ((pname = XrdPssSys::valProt(path, n, 1))) path += n;
+   if ((pname = XrdPssUtils::valProt(path, n, 1))) path += n;
       else {if (!hdrLen) return -ENOTSUP;
             n = snprintf(pbuff, pblen, hdrData, theID, thePath);
-            if (n >= pblen || !uInfo.addCGI(pbuff+n, pblen-n))
+            if (n >= pblen || !uInfo.addCGI(pbuff, pbuff+n, pblen-n))
                return -ENAMETOOLONG;
             return 0;
            }
@@ -1060,7 +1098,7 @@ int XrdPssSys::P2OUT(char *pbuff, int pblen, XrdPssUrlInfo &uInfo)
        if (Police[PolObj] && !P2DST(retc, hBuff, sizeof(hBuff), PolObj,
                                     path+(*path == '/' ? 1:0))) return 0;
        n = snprintf(pbuff, pblen, "%s%s%s", pname, theID, path);
-       if (n >= pblen || !uInfo.addCGI(pbuff+n, pblen-n))
+       if (n >= pblen || !uInfo.addCGI(pbuff, pbuff+n, pblen-n))
           return -ENAMETOOLONG;
        return 0;
       }
@@ -1078,7 +1116,7 @@ int XrdPssSys::P2OUT(char *pbuff, int pblen, XrdPssUrlInfo &uInfo)
 
 // Make sure the path will fit
 //
-   if (n >= pblen || !uInfo.addCGI(pbuff+n, pblen-n))
+   if (n >= pblen || !uInfo.addCGI(pbuff, pbuff+n, pblen-n))
       return -ENAMETOOLONG;
 
 // All done
@@ -1125,7 +1163,9 @@ int XrdPssSys::P2URL(char *pbuff, int pblen, XrdPssUrlInfo &uInfo, bool doN2N)
 // Add any cgi information
 //
    if (uInfo.hasCGI())
-      {if (!uInfo.addCGI(pbuff+pfxLen, pblen-pfxLen)) return -ENAMETOOLONG;}
+      {if (!uInfo.addCGI(pbuff, pbuff+pfxLen, pblen-pfxLen))
+          return -ENAMETOOLONG;
+      }
 
 // All done
 //
