@@ -40,15 +40,14 @@
 
 #include "XrdVersion.hh"
 
-#include "XrdFfs/XrdFfsDent.hh"
-#include "XrdFfs/XrdFfsMisc.hh"
-#include "XrdFfs/XrdFfsQueue.hh"
-
 #include "XrdNet/XrdNetAddr.hh"
 #include "XrdNet/XrdNetUtils.hh"
 #include "XrdNet/XrdNetSecurity.hh"
 
 #include "XrdPss/XrdPss.hh"
+#include "XrdPss/XrdPssTrace.hh"
+#include "XrdPss/XrdPssUrlInfo.hh"
+#include "XrdPss/XrdPssUtils.hh"
 
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
@@ -56,19 +55,22 @@
 #include "XrdSys/XrdSysPthread.hh"
 
 #include "XrdOuc/XrdOuca2x.hh"
-#include "XrdOuc/XrdOucCache2.hh"
+#include "XrdOuc/XrdOucCache.hh"
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucExport.hh"
 #include "XrdOuc/XrdOucN2NLoader.hh"
-#include "XrdOuc/XrdOucPinLoader.hh"
 #include "XrdOuc/XrdOucPsx.hh"
 #include "XrdOuc/XrdOucStream.hh"
 #include "XrdOuc/XrdOucTList.hh"
 #include "XrdOuc/XrdOucUtils.hh"
-#include "XrdOuc/XrdOucCache2.hh"
 
 #include "XrdPosix/XrdPosixConfig.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
+#include "XrdPosix/XrdPosixXrootdPath.hh"
+
+#include "XrdSecsss/XrdSecsssID.hh"
+
+#include "XrdXrootd/XrdXrootdGStream.hh"
 
 /******************************************************************************/
 /*                               d e f i n e s                                */
@@ -83,6 +85,8 @@
 #define TS_PSX(x,m)    if (!strcmp(x,var)) \
                           return (psxConfig->m(&eDest, Config) ? 0 : 1);
 
+#define TS_DBG(x,m)    if (!strcmp(x,var)) {SysTrace.What |= m; return 0;}
+
 /*******x**********************************************************************/
 /*                               G l o b a l s                                */
 /******************************************************************************/
@@ -90,34 +94,24 @@
 const char  *XrdPssSys::ConfigFN;       // -> Pointer to the config file name
 const char  *XrdPssSys::myHost;
 const char  *XrdPssSys::myName;
-uid_t        XrdPssSys::myUid     =  geteuid();
-gid_t        XrdPssSys::myGid     =  getegid();
 
 XrdOucPListAnchor XrdPssSys::XPList;
 
 XrdNetSecurity   *XrdPssSys::Police[XrdPssSys::PolNum] = {0, 0};
 
 XrdOucTList *XrdPssSys::ManList   =  0;
-const char  *XrdPssSys::urlPlain  =  0;
-int          XrdPssSys::urlPlen   =  0;
+const char  *XrdPssSys::protName  =  "root:";
+const char  *XrdPssSys::hdrData   =  "";
 int          XrdPssSys::hdrLen    =  0;
-const char  *XrdPssSys::hdrData   =  0;
-const char  *XrdPssSys::urlRdr    =  0;
 int          XrdPssSys::Streams   =512;
 int          XrdPssSys::Workers   = 16;
+int          XrdPssSys::Trace     =  0;
+int          XrdPssSys::dcaCTime  =  0;
 
-char         XrdPssSys::allChmod  =  0;
-char         XrdPssSys::allMkdir  =  0;
-char         XrdPssSys::allMv     =  0;
-char         XrdPssSys::allRm     =  0;
-char         XrdPssSys::allRmdir  =  0;
-char         XrdPssSys::allTrunc  =  0;
-
-bool         XrdPssSys::outProxy  = false;
-bool         XrdPssSys::pfxProxy  = false;
 bool         XrdPssSys::xLfn2Pfn  = false;
-
-char         XrdPssSys::cfgDone   =  0;
+bool         XrdPssSys::dcaCheck  = false;
+bool         XrdPssSys::dcaWorld  = false;
+bool         XrdPssSys::deferID   = false;
 
 namespace XrdProxy
 {
@@ -127,39 +121,31 @@ extern XrdSysError      eDest;
 
 extern XrdOucSid       *sidP;
 
+extern XrdOucEnv       *envP;
+
+extern XrdSecsssID     *idMapper; // -> Auth ID mapper
+
+extern bool             idMapAll;
+
+extern bool             outProxy; // True means outgoing proxy
+
+extern bool             xrdProxy; // True means dest using xroot protocol
+
+extern XrdSysTrace      SysTrace;
+
 static const int maxHLen = 1024;
 }
 
 namespace
 {
 XrdOucPsx *psxConfig;
+
+XrdSecsssID::authType sssMap;      // persona setting
+
+std::vector<const char *> protVec;    // Additional wanted protocols
 }
 
 using namespace XrdProxy;
-  
-/******************************************************************************/
-/*            E x t e r n a l   T h r e a d   I n t e r f a c e s             */
-/******************************************************************************/
-
-void *XrdPssConfigFfs(void *carg)
-{
-   XrdPssSys *myPSS = (XrdPssSys *)carg;
-
-// Initialize the Ffs (we don't use xrd_init() as it messes up the settings
-// We also do not initialize secsss as we don't know how to effectively use it.
-//
-// XrdFfsMisc_xrd_secsss_init();
-   XrdFfsMisc_refresh_url_cache(myPSS->urlRdr);
-   XrdFfsDent_cache_init();
-   XrdFfsQueue_create_workers(myPSS->Workers);
-
-// Tell everyone waiting for this initialization to complete. We use the trick
-// that in all systems a caharcter is atomically accessed and a single change
-// in value will be detected in a consistent way.
-//
-   myPSS->cfgDone = 1;
-   return (void *)0;
-}
 
 /******************************************************************************/
 /*                             C o n f i g u r e                              */
@@ -174,20 +160,8 @@ int XrdPssSys::Configure(const char *cfn)
 
   Output:   0 upon success or !0 otherwise.
 */
-   struct {const char *Typ; char *Loc;} Fwd[] = {{" ch", &allChmod},
-                                                 {" mk", &allMkdir},
-                                                 {" mv", &allMv   },
-                                                 {" rd", &allRmdir},
-                                                 {" rm", &allRm   },
-                                                 {" tr", &allTrunc},
-                                                 {0,     0        }
-                                                };
-   const char *xP;
-   XrdOucPList *fP;
-   pthread_t tid;
-   char *eP, theRdr[maxHLen+1024];
-   int i, hpLen, NoGo = 0;
-   bool haveFwd = false;
+   char theRdr[maxHLen];
+   int NoGo = 0;
 
 // Get environmental values
 //
@@ -199,14 +173,16 @@ int XrdPssSys::Configure(const char *cfn)
 //
    XrdOucEnv::Export("XRDXROOTD_NOPOSC", "1");
 
-// Create a configurator
+// Create a configurator. It will be deleted when we are done.
 //
-   psxConfig = new XrdOucPsx(myVersion, cfn); // We be deleted later
+   psxConfig = new XrdOucPsx(myVersion, cfn, eDest.logger(), envP);
 
 // Set debug level if so wanted
 //
-   XrdPosixConfig::setLogger(eDest.logger());
-   if (getenv("XRDDEBUG")) psxConfig->traceLvl = 4;
+   if (getenv("XRDDEBUG"))
+      {psxConfig->traceLvl = 4;
+       SysTrace.What |=  TRACEPSS_Debug;
+      }
 
 // Set the defaault number of worker threads for the client
 //
@@ -231,6 +207,10 @@ int XrdPssSys::Configure(const char *cfn)
        return 1;
       }
 
+// Check if we should configure authentication security mapping
+//
+   if (sssMap && !ConfigMapID()) return 1;
+
 // Handle the local root here
 //
    if (LocalRoot) psxConfig->SetRoot(LocalRoot);
@@ -250,21 +230,29 @@ int XrdPssSys::Configure(const char *cfn)
 //
    if (!(psxConfig->ConfigSetup(eDest))) return 1;
 
-// Complete initialization (this cannot fail)
+// Complete initialization (we would set the env pointer here)
 //
-   XrdPosixConfig::SetConfig(*psxConfig);
+   if (!XrdPosixConfig::SetConfig(*psxConfig)) return 1;
 
 // Save the N2N library pointer if we will be using it
 //
    if (psxConfig->xLfn2Pfn) xLfn2Pfn = (theN2N = psxConfig->theN2N) != 0;
 
+// If we have a cache then save it and check if we need to tell
+// xrootd we allow a redirect on a read (this is complicated).
+// ??? Why are we doing this
+// if (psxConfig->theCache2 && dcaCTime)
+//    {char buff[32];
+//     sprintf(buff, "%d", dcaCTime);
+//     XrdOucEnv::Export("XRDXROOTD_CACHERDRDR", buff);
+//    }
+
 // All done with the configurator
 //
    delete psxConfig;
 
 // Allocate an Xroot proxy object (only one needed here). Tell it to not
-// shadow open files with real file descriptors (we will be honest). This can
-// be done before we initialize the ffs.
+// shadow open files with real file descriptors (we will be honest).
 //
    Xroot = new XrdPosixXrootd(-32768, 16384);
 
@@ -272,71 +260,55 @@ int XrdPssSys::Configure(const char *cfn)
 //
    if (Streams) sidP = new XrdOucSid((Streams > 8192 ? 8192 : Streams));
 
-// If this is an outgoing proxy then we are done
+// Tell any security manager we are a proxy as this will force it to use our
+// credentials. We don't support credential forwarding, yet. If we did we would
+// also set XrdSecPROXYCREDS to accomplish that feat.
 //
-   if (outProxy)
-      {if (!ManList) strcpy(theRdr, "=");
-          else sprintf(theRdr, "= %s:%d", ManList->text, ManList->val);
-       XrdOucEnv::Export("XRDXROOTD_PROXY", theRdr);
-       if (ManList)
-          {hdrLen = sprintf(theRdr, "root://%%s%s:%d/%%s%%s%%s",
-                            ManList->text, ManList->val);
-           hdrData = strdup(theRdr);
-          }
-       return 0;
+    XrdOucEnv::Export("XrdSecPROXY", "1");
+
+// Add the origin protocl to the recognized list of protocol names
+//
+   if (!XrdPosixXrootPath::AddProto(protName))
+      {eDest.Emsg("Config", "Unable to add origin protocol to protocol list.");
+       return 1;
       }
 
-// Build the URL header
+// Add any other protocols to the recognized list of protocol names
 //
-   if (!(hpLen = buildHdr())) return 1;
-
-// Create a plain url for future use
-//
-   urlPlen = sprintf(theRdr, hdrData, "", "", "", "", "", "", "", "");
-   urlPlain= strdup(theRdr);
-
-// Export the origin
-//
-   theRdr[urlPlen-1] = 0;
-   XrdOucEnv::Export("XRDXROOTD_PROXY", theRdr+hpLen);
-   theRdr[urlPlen-1] = '/';
-
-// Copy out the forwarding that might be happening via the ofs
-//
-   i = 0;
-   if ((eP = getenv("XRDOFS_FWD")))
-      while(Fwd[i].Typ)
-           {if (!strstr(eP, Fwd[i].Typ)) {*(Fwd[i].Loc) = 1; haveFwd = true;}
-            i++;
+   if (protVec.size())
+      {for (int i = 0; i < (int)protVec.size(); i++)
+           {if (!XrdPosixXrootPath::AddProto(protVec[i]))
+               {eDest.Emsg("Config", "Unable to add", protVec[i],
+                                     "protocol to protocol list.");
+                return 1;
+               }
            }
+       protVec.clear();
+      }
 
-// We would really like that the Ffs interface use the generic method of
-// keeping track of data servers. It does not and it even can't handle more
-// than one export (really). But it does mean we need to give it a valid one.
+// Construct the redirector name:port (we might not have one) export it
 //
-   if (!(eP = getenv("XRDEXPORTS")) || *eP != '/') xP = "/tmp";
-      else if ((xP = rindex(eP, ' '))) xP++;
-              else xP = eP;
+   const char *outeq = (outProxy ? "= " : "");
+   if (ManList) sprintf(theRdr, "%s%s:%d", outeq, ManList->text, ManList->val);
+      else strcpy(theRdr, outeq);
+   XrdOucEnv::Export("XRDXROOTD_PROXY",  theRdr);
+   XrdOucEnv::Export("XRDXROOTD_ORIGIN", theRdr); // Backward compatability
 
-// Setup the redirection url
+// Construct the contact URL header
 //
-   strcpy(&theRdr[urlPlen], xP); urlRdr = strdup(theRdr);
+   if (ManList)               //<prot><id>@<host>:<port>/<path>
+      {hdrLen = sprintf(theRdr, "%s%%s%s:%d/%%s",
+                        protName, ManList->text, ManList->val);
+       hdrData = strdup(theRdr);
+      }
 
 // Check if we have any r/w exports as this will determine whether or not we
-// need to initialize the ffs (we'd rather ot if at all possible).
+// need to initialize any r/w cache. Currently, we don't support this so we
+// have no particular initialization to do.
 //
-   fP = XPList.First();
-   while(fP && !(fP->Flag() & XRDEXP_NOTRW)) fP = fP->Next();
-
-// We avoid initializing the ffs if we have no r/w paths or there is no need to
-// forward r/w metadata operations (e.g. rm, rmdir, etc).
-//
-   if (!fP || !haveFwd) return 0;
-
-// Now spwan a thread to complete ffs initialization which may hang for a while
-//
-   if (XrdSysThread::Run(&tid, XrdPssConfigFfs, (void *)this, 0, "Ffs Config"))
-      {eDest.Emsg("Config", errno, "start ffs configurator"); return 1;}
+// XrdOucPList *fP = XPList.First();
+// while(fP && !(fP->Flag() & XRDEXP_NOTRW)) fP = fP->Next();
+// if (!fP) . . .
 
 // All done
 //
@@ -347,38 +319,67 @@ int XrdPssSys::Configure(const char *cfn)
 /*                     P r i v a t e   F u n c t i o n s                      */
 /******************************************************************************/
 /******************************************************************************/
-/*                              b u i l d H d r                               */
+/*                           C o n f i g M a p I D                            */
 /******************************************************************************/
   
-int XrdPssSys::buildHdr()
+bool XrdPssSys::ConfigMapID()
 {
-   XrdOucTList *tp = ManList;
-   char buff[maxHLen], *pb;
-   int n, hpLen, bleft = sizeof(buff);
+   XrdSecsssCon *conTracker;
+   bool isOK, Debug = (SysTrace.What & TRACEPSS_Debug) != 0;
 
-// Fill in start of header
+// If this is a generic static ID mapping, we are done
 //
-   strcpy(buff, "root://"); hpLen = strlen(buff);
-   pb = buff+hpLen; bleft -= hpLen;
+   if (sssMap == XrdSecsssID::idStatic) return true;
 
-// The redirector list must fit into 1K bytes (along with header)
+// For optimzation we also note if we have a cache in he way of the map
 //
-   while(tp)
-        {n = snprintf(pb, bleft, "%%s%s:%d%c", tp->text, tp->val,
-                                              (tp->next ? ',':'/'));
-         if (n >= bleft) break;
-         pb += n; bleft -= n;
-         tp = tp->next;
-        }
+   deferID = psxConfig->hasCache();
 
-   if (tp)
-      {eDest.Emsg("Config", "Too many proxy service managers specified.");
-       return 0;
+// Now that we did the cache thing, currently we don't support client personas
+// with a cache because aren't able to tell which client will be used.
+//
+   if (deferID)
+      {eDest.Emsg("Config", "Client personas are not supported for "
+                            "caching proxy servers.");
+       return false;
       }
 
-   hdrData = strdup(buff);
-   hdrLen  = strlen(buff);
-   return hpLen;
+// If this server is only a forwarding proxy server, we can't support client
+// personas either because we don't control the URL. However, if we have an
+// origin then simply warn that the client persona applies to the origin.
+//
+   if (outProxy)
+      {if (!ManList)
+          {eDest.Emsg("Config", "Client personas are not supported for "
+                                "strictly forwarding proxy servers.");
+           return false;
+          }
+       eDest.Say("Config warning: client personas only apply to "
+                 "the origin server!");
+      }
+
+// We need to get a connection tracker object from the posix interface.
+// However, we only need it if we are actually mapping id's.
+//
+   if (sssMap == XrdSecsssID::idStaticM) conTracker = 0;
+      else conTracker = XrdPosixConfig::conTracker(Debug);
+
+// Get an mapper object
+//
+   idMapper = new XrdSecsssID(sssMap, 0, conTracker, &isOK);
+   if (!isOK)
+      {eDest.Emsg("Config", "Unable to render persona; persona mapper failed!");
+       return false;
+      }
+
+// If ths is a server persona then we don't need the mapper; abandon it.
+//
+   if (sssMap == XrdSecsssID::idStaticM) idMapper = 0;
+      else XrdPssUrlInfo::setMapID(true);
+
+// We are all done
+//
+   return true;
 }
 
 /******************************************************************************/
@@ -406,6 +407,8 @@ int XrdPssSys::ConfigProc(const char *Cfn)
        return 1;
       }
    Config.Attach(cfgFD);
+   static const char *cvec[] = { "*** pss (oss) plugin config:", 0 };
+   Config.Capture(cvec);
 
 // Now start reading records until eof.
 //
@@ -445,13 +448,17 @@ int XrdPssSys::ConfigXeq(char *var, XrdOucStream &Config)
    TS_PSX("memcache",      ParseCache);  // Backward compatibility
    TS_PSX("cache",         ParseCache);
    TS_PSX("cachelib",      ParseCLib);
+   TS_PSX("ccmlib",        ParseMLib);
    TS_PSX("ciosync",       ParseCio);
    TS_Xeq("config",        xconf);
+   TS_Xeq("dca",           xdca);
    TS_Xeq("defaults",      xdef);
+   TS_DBG("debug",         TRACEPSS_Debug);
    TS_Xeq("export",        xexp);
    TS_PSX("inetmode",      ParseINet);
    TS_Xeq("origin",        xorig);
    TS_Xeq("permit",        xperm);
+   TS_Xeq("persona",       xpers);
    TS_PSX("setopt",        ParseSet);
    TS_PSX("trace",         ParseTrace);
 
@@ -475,18 +482,6 @@ int XrdPssSys::ConfigXeq(char *var, XrdOucStream &Config)
    return 0;
 }
   
-/******************************************************************************/
-/*                             g e t D o m a i n                              */
-/******************************************************************************/
-
-const char *XrdPssSys::getDomain(const char *hName)
-{
-   const char *dot = index(hName, '.');
-
-   if (dot) return dot+1;
-   return hName;
-}
-  
 /******************************************************************************/
 /*                                 x c o n f                                  */
 /******************************************************************************/
@@ -537,6 +532,56 @@ do{for (i = 0; i < numopts; i++) if (!strcmp(Xopts[i].Key, val)) break;
    return 0;
 }
 
+/******************************************************************************/
+/*                                  x d c a                                   */
+/******************************************************************************/
+
+/* Function: xdca
+
+   Purpose:  To parse the directive: dca [group|world] [recheck {<tm> | off}]
+
+             <tm>      recheck for applicability every <tm> interval
+             world     When specified, files are made world deadable.
+                       Otherwise, they are only made group readable.
+
+   Output: 0 upon success or 1 upon failure.
+*/
+
+int XrdPssSys::xdca(XrdSysError *errp, XrdOucStream &Config)
+{
+    static const int maxsz = 0x7fffffff;
+    char *val;
+
+// Preset the defaults
+//
+   dcaCheck = true;
+   dcaCTime = 0;
+   dcaWorld = false;
+
+// If no options then we are done
+//
+   while((val = Config.GetWord()))
+        {     if (!strcmp(val, "world")) dcaWorld = true;
+         else if (!strcmp(val, "group")) dcaWorld = false;
+         else if (!strcmp(val, "recheck"))
+                 {if (!strcmp(val, "off")) dcaCTime = 0;
+                     else {if (!(val = Config.GetWord()))
+                              {errp->Emsg("Config",
+                                          "dca recheck value not specified");
+                               return 1;
+                              }
+                           if (XrdOuca2x::a2tm(*errp,"dca recheck",val,
+                                               &dcaCTime,10,maxsz)) return 1;
+                          }
+                 }
+         else {errp->Emsg("Config","invalid dca option -", val); return 1;}
+        }
+
+// All done
+//
+   return 0;
+}
+  
 /******************************************************************************/
 /*                                  x d e f                                   */
 /******************************************************************************/
@@ -592,9 +637,12 @@ int XrdPssSys::xexp(XrdSysError *Eroute, XrdOucStream &Config)
 
 /* Function: xorig
 
-   Purpose:  Parse: origin {= [<dest>] | <dest>}
+   Purpose:  Parse: origin {=[<prot>,<prot>,...] [<dest>] | <dest>}
 
-   Where:    <dest> <host>[+][:<port>|<port>]
+                                                                                 d
+   where:    <dest> <host>[+][:<port>|<port>] or a URL of the form
+                    <prot>://<dest>[:<port>] where <prot> is one
+                    http, https, root, xroot
 
    Output: 0 upon success or !0 upon failure.
 */
@@ -602,8 +650,9 @@ int XrdPssSys::xexp(XrdSysError *Eroute, XrdOucStream &Config)
 int XrdPssSys::xorig(XrdSysError *errp, XrdOucStream &Config)
 {
     XrdOucTList *tp = 0;
-    char *val, *mval = 0;
+    char *val, *colon, *slash, *mval = 0;
     int  i, port = 0;
+    bool isURL;
 
 //  We are looking for regular managers. These are our points of contact
 //
@@ -612,18 +661,70 @@ int XrdPssSys::xorig(XrdSysError *errp, XrdOucStream &Config)
 
 // Check for outgoing proxy
 //
-   if (!strcmp(val, "="))
-      {pfxProxy = outProxy = true;
+   if (*val == '=')
+      {outProxy = true;
+       if (*(val+1))
+          {std::vector<char *> pVec;
+           char *pData = strdup(val+1);
+           const char *pName;
+           protVec.clear();
+           if (!XrdPssUtils::Vectorize(pData, pVec, ','))
+              {errp->Emsg("Config", "Malformed forwarding specification");
+               free(pData);
+               return 1;
+              }
+           protVec.reserve(pVec.size());
+           for (int i = 0; i < (int)pVec.size(); i++)
+               {int n = strlen(pVec[i]);
+                if (!(pName = XrdPssUtils::valProt(pVec[i], n, 3)))
+                   {errp->Emsg("Config","Unsupported forwarding protocol -",pVec[i]);
+                    free(pData);
+                    return 1;
+                   }
+                protVec.push_back(pName);
+               }
+           free(pData);
+          }
        if (!(val = Config.GetWord())) return 0;
       }
-      else pfxProxy = outProxy = false;
-   mval = strdup(val);
+      else outProxy = false;
+
+// Check if the <dest> is a url, if so, the protocol, must be supported
+//
+   if ((colon = index(val, ':')) && *(colon+1) == '/' && *(colon+2) == '/')
+      {int pnlen;
+       protName = XrdPssUtils::valProt(val, pnlen);
+       if (!protName)
+          {errp->Emsg("Config", "Unsupported origin protocol -", val);
+           return 1;
+          }
+       if (*val == 'x') protName++;
+       xrdProxy = (*val == 'r');
+       val += pnlen;
+       if ((slash = index(val, '/')))
+          {if (*(slash+1))
+              {errp->Emsg("Config","badly formed origin URL"); return 1;}
+           *slash = 0;
+          }
+       mval = strdup(val);
+       isURL = true;
+      } else {
+       protName = "root://";
+       mval = strdup(val);
+       isURL = false;
+       xrdProxy = true;
+      }
 
 // Check if there is a port number. This could be as ':port' or ' port'.
 //
-    if (!(val = index(mval,':'))) val = Config.GetWord();
+    if (!(val = index(mval,':')) && !isURL) val = Config.GetWord();
        else {*val = '\0'; val++;}
 
+// At this point, make sure we actually have a host name
+//
+   if (!(*mval))
+      {errp->Emsg("Config","origin host name not specified"); return 1;}
+
 // Validate the port number
 //
     if (val)
@@ -660,8 +761,9 @@ int XrdPssSys::xorig(XrdSysError *errp, XrdOucStream &Config)
 // We now set the default dirlist flag based on whether the origin is in or out
 // of domain. Composite listings are normally disabled for out of domain nodes.
 //
-   if (!index(mval, '.') || !strcmp(getDomain(mval), getDomain(myHost)))
-      XrdPosixXrootd::setEnv("DirlistDflt", 1);
+   if (!index(mval, '.')
+   || !strcmp(XrdPssUtils::getDomain(mval), XrdPssUtils::getDomain(myHost)))
+      XrdPosixConfig::SetEnv("DirlistDflt", 1);
 
 // All done
 //
@@ -707,3 +809,70 @@ do {if (!(val = Config.GetWord()))
 
     return 0;
 }
+  
+/******************************************************************************/
+/*                                 x p e r s                                  */
+/******************************************************************************/
+
+/* Function: xpers
+
+   Purpose:  To parse the directive: persona {client | server} [options]
+
+   options:  [[non]strict] [[no]verify]
+
+                    client    proxy client's identity via sss authentication
+                    server    use server's identity at end point
+                    strict    all requests must use the client persona
+                    nonstrict certain requests can use a server persona
+                    noverify  do not verify endpoint
+                    verify    verify endpoint
+
+   Output: 0 upon success or !0 upon failure.
+*/
+
+int XrdPssSys::xpers(XrdSysError *Eroute, XrdOucStream &Config)
+{   char *val;
+    bool isClient = false, strict = false;
+    int doVer = -1;
+
+// Make sure a parameter was specified
+//
+   if (!(val = Config.GetWord()))
+      {Eroute->Emsg("Config", "persona not specified"); return 1;}
+
+// Check for persona
+//
+        if (!strcmp(val, "client")) isClient = true;
+   else if (!strcmp(val, "server")) isClient = false;
+   else {Eroute->Emsg("Config", "Invalid persona - ", val); return 1;}
+
+// Process the subsequent options
+//
+   while ((val = Config.GetWord()))
+         {     if (!strcmp(val, "strict"     )) strict = true;
+          else if (!strcmp(val, "nonstrict"  )) strict = false;
+          else if (!strcmp(val, "verify"     )) doVer  = 1;
+          else if (!strcmp(val, "noverify"   )) doVer  = 0;
+          else {Eroute->Emsg("Config", "Invalid persona option - ", val);
+                return 1;
+               }
+      }
+
+// Resolve options vs persona
+//
+   if (isClient)
+      {idMapAll = (strict ? true : false);
+       if (doVer < 0) doVer = 1;
+      }
+
+// Now record the information for future processin
+//
+   if (isClient) sssMap = (doVer ? XrdSecsssID::idMappedM
+                                 : XrdSecsssID::idMapped);
+      else       sssMap = (doVer ? XrdSecsssID::idStaticM
+                                 : XrdSecsssID::idStatic);
+
+// All done
+//
+    return 0;
+}
