@@ -30,7 +30,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <dirent.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <memory.h>
 #include <string.h>
@@ -40,6 +39,7 @@
 #include <sys/stat.h>
 
 #include "XrdVersion.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysLogger.hh"
@@ -197,17 +197,26 @@ const char *XrdSfsNativeDirectory::nextEntry()
 
 // Read the next directory entry
 //
+#ifdef __linux__
    errno = 0;
+   rp = readdir(dh);
+   if (!rp)
+      {if (!(retc = errno)) {ateof = 1; error.clear();}
+          else XrdSfsNative::Emsg(epname,error,retc,"read directory",fname);
+       d_pnt->d_name[0] = '\0';
+       return (const char *)0;
+      }
+    return (const char *)(rp->d_name);
+#else
    if ((retc = readdir_r(dh, d_pnt, &rp)))
-      {if (retc && errno != 0)
-          XrdSfsNative::Emsg(epname,error,retc,"read directory",fname);
+      {XrdSfsNative::Emsg(epname,error,retc,"read directory",fname);
        d_pnt->d_name[0] = '\0';
        return (const char *)0;
       }
 
 // Check if we have reached end of file
 //
-   if (retc || !rp || !d_pnt->d_name[0])
+   if (!rp || !d_pnt->d_name[0])
       {ateof = 1;
        error.clear();
        return (const char *)0;
@@ -216,6 +225,7 @@ const char *XrdSfsNativeDirectory::nextEntry()
 // Return the actual entry
 //
    return (const char *)(d_pnt->d_name);
+#endif
 }
 
 /******************************************************************************/
@@ -377,7 +387,7 @@ int      XrdSfsNativeFile::fctl(const int               cmd,
 
 // We don't support this
 //
-   out_error.setErrInfo(EEXIST, "fctl operation not supported");
+   out_error.setErrInfo(ENOTSUP, "fctl operation not supported");
    return SFS_ERROR;
 }
   
@@ -1015,13 +1025,13 @@ int XrdSfsNative::Emsg(const char    *pfx,    // Message prefix value
                        const char    *op,     // Operation being performed
                        const char    *target) // The target (e.g., fname)
 {
-    char *etext, buffer[MAXPATHLEN+80], unkbuff[64];
+    const char *etext;
+    char buffer[MAXPATHLEN+80];
 
 // Get the reason for the error
 //
    if (ecode < 0) ecode = -ecode;
-   if (!(etext = strerror(ecode)))
-      {sprintf(unkbuff, "reason unknown (%d)", ecode); etext = unkbuff;}
+   etext = XrdSysE2T(ecode);
 
 // Format the error message
 //
