@@ -36,7 +36,6 @@
 #include "XrdOuc/XrdOucERoute.hh"
 #include "XrdOuc/XrdOucErrInfo.hh"
 #include "XrdSfs/XrdSfsDio.hh"
-#include "XrdSfs/XrdSfsXio.hh"
 #include "XrdSsi/XrdSsiAlert.hh"
 #include "XrdSsi/XrdSsiFileReq.hh"
 #include "XrdSsi/XrdSsiFileResource.hh"
@@ -45,6 +44,7 @@
 #include "XrdSsi/XrdSsiService.hh"
 #include "XrdSsi/XrdSsiSfs.hh"
 #include "XrdSsi/XrdSsiStream.hh"
+#include "XrdSsi/XrdSsiStats.hh"
 #include "XrdSsi/XrdSsiTrace.hh"
 #include "XrdSsi/XrdSsiUtils.hh"
 #include "XrdSys/XrdSysError.hh"
@@ -66,6 +66,7 @@ namespace XrdSsi
 extern XrdSysError    Log;
 extern XrdScheduler  *Sched;
 extern XrdSsiService *Service;
+extern XrdSsiStats    Stats;
 };
 
 using namespace XrdSsi;
@@ -100,7 +101,7 @@ int             XrdSsiFileReq::freeMax = 256;
 /*                              A c t i v a t e                               */
 /******************************************************************************/
   
-void XrdSsiFileReq::Activate(XrdOucBuffer *oP, XrdSfsXioHandle *bR, int rSz)
+void XrdSsiFileReq::Activate(XrdOucBuffer *oP, XrdSfsXioHandle bR, int rSz)
 {
    EPNAME("Activate");
 
@@ -108,6 +109,14 @@ void XrdSsiFileReq::Activate(XrdOucBuffer *oP, XrdSfsXioHandle *bR, int rSz)
 //
    DEBUGXQ((oP ? "oucbuff" : "sfsbuff") <<" rqsz=" <<rSz);
 
+// Do statistics
+//
+   Stats.statsMutex.Lock();
+   Stats.ReqCount++;
+   Stats.ReqBytes += rSz;
+   if (rSz > Stats.ReqMaxsz) Stats.ReqMaxsz = rSz;
+   Stats.statsMutex.UnLock();
+
 // Set request buffer pointers
 //
    oucBuff = oP;
@@ -134,6 +143,10 @@ void XrdSsiFileReq::Alert(XrdSsiRespInfoMsg &aMsg)
    aMsg.GetMsg(msgLen);
    DEBUGXQ(msgLen <<" byte alert presented wtr=" <<respWait);
 
+// Add up statistics
+//
+   Stats.Bump(Stats.ReqAlerts);
+
 // Lock this object
 //
    frqMutex.Lock();
@@ -229,6 +242,10 @@ void XrdSsiFileReq::BindDone()
 //
    DEBUGXQ("Bind called; for request " <<reqID);
 
+// Collect statistics
+//
+   Stats.Bump(Stats.ReqBound);
+
 // Processing depends on the current state. Only listed states are valid.
 // When the state is done, a finished event occuured between the time the
 // request was handed off to the service but before the service bound it.
@@ -264,6 +281,10 @@ void XrdSsiFileReq::Dispose()
 //
    DEBUGXQ("Recycling request...");
 
+// Collect statistics
+//
+   Stats.Bump(Stats.ReqBound, -1);
+
 // Simply recycle the object
 //
    Recycle();
@@ -286,12 +307,14 @@ void XrdSsiFileReq::DoIt()
          {case isNew:    myState = xqReq; urState = isBegun;
                          DEBUGXQ("Calling service processor");
                          frqMutex.UnLock();
+                         Stats.Bump(Stats.ReqProcs);
                          Service->ProcessRequest((XrdSsiRequest      &)*this,
                                                  (XrdSsiFileResource &)*fileR);
                          return;
                          break;
           case isAbort:  DEBUGXQ("Skipped calling service processor");
                          frqMutex.UnLock();
+                         Stats.Bump(Stats.ReqAborts);
                          Recycle();
                          return;
                          break;
@@ -300,6 +323,8 @@ void XrdSsiFileReq::DoIt()
                          if (respWait) WakeUp();
                          if (finWait)  finWait->Post();
                          frqMutex.UnLock();
+                         Stats.Bump(Stats.ReqFinished);
+                         if (cancel) Stats.Bump(Stats.ReqCancels);
                          Finished(cancel);  // This object may be deleted!
                          return;
                          break;
@@ -361,6 +386,10 @@ int XrdSsiFileReq::Emsg(const char    *pfx,    // Message prefix value
 {
    char buffer[2048];
 
+// Count errors
+//
+   Stats.Bump(Stats.SsiErrs);
+
 // Get correct error code
 //
    if (ecode < 0) ecode = -ecode;
@@ -389,6 +418,10 @@ int XrdSsiFileReq::Emsg(const char    *pfx,    // Message prefix value
    char buffer[2048];
    int  eNum;
 
+// Count errors
+//
+   Stats.Bump(Stats.SsiErrs);
+
 // Get correct error code and message
 //
    eMsg = eObj.Get(eNum).c_str();
@@ -440,6 +473,7 @@ void XrdSsiFileReq::Finalize()
                          urState = isAbort;
                          cbInfo  = 0;
                          sessN   = "???";
+                         Stats.Bump(Stats.ReqAborts);
                          return;
                          break;
 
@@ -452,22 +486,28 @@ void XrdSsiFileReq::Finalize()
                          mHelper.UnLock();
                          wt4fin.Wait();
                         }
+                         sessN   = "n/a";
                          return;
 
           // Request is bound so we can finish right off.
           //
-          case isBound:  if (strBuff) {strBuff->Recycle(); strBuff = 0;}
+          case isBound:  urState = isDone;
+                         if (strBuff) {strBuff->Recycle(); strBuff = 0;}
                          DEBUGXQ("Calling Finished(" <<cancel <<')');
                          if (respWait) WakeUp();
                          mHelper.UnLock();
+                         Stats.Bump(Stats.ReqFinished);
+                         if (cancel) Stats.Bump(Stats.ReqCancels);
                          Finished(cancel); // This object may be deleted!
+                         sessN   = "n/a";
                          return;
                          break;
 
           // The following two cases may happen but it's safe to ignore them.
           //
           case isAbort:
-          case isDone:   return;
+          case isDone:   sessN = "bad";
+                         return;
                          break;
           default:       break;
          }
@@ -489,12 +529,13 @@ char *XrdSsiFileReq::GetRequest(int &rLen)
 // Do some debugging
 //
    DEBUGXQ("sz=" <<reqSize);
+   Stats.Bump(Stats.ReqGets);
 
 // The request may come from a ouc buffer or an sfs buffer
 //
    rLen = reqSize;
    if (oucBuff) return oucBuff->Data();
-   return sfsBref->Buffer();
+   return XrdSfsXio::Buffer(sfsBref);
 }
 
 /******************************************************************************/
@@ -559,23 +600,28 @@ bool XrdSsiFileReq::ProcessResponse(const XrdSsiErrInfo  &eInfo,
          {case XrdSsiRespInfo::isData:
                DEBUGXQ("Resp data sz="<<Resp.blen);
                respLen = Resp.blen;
+               Stats.Bump(Stats.RspData);
                break;
           case XrdSsiRespInfo::isError:
                DEBUGXQ("Resp err rc="<<Resp.eNum<<" msg="<<Resp.eMsg);
                respLen = 0;
+               Stats.Bump(Stats.RspErrs);
                break;
           case XrdSsiRespInfo::isFile:
                DEBUGXQ("Resp file fd="<<Resp.fdnum<<" sz="<<Resp.fsize);
                fileSz  = Resp.fsize;
                respOff = 0;
+               Stats.Bump(Stats.RspFile);
                break;
           case XrdSsiRespInfo::isStream:
                DEBUGXQ("Resp strm");
                respLen = 0;
+               Stats.Bump(Stats.RspStrm);
                break;
           default:
                DEBUGXQ("Resp invalid!!!!");
                return false;
+               Stats.Bump(Stats.RspBad);
                break;
          }
 
@@ -748,7 +794,7 @@ void XrdSsiFileReq::Recycle()
 // and sfs buffer, put it on the defered release queue.
 //
         if (oucBuff) {oucBuff->Recycle(); oucBuff = 0;}
-   else if (sfsBref) {sfsBref->Recycle(); sfsBref = 0;}
+   else if (sfsBref) {XrdSfsXio::Reclaim(sfsBref);}
    reqSize = 0;
 
 // Add to queue unless we have too many of these. If we add it back to the
@@ -777,11 +823,12 @@ void XrdSsiFileReq::RelRequestBuffer()
 // Do some debugging
 //
    DEBUGXQ("called");
+   Stats.Bump(Stats.ReqRelBuf);
 
 // Release buffers
 //
         if (oucBuff) {oucBuff->Recycle(); oucBuff = 0;}
-   else if (sfsBref) {sfsBref->Recycle(); sfsBref = 0;}
+   else if (sfsBref) {XrdSfsXio::Reclaim(sfsBref);}
    reqSize = 0;
 }
 
@@ -1000,4 +1047,5 @@ void XrdSsiFileReq::WakeUp(XrdSsiAlert *aP) // Called with frqMutex locked!
 //
    respWait = false;
    respCB->Done(respCode, wuInfo, sessN);
+   Stats.Bump(Stats.RspCallBK);
 }
