@@ -27,7 +27,6 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
 #include <fcntl.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -47,7 +46,6 @@
 #include "XrdSec/XrdSecEntity.hh"
 
 #include "XrdSfs/XrdSfsAio.hh"
-#include "XrdSfs/XrdSfsXio.hh"
 
 #include "XrdSsi/XrdSsiEntity.hh"
 #include "XrdSsi/XrdSsiFileSess.hh"
@@ -55,10 +53,12 @@
 #include "XrdSsi/XrdSsiRRInfo.hh"
 #include "XrdSsi/XrdSsiService.hh"
 #include "XrdSsi/XrdSsiSfs.hh"
+#include "XrdSsi/XrdSsiStats.hh"
 #include "XrdSsi/XrdSsiStream.hh"
 #include "XrdSsi/XrdSsiTrace.hh"
 #include "XrdSsi/XrdSsiUtils.hh"
 
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
   
 /******************************************************************************/
@@ -70,9 +70,12 @@ namespace XrdSsi
 extern XrdOucBuffPool   *BuffPool;
 extern XrdSsiProvider   *Provider;
 extern XrdSsiService    *Service;
+extern XrdSsiStats       Stats;
 extern XrdSysError       Log;
 extern int               respWT;
-};
+extern int               minRSZ;
+extern int               maxRSZ;
+}
 
 using namespace XrdSsi;
 
@@ -116,8 +119,6 @@ int             XrdSsiFileSess::freeAbs  = 200;
 
 bool            XrdSsiFileSess::authDNS  = false;
 
-int             XrdSsiFileSess::maxRSZ   = 0;
-
 /******************************************************************************/
 /*                                 A l l o c                                  */
 /******************************************************************************/
@@ -202,6 +203,7 @@ bool XrdSsiFileSess::AttnInfo(XrdOucErrInfo &eInfo, const XrdSsiRespInfo *respP,
       {attnResp->ioV[2].iov_base = (void *)respP->mdata;
        attnResp->ioV[2].iov_len  =         respP->mdlen; ioN = 3;
        attnResp->aHdr.mdLen = htonl(respP->mdlen);
+       Stats.Bump(Stats.RspMDBytes, respP->mdlen);
        if (QTRACE(Debug))
           {char hexBuff[16],dotBuff[4];
            DEBUG(reqID <<':' <<gigID <<' ' <<respP->mdlen <<" byte metadata (0x"
@@ -251,6 +253,13 @@ int XrdSsiFileSess::close(bool viaDel)
 //
    DEBUG((gigID ? gigID : "???") <<" del=" <<viaDel);
 
+// Collect statistics if this is a delete which implies a lost connection
+//
+   if (viaDel)
+      {int rCnt = rTab.Num();
+       if (rCnt) Stats.Bump(Stats.ReqFinForce, rCnt);
+      }
+
 // Run through all outstanding requests and comlete them
 //
    rTab.Reset();
@@ -310,6 +319,7 @@ int XrdSsiFileSess::fctl(const int           cmd,
 //
    if (rqstP->WantResponse(*eInfo))
       {DEBUG(reqID <<':' <<gigID <<" resp ready");
+       Stats.Bump(Stats.RspReady);
        return SFS_DATAVEC;
       }
 
@@ -318,6 +328,7 @@ int XrdSsiFileSess::fctl(const int           cmd,
    DEBUG(reqID <<':' <<gigID <<" resp not ready");
    eInfo->setErrCB((XrdOucEICB *)rqstP);
    eInfo->setErrInfo(respWT, "");
+   Stats.Bump(Stats.RspUnRdy);
    return SFS_STARTED;
 }
 
@@ -349,7 +360,7 @@ void XrdSsiFileSess::Init(XrdOucErrInfo &einfo, const char *user, bool forReuse)
   
 bool XrdSsiFileSess::NewRequest(unsigned int     reqid,
                                 XrdOucBuffer    *oP,
-                                XrdSfsXioHandle *bR,
+                                XrdSfsXioHandle  bR,
                                 int              rSz)
 {
    XrdSsiFileReq *reqP;
@@ -434,6 +445,7 @@ int XrdSsiFileSess::open(const char         *path,      // In
                eNum = errInfo.GetArg();
                DEBUG(path <<" --> " <<eText <<':' <<eNum);
                eInfo->setErrInfo(eNum, eText);
+               Stats.Bump(Stats.ReqRedir);
                return SFS_REDIRECT;
                break;
           case EBUSY:
@@ -442,12 +454,14 @@ int XrdSsiFileSess::open(const char         *path,      // In
                DEBUG(path <<" dly " <<eNum <<' ' <<eText);
                if (eNum <= 0) eNum = 1;
                eInfo->setErrInfo(eNum, eText);
+               Stats.Bump(Stats.ReqStalls);
                return eNum;
                break;
           default:
-               if (!eText || !(*eText)) eText = strerror(eNum);
+               if (!eText || !(*eText)) eText = XrdSysE2T(eNum);
                DEBUG(path <<" err " <<eNum <<' ' <<eText);
                eInfo->setErrInfo(eNum, eText);
+               Stats.Bump(Stats.ReqPrepErrs);
                return SFS_ERROR;
                break;
          };
@@ -456,6 +470,7 @@ int XrdSsiFileSess::open(const char         *path,      // In
 //
    Log.Emsg(epname, "Provider redirect returned no target host name!");
    eInfo->setErrInfo(ENOMSG, "Server logic error");
+   Stats.Bump(Stats.ReqPrepErrs);
    return SFS_ERROR;
 }
   
@@ -622,7 +637,7 @@ int XrdSsiFileSess::truncate(XrdSfsFileOffset  flen)  // In
 // Process request (this can only be a cancel request)
 //
    if (reqXQ != XrdSsiRRInfo::Can)
-      return XrdSsiUtils::Emsg(epname, ENOSYS, "trunc", gigID, *eInfo);
+      return XrdSsiUtils::Emsg(epname, ENOTSUP, "trunc", gigID, *eInfo);
 
 // Perform the cancellation
 //
@@ -693,17 +708,13 @@ XrdSfsXferSize XrdSsiFileSess::write(XrdSfsFileOffset  offset,    // In
 // the request object, and then activate it for processing.
 //
    if (reqSize == blen && xioP)
-      {XrdSfsXioHandle *bRef;
-       XrdSfsXio::XioStatus xStat = xioP->Swap(buff, bRef);
-       if (xStat != XrdSfsXio::allOK)
-          {char etxt[16];
-           sprintf(etxt, "%d", xStat);
-           Log.Emsg(epname, "Xio.Swap() return error status of ", etxt);
-           return XrdSsiUtils::Emsg(epname, ENOMEM, "write", gigID, *eInfo);
-          }
-       if (!NewRequest(reqID, 0, bRef, reqPass))
-          return XrdSsiUtils::Emsg(epname, ENOMEM, "write", gigID, *eInfo);
-       return blen;
+      {XrdSfsXioHandle bRef = xioP->Claim(buff, reqSize, minRSZ);
+       if (!bRef)
+          {if (errno) Log.Emsg(epname,"Xio.Claim() failed;",XrdSysE2T(errno));}
+          else {if (!NewRequest(reqID, 0, bRef, reqPass))
+                   return XrdSsiUtils::Emsg(epname,ENOMEM,"write",gigID,*eInfo);
+                return blen;
+               }
       }
 
 // The full request is not present, so get a buffer to piece it together
