@@ -31,7 +31,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "XrdSsi/XrdSsiPacer.hh"
 #include "XrdSsi/XrdSsiRespInfo.hh"
 #include "XrdSsi/XrdSsiResponder.hh"
 #include "XrdSsi/XrdSsiRequest.hh"
@@ -41,8 +40,20 @@
 /*                        S t a t i c   M e m b e r s                         */
 /******************************************************************************/
   
-XrdSsiMutex XrdSsiRequest::ubMutex;
+namespace XrdSsi
+{
+XrdSsiMutex ubMutex(XrdSsiMutex::Recursive);
+}
 
+/******************************************************************************/
+/*                           C o n s t r u c t o r                            */
+/******************************************************************************/
+
+XrdSsiRequest::XrdSsiRequest(const char *reqid, uint16_t tmo)
+                            : reqID(reqid), rrMutex(&XrdSsi::ubMutex),
+                              theRespond(0), rsvd1(0), epNode(0),
+                              detTTL(0), tOut(0), onClient(true), flags(0) {}
+  
 /******************************************************************************/
 /* Private:                     C l e a n U p                                */
 /******************************************************************************/
@@ -57,7 +68,7 @@ void XrdSsiRequest::CleanUp()
    errInfo.Clr();
    epNode = 0;
    XrdSsiMutex *mP = rrMutex;
-   rrMutex = &ubMutex;
+   rrMutex = &XrdSsi::ubMutex;
    mP->UnLock();
 }
 
@@ -120,6 +131,28 @@ bool XrdSsiRequest::Finished(bool cancel)
    return respP != 0;
 }
 
+/******************************************************************************/
+/*                           G e t E n d P o i n t                            */
+/******************************************************************************/
+  
+std::string XrdSsiRequest::GetEndPoint()
+{
+   XrdSsiMutexMon lck(rrMutex);
+   std::string epName(epNode ? epNode : "");
+   return epName;
+}
+  
+/******************************************************************************/
+/*                           G e t M e t a d a t a                            */
+/******************************************************************************/
+
+const char *XrdSsiRequest::GetMetadata(int &dlen)
+{
+   XrdSsiMutexMon lck(rrMutex);
+   if ((dlen = Resp.mdlen)) return Resp.mdata;
+   return 0;
+}
+  
 /******************************************************************************/
 /*                       G e t R e s p o n s e D a t a                        */
 /******************************************************************************/
@@ -144,16 +177,24 @@ void XrdSsiRequest::GetResponseData(char *buff, int  blen)
 }
 
 /******************************************************************************/
-/*                   R e s t a r t D a t a R e s p o n s e                    */
+/*                  R e l e a s e R e q u e s t B u f f e r                   */
 /******************************************************************************/
   
-XrdSsiRequest::RDR_Info XrdSsiRequest::RestartDataResponse
-                                      (XrdSsiRequest::RDR_How  rhow,
-                                       const char             *reqid
-                                      )
+void XrdSsiRequest::ReleaseRequestBuffer()
 {
-   RDR_Info rInfo;
+   XrdSsiMutexMon lck(rrMutex);
+   RelRequestBuffer();
+}
 
-   XrdSsiPacer::Run(rInfo, rhow, reqid);
-   return rInfo;
+/******************************************************************************/
+/*                              S e t R e t r y                               */
+/******************************************************************************/
+  
+void XrdSsiRequest::SetRetry(bool onoff)
+{
+
+// Set flag as needed
+//
+if (onoff) flags |=  isaRetry;
+   else    flags &= ~isaRetry;
 }
