@@ -81,7 +81,9 @@ using namespace XrdSsi;
 namespace
 {
    std::string dsProperty("DataServer");
-   const char *tident = 0;
+   XrdSsiMutex sidMutex;
+
+   Atomic(uint32_t) sidVal(0);
 }
 
 /******************************************************************************/
@@ -127,6 +129,7 @@ XrdSsiSessReal::~XrdSsiSessReal()
 {
    XrdSsiTaskReal *tP;
 
+   if (resKey)   free(resKey);
    if (sessName) free(sessName);
    if (sessNode) free(sessNode);
 
@@ -138,8 +141,9 @@ XrdSsiSessReal::~XrdSsiSessReal()
 /******************************************************************************/
   
 void XrdSsiSessReal::InitSession(XrdSsiServReal *servP, const char *sName,
-                                 int uent, bool hold)
+                                 int uent, bool hold, bool newSID)
 {
+   EPNAME("InitSession");
    requestP  = 0;
    uEnt      = uent;
    attBase   = 0;
@@ -150,10 +154,22 @@ void XrdSsiSessReal::InitSession(XrdSsiServReal *servP, const char *sName,
    isHeld    = hold;
    inOpen    = false;
    noReuse   = false;
+   if (resKey) {free(resKey); resKey = 0;}
    if (sessName) free(sessName);
    sessName  = (sName ? strdup(sName) : 0);
    if (sessNode) free(sessNode);
    sessNode  = 0;
+   if (newSID)
+      {if (servP == 0) sessID = 0xffffffff;
+          else {Atomic_BEG(sidMutex);
+                sessID = Atomic_INC(sidVal);
+                Atomic_END(sidMutex);
+                snprintf(tident, sizeof(tident), "S %u#", sessID);
+                DEBUG("new sess for "<<sName<<" uent="<<uent<<" hold="<<hold);
+               }
+      } else {
+       DEBUG("reuse sess for "<<sName<<" uent="<<uent<<" hold="<<hold);
+      }
 }
 
 /******************************************************************************/
@@ -177,17 +193,17 @@ XrdSsiTaskReal *XrdSsiSessReal::NewTask(XrdSsiRequest *reqP)
             alocLeft--;
            }
 
-// We always set a new task ID to avoid ID collisions. his is good for over
+// We always set a new task ID to avoid ID collisions. This is good for over
 // 194 days if we have 1 request/second. In practice. this will work for a
 // couple of years before wrapping. By then the ID's should be free.
 //
-   tP->SetTaskID(nextTID++);
+   tP->SetTaskID(nextTID++, sessID);
    nextTID &= XrdSsiRRInfo::idMax;
 
 // Initialize the task and return its pointer
 //
    tP->Init(reqP, reqP->GetTimeOut());
-   DEBUG("Task=" <<tP <<" processing id=" <<nextTID-1);
+   DEBUG("New task=" <<tP <<" id=" <<tP->ID());
 
 // Insert the task into our list of tasks
 //
@@ -211,12 +227,17 @@ bool XrdSsiSessReal::Provision(XrdSsiRequest *reqP, const char *epURL)
    EPNAME("Provision");
    XrdCl::XRootDStatus epStatus;
    XrdSsiMutexMon rHelp(&sessMutex);
+   XrdCl::OpenFlags::Flags oFlags = XrdCl::OpenFlags::Read;
+
+// Set retry flag as appropriate
+//
+   if (XrdSsiRRAgent::isaRetry(reqP, true)) oFlags |= XrdCl::OpenFlags::Refresh;
 
 // Issue the open and if the open was started, return success.
 //
    DEBUG("Provisioning " <<epURL);
-   epStatus = epFile.Open((const std::string)epURL,
-                          XrdCl::OpenFlags::Read, (XrdCl::Access::Mode)0,
+   epStatus = epFile.Open((const std::string)epURL, oFlags,
+                          (XrdCl::Access::Mode)0,
                           (XrdCl::ResponseHandler *)this,
                           reqP->GetTimeOut());
 
@@ -244,11 +265,17 @@ bool XrdSsiSessReal::Provision(XrdSsiRequest *reqP, const char *epURL)
   
 void XrdSsiSessReal::RelTask(XrdSsiTaskReal *tP) // sessMutex locked!
 {
+   EPNAME("RelTask");
+
+// Do some debugging here
+//
+   DEBUG((isHeld ? "Recycling":"Deleting")<<" task="<<tP<<" id=" <<tP->ID());
 
 // Delete this task or place it on the free list
 //
    if (!isHeld) delete tP;
-      else {tP->attList.next = freeTask;
+      else {tP->ClrEvent();
+            tP->attList.next = freeTask;
             freeTask = tP;
            }
 }
@@ -305,6 +332,7 @@ void XrdSsiSessReal::Shutdown(XrdCl::XRootDStatus &epStatus, bool onClose)
                        sessName, sessNode, eNum);
        Log.Emsg("Shutdown", mBuff, eText.c_str());
        sessMutex.UnLock();
+       myService->Recycle(this, false);
       } else {
        if (sessName) {free(sessName); sessName = 0;}
        if (sessNode) {free(sessNode); sessNode = 0;}
@@ -319,44 +347,55 @@ void XrdSsiSessReal::Shutdown(XrdCl::XRootDStatus &epStatus, bool onClose)
   
 void XrdSsiSessReal::TaskFinished(XrdSsiTaskReal *tP)
 {
+   EPNAME("TaskFin");
 // Lock our mutex
 //
    sessMutex.Lock();
 
-// Remove task from the task list if it's in it
+// Remove task from the task list if it's in it and release the task object.
 //
    if (tP == attBase || tP->attList.next != tP)
       {REMOVE(attBase, attList, tP);}
-
-// Clear any pending task events and decrease active count
-//
-   tP->ClrEvent();
+   RelTask(tP);
 
 // Return the request entry number
 //
    XrdSsi::sidScale.retEnt(uEnt);
 
-// Place the task on the free list. If we can shutdown, then unprovision which
-// will drive a shutdown. The returns without the sessMutex, otherwise we must
+// If we are waiting for a provision to finish, simply exit as the event
+// handler will notice that there is no task and will unprovision. Otherwise
+//
+
+
+// if we can shutdown, then unprovision which will drive a shutdown. Note
+// that Unprovision() returns without the sessMutex, otherwise we must
 // unlock it before we return.
 //
-   RelTask(tP);
-   if (!isHeld && !attBase) Unprovision();
-      else sessMutex.UnLock();
+   if (!inOpen)
+      {if (!isHeld && !attBase) Unprovision();
+          else sessMutex.UnLock();
+      } else {
+       DEBUG("Unprovision deferred for " <<sessName);
+       sessMutex.UnLock();
+      }
 }
 
 /******************************************************************************/
 /*                                U n H o l d                                 */
 /******************************************************************************/
   
-void XrdSsiSessReal::UnHold()
+void XrdSsiSessReal::UnHold(bool cleanup)
 {
    XrdSsiMutexMon sessMon(sessMutex);
 
+// Immediately stopo reuse of this object
+//
+   if (isHeld && resKey && myService) myService->StopReuse(resKey);
+
 // Turn off the hold flag and if we have no attached tasks, schedule shutdown
 //
    isHeld = false;
-   if (!attBase) XrdSsi::schedP->Schedule(new CleanUp(this));
+   if (cleanup && !attBase) XrdSsi::schedP->Schedule(new CleanUp(this));
 }
 
 /******************************************************************************/
@@ -373,7 +412,6 @@ void XrdSsiSessReal::Unprovision() // Called with sessMutex locked!
 // Clear any pending events
 //
    DEBUG("Closing " <<sessName);
-   ClrEvent();
 
 // If the file is not open (it might be due to an open error) then do a
 // shutdown right away. Otherwise, try to close if successful the event
@@ -396,7 +434,7 @@ bool XrdSsiSessReal::XeqEvent(XrdCl::XRootDStatus *status,
 // Lock out mutex. Note that events like shutdown unlock the mutex
 //
    sessMutex.Lock();
-   XrdSsiTaskReal *tP = attBase;
+   XrdSsiTaskReal *ztP, *ntP, *tP = attBase;
 
 // If we are not in the open phase then this is due to a close event. Simply
 // do a shutdown and return to stop event processing.
@@ -448,11 +486,14 @@ bool XrdSsiSessReal::XeqEvent(XrdCl::XRootDStatus *status,
        sessNode = strdup(currNode.c_str());
       } else sessNode = strdup("Unknown!");
 
-// Execute each pending request.
+// Execute each pending request. Make sure not to reference the task object
+// chain pointer after invoking SendRequest() as it may become invalid.
 //
-   do {if (!tP->SendRequest(sessNode)) noReuse = true;
-       tP = tP->attList.next;
-      } while(tP != attBase);
+   ztP = attBase;
+   do {ntP = tP->attList.next;
+       if (!tP->SendRequest(sessNode)) noReuse = true;
+       tP = ntP;
+      } while(tP != ztP);
 
 // We are done, field the next event
 //
