@@ -50,6 +50,7 @@
 #include "XrdSsi/XrdSsiProvider.hh"
 #include "XrdSsi/XrdSsiSfs.hh"
 #include "XrdSsi/XrdSsiSfsConfig.hh"
+#include "XrdSsi/XrdSsiStats.hh"
 #include "XrdSsi/XrdSsiTrace.hh"
 
 #include "XrdCms/XrdCmsClient.hh"
@@ -69,6 +70,7 @@
 #include "XrdSec/XrdSecEntity.hh"
 #include "XrdSfs/XrdSfsAio.hh"
 #include "XrdSfs/XrdSfsInterface.hh"
+#include "XrdSfs/XrdSfsFlags.hh"
 
 #include "XrdVersion.hh"
 
@@ -80,18 +82,12 @@
 /*                V e r s i o n   I d e n t i f i c a t i o n                 */
 /******************************************************************************/
   
-XrdVERSIONINFO(XrdSfsGetFileSystem,ssi);
+XrdVERSIONINFO(XrdSfsGetFileSystem2,ssi);
 
 /******************************************************************************/
 /*                        G l o b a l   O b j e c t s                         */
 /******************************************************************************/
 
-namespace
-{
-XrdSsiSfsConfig *Config;
-
-};
-
 namespace XrdSsi
 {
 extern XrdSsiProvider   *Provider;
@@ -109,6 +105,8 @@ extern XrdSysError       Log;
 extern XrdSysLogger     *Logger;
 
 extern XrdSysTrace       Trace;
+
+extern XrdSsiStats       Stats;
 };
 
 using namespace XrdSsi;
@@ -125,17 +123,18 @@ int               XrdSsiSfs::freeMax = 256;
   
 extern "C"
 {
-XrdSfsFileSystem *XrdSfsGetFileSystem(XrdSfsFileSystem *nativeFS,
-                                      XrdSysLogger     *logger,
-                                      const char       *configFn)
+XrdSfsFileSystem *XrdSfsGetFileSystem2(XrdSfsFileSystem *nativeFS,
+                                       XrdSysLogger     *logger,
+                                       const char       *configFn,
+                                       XrdOucEnv        *envP)
 {
    static XrdSsiSfs       mySfs;
    static XrdSsiSfsConfig myConfig;
 
 // Set pointer to the config and file system
 //
-   Config = &myConfig;
    theFS  = nativeFS;
+   Stats.setFS(nativeFS);
 
 // No need to herald this as it's now the default filesystem
 //
@@ -146,10 +145,11 @@ XrdSfsFileSystem *XrdSfsGetFileSystem(XrdSfsFileSystem *nativeFS,
 
 // Initialize the subsystems
 //
-   if (!myConfig.Configure(configFn)) return 0;
+   if (!myConfig.Configure(configFn, envP)) return 0;
 
 // All done, we can return the callout vector to these routines.
 //
+   mySfs.setFeatures(nativeFS);
    return &mySfs;
 }
 }
@@ -219,17 +219,6 @@ int XrdSsiSfs::Emsg(const char    *pfx,    // Message prefix value
     einfo.setErrInfo(ecode, buffer);
     return SFS_ERROR;
 }
-  
-/******************************************************************************/
-/*                               E n v I n f o                                */
-/******************************************************************************/
-  
-void XrdSsiSfs::EnvInfo(XrdOucEnv *envP)
-{
-    if (!envP) Log.Emsg("EnvInfo", "No environmental information passed!");
-    if (!envP || !Config->Configure(envP)) abort();
-}
-
 
 /******************************************************************************/
 /*                                e x i s t s                                 */
@@ -340,10 +329,9 @@ int XrdSsiSfs::fsctl(const int               cmd,
 
 int XrdSsiSfs::getStats(char *buff, int blen)
 {
-// Reroute this if we can
+// Return statustics
 //
-   if (theFS) return theFS->getStats(buff, blen);
-   return 0;
+   return Stats.Stats(buff, blen);
 }
 
 /******************************************************************************/
@@ -445,6 +433,17 @@ int XrdSsiSfs::rename(const char             *old_name,  // In
    return SFS_ERROR;
 }
 
+/******************************************************************************/
+/* Private:                  s e t F e a t u r e s                            */
+/******************************************************************************/
+
+void XrdSsiSfs::setFeatures(XrdSfsFileSystem *prevFS)
+{
+   uint64_t fSet = (prevFS ? prevFS->Features() : 0);
+
+   FeatureSet = fSet | XrdSfs::hasSXIO;
+}
+  
 /******************************************************************************/
 /* Private:                        S p l i t                                  */
 /******************************************************************************/
