@@ -27,7 +27,6 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
 #include <fcntl.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -43,9 +42,18 @@
 #include <iostream>
 
 #include "XrdSsi/XrdSsiShMam.hh"
+#include "XrdSys/XrdSysE2T.hh"
 
 using namespace std;
 
+/* Gentoo removed OF from their copy of zconf.h but we need it here.
+   See https://bugs.gentoo.org/show_bug.cgi?id=383179 for the sad history.
+   This patch modelled after https://trac.osgeo.org/gdal/changeset/24622
+*/
+#ifndef OF
+#define OF(args) args
+#endif
+
 /******************************************************************************/
 /*   S h a r e d   M e m o r y   I n f o r m a t i o n   S t r u c t u r e    */
 /******************************************************************************/
@@ -188,7 +196,7 @@ inline bool ShMam_Flush(void *memP, int sOpt)
 {
    if (msync((void *)((uintptr_t)memP & PageMask), PageSize, sOpt))
       return true;
-   cerr <<"ShMam: msync() failed; " <<strerror(errno) <<endl;
+   cerr <<"ShMam: msync() failed; " <<XrdSysE2T(errno) <<endl;
    return false;
 }
 */
@@ -198,7 +206,7 @@ inline bool ShMam_Flush(void *memP, int mLen, int sOpt)
    uintptr_t memE = ((uintptr_t)memP) + mLen;
    int rc;
    if ((rc = msync((void *)memB, memE-memB, sOpt)))
-      cerr <<"ShMam: msync() failed; " <<strerror(errno) <<endl;
+      cerr <<"ShMam: msync() failed; " <<XrdSysE2T(errno) <<endl;
    return rc == 0;
 }
 */
@@ -761,7 +769,7 @@ bool XrdSsiShMam::ExportIt(bool fLocked)
               noGo = true;
           } else noGo = true;
        if (noGo) cerr <<"SsiShMam: Unable to update version for " <<shmPath
-                      <<"; " <<strerror(errno) <<endl;
+                      <<"; " <<XrdSysE2T(errno) <<endl;
        close(oldFD);
       }
 
@@ -831,7 +839,7 @@ bool XrdSsiShMam::Flush()
 //
    if (rc)
       {rc = errno;
-       cerr <<"ShMam: msync() failed; " <<strerror(errno) <<endl;
+       cerr <<"ShMam: msync() failed; " <<XrdSysE2T(errno) <<endl;
        errno = rc; rc = -1;
       }
 
@@ -997,7 +1005,7 @@ int  XrdSsiShMam::Info(const char *vname, char *buff, int blen)
 
 // Return variable not supported
 //
-   errno = ENOSYS;
+   errno = ENOTSUP;
    return -1;
 }
 
