@@ -65,8 +65,6 @@ XrdSsiSessReal voidSession(0, "voidSession", 0);
 
 std::string pName("ReadRecovery");
 std::string pValue("false");
-
-const char *tident  = 0;
 char        zedData = 0;
 }
 
@@ -145,7 +143,7 @@ bool XrdSsiTaskReal::Ask4Resp()
 
 // Do some debugging
 //
-   DEBUG("Calling fcntl id=" <<tskID);
+   DEBUG("Calling fcntl.");
 
 // Issue the command to field the response
 //
@@ -186,10 +184,10 @@ void XrdSsiTaskReal::Finished(XrdSsiRequest        &rqstR,
 
 // Do some debugging
 //
-   DEBUG("Request="<<&rqstR<<" cancel="<<cancel<<" task="<<this);
+   DEBUG("Request="<<&rqstR<<" cancel="<<cancel);
 
 // We should do an unbind here but that is overkill. All we need to do is
-// reset the mutex to point to the unbound mutex to keep things safe.
+// clear the pointer to the request object.
 //
    XrdSsiRRAgent::ResetResponder(this);
 
@@ -197,7 +195,7 @@ void XrdSsiTaskReal::Finished(XrdSsiRequest        &rqstR,
 // handler will clean things up.
 //
    if (Kill()) sessP->TaskFinished(this);
-      else {DEBUG("Removal deferred; Task="<<this<<" id=" <<tskID);}
+      else {DEBUG("Task removal deferred.");}
 }
 
 /******************************************************************************/
@@ -219,7 +217,7 @@ XrdSsiTaskReal::respType XrdSsiTaskReal::GetResp(XrdCl::AnyObject **respP,
 //
    response->Get(buffP);
    if (!buffP || !(cdP = buffP->GetBuffer()))
-      {DEBUG("Responding with stream id=" <<tskID);
+      {DEBUG("Responding with stream.");
        return isStream;
       }
 
@@ -237,8 +235,7 @@ XrdSsiTaskReal::respType XrdSsiTaskReal::GetResp(XrdCl::AnyObject **respP,
    if (mdP->tag == XrdSsiRRInfoAttn::alrtResp)
       {char hexBuff[16],dotBuff[4];
        dbuff = cdP+pxL; dbL = mdL;
-       DEBUG("Posting " <<dbL <<" byte alert (0x" <<DUMPIT(dbuff,dbL)
-             <<"); id=" <<tskID);
+       DEBUG("Posting "<<dbL<<" byte alert (0x"<<DUMPIT(dbuff,dbL)<<")");
        return isAlert;
       }
 
@@ -246,8 +243,7 @@ XrdSsiTaskReal::respType XrdSsiTaskReal::GetResp(XrdCl::AnyObject **respP,
 //
    if (mdL)
       {char hexBuff[16],dotBuff[4];
-       DEBUG(mdL <<" byte metadata (0x" <<DUMPIT(cdP+pxL,mdL)
-                 <<") set; id=" <<tskID);
+       DEBUG(mdL <<" byte metadata set (0x" <<DUMPIT(cdP+pxL,mdL)<<")");
        SetMetadata(cdP+pxL, mdL);
       }
 
@@ -256,10 +252,10 @@ XrdSsiTaskReal::respType XrdSsiTaskReal::GetResp(XrdCl::AnyObject **respP,
         if (mdP->tag == XrdSsiRRInfoAttn::fullResp)
            {dbuff = (dbL ? cdP+mdL+pxL : &zedData);
             xResp = isData;
-            DEBUG("Responding with data id=" <<tskID);
+            DEBUG("Responding with " <<dbL <<" data bytes.");
            }
    else    {xResp = isStream;
-            DEBUG("Responding with stream id=" <<tskID);
+            DEBUG("Responding with stream.");
            }
 
 // Save the response buffer if we will be referecing it later
@@ -285,12 +281,7 @@ bool XrdSsiTaskReal::Kill() // Called with session mutex locked!
 
 // Do some debugging
 //
-   DEBUG("Status="<<statName[tStat]<<" defer=" <<defer<<" mhPend="<<mhPend
-         <<" id=" <<tskID);
-
-// Regardless of the state, remove this task from the hold queue if there
-//
-   Reset();
+   DEBUG("Status="<<statName[tStat]<<" defer=" <<defer<<" mhPend="<<mhPend);
 
 // Affect proper procedure
 //
@@ -322,6 +313,7 @@ bool XrdSsiTaskReal::Kill() // Called with session mutex locked!
    if (tStat == isWrite && mhPend)
       {XrdSysSemaphore wSem(0);
        wPost = &wSem;
+       DEBUG("Waiting for write event.");
        sessP->UnLock();
        wSem.Wait();
        sessP->Lock();
@@ -332,7 +324,7 @@ bool XrdSsiTaskReal::Kill() // Called with session mutex locked!
 // that, for now, we ignore any errors as we don't have a recovery plan.
 //
    rInfo.Id(tskID); rInfo.Cmd(XrdSsiRRInfo::Can);
-   DEBUG("Sending cancel request id=" <<tskID);
+   DEBUG("Sending cancel request.");
    XrdCl::XRootDStatus Status = sessP->epFile.Truncate(rInfo.Info(), tmOut);
 
 
@@ -340,35 +332,9 @@ bool XrdSsiTaskReal::Kill() // Called with session mutex locked!
 // the message handler will dispose of the task.
 //
    tStat = isDead;
+   DEBUG("Returning " <<!(mhPend || defer));
    return !(mhPend || defer);
 }
-  
-/******************************************************************************/
-/*                               R e d r i v e                                */
-/******************************************************************************/
-  
-void XrdSsiTaskReal::Redrive()
-{
-   EPNAME("TaskRedrive");
-   XrdSsiRequest::PRD_Xeq prdVal;
-   bool last = tStat == isDone;
-
-// Simply call data response method again
-//
-   sessP->UnLock();
-   DEBUG("Redriving ProcessResponseData; len="<<dataRlen<<" last="<<last);
-   prdVal = rqstP->ProcessResponseData(XrdSsiRRAgent::ErrInfoRef(rqstP),
-                                       dataBuff, dataRlen, last);
-   switch(prdVal)
-         {case XrdSsiRequest::PRD_Normal:                               break;
-          case XrdSsiRequest::PRD_Hold:    Hold(0);                     break;
-          case XrdSsiRequest::PRD_HoldLcl: Hold(rqstP->GetRequestID()); break;
-          default: char mBuff[32];
-                   snprintf(mBuff, sizeof(mBuff), "%d", prdVal);
-                   Log.Emsg("TaskRedrive", "ProcessResponseData() returned "
-                                           " invalid enum - ", mBuff);
-         }
-}
 
 /******************************************************************************/
 /* Private:                      R e s p E r r                                */
@@ -387,7 +353,10 @@ bool XrdSsiTaskReal::RespErr(XrdCl::XRootDStatus *status) // Session is locked!
    tStat  = isDone;
    mhPend = false;
    defer  = false;
-   if (sessP) sessP->UnLock();
+   if (sessP)
+      {sessP->UnHold(false);
+       sessP->UnLock();
+      }
 
 // Reflect an error to the request object.
 //
@@ -459,12 +428,13 @@ bool XrdSsiTaskReal::SendRequest(const char *node)
    int                 reqBlen;
 
 // We must be in pend state to send a request. If we are not then the request
-// must have been cancelled. It also means we have a logic error since this
-// should never have happened. Issue a message and ignore this request.
+// must have been cancelled. It also means we have a logic error if the
+// state is not isDead as we can't finish off the task and leak memory.
 //
    if (tStat != isPend)
-      {Log.Emsg("SendRequest", "Invalid state", statName[tStat],
-                               "; should be isPend!");
+      {if (tStat == isDead) sessP->TaskFinished(this);
+          else Log.Emsg("SendRequest", "Invalid state", statName[tStat],
+                                       "; should be isPend!");
        return false;
       }
 
@@ -472,9 +442,19 @@ bool XrdSsiTaskReal::SendRequest(const char *node)
 //
    XrdSsiRRAgent::SetNode(XrdSsiRRAgent::Request(this), node);
 
-// Get the request information
+// Get the request information. Make sure to defer Finish() calls.
 //
+   defer = true;
    reqBuff = XrdSsiRRAgent::Request(this)->GetRequest(reqBlen);
+   defer = false;
+
+// It's possible that GetRequest() called finished so process that here.
+//
+   if (tStat == isDead)
+      {sessP->TaskFinished(this);
+       return false;
+      }
+
 
 // Construct the info for this request
 //
@@ -525,7 +505,7 @@ int XrdSsiTaskReal::SetBuff(XrdSsiErrInfo &eRef,
 
 // Check if this is a proper call or we have reached EOF
 //
-   DEBUG("Sync Status=" <<statName[tStat] <<" id=" <<tskID);
+   DEBUG("ReadSync status=" <<statName[tStat]);
    if (tStat != isReady)
       {if (tStat == isDone) return 0;
        eRef.Set("Stream is not active", ENODEV);
@@ -541,6 +521,7 @@ int XrdSsiTaskReal::SetBuff(XrdSsiErrInfo &eRef,
    epStatus = sessP->epFile.Read(rrInfo.Info(),(uint32_t)blen,buff,ubRead,tmOut);
    if (epStatus.IsOK())
       {if (ibRead < blen) {tStat = isDone; last = true;}
+       DEBUG("ReadSync returning " <<ibRead <<" bytes.");
        return ibRead;
       }
 
@@ -548,7 +529,7 @@ int XrdSsiTaskReal::SetBuff(XrdSsiErrInfo &eRef,
 //
    XrdSsiUtils::SetErr(epStatus, eRef);
    tStat = isDone;
-   DEBUG("Task Sync SetBuff error id=" <<tskID);
+   DEBUG("ReadSync error; " <<epStatus.ToStr());
    return -1;
 }
 
@@ -563,7 +544,7 @@ bool XrdSsiTaskReal::SetBuff(XrdSsiErrInfo &eRef, char *buff, int blen)
 
 // Check if this is a proper call or we have reached EOF
 //
-   DEBUG("Async Status=" <<statName[tStat] <<" id=" <<tskID);
+   DEBUG("ReadAsync Status=" <<statName[tStat]);
    if (tStat != isReady)
       {eRef.Set("Stream is not active", ENODEV);
        return false;
@@ -601,7 +582,7 @@ bool XrdSsiTaskReal::SetBuff(XrdSsiErrInfo &eRef, char *buff, int blen)
 //
    XrdSsiUtils::SetErr(epStatus, eRef);
    tStat = isDone;
-   DEBUG("Task Async SetBuff error id=" <<tskID);
+   DEBUG("ReadAsync error; " <<epStatus.ToStr());
    return false;
 }
 
@@ -620,7 +601,7 @@ bool XrdSsiTaskReal::XeqEnd(bool getLock)
 // Check if finished has been called while we were defered
 //
    if (tStat == isDead)
-      {DEBUG("Task Handler calling TaskFinished.");
+      {DEBUG("Calling TaskFinished; mhPend="<<mhPend<<" defer="<<defer);
        sessP->UnLock();
        sessP->TaskFinished(this);
        return false;
@@ -647,7 +628,6 @@ bool XrdSsiTaskReal::XeqEvent(XrdCl::XRootDStatus *status,
    char *dBuff;
    union {uint32_t ubRead; int ibRead;};
    int dLen;
-   XrdSsiRequest::PRD_Xeq prdVal;
    bool last, aOK = status->IsOK();
 
 // Obtain a lock and indicate the any Finish() calls should be defered until
@@ -660,19 +640,19 @@ bool XrdSsiTaskReal::XeqEvent(XrdCl::XRootDStatus *status,
 
 // Do some debugging
 //
-   DEBUG(" sess="<<(sessP==&voidSession?"no":"ok") <<" id=" <<tskID
-              <<" aOK="<<aOK<<' '<<statName[tStat]
-              <<" clrT=" <<Xrd::hex <<myCaller
-              <<" xeqT=" <<pthread_self() <<Xrd::dec);
+   DEBUG(" aOK="<<aOK<<" status="<<statName[tStat]);
 
 // Affect proper response
 //
    switch(tStat)
          {case isWrite:
                if (!aOK) return RespErr(status); // Unlocks the mutex!
-               DEBUG("Write complete id=" <<tskID);
-               if (wPost) {wPost->Post(); wPost = 0;}
-                  else {DEBUG("Calling RelBuff id=" <<tskID);
+               DEBUG("Write completed.");
+               if (wPost)
+                  {DEBUG("Posting killer.");
+                   wPost->Post(); wPost = 0;
+                  }
+                  else {DEBUG("Calling RelBuff.");
                         ReleaseRequestBuffer();
                         if (tStat == isWrite) return Ask4Resp();
                        }
@@ -684,7 +664,7 @@ bool XrdSsiTaskReal::XeqEvent(XrdCl::XRootDStatus *status,
                   {case isAlert:  aMsg = new AlertMsg(*respP, dBuff, dLen);
                                   *respP = 0;
                                   sessP->UnLock();
-                                  rqstP->Alert(*aMsg);
+                                  XrdSsiRRAgent::Alert(*rqstP, *aMsg);
                                   sessP->Lock();
                                   if (tStat == isSync) return Ask4Resp();
                                      else return XeqEnd(false);
@@ -745,23 +725,11 @@ bool XrdSsiTaskReal::XeqEvent(XrdCl::XRootDStatus *status,
    last = tStat == isDone;
    sessP->UnLock();
    DEBUG("Calling ProcessResponseData; len="<<ibRead<<" last="<<last);
-   prdVal = rqstP->ProcessResponseData(XrdSsiRRAgent::ErrInfoRef(rqstP),
-                                       dBuff, ibRead, last);
+   rqstP->ProcessResponseData(XrdSsiRRAgent::ErrInfoRef(rqstP),
+                              dBuff, ibRead, last);
 
 // If finished was called then we need stop any further action
 //
    if (!XeqEnd(true)) return false;
-
-// The processor requested a hold on further action for this request
-//
-   switch(prdVal)
-         {case XrdSsiRequest::PRD_Normal:                               break;
-          case XrdSsiRequest::PRD_Hold:    Hold(0);                     break;
-          case XrdSsiRequest::PRD_HoldLcl: Hold(rqstP->GetRequestID()); break;
-          default: char mBuff[32];
-                   snprintf(mBuff, sizeof(mBuff), "%d", prdVal);
-                   Log.Emsg("TaskXeqEvent", "ProcessResponseData() returned "
-                                           " invalid enum - ", mBuff);
-         }
    return true;
 }
