@@ -43,6 +43,7 @@
 #include "XrdOuc/XrdOucString.hh"
 
 #include "XrdSut/XrdSutAux.hh"
+#include "XrdSut/XrdSutRndm.hh"
 #include "XrdSut/XrdSutTrace.hh"
 
 static const char *gXRSBucketTypes[] = {
@@ -447,6 +448,13 @@ int XrdSutResolve(XrdOucString &path,
    // Replace <user>, if defined
    if (us && strlen(us) > 0) path.replace("<user>", us);
 
+   // Replace <rtag>, if defined
+   if (path.find("<rtag>") != STR_NPOS) {
+      XrdOucString rtag;
+      XrdSutRndm::GetString(2,6,rtag);
+      path.replace("<rtag>", rtag);
+   }
+
    // Done
    return 0;
 }
