@@ -29,7 +29,6 @@
 /******************************************************************************/
   
 #include <ctype.h>
-#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #ifndef WIN32
@@ -43,6 +42,7 @@
 #endif
 
 #include "XrdSys/XrdSysDNS.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 #include "XrdSys/XrdSysPthread.hh"
@@ -54,7 +54,7 @@
 int XrdSysDNS::getHostAddr(const  char     *InetName,
                            struct sockaddr  InetAddr[],
                                   int       maxipa,
-                                  char    **errtxt)
+                           const  char    **errtxt)
 {
 #ifdef HAVE_NAMEINFO
    struct addrinfo   *rp, *np, *pnp=0;
@@ -158,7 +158,7 @@ int XrdSysDNS::getHostAddr(const  char     *InetName,
   
 int XrdSysDNS::getAddrName(const char *InetName,
                            int maxipa, char **Addr, char **Name,
-                           char    **errtxt)
+                           const char **errtxt)
 {
 
 // Host or address and output arrays must be defined
@@ -184,7 +184,8 @@ int XrdSysDNS::getAddrName(const char *InetName,
 
       // The name
       char *names[1] = {0};
-      int hn = getHostName((struct sockaddr &)ip[i], names, 1, errtxt);
+      struct sockaddr *ipaddr = (struct sockaddr *)&ip[i];
+      int hn = getHostName(*ipaddr, names, 1, errtxt);
       if (hn)
          Name[i] = strdup(names[0]);
       else
@@ -220,7 +221,7 @@ char *XrdSysDNS::getHostID(struct sockaddr &InetAddr)
 /*               g e t H o s t N a m e   ( V a r i a n t   1 )                */
 /******************************************************************************/
 
-char *XrdSysDNS::getHostName(const char *InetName, char **errtxt)
+char *XrdSysDNS::getHostName(const char *InetName, const char **errtxt)
 {
    char myname[256];
    const char *hp;
@@ -246,7 +247,7 @@ char *XrdSysDNS::getHostName(const char *InetName, char **errtxt)
 /*               g e t H o s t N a m e   ( V a r i a n t   2 )                */
 /******************************************************************************/
 
-char *XrdSysDNS::getHostName(struct sockaddr &InetAddr, char **errtxt)
+char *XrdSysDNS::getHostName(struct sockaddr &InetAddr, const char **errtxt)
 {
      char *result;
      if (getHostName(InetAddr, &result, 1, errtxt)) return result;
@@ -267,7 +268,7 @@ char *XrdSysDNS::getHostName(struct sockaddr &InetAddr, char **errtxt)
 int XrdSysDNS::getHostName(struct sockaddr &InetAddr,
                                   char     *InetName[],
                                   int       maxipn,
-                                  char    **errtxt)
+                           const  char    **errtxt)
 {
    char mybuff[256];
    int i, rc;
@@ -396,7 +397,7 @@ int XrdSysDNS::getHostName(struct sockaddr &InetAddr,
   
 int XrdSysDNS::getPort(const char  *servname,
                        const char  *servtype,
-                             char **errtxt)
+                       const char **errtxt)
 {
    int rc;
 
@@ -437,7 +438,7 @@ int XrdSysDNS::getPort(const char  *servname,
  
 /******************************************************************************/
 
-int XrdSysDNS::getPort(int fd, char **errtxt)
+int XrdSysDNS::getPort(int fd, const char **errtxt)
 {
    struct sockaddr InetAddr;
    struct sockaddr_in *ip = (struct sockaddr_in *)&InetAddr;
@@ -497,7 +498,7 @@ int XrdSysDNS::getProtoID(const char *pname)
   
 int XrdSysDNS::Host2Dest(const char      *hostname,
                          struct sockaddr &DestAddr,
-                         char           **errtxt)
+                         const char     **errtxt)
 { char *cp, hbuff[256];
   int port, i;
   struct sockaddr_in InetAddr;
@@ -519,7 +520,8 @@ int XrdSysDNS::Host2Dest(const char      *hostname,
 
 // Convert hostname to an ip address
 //
-   if (!getHostAddr(hbuff, (struct sockaddr &)InetAddr, errtxt)) return 0;
+   struct sockaddr *ip = (struct sockaddr *)&InetAddr;
+   if (!getHostAddr(hbuff, *ip, errtxt)) return 0;
 
 // Insert port number in address
 //
@@ -547,7 +549,8 @@ int XrdSysDNS::Host2IP(const char *hname, unsigned int *ipaddr)
 
 // Convert hostname to an ascii ip address
 //
-   if (!getHostAddr(hname, (struct sockaddr &)InetAddr)) return 0;
+   struct sockaddr *ip = (struct sockaddr *)&InetAddr;
+   if (!getHostAddr(hname, *ip)) return 0;
    if (ipaddr) memcpy(ipaddr, &InetAddr.sin_addr, sizeof(unsigned int));
    return 1;
 }
@@ -690,7 +693,7 @@ int XrdSysDNS::isMatch(const char *HostName, char *HostPat)
 /*                              P e e r n a m e                               */
 /******************************************************************************/
   
-char *XrdSysDNS::Peername(int snum, struct sockaddr *sap, char **errtxt)
+char *XrdSysDNS::Peername(int snum, struct sockaddr *sap, const char **errtxt)
 {
    struct sockaddr addr;
    SOCKLEN_t addrlen = sizeof(addr);
@@ -743,9 +746,9 @@ char *XrdSysDNS::LowCase(char *str)
 /*                                 s e t E T                                  */
 /******************************************************************************/
   
-int XrdSysDNS::setET(char **errtxt, int rc)
+int XrdSysDNS::setET(const char **errtxt, int rc)
 {
-    if (rc) *errtxt = strerror(rc);
+    if (rc) *errtxt = XrdSysE2T(rc);
        else *errtxt = (char *)"unexpected error";
     return 0;
 }
@@ -754,7 +757,7 @@ int XrdSysDNS::setET(char **errtxt, int rc)
 /*                               s e t E T n i                                */
 /******************************************************************************/
   
-int XrdSysDNS::setETni(char **errtxt, int rc)
+int XrdSysDNS::setETni(const char **errtxt, int rc)
 {
 #ifndef HAVE_NAMEINFO
     return setET(errtxt, rc);
