@@ -28,12 +28,12 @@
 /******************************************************************************/
 
 #include <ctype.h>
-#include <errno.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <limits.h>
 
 #include "XrdSys/XrdSysAtomics.hh"
+#include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysFD.hh"
 #include "XrdSys/XrdSysPthread.hh"
 #include "XrdXml/XrdXmlMetaLink.hh"
@@ -139,7 +139,7 @@ XrdOucFileInfo *XrdXmlMetaLink::Convert(const char *fname, int blen)
    if (!(reader = XrdXmlReader::GetReader(fname, encType)))
       {eCode =  errno;
        snprintf(eText, sizeof(eText), "%s trying to read %s",
-               (errno ? strerror(errno) : "Unknow error"), fname);
+               (errno ? XrdSysE2T(errno) : "Unknow error"), fname);
        return 0;
       }
 
@@ -300,7 +300,11 @@ bool XrdXmlMetaLink::GetFile(const char *scope)
 //
    if (!reader->GetElement(fileElem, needFile))
       {if ((etext = reader->GetError(eCode)))
-          strncpy(eText, etext, sizeof(eText));
+	  {size_t len = strlen(etext);
+           if(len > sizeof(eText)-1) len=sizeof(eText)-1;
+           memcpy(eText, etext, len);
+	   eText[len]=0;
+	  }
        return false;
       }
 
@@ -417,7 +421,12 @@ void XrdXmlMetaLink::GetRdrError(const char *why)
 {
    const char *etext = reader->GetError(eCode);
 
-   if (etext) strncpy(eText, etext, sizeof(eText));
+   if (etext) 
+      {size_t len = strlen(etext);
+       if(len > sizeof(eText)-1) len = sizeof(eText)-1;
+       memcpy(eText, etext, len);
+       eText[len]=0;
+      }
       else {snprintf(eText, sizeof(eText), "End of xml while %s", why);
             eCode = EIDRM;
            }
@@ -548,7 +557,7 @@ bool XrdXmlMetaLink::PutFile(const char *buff, int blen)
 // We failed
 //
    eCode = errno;
-   snprintf(eText, sizeof(eText), "%s %s %s", strerror(eCode), what, tmpFn);
+   snprintf(eText, sizeof(eText), "%s %s %s", XrdSysE2T(eCode), what, tmpFn);
    unlink(tmpFn);
    return false;
 }
