@@ -55,8 +55,6 @@
 /*                     S t a t i c   A l l o c a t i o n                      */
 /******************************************************************************/
 
-XrdScheduler      *XrdXrootdMonitor::Sched      = 0;
-XrdSysError       *XrdXrootdMonitor::eDest      = 0;
 char              *XrdXrootdMonitor::idRec      = 0;
 int                XrdXrootdMonitor::idLen      = 0;
 char              *XrdXrootdMonitor::Dest1      = 0;
@@ -67,7 +65,6 @@ int                XrdXrootdMonitor::monMode2   = 0;
 XrdNetMsg         *XrdXrootdMonitor::InetDest2  = 0;
 XrdXrootdMonitor  *XrdXrootdMonitor::altMon     = 0;
 XrdSysMutex        XrdXrootdMonitor::windowMutex;
-kXR_int32          XrdXrootdMonitor::startTime  = 0;
 int                XrdXrootdMonitor::monRlen    = 0;
 XrdXrootdMonitor::MonRdrBuff
                    XrdXrootdMonitor::rdrMon[XrdXrootdMonitor::rdrMax];
@@ -88,7 +85,6 @@ int                XrdXrootdMonitor::rdrTOD     = 0;
 int                XrdXrootdMonitor::rdrWin     = 0;
 int                XrdXrootdMonitor::rdrNum     = 3;
 kXR_int32          XrdXrootdMonitor::sizeWindow = 60;
-long long          XrdXrootdMonitor::mySID      = 0;
 char               XrdXrootdMonitor::sidName[16]= {0};
 short              XrdXrootdMonitor::sidSize    = 0;
 char               XrdXrootdMonitor::monINFO    = 0;
@@ -107,6 +103,25 @@ char               XrdXrootdMonitor::monCLOCK   = 0;
   
 extern          XrdOucTrace       *XrdXrootdTrace;
 
+namespace XrdXrootdMonInfo
+{
+
+inline static int32_t  InitStartTime()
+{
+  return htonl( time( 0 ) );
+}
+
+XrdScheduler   *Sched     = 0;
+XrdSysError    *eDest     = 0;
+char           *monHost   = 0;
+long long       mySID     = 0;
+int32_t         startTime = InitStartTime();
+int             seq       = 0;
+XrdSysMutex     seqMutex;
+}
+
+using namespace XrdXrootdMonInfo;
+
 /******************************************************************************/
 /*                         L o c a l   D e f i n e s                          */
 /******************************************************************************/
@@ -437,7 +452,7 @@ void XrdXrootdMonitor::Defaults(char *dest1, int mode1, char *dest2, int mode2)
 
 void XrdXrootdMonitor::Defaults(int msz,   int rsz,   int wsz,
                                 int flush, int flash, int idt, int rnm,
-                                int fsint, int fsopt, int fsion)
+                                int fbsz, int fsint, int fsopt, int fsion)
 {
 
 // Set default window size and flush time
@@ -452,7 +467,7 @@ void XrdXrootdMonitor::Defaults(int msz,   int rsz,   int wsz,
 
 // Set the fstat defaults
 //
-   XrdXrootdMonFile::Defaults(fsint, fsopt, fsion);
+   XrdXrootdMonFile::Defaults(fsint, fsopt, fsion, fbsz);
    monFSTAT = fsint != 0;
 
 // Set default monitor buffer size
@@ -552,7 +567,6 @@ int XrdXrootdMonitor::Init(XrdScheduler *sp,    XrdSysError *errp,
 //
    Sched = sp;
    eDest = errp;
-   startTime = htonl(Now);
 
 // Generate our server ID
 //
@@ -565,6 +579,7 @@ int XrdXrootdMonitor::Init(XrdScheduler *sp,    XrdSysError *errp,
    if (sidSize >= (int)sizeof(sidName)) sName[sizeof(sidName)-1] = 0;
    strcpy(sidName, sName);
    free(sName);
+   monHost = strdup(iHost);
 
 // There is nothing to do unless we have been enabled via Defaults()
 //
@@ -620,7 +635,7 @@ int XrdXrootdMonitor::Init(XrdScheduler *sp,    XrdSysError *errp,
 // If we are monitoring file stats then start that up
 //
    if (!Sched || !monFSTAT) monFSTAT = 0;
-      else if (!XrdXrootdMonFile::Init(Sched, eDest)) return 0;
+      else if (!XrdXrootdMonFile::Init()) return 0;
 
 // If we are not monitoring redirections, we are done!
 //
@@ -893,20 +908,12 @@ unsigned char XrdXrootdMonitor::do_Shift(long long xTot, unsigned int &xVal)
   
 void XrdXrootdMonitor::fillHeader(XrdXrootdMonHeader *hdr,
                                   const char          id, int size)
-{  static XrdSysMutex seqMutex;
-   static int         seq = 0;
-          int         myseq;
-
-// Generate a new sequence number
-//
-   seqMutex.Lock();
-   myseq = 0x00ff & (seq++);
-   seqMutex.UnLock();
+{
 
 // Fill in the header
 //
    hdr->code = static_cast<kXR_char>(id);
-   hdr->pseq = static_cast<kXR_char>(myseq);
+// hdr->pseq = static_cast<kXR_char>(myseq);  // Filled in Send()
    hdr->plen = htons(static_cast<uint16_t>(size));
    hdr->stod = startTime;
 }
@@ -1039,22 +1046,31 @@ void XrdXrootdMonitor::Mark()
 /*                                  S e n d                                   */
 /******************************************************************************/
   
-int XrdXrootdMonitor::Send(int monMode, void *buff, int blen)
+int XrdXrootdMonitor::Send(int monMode, void *buff, int blen, bool setseq)
 {
 #ifndef NODEBUG
     const char *TraceID = "Monitor";
 #endif
     static XrdSysMutex sendMutex;
+    static int seq1=0, seq2=0;
+    XrdXrootdMonHeader *mHdr=0;
     int rc1, rc2;
 
+// If we are to set sequence numbers, recast the buffer. We are assured that
+// the buffer always starts with the standard monitor header.
+//
+   if (setseq) mHdr = static_cast<XrdXrootdMonHeader*>(buff);
+
     sendMutex.Lock();
     if (monMode & monMode1 && InetDest1)
-       {rc1  = InetDest1->Send((char *)buff, blen);
+       {if (mHdr) mHdr->pseq = (seq1++) & 0xff;
+        rc1  = InetDest1->Send((char *)buff, blen);
         TRACE(DEBUG,blen <<" bytes sent to " <<Dest1 <<" rc=" <<rc1);
        }
        else rc1 = 0;
     if (monMode & monMode2 && InetDest2)
-       {rc2  = InetDest2->Send((char *)buff, blen);
+       {if (mHdr) mHdr->pseq = (seq2++) & 0xff;
+        rc2  = InetDest2->Send((char *)buff, blen);
         TRACE(DEBUG,blen <<" bytes sent to " <<Dest2 <<" rc=" <<rc2);
        }
        else rc2 = 0;
