@@ -29,12 +29,16 @@
  
 #include "XrdVersion.hh"
 
-#include "XrdSfs/XrdSfsInterface.hh"
+#include "XProtocol/XProtocol.hh"
+
 #include "Xrd/XrdBuffer.hh"
 #include "Xrd/XrdLink.hh"
-#include "XProtocol/XProtocol.hh"
 #include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSec/XrdSecProtect.hh"
+#include "XrdSfs/XrdSfsFlags.hh"
+#include "XrdSfs/XrdSfsInterface.hh"
+#include "XrdSys/XrdSysAtomics.hh"
 #include "XrdSys/XrdSysTimer.hh"
 #include "XrdXrootd/XrdXrootdAio.hh"
 #include "XrdXrootd/XrdXrootdFile.hh"
@@ -59,15 +63,10 @@ XrdXrootdXPath        XrdXrootdProtocol::RQList;
 XrdXrootdXPath        XrdXrootdProtocol::XPList;
 XrdSfsFileSystem     *XrdXrootdProtocol::osFS;
 XrdSfsFileSystem     *XrdXrootdProtocol::digFS    = 0;
-char                 *XrdXrootdProtocol::FSLib[2] = {0, 0};
-int                   XrdXrootdProtocol::FSLvn[2] = {0, 0};
-char                 *XrdXrootdProtocol::digLib   = 0;
-char                 *XrdXrootdProtocol::digParm  = 0;
 XrdXrootdFileLock    *XrdXrootdProtocol::Locker;
 XrdSecService        *XrdXrootdProtocol::CIA      = 0;
 XrdSecProtector      *XrdXrootdProtocol::DHS      = 0;
-char                 *XrdXrootdProtocol::SecLib   = 0;
-char                 *XrdXrootdProtocol::pidPath  = strdup("/tmp");
+XrdTlsContext        *XrdXrootdProtocol::tlsCtx   = 0;
 XrdScheduler         *XrdXrootdProtocol::Sched;
 XrdBuffManager       *XrdXrootdProtocol::BPool;
 XrdSysError           XrdXrootdProtocol::eDest(0, "Xrootd");
@@ -76,6 +75,7 @@ XrdXrootdJob         *XrdXrootdProtocol::JobCKS   = 0;
 char                 *XrdXrootdProtocol::JobCKT   = 0;
 XrdOucTList          *XrdXrootdProtocol::JobCKTLST= 0;
 XrdOucReqID          *XrdXrootdProtocol::PrepID   = 0;
+uint64_t              XrdXrootdProtocol::fsFeatures = 0;
 
 char                 *XrdXrootdProtocol::Notify = 0;
 const char           *XrdXrootdProtocol::myCName= 0;
@@ -84,8 +84,7 @@ int                   XrdXrootdProtocol::hailWait;
 int                   XrdXrootdProtocol::readWait;
 int                   XrdXrootdProtocol::Port;
 int                   XrdXrootdProtocol::Window;
-int                   XrdXrootdProtocol::WANPort;
-int                   XrdXrootdProtocol::WANWindow;
+int                   XrdXrootdProtocol::tlsPort = 0;
 char                  XrdXrootdProtocol::isRedir = 0;
 char                  XrdXrootdProtocol::JobLCL  = 0;
 char                  XrdXrootdProtocol::JobCKCGI=0;
@@ -118,7 +117,16 @@ int                   XrdXrootdProtocol::myPID = static_cast<int>(getpid());
 int                   XrdXrootdProtocol::myRole = 0;
 int                   XrdXrootdProtocol::myRolf = 0;
 
+gid_t                 XrdXrootdProtocol::myGID  = 0;
+uid_t                 XrdXrootdProtocol::myUID  = 0;
+int                   XrdXrootdProtocol::myGNLen= 1;
+int                   XrdXrootdProtocol::myUNLen= 1;
+const char           *XrdXrootdProtocol::myGName= "?";
+const char           *XrdXrootdProtocol::myUName= "?";
+time_t                XrdXrootdProtocol::keepT  = 86400; // 24 hours
+
 int                   XrdXrootdProtocol::PrepareLimit = -1;
+bool                  XrdXrootdProtocol::PrepareAlt = false;
 bool                  XrdXrootdProtocol::LimitError = true;
 
 struct XrdXrootdProtocol::RD_Table XrdXrootdProtocol::Route[RD_Num];
@@ -126,6 +134,13 @@ int                   XrdXrootdProtocol::OD_Stall = 33;
 bool                  XrdXrootdProtocol::OD_Bypass= false;
 bool                  XrdXrootdProtocol::OD_Redir = false;
 
+int                   XrdXrootdProtocol::usxMaxNsz= kXR_faMaxNlen;
+int                   XrdXrootdProtocol::usxMaxVsz= kXR_faMaxVlen;
+char                 *XrdXrootdProtocol::usxParms = 0;
+
+char                  XrdXrootdProtocol::tlsCap   = 0;
+char                  XrdXrootdProtocol::tlsNot   = 0;
+
 /******************************************************************************/
 /*            P r o t o c o l   M a n a g e m e n t   S t a c k s             */
 /******************************************************************************/
@@ -197,43 +212,43 @@ int XrdgetProtocolPort(const char *pname, char *parms, XrdProtocol_Config *pi)
 /******************************************************************************/
 /*               X r d P r o t o c o l X r o o t d   C l a s s                */
 /******************************************************************************/
+
+namespace
+{
+XrdSfsXioImpl SfsXioImpl(&XrdXrootdProtocol::Buffer,
+                         &XrdXrootdProtocol::Reclaim);
+}
+
 /******************************************************************************/
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
 
 XrdXrootdProtocol::XrdXrootdProtocol() 
-                    : XrdProtocol("xrootd protocol handler"), ProtLink(this),
-                      Entity("")
+                    : XrdProtocol("xrootd protocol handler"),
+                      XrdSfsXio(SfsXioImpl),
+                      ProtLink(this), Entity(0), AppName(0)
 {
    Reset();
 }
 
 /******************************************************************************/
-/*                   A s s i g n m e n t   O p e r a t o r                    */
+/* protected:                     g e t S I D                                 */
 /******************************************************************************/
 
-XrdXrootdProtocol XrdXrootdProtocol::operator =(const XrdXrootdProtocol &rhs)
+unsigned int XrdXrootdProtocol::getSID()
 {
-// Reset all common fields
-//
-   abort();
-   Reset();
+   static XrdSysMutex  SidMutex;
+   static unsigned int Sid = 1;
+   unsigned int theSid;
 
-// Now copy the relevant fields only
+// Generate unqiue number for this server instance
 //
-   Link          = rhs.Link;
-   Link->setRef(1);      // Keep the link stable until we dereference it
-   Status        = rhs.Status;
-   myFile        = rhs.myFile;
-   myIOLen       = rhs.myIOLen;
-   myOffset      = rhs.myOffset;
-   Response      = rhs.Response;
-   memcpy((void *)&Request,(const void *)&rhs.Request, sizeof(Request));
-   Client        = rhs.Client;
-   AuthProt      = rhs.AuthProt;
-   return *this;
+   AtomicBeg(SidMutex);
+   theSid = AtomicInc(Sid);
+   AtomicEnd(SidMutex);
+   return theSid;
 }
-
+  
 /******************************************************************************/
 /*                                 M a t c h                                  */
 /******************************************************************************/
@@ -243,11 +258,8 @@ XrdXrootdProtocol XrdXrootdProtocol::operator =(const XrdXrootdProtocol &rhs)
 XrdProtocol *XrdXrootdProtocol::Match(XrdLink *lp)
 {
 static  const int hsSZ = sizeof(ClientInitHandShake);
-static  const int prSZ = sizeof(ClientProtocolRequest);
-static  const int hpSZ = hsSZ + prSZ;
-        char hsbuff[hpSZ];
+        char hsbuff[hsSZ];
         struct ClientInitHandShake   *hsData = (ClientInitHandShake *)hsbuff;
-        struct ClientProtocolRequest *hsRqst = (ClientProtocolRequest *)(hsbuff + hsSZ);
 
 static  struct hs_response
                {kXR_unt16 streamid;
@@ -255,17 +267,15 @@ static  struct hs_response
                 kXR_unt32 rlen;   // Specified as kXR_int32 in doc!
                 kXR_unt32 pval;   // Specified as kXR_int32 in doc!
                 kXR_unt32 styp;   // Specified as kXR_int32 in doc!
-               } hsresp={0, 0, htonl(8), // isRedir == 'M' -> MetaManager
-                         htonl(kXR_PROTOCOLVERSION),
+               } hsresp={0, 0, htonl(8), htonl(kXR_PROTOCOLVERSION),
                          (isRedir ? htonl((unsigned int)kXR_LBalServer)
                                   : htonl((unsigned int)kXR_DataServer))};
-
 XrdXrootdProtocol *xp;
 int dlen, rc;
 
 // Peek at the first 20 bytes of data
 //
-   if ((dlen = lp->Peek(hsbuff, hpSZ, hailWait)) < hsSZ)
+   if ((dlen = lp->Peek(hsbuff, hsSZ, hailWait)) < hsSZ)
       {if (dlen <= 0) lp->setEtext("handshake not received");
        return (XrdProtocol *)0;
       }
@@ -281,28 +291,10 @@ int dlen, rc;
    if (hsData->first || hsData->second || hsData->third
    ||  hsData->fourth != 4 || hsData->fifth != ROOTD_PQ) return 0;
 
-// Optimized clients using protocol 2.9.7 or above will piggy-back a protocol
-// request with the handshake. We optimize the response here as well.
+// Send the handshake response. We used optimize the subsequent protocol
+// request sent with handshake but the protocol request is now overloaded.
 //
-   if (dlen != hpSZ||ntohs(hsRqst->requestid) != kXR_protocol||hsRqst->dlen)
-      {dlen = hsSZ;
-       rc = lp->Send((char *)&hsresp, sizeof(hsresp));
-      } else {
-        struct {struct ServerResponseHeader        Hdr;
-                struct ServerResponseBody_Protocol Rsp;
-               }                                   hsprot;
-        struct iovec iov[2] = {{(char *)&hsresp, sizeof(hsresp)},
-                               {(char *)&hsprot, 0}
-                              };
-        int rspLen;
-        memcpy(&Request, hsRqst, sizeof(Request));
-        memcpy(hsprot.Hdr.streamid,hsRqst->streamid,sizeof(hsprot.Hdr.streamid));
-        rspLen              = do_Protocol(&hsprot.Rsp);
-        hsprot.Hdr.dlen     = htonl(rspLen);
-        hsprot.Hdr.status   = 0;
-        iov[1].iov_len      = sizeof(hsprot.Hdr) + rspLen;
-        rc = lp->Send(iov, 2, sizeof(hsresp)+sizeof(hsprot.Hdr)+rspLen);
-       }
+   rc = lp->Send((char *)&hsresp, sizeof(hsresp));
 
 // Verify that our handshake response was actually sent
 //
@@ -313,7 +305,7 @@ int dlen, rc;
 
 // We can now read all 20 bytes and discard them (no need to wait for it)
 //
-   if (lp->Recv(hsbuff, dlen) != dlen)
+   if (lp->Recv(hsbuff, hsSZ) != hsSZ)
       {lp->setEtext("reread failed");
        return (XrdProtocol *)0;
       }
@@ -452,7 +444,7 @@ int XrdXrootdProtocol::Process2()
              case kXR_bind:     return do_Bind();
              default:           Response.Send(kXR_InvalidRequest,
                                 "Invalid request; user not logged in");
-                                return Link->setEtext("protocol sequence error 1");
+                                return Link->setEtext("request without login");
             }
 
 // Help the compiler, select the the high activity requests (the ones with
@@ -463,9 +455,13 @@ int XrdXrootdProtocol::Process2()
          {case kXR_read:     return do_Read();
           case kXR_readv:    return do_ReadV();
           case kXR_write:    return do_Write();
+          case kXR_writev:   return do_WriteV();
+          case kXR_pgread:   return do_PgRead();
+          case kXR_pgwrite:  return do_PgWrite();
           case kXR_sync:     ReqID.setID(Request.header.streamid);
                              return do_Sync();
-          case kXR_close:    return do_Close();
+          case kXR_close:    ReqID.setID(Request.header.streamid);
+                             return do_Close();
           case kXR_truncate: ReqID.setID(Request.header.streamid);
                              if (!Request.header.dlen) return do_Truncate();
                              break;
@@ -514,8 +510,7 @@ int XrdXrootdProtocol::Process2()
 //
    switch(Request.header.requestid)
          {case kXR_open:      return do_Open();
-          case kXR_getfile:   return do_Getfile();
-          case kXR_putfile:   return do_Putfile();
+          case kXR_gpfile:    return do_gpFile();
           default:            break;
          }
 
@@ -526,10 +521,9 @@ int XrdXrootdProtocol::Process2()
 // Now process whatever we have
 //
    switch(Request.header.requestid)
-         {case kXR_admin:     if (Status & XRD_ADMINUSER) return do_Admin();
-                                 else break;
-          case kXR_chmod:     return do_Chmod();
+         {case kXR_chmod:     return do_Chmod();
           case kXR_dirlist:   return do_Dirlist();
+          case kXR_fattr:     return do_FAttr();
           case kXR_locate:    return do_Locate();
           case kXR_mkdir:     return do_Mkdir();
           case kXR_mv:        return do_Mv();
@@ -612,9 +606,10 @@ void XrdXrootdProtocol::Recycle(XrdLink *lp, int csec, const char *reason)
 //
    if (lp)
       {XrdSysTimer::s2hms(csec, ctbuff, sizeof(ctbuff));
-       if (reason) {snprintf(buff, sizeof(buff), "%s (%s)", ctbuff, reason);
-                    sfxp = buff;
-                   } else sfxp = ctbuff;
+       if (reason && strcmp(reason, "hangup"))
+          {snprintf(buff, sizeof(buff), "%s (%s)", ctbuff, reason);
+           sfxp = buff;
+          } else sfxp = ctbuff;
 
        eDest.Log(SYS_LOG_02, "Xeq", lp->ID, (char *)What, sfxp);
       }
@@ -626,9 +621,9 @@ void XrdXrootdProtocol::Recycle(XrdLink *lp, int csec, const char *reason)
 //
    if (Status == XRD_BOUNDPATH && Stream[0])
       {Stream[0]->streamMutex.Lock();
-       isDead = 1;
+       isDead = true;
        if (isActive)
-          {isActive = 0;
+          {isActive = false;
            Stream[0]->Link->setRef(-1);
           }
        Stream[0]->streamMutex.UnLock();
@@ -658,6 +653,113 @@ void XrdXrootdProtocol::Recycle(XrdLink *lp, int csec, const char *reason)
    if (Response.isOurs()) ProtStack.Push(&ProtLink);
 }
 
+/******************************************************************************/
+/*                               S t a t G e n                                */
+/******************************************************************************/
+  
+int XrdXrootdProtocol::StatGen(struct stat &buf, char *xxBuff, int xxLen,
+                               bool xtnd)
+{
+   const mode_t isReadable = (S_IRUSR | S_IRGRP | S_IROTH);
+   const mode_t isWritable = (S_IWUSR | S_IWGRP | S_IWOTH);
+   const mode_t isExecable = (S_IXUSR | S_IXGRP | S_IXOTH);
+   uid_t theuid;
+   gid_t thegid;
+   union {long long uuid; struct {int hi; int lo;} id;} Dev;
+   long long fsz;
+   int m, n, flags = 0;
+
+// Get the right uid/gid
+//
+   theuid = (Client && Client->uid ? Client->uid : myUID);
+   thegid = (Client && Client->gid ? Client->gid : myGID);
+
+// Compute the unique id
+//
+   Dev.id.lo = buf.st_ino;
+   Dev.id.hi = buf.st_dev;
+
+// Compute correct setting of the readable flag
+//
+   if (buf.st_mode & isReadable
+   &&((buf.st_mode & S_IRUSR && theuid == buf.st_uid)
+   || (buf.st_mode & S_IRGRP && thegid == buf.st_gid)
+   ||  buf.st_mode & S_IROTH)) flags |= kXR_readable;
+
+// Compute correct setting of the writable flag
+//
+   if (buf.st_mode & isWritable
+   &&((buf.st_mode & S_IWUSR && theuid == buf.st_uid)
+   || (buf.st_mode & S_IWGRP && thegid == buf.st_gid)
+   ||  buf.st_mode & S_IWOTH)) flags |= kXR_writable;
+
+// Compute correct setting of the execable flag
+//
+   if (buf.st_mode & isExecable
+   &&((buf.st_mode & S_IXUSR && theuid == buf.st_uid)
+   || (buf.st_mode & S_IXGRP && thegid == buf.st_gid)
+   ||  buf.st_mode & S_IXOTH)) flags |= kXR_xset;
+
+// Compute the other flag settings
+//
+        if (!Dev.uuid)                         flags |= kXR_offline;
+        if (S_ISDIR(buf.st_mode))              flags |= kXR_isDir;
+   else if (!S_ISREG(buf.st_mode))             flags |= kXR_other;
+   else{if (buf.st_mode & XRDSFS_POSCPEND)     flags |= kXR_poscpend;
+        if ((buf.st_rdev & XRDSFS_RDVMASK) == 0)
+           {if (buf.st_rdev & XRDSFS_OFFLINE)  flags |= kXR_offline;
+            if (buf.st_rdev & XRDSFS_HASBKUP)  flags |= kXR_bkpexist;
+           }
+       }
+   fsz = static_cast<long long>(buf.st_size);
+
+// Format the default response: <devid> <size> <flags> <mtime>
+//
+   m = snprintf(xxBuff, xxLen, "%lld %lld %d %ld",
+                Dev.uuid, fsz, flags, buf.st_mtime);
+// if (!xtnd || m >= xxLen) return xxLen;
+//
+
+// Format extended response: <ctime> <atime> <mode>
+//
+   char *origP = xxBuff;
+   char *nullP = xxBuff + m++;
+   xxBuff += m; xxLen -= m;
+   n = snprintf(xxBuff, xxLen, "%ld %ld %04o ",
+                buf.st_ctime, buf.st_atime, buf.st_mode&07777);
+   if (n >= xxLen) return m;
+   xxBuff += n; xxLen -= n;
+
+// Tack on owner
+//
+   if (buf.st_uid == myUID)
+      {if (myUNLen >= xxLen) return m;
+       strcpy(xxBuff, myUName);
+       n = myUNLen;
+      } else {
+       if (!(n = XrdOucUtils::UidName(buf.st_uid,xxBuff,xxLen,keepT))) return m;
+      }
+   xxBuff += n;
+   *xxBuff++ = ' ';
+   xxLen -= (n+1);
+
+// Tack on group
+//
+   if (buf.st_gid == myGID)
+      {if (myGNLen >= xxLen) return m;
+       strcpy(xxBuff, myGName);
+       n = myUNLen;
+      } else {
+       if (!(n = XrdOucUtils::GidName(buf.st_gid,xxBuff,xxLen,keepT))) return m;
+      }
+   xxBuff += n+1;
+
+// All done, return full response
+//
+   *nullP = ' ';
+   return xxBuff - origP;
+}
+
 /******************************************************************************/
 /*                                 S t a t s                                  */
 /******************************************************************************/
@@ -672,10 +774,17 @@ int XrdXrootdProtocol::Stats(char *buff, int blen, int do_sync)
        cumReads += numReads; numReads  = 0;
        SI->prerCnt += numReadP;
        cumReadP += numReadP; numReadP = 0;
+
        SI->rvecCnt += numReadV;
        cumReadV += numReadV; numReadV = 0;
        SI->rsegCnt += numSegsV;
        cumSegsV += numSegsV; numSegsV = 0;
+
+       SI->wvecCnt += numWritV;
+       cumWritV += numWritV; numWritV = 0;
+       SI->wsegCnt += numSegsW;
+       cumSegsW += numSegsW, numSegsW = 0;
+
        SI->writeCnt += numWrites;
        cumWrites+= numWrites;numWrites = 0;
        SI->statsMutex.UnLock();
@@ -686,6 +795,75 @@ int XrdXrootdProtocol::Stats(char *buff, int blen, int do_sync)
    return SI->Stats(buff, blen, do_sync);
 }
   
+/******************************************************************************/
+/*                     X r d S f s X i o   M e t h o d s                      */
+/******************************************************************************/
+/******************************************************************************/
+/* Static:                        B u f f e r                                 */
+/******************************************************************************/
+  
+char *XrdXrootdProtocol::Buffer(XrdSfsXioHandle h, int *bsz)
+{
+   XrdBuffer *xbP = (XrdBuffer *)h;
+
+   if (h)
+      {if (bsz) *bsz = xbP->bsize;
+       return xbP->buff;
+      }
+   if (bsz) *bsz = 0;
+   return 0;
+}
+
+/******************************************************************************/
+/*                                 C l a i m                                  */
+/******************************************************************************/
+
+XrdSfsXioHandle XrdXrootdProtocol::Claim(const char *buff, int datasz,
+                                                           int minasz)
+{
+
+// Qualify swap choice
+//
+   if (minasz >= argp->bsize || datasz >= argp->bsize/2) return Swap(buff);
+   errno = 0;
+   return 0;
+}
+  
+/******************************************************************************/
+/* Static:                       R e c l a i m                                */
+/******************************************************************************/
+  
+void XrdXrootdProtocol::Reclaim(XrdSfsXioHandle h)
+{
+
+   if (h) BPool->Release((XrdBuffer *)h);
+}
+  
+/******************************************************************************/
+/*                                  S w a p                                   */
+/******************************************************************************/
+  
+XrdSfsXioHandle XrdXrootdProtocol::Swap(const char *buff, XrdSfsXioHandle h)
+{
+   XrdBuffer *oldBP = argp;
+
+// Verify the context and linkage and if OK, swap buffers
+//
+   if (Request.header.requestid != kXR_write) errno = ENOTSUP;
+      else if (buff != argp->buff) errno = EINVAL;
+              else {if (h)
+                       {argp = (XrdBuffer *)h;
+                        return oldBP;
+                       } else {
+                        argp = BPool->Obtain(argp->bsize);
+                        if (argp) return oldBP;
+                        argp = oldBP;
+                        errno = ENOBUFS;
+                       }
+                   }
+   return 0;
+}
+  
 /******************************************************************************/
 /*                       P r i v a t e   M e t h o d s                        */
 /******************************************************************************/
@@ -695,18 +873,18 @@ int XrdXrootdProtocol::Stats(char *buff, int blen, int do_sync)
   
 int XrdXrootdProtocol::CheckSum(XrdOucStream *Stream, char **argv, int argc)
 {
-   XrdOucErrInfo myInfo("CheckSum");
    int rc, ecode;
 
-// The arguments must have <name> <cstype> <path> (i.e. argc >= 3)
+// The arguments must have <name> <cstype> <path> <tident> (i.e. argc >= 4)
 //
-   if (argc < 3)
+   if (argc < 4)
       {Stream->PutLine("Internal error; not enough checksum args!");
        return 8;
       }
 
 // Issue the checksum calculation (that's all we do here).
 //
+   XrdOucErrInfo myInfo(argv[3]);
    rc = osFS->chksum(XrdSfsFileSystem::csCalc, argv[1], argv[2], myInfo);
 
 // Return result regardless of what it is
@@ -755,7 +933,7 @@ void XrdXrootdProtocol::Cleanup()
       {streamMutex.Lock();
        for (i = 1; i < maxStreams; i++)
            if (Stream[i])
-              {Stream[i]->isBound = 0; Stream[i]->Stream[0] = 0;
+              {Stream[i]->isBound = false; Stream[i]->Stream[0] = 0;
                if (Stream[i]->isDead) Stream[i]->Recycle(0, 0, 0);
                   else Stream[i]->Link->Close();
                Stream[i] = 0;
@@ -778,6 +956,14 @@ void XrdXrootdProtocol::Cleanup()
 //
    while((pioP = pioFirst)) {pioFirst = pioP->Next; pioP->Recycle();}
    while((pioP = pioFree )) {pioFree  = pioP->Next; pioP->Recycle();}
+
+// Handle writev appendage
+//
+   if (wvInfo) {free(wvInfo); wvInfo = 0;}
+
+// Release aplication name
+//
+   if (AppName) {free(AppName); AppName = 0;}
 }
   
 /******************************************************************************/
@@ -804,6 +990,46 @@ int XrdXrootdProtocol::getData(const char *dtype, char *buff, int blen)
    return 0;
 }
 
+/******************************************************************************/
+/*                             g e t P a t h I D                              */
+/******************************************************************************/
+
+int XrdXrootdProtocol::getPathID(bool isRead)
+{
+   XrdXrootdProtocol *pp;;
+   long long sBytes, bestBytes = 0;
+   int  bestStream = 0;
+   bool useS, sBusy, bestIsBusy = false;
+
+// Find a working stream that is least loaded and has at least one free
+// parallel I/O object (it might not be the fastest). We allow for a delay
+// of 50ms so as not to pick a busy stream.
+//
+   for (int i = 1; i < maxStreams; i++)
+       {if (!(pp = Stream[i])) break;
+        pp->streamMutex.Lock();
+        if (!(pp->isDead || pp->isNOP))
+           {sBytes = (isRead ? pp->bytes2send : pp->bytes2recv);
+            sBusy  = (pp->pioFree == 0);
+                 if (!bestStream)        useS = true;
+            else if (bestBytes > sBytes) useS = !sBusy || bestIsBusy;
+            else if (!bestIsBusy)        useS = false;
+            else if (!sBusy)             useS = (bestBytes+5248000 >= sBytes);
+            else                         useS = false;
+            if (useS)
+               {bestBytes  = sBytes;
+                bestStream = i;
+                bestIsBusy = sBusy;
+               }
+           }
+        pp->streamMutex.UnLock();
+       }
+
+// All done, return result
+//
+   return bestStream;
+}
+  
 /******************************************************************************/
 /*                                 R e s e t                                  */
 /******************************************************************************/
@@ -821,17 +1047,23 @@ void XrdXrootdProtocol::Reset()
    myOffset           = 0;
    myIOLen            = 0;
    myStalls           = 0;
+   myFlags            = 0;
    myAioReq           = 0;
    myFile             = 0;
+   wvInfo             = 0;
    numReads           = 0;
    numReadP           = 0;
    numReadV           = 0;
    numSegsV           = 0;
+   numWritV           = 0;
+   numSegsW           = 0;
    numWrites          = 0;
    numFiles           = 0;
    cumReads           = 0;
    cumReadV           = 0;
    cumSegsV           = 0;
+   cumWritV           = 0;
+   cumSegsW           = 0;
    cumWrites          = 0;
    totReadP           = 0;
    hcPrev             =13;
@@ -843,15 +1075,22 @@ void XrdXrootdProtocol::Reset()
    mySID              = 0;
    CapVer             = 0;
    clientPV           = 0;
+   clientRN           = 0;
    reTry              = 0;
    PathID             = 0;
    rvSeq              = 0;
+   wvSeq              = 0;
+   doTLS              = tlsNot; // Assume client is not capable. This will be
+   ableTLS            = false;  // resolved during the kXR_protocol interchange.
+   isTLS              = false;  // Made true when link converted to TLS
    pioFree = pioFirst = pioLast = 0;
-   isActive = isDead  = isNOP = isBound = 0;
+   bytes2recv = bytes2send = 0;
+   isActive = isDead  = isNOP = isBound = false;
    sigNeed = sigHere = sigRead = false;
    sigWarn = true;
    rdType             = 0;
-   memset(&Entity, 0, sizeof(Entity));
+   Entity.Reset(0);
    memset(Stream,  0, sizeof(Stream));
    PrepareCount       = 0;
+   if (AppName) {free(AppName); AppName = 0;}
 }
