@@ -180,7 +180,7 @@ void FileCopyTest::UploadTestFunc()
   int fd = -1;
   CPPUNIT_ASSERT_ERRNO( (fd=open( localFile.c_str(), O_RDONLY )) > 0 )
   CPPUNIT_ASSERT_XRDST( f.Open( fileUrl,
-                                OpenFlags::Delete|OpenFlags::Append ) );
+                                OpenFlags::Delete|OpenFlags::Update ) );
 
   //----------------------------------------------------------------------------
   // Read the data
@@ -307,8 +307,7 @@ namespace
 
     private:
       bool pCancel;
-};
-
+  };
 }
 
 //------------------------------------------------------------------------------
@@ -339,11 +338,16 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
   std::string targetPath   = dataPath + "/tpcFile";
   std::string targetURL    = manager2 + "/" + targetPath;
   std::string metalinkURL  = metamanager + "/" + dataPath + "/metalink/mlTpcTest.meta4";
+  std::string metalinkURL2 = metamanager + "/" + dataPath + "/metalink/mlZipTest.meta4";
   std::string zipURL       = metamanager + "/" + dataPath + "/data.zip";
+  std::string zipURL2      = metamanager + "/" + dataPath + "/large.zip";
   std::string fileInZip    = "paper.txt";
+  std::string fileInZip2   = "bible.txt";
   std::string xcpSourceURL = metamanager + "/" + dataPath + "/1db882c8-8cd6-4df1-941f-ce669bad3458.dat";
+  std::string localFile    = "/data/localfile.dat";
 
-  CopyProcess  process1, process2, process3, process4, process5, process6, process7;
+  CopyProcess  process1, process2, process3, process4, process5, process6, process7, process8, process9,
+               process10, process11, process12, process13;
   PropertyList properties, results;
   FileSystem fs( manager2 );
 
@@ -362,6 +366,56 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
     CPPUNIT_ASSERT_XRDST( process6.Run(0) );
     CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
     properties.Clear();
+
+    //--------------------------------------------------------------------------
+    // Copy from a ZIP archive (compressed) and validate the zcrc32 checksum
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Set( "source",       zipURL2 );
+    properties.Set( "target",       targetURL );
+    properties.Set( "checkSumMode",   "end2end"     );
+    properties.Set( "checkSumType",   "zcrc32"      );
+    properties.Set( "zipArchive",   true      );
+    properties.Set( "zipSource",    fileInZip2 );
+    CPPUNIT_ASSERT_XRDST( process10.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process10.Prepare() );
+    CPPUNIT_ASSERT_XRDST( process10.Run(0) );
+    CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+    properties.Clear();
+
+    //--------------------------------------------------------------------------
+    // Copy with `--rm-bad-cksum`
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Set( "source",         sourceURL );
+    properties.Set( "target",         targetURL );
+    properties.Set( "checkSumMode",   "end2end" );
+    properties.Set( "checkSumType",   "auto"    );
+    properties.Set( "checkSumPreset", "bad-value" ); //< provide wrong checksum value, so the check fails and the file gets removed
+    properties.Set( "rmOnBadCksum",   true        );
+    CPPUNIT_ASSERT_XRDST( process12.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process12.Prepare() );
+    CPPUNIT_ASSERT_XRDST_NOTOK( process12.Run(0), XrdCl::errCheckSumError );
+    XrdCl::StatInfo *info = 0;
+    XrdCl::XRootDStatus status = fs.Stat( targetPath, info );
+    CPPUNIT_ASSERT_XRDST( status.status == XrdCl::stError && st.code == XrdCl::errNotFound );
+    properties.Clear();
+
+    //--------------------------------------------------------------------------
+    // Copy with `--zip-mtln-cksum`
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Set( "source",         metalinkURL2 );
+    properties.Set( "target",         targetURL    );
+    properties.Set( "checkSumMode",   "end2end"    );
+    properties.Set( "checkSumType",   "zcrc32"     );
+    XrdCl::Env *env = XrdCl::DefaultEnv::GetEnv();
+    env->PutInt( "ZipMtlnCksum", 1 );
+    CPPUNIT_ASSERT_XRDST( process13.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process13.Prepare() );
+    CPPUNIT_ASSERT_XRDST_NOTOK( process13.Run(0), XrdCl::errCheckSumError );
+    env->PutInt( "ZipMtlnCksum", 0 );
+    CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
   }
 
   //----------------------------------------------------------------------------
@@ -392,6 +446,58 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
   CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
   properties.Clear();
 
+  //----------------------------------------------------------------------------
+  // Copy to local fs
+  //----------------------------------------------------------------------------
+  results.Clear();
+  properties.Set( "source", sourceURL );
+  properties.Set( "target", "file://localhost" + localFile );
+  properties.Set( "checkSumMode", "end2end" );
+  properties.Set( "checkSumType", "zcrc32"  );
+  CPPUNIT_ASSERT_XRDST( process8.AddJob( properties, &results ) );
+  CPPUNIT_ASSERT_XRDST( process8.Prepare() );
+  CPPUNIT_ASSERT_XRDST( process8.Run(0) );
+  properties.Clear();
+
+  //----------------------------------------------------------------------------
+  // Copy from local fs with extended attributes
+  //----------------------------------------------------------------------------
+
+  // set extended attributes in the local source file
+  File lf;
+  CPPUNIT_ASSERT_XRDST( lf.Open( "file://localhost" + localFile, OpenFlags::Write ) );
+  std::vector<xattr_t> attrs; attrs.push_back( xattr_t( "foo", "bar" ) );
+  std::vector<XAttrStatus> result;
+  CPPUNIT_ASSERT_XRDST( lf.SetXAttr( attrs, result ) );
+  CPPUNIT_ASSERT( result.size() == 1 );
+  CPPUNIT_ASSERT_XRDST( result.front().status );
+  CPPUNIT_ASSERT_XRDST( lf.Close() );
+
+  results.Clear();
+  properties.Set( "source", "file://localhost" + localFile );
+  properties.Set( "target", targetURL );
+  properties.Set( "checkSumMode", "end2end" );
+  properties.Set( "checkSumType", "zcrc32"  );
+  properties.Set( "preserveXAttr", true );
+  CPPUNIT_ASSERT_XRDST( process9.AddJob( properties, &results ) );
+  CPPUNIT_ASSERT_XRDST( process9.Prepare() );
+  CPPUNIT_ASSERT_XRDST( process9.Run(0) );
+  properties.Clear();
+
+  // now test if the xattrs were preserved
+  std::vector<XAttr> xattrs;
+  CPPUNIT_ASSERT_XRDST( fs.ListXAttr( targetPath, xattrs ) );
+  CPPUNIT_ASSERT( xattrs.size() == 1 );
+  XAttr &xattr = xattrs.front();
+  CPPUNIT_ASSERT_XRDST( xattr.status );
+  CPPUNIT_ASSERT( xattr.name == "foo" && xattr.value == "bar" );
+
+  //----------------------------------------------------------------------------
+  // Cleanup
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+  CPPUNIT_ASSERT( remove( localFile.c_str() ) == 0 );
+
   //----------------------------------------------------------------------------
   // Initialize and run the copy
   //----------------------------------------------------------------------------
@@ -399,18 +505,29 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
   properties.Set( "target",       targetURL );
   properties.Set( "checkSumMode", "end2end" );
   properties.Set( "checkSumType", "zcrc32"  );
-
   if( thirdParty )
     properties.Set( "thirdParty",   "only"    );
-
   CPPUNIT_ASSERT_XRDST( process1.AddJob( properties, &results ) );
   CPPUNIT_ASSERT_XRDST( process1.Prepare() );
   CPPUNIT_ASSERT_XRDST( process1.Run(0) );
+  CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+  properties.Clear(); 
 
   //----------------------------------------------------------------------------
-  // Cleanup
+  // Copy with `auto` checksum
   //----------------------------------------------------------------------------
+  results.Clear();
+  properties.Set( "source",       sourceURL );
+  properties.Set( "target",       targetURL );
+  properties.Set( "checkSumMode", "end2end" );
+  properties.Set( "checkSumType", "auto"    );
+  if( thirdParty )
+    properties.Set( "thirdParty",   "only"    );
+  CPPUNIT_ASSERT_XRDST( process11.AddJob( properties, &results ) );
+  CPPUNIT_ASSERT_XRDST( process11.Prepare() );
+  CPPUNIT_ASSERT_XRDST( process11.Run(0) );
   CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+  properties.Clear();
 
   // the further tests are only valid for third party copy for now
   if( !thirdParty )
@@ -419,34 +536,37 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
   //----------------------------------------------------------------------------
   // Abort the copy after 100MB
   //----------------------------------------------------------------------------
-  CancelProgressHandler progress;
-  CPPUNIT_ASSERT_XRDST( process2.AddJob( properties, &results ) );
-  CPPUNIT_ASSERT_XRDST( process2.Prepare() );
-  CPPUNIT_ASSERT_XRDST_NOTOK( process2.Run(&progress), errErrorResponse );
-  CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+//  CancelProgressHandler progress;
+//  CPPUNIT_ASSERT_XRDST( process2.AddJob( properties, &results ) );
+//  CPPUNIT_ASSERT_XRDST( process2.Prepare() );
+//  CPPUNIT_ASSERT_XRDST_NOTOK( process2.Run(&progress), errErrorResponse );
+//  CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
 
   //----------------------------------------------------------------------------
   // Copy from a non-existent source
   //----------------------------------------------------------------------------
   results.Clear();
-  properties.Set( "source", "root://localhost:9999//test" );
+  properties.Set( "source",      "root://localhost:9999//test" );
+  properties.Set( "target",      targetURL );
   properties.Set( "initTimeout", 10 );
+  properties.Set( "thirdParty",  "only"    );
   CPPUNIT_ASSERT_XRDST( process3.AddJob( properties, &results ) );
   CPPUNIT_ASSERT_XRDST( process3.Prepare() );
-  CPPUNIT_ASSERT_XRDST_NOTOK( process3.Run(&progress), errOperationExpired );
+  XrdCl::XRootDStatus status = process3.Run(0);
+  CPPUNIT_ASSERT( !status.IsOK() && ( status.code == errOperationExpired || status.code == errConnectionError ) );
 
   //----------------------------------------------------------------------------
   // Copy to a non-existent target
   //----------------------------------------------------------------------------
   results.Clear();
-  properties.Set( "source", sourceURL );
-  properties.Set( "target", "root://localhost:9999//test" );
+  properties.Set( "source",      sourceURL );
+  properties.Set( "target",      "root://localhost:9999//test" );
   properties.Set( "initTimeout", 10 );
+  properties.Set( "thirdParty",  "only"    );
   CPPUNIT_ASSERT_XRDST( process4.AddJob( properties, &results ) );
   CPPUNIT_ASSERT_XRDST( process4.Prepare() );
-  CPPUNIT_ASSERT_XRDST_NOTOK( process4.Run(0), errOperationExpired );
-
-
+  status = process4.Run(0);
+  CPPUNIT_ASSERT( !status.IsOK() && ( status.code == errOperationExpired || status.code == errConnectionError ) );
 }
 
 //------------------------------------------------------------------------------
