@@ -19,11 +19,14 @@
 #include <cppunit/extensions/HelperMacros.h>
 #include <XrdCl/XrdClFileSystem.hh>
 #include <XrdCl/XrdClFile.hh>
+#include "XrdCl/XrdClDefaultEnv.hh"
+#include "XrdCl/XrdClPlugInManager.hh"
 #include "CppUnitXrdHelpers.hh"
 
 #include <pthread.h>
 
 #include "TestEnv.hh"
+#include "IdentityPlugIn.hh"
 
 using namespace XrdClTests;
 
@@ -48,6 +51,8 @@ class FileSystemTest: public CppUnit::TestCase
       CPPUNIT_TEST( DirListTest );
       CPPUNIT_TEST( SendInfoTest );
       CPPUNIT_TEST( PrepareTest );
+      CPPUNIT_TEST( XAttrTest );
+      CPPUNIT_TEST( PlugInTest );
     CPPUNIT_TEST_SUITE_END();
     void LocateTest();
     void MvTest();
@@ -63,6 +68,8 @@ class FileSystemTest: public CppUnit::TestCase
     void DirListTest();
     void SendInfoTest();
     void PrepareTest();
+    void XAttrTest();
+    void PlugInTest();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION( FileSystemTest );
@@ -125,11 +132,26 @@ void FileSystemTest::MvTest()
   std::string filePath1 = remoteFile;
   std::string filePath2 = remoteFile + "2";
 
+
+  LocationInfo *info = 0;
   FileSystem fs( url );
 
+  // move the file
   CPPUNIT_ASSERT_XRDST( fs.Mv( filePath1, filePath2 ) );
+  // make sure it's not there
+  CPPUNIT_ASSERT_XRDST_NOTOK( fs.Locate( filePath1, OpenFlags::Refresh, info ),
+                              errErrorResponse );
+  // make sure the destination is there
+  CPPUNIT_ASSERT_XRDST( fs.Locate( filePath2, OpenFlags::Refresh, info ) );
+  delete info;
+  // move it back
   CPPUNIT_ASSERT_XRDST( fs.Mv( filePath2, filePath1 ) );
-
+  // make sure it's there
+  CPPUNIT_ASSERT_XRDST( fs.Locate( filePath1, OpenFlags::Refresh, info ) );
+  delete info;
+  // make sure the other one is gone
+  CPPUNIT_ASSERT_XRDST_NOTOK( fs.Locate( filePath2, OpenFlags::Refresh, info ),
+                              errErrorResponse );
 }
 
 //------------------------------------------------------------------------------
@@ -393,6 +415,79 @@ void FileSystemTest::DeepLocateTest()
   delete locations;
 }
 
+//------------------------------------------------------------------------------
+// Handler for chunked DirList
+//------------------------------------------------------------------------------
+class DirListChunkedHandler : public XrdCl::ResponseHandler
+{
+  public:
+
+    //--------------------------------------------------------------------------
+    // Constructor
+    //--------------------------------------------------------------------------
+    DirListChunkedHandler( std::set<std::string> &dirls ) : sem( 0 ), dirls( dirls )
+    {
+
+    }
+
+    //--------------------------------------------------------------------------
+    //
+    //--------------------------------------------------------------------------
+    void HandleResponse( XrdCl::XRootDStatus *status, XrdCl::AnyObject *response )
+    {
+      using namespace XrdCl;
+
+      CPPUNIT_ASSERT_XRDST( *status );
+
+      //------------------------------------------------------------------------
+      // concatenate chunks
+      //------------------------------------------------------------------------
+      if( status->IsOK() )
+      {
+        DirectoryList *list = 0;
+        response->Get( list );
+
+        for( auto itr = list->Begin(); itr != list->End(); ++itr )
+        {
+          DirectoryList::ListEntry *entry = *itr;
+          dirls.insert( entry->GetName() );
+        }      }
+
+      //------------------------------------------------------------------------
+      // if it is the final response we can post the semaphore
+      //------------------------------------------------------------------------
+      if( IsFinal( status ) ) sem.Post();
+
+      //------------------------------------------------------------------------
+      // Clean up
+      //------------------------------------------------------------------------
+      delete status;
+      delete response;
+    }
+
+    //--------------------------------------------------------------------------
+    // Wait until we have all the data
+    //--------------------------------------------------------------------------
+    void Wait()
+    {
+      sem.Wait();
+    }
+
+  private:
+
+    //--------------------------------------------------------------------------
+    // @return : true if it is the final response
+    //--------------------------------------------------------------------------
+    bool IsFinal( XrdCl::XRootDStatus *status )
+    {
+      return !( status->IsOK() && status->code == XrdCl::suContinue );
+    }
+
+    XrdSysSemaphore sem;
+
+    std::set<std::string> &dirls;
+};
+
 //------------------------------------------------------------------------------
 // Dir list
 //------------------------------------------------------------------------------
@@ -426,7 +521,53 @@ void FileSystemTest::DirListTest()
   CPPUNIT_ASSERT( list );
   CPPUNIT_ASSERT( list->GetSize() == 40000 );
 
+  std::set<std::string> dirls1;
+  for( auto itr = list->Begin(); itr != list->End(); ++itr )
+  {
+    DirectoryList::ListEntry *entry = *itr;
+    dirls1.insert( entry->GetName() );
+  }
+
   delete list;
+  list = 0;
+
+  //----------------------------------------------------------------------------
+  // Now do a chunked query
+  //----------------------------------------------------------------------------
+  std::set<std::string> dirls2;
+
+  LocationInfo *info = 0;
+  CPPUNIT_ASSERT_XRDST( fs.DeepLocate( lsPath, OpenFlags::PrefName, info ) );
+  CPPUNIT_ASSERT( info );
+
+  for( auto itr = info->Begin(); itr != info->End(); ++itr )
+  {
+    DirListChunkedHandler handler( dirls2 );
+    FileSystem fs1( std::string( itr->GetAddress() ) );
+    CPPUNIT_ASSERT_XRDST( fs1.DirList( lsPath, DirListFlags::Stat | DirListFlags::Chunked, &handler ) );
+    handler.Wait();
+  }
+  delete info;
+  info = 0;
+
+  CPPUNIT_ASSERT( dirls1 == dirls2 );
+
+  //----------------------------------------------------------------------------
+  // Now list an empty directory
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( fs.MkDir( "/data/empty", MkDirFlags::None, Access::None ) );
+  CPPUNIT_ASSERT_XRDST( fs.DeepLocate( "/data/empty", OpenFlags::PrefName, info ) );
+  CPPUNIT_ASSERT( info->GetSize() );
+  FileSystem fs3( info->Begin()->GetAddress() );
+  CPPUNIT_ASSERT_XRDST( fs3.DirList( "/data/empty", DirListFlags::Stat, list ) );
+  CPPUNIT_ASSERT( list );
+  CPPUNIT_ASSERT( list->GetSize() == 0 );
+  CPPUNIT_ASSERT_XRDST( fs.RmDir( "/data/empty" ) );
+
+  delete list;
+  list = 0;
+  delete info;
+  info = 0;
 }
 
 
@@ -490,3 +631,122 @@ void FileSystemTest::PrepareTest()
   CPPUNIT_ASSERT( id->GetSize() );
   delete id;
 }
+
+//------------------------------------------------------------------------------
+// Extended attributes test
+//------------------------------------------------------------------------------
+void FileSystemTest::XAttrTest()
+{
+  using namespace XrdCl;
+
+  //----------------------------------------------------------------------------
+  // Get the environment variables
+  //----------------------------------------------------------------------------
+  Env *testEnv = TestEnv::GetEnv();
+
+  std::string address;
+  std::string remoteFile;
+
+  CPPUNIT_ASSERT( testEnv->GetString( "DiskServerURL", address ) );
+  CPPUNIT_ASSERT( testEnv->GetString( "RemoteFile",    remoteFile ) );
+
+  URL url( address );
+  CPPUNIT_ASSERT( url.IsValid() );
+
+  FileSystem fs( url );
+
+  std::map<std::string, std::string> attributes
+  {
+      std::make_pair( "version",  "v1.2.3-45" ),
+      std::make_pair( "checksum", "2ccc0e85556a6cd193dd8d2b40aab50c" ),
+      std::make_pair( "index",    "4" )
+  };
+
+  //----------------------------------------------------------------------------
+  // Test SetXAttr
+  //----------------------------------------------------------------------------
+  std::vector<xattr_t> attrs;
+  auto itr1 = attributes.begin();
+  for( ; itr1 != attributes.end() ; ++itr1 )
+    attrs.push_back( std::make_tuple( itr1->first, itr1->second ) );
+
+  std::vector<XAttrStatus> result1;
+  CPPUNIT_ASSERT_XRDST( fs.SetXAttr( remoteFile, attrs, result1 ) );
+
+  auto itr2 = result1.begin();
+  for( ; itr2 != result1.end() ; ++itr2 )
+    CPPUNIT_ASSERT_XRDST( itr2->status );
+  result1.clear();
+
+  //----------------------------------------------------------------------------
+  // Test GetXAttr
+  //----------------------------------------------------------------------------
+  std::vector<std::string> names;
+  itr1 = attributes.begin();
+  for( ; itr1 != attributes.end() ; ++itr1 )
+    names.push_back( itr1->first );
+
+  std::vector<XAttr> result2;
+  CPPUNIT_ASSERT_XRDST( fs.GetXAttr( remoteFile, names, result2 ) );
+
+  auto itr3 = result2.begin();
+  for( ; itr3 != result2.end() ; ++itr3 )
+  {
+    CPPUNIT_ASSERT_XRDST( itr3->status );
+    auto match = attributes.find( itr3->name );
+    CPPUNIT_ASSERT( match != attributes.end() );
+    CPPUNIT_ASSERT( match->second == itr3->value );
+  }
+  result2.clear();
+
+  //----------------------------------------------------------------------------
+  // Test ListXAttr
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( fs.ListXAttr( remoteFile, result2 ) );
+
+  itr3 = result2.begin();
+  for( ; itr3 != result2.end() ; ++itr3 )
+  {
+    CPPUNIT_ASSERT_XRDST( itr3->status );
+    auto match = attributes.find( itr3->name );
+    CPPUNIT_ASSERT( match != attributes.end() );
+    CPPUNIT_ASSERT( match->second == itr3->value );
+  }
+
+  result2.clear();
+
+  //----------------------------------------------------------------------------
+  // Test DelXAttr
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( fs.DelXAttr( remoteFile, names, result1 ) );
+
+  itr2 = result1.begin();
+  for( ; itr2 != result1.end() ; ++itr2 )
+    CPPUNIT_ASSERT_XRDST( itr2->status );
+
+  result1.clear();
+}
+
+//------------------------------------------------------------------------------
+// Plug-in test
+//------------------------------------------------------------------------------
+void FileSystemTest::PlugInTest()
+{
+  XrdCl::PlugInFactory *f = new IdentityFactory;
+  XrdCl::DefaultEnv::GetPlugInManager()->RegisterDefaultFactory(f);
+  LocateTest();
+  MvTest();
+  ServerQueryTest();
+  TruncateRmTest();
+  MkdirRmdirTest();
+  ChmodTest();
+  PingTest();
+  StatTest();
+  StatVFSTest();
+  ProtocolTest();
+  DeepLocateTest();
+  DirListTest();
+  SendInfoTest();
+  PrepareTest();
+  XrdCl::DefaultEnv::GetPlugInManager()->RegisterDefaultFactory(0);
+}
