@@ -33,6 +33,7 @@
 #include "XrdCl/XrdClXRootDMsgHandler.hh"
 #include "XrdCl/XrdClCopyProcess.hh"
 #include "XrdCl/XrdClZipArchiveReader.hh"
+#include "XrdCl/XrdClConstants.hh"
 
 using namespace XrdClTests;
 
@@ -48,7 +49,9 @@ class FileTest: public CppUnit::TestCase
       CPPUNIT_TEST( WriteTest );
       CPPUNIT_TEST( WriteVTest );
       CPPUNIT_TEST( VectorReadTest );
+      CPPUNIT_TEST( VectorWriteTest );
       CPPUNIT_TEST( VirtualRedirectorTest );
+      CPPUNIT_TEST( XAttrTest );
       CPPUNIT_TEST( PlugInTest );
     CPPUNIT_TEST_SUITE_END();
     void RedirectReturnTest();
@@ -56,7 +59,9 @@ class FileTest: public CppUnit::TestCase
     void WriteTest();
     void WriteVTest();
     void VectorReadTest();
+    void VectorWriteTest();
     void VirtualRedirectorTest();
+    void XAttrTest();
     void PlugInTest();
 };
 
@@ -86,18 +91,6 @@ void FileTest::RedirectReturnTest()
   std::string path = dataPath + "/cb4aacf1-6f28-42f2-b68a-90a73460f424.dat";
   std::string fileUrl = address + "/" + path;
 
-  //----------------------------------------------------------------------------
-  // Get the SID manager
-  //----------------------------------------------------------------------------
-  PostMaster *postMaster = DefaultEnv::GetPostMaster();
-  AnyObject   sidMgrObj;
-  SIDManager *sidMgr    = 0;
-  Status      st;
-  st = postMaster->QueryTransport( url, XRootDQuery::SIDManager, sidMgrObj );
-
-  CPPUNIT_ASSERT( st.IsOK() );
-  sidMgrObj.Get( sidMgr );
-
   //----------------------------------------------------------------------------
   // Build the open request
   //----------------------------------------------------------------------------
@@ -202,11 +195,12 @@ void FileTest::ReadTest()
   CPPUNIT_ASSERT_XRDST( f.Close() );
 
   //----------------------------------------------------------------------------
-  // Read ZIP archive test
+  // Read ZIP archive test (uncompressed)
   //----------------------------------------------------------------------------
   std::string archiveUrl = address + "/" + dataPath + "/data.zip";
 
-  ZipArchiveReader zip;
+  File zipsrc;
+  ZipArchiveReader zip( zipsrc );
   CPPUNIT_ASSERT_XRDST( zip.Open( archiveUrl ) );
 
   //----------------------------------------------------------------------------
@@ -328,6 +322,7 @@ void FileTest::WriteTest()
   delete [] buffer2;
   delete [] buffer3;
   delete [] buffer4;
+  delete response;
   delete stat;
 }
 
@@ -482,6 +477,134 @@ void FileTest::VectorReadTest()
   delete [] buffer2;
 }
 
+void gen_random_str(char *s, const int len)
+{
+    static const char alphanum[] =
+        "0123456789"
+        "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+        "abcdefghijklmnopqrstuvwxyz";
+
+    for (int i = 0; i < len - 1; ++i)
+    {
+        s[i] = alphanum[rand() % (sizeof(alphanum) - 1)];
+    }
+
+    s[len - 1] = 0;
+}
+
+//------------------------------------------------------------------------------
+// Vector write test
+//------------------------------------------------------------------------------
+void FileTest::VectorWriteTest()
+{
+  using namespace XrdCl;
+
+  //----------------------------------------------------------------------------
+  // Initialize
+  //----------------------------------------------------------------------------
+  Env *testEnv = TestEnv::GetEnv();
+
+  std::string address;
+  std::string dataPath;
+
+  CPPUNIT_ASSERT( testEnv->GetString( "MainServerURL", address ) );
+  CPPUNIT_ASSERT( testEnv->GetString( "DataPath", dataPath ) );
+
+  URL url( address );
+  CPPUNIT_ASSERT( url.IsValid() );
+
+  std::string filePath = dataPath + "/a048e67f-4397-4bb8-85eb-8d7e40d90763.dat";
+  std::string fileUrl = address + "/";
+  fileUrl += filePath;
+
+  //----------------------------------------------------------------------------
+  // Build a random chunk list for vector write
+  //----------------------------------------------------------------------------
+
+  const uint32_t MB = 1024*1024;
+  const uint32_t GB = 1000*MB; // maybe that's not 100% precise but that's
+                               // what we have in our testbed
+
+  time_t seed = time( 0 );
+  srand( seed );
+  DefaultEnv::GetLog()->Info( UtilityMsg,
+      "Carrying out the VectorWrite test with seed: %d", seed );
+
+  // figure out how many chunks are we going to write/read
+  size_t nbChunks = rand() % 100 + 1;
+
+  XrdCl::ChunkList chunks;
+  size_t   min_offset = 0;
+  uint32_t expectedCrc32 = 0;
+  size_t   totalSize = 0;
+
+  for( size_t i = 0; i < nbChunks; ++i )
+  {
+    // figure out the offset
+    size_t offset = min_offset + rand() % ( GB - min_offset + 1 );
+
+    // figure out the size
+    size_t size = MB + rand() % ( MB + 1 );
+    if( offset + size >= GB )
+      size = GB - offset;
+
+    // generate random string of given size
+    char *buffer = new char[size];
+    gen_random_str( buffer, size );
+
+    // calculate expected checksum
+    expectedCrc32 = Utils::UpdateCRC32( expectedCrc32, buffer, size );
+    totalSize += size;
+    chunks.push_back( XrdCl::ChunkInfo( offset, size, buffer ) );
+
+    min_offset = offset + size;
+    if( min_offset >= GB )
+      break;
+  }
+
+  //----------------------------------------------------------------------------
+  // Open the file
+  //----------------------------------------------------------------------------
+  File f;
+  CPPUNIT_ASSERT_XRDST( f.Open( fileUrl, OpenFlags::Update ) );
+
+  //----------------------------------------------------------------------------
+  // First do a VectorRead so we can revert to the original state
+  //----------------------------------------------------------------------------
+  char *buffer1 = new char[totalSize];
+  VectorReadInfo *info1 = 0;
+  CPPUNIT_ASSERT_XRDST( f.VectorRead( chunks, buffer1, info1 ) );
+
+  //----------------------------------------------------------------------------
+  // Then do the VectorWrite
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( f.VectorWrite( chunks ) );
+
+  //----------------------------------------------------------------------------
+  // Now do a vector read and verify that the checksum is the same
+  //----------------------------------------------------------------------------
+  char *buffer2 = new char[totalSize];
+  VectorReadInfo *info2 = 0;
+  CPPUNIT_ASSERT_XRDST( f.VectorRead( chunks, buffer2, info2 ) );
+
+  CPPUNIT_ASSERT( info2->GetSize() == totalSize );
+  uint32_t crc32 = Utils::ComputeCRC32( buffer2, totalSize );
+  CPPUNIT_ASSERT( crc32 == expectedCrc32 );
+
+  //----------------------------------------------------------------------------
+  // And finally revert to the original state
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( f.VectorWrite( info1->GetChunks() ) );
+  CPPUNIT_ASSERT_XRDST( f.Close() );
+
+  delete info1;
+  delete info2;
+  delete [] buffer1;
+  delete [] buffer2;
+  for( auto itr = chunks.begin(); itr != chunks.end(); ++itr )
+    delete[] (char*)itr->buffer;
+}
+
 void FileTest::VirtualRedirectorTest()
 {
   using namespace XrdCl;
@@ -517,7 +640,8 @@ void FileTest::VirtualRedirectorTest()
   //----------------------------------------------------------------------------
   CPPUNIT_ASSERT_XRDST( f1.Open( mlUrl1, OpenFlags::Read ) );
   CPPUNIT_ASSERT( f1.GetProperty( key, value ) );
-  CPPUNIT_ASSERT( value == fileUrl );
+  URL lastUrl( value );
+  CPPUNIT_ASSERT( lastUrl.GetLocation() == fileUrl );
   CPPUNIT_ASSERT_XRDST( f1.Close() );
 
   //----------------------------------------------------------------------------
@@ -526,7 +650,8 @@ void FileTest::VirtualRedirectorTest()
   //----------------------------------------------------------------------------
   CPPUNIT_ASSERT_XRDST( f2.Open( mlUrl2, OpenFlags::Read ) );
   CPPUNIT_ASSERT( f2.GetProperty( key, value ) );
-  CPPUNIT_ASSERT( value == fileUrl );
+  URL lastUrl2( value );
+  CPPUNIT_ASSERT( lastUrl2.GetLocation() == fileUrl );
   CPPUNIT_ASSERT_XRDST( f2.Close() );
 
   //----------------------------------------------------------------------------
@@ -544,7 +669,8 @@ void FileTest::VirtualRedirectorTest()
 
   CPPUNIT_ASSERT_XRDST( f4.Open( mlUrl4, OpenFlags::Read ) );
   CPPUNIT_ASSERT( f4.GetProperty( key, value ) );
-  CPPUNIT_ASSERT( value == replica1 );
+  URL lastUrl3( value );
+  CPPUNIT_ASSERT( lastUrl3.GetLocation() == replica1 );
   CPPUNIT_ASSERT_XRDST( f4.Close() );
   //----------------------------------------------------------------------------
   // Delete the replica that has been selected by the virtual redirector
@@ -556,7 +682,8 @@ void FileTest::VirtualRedirectorTest()
   //----------------------------------------------------------------------------
   CPPUNIT_ASSERT_XRDST( f4.Open( mlUrl4, OpenFlags::Read ) );
   CPPUNIT_ASSERT( f4.GetProperty( key, value ) );
-  CPPUNIT_ASSERT( value == replica2 );
+  URL lastUrl4( value );
+  CPPUNIT_ASSERT( lastUrl4.GetLocation() == replica2 );
   CPPUNIT_ASSERT_XRDST( f4.Close() );
   //----------------------------------------------------------------------------
   // Recreate the deleted file
@@ -570,6 +697,99 @@ void FileTest::VirtualRedirectorTest()
   CPPUNIT_ASSERT_XRDST( process.Run(0) );
 }
 
+void FileTest::XAttrTest()
+{
+  using namespace XrdCl;
+
+  //----------------------------------------------------------------------------
+  // Get the environment variables
+  //----------------------------------------------------------------------------
+  Env *testEnv = TestEnv::GetEnv();
+
+  std::string address;
+  std::string dataPath;
+
+  CPPUNIT_ASSERT( testEnv->GetString( "DiskServerURL", address ) );
+  CPPUNIT_ASSERT( testEnv->GetString( "DataPath", dataPath ) );
+
+  std::string filePath = dataPath + "/a048e67f-4397-4bb8-85eb-8d7e40d90763.dat";
+  std::string fileUrl = address + "/" + filePath;
+
+  URL url( address );
+  CPPUNIT_ASSERT( url.IsValid() );
+
+
+  File file;
+  CPPUNIT_ASSERT_XRDST( file.Open( fileUrl, OpenFlags::Update ) );
+
+  std::map<std::string, std::string> attributes
+  {
+      std::make_pair( "version",  "v1.2.3-45" ),
+      std::make_pair( "checksum", "2ccc0e85556a6cd193dd8d2b40aab50c" ),
+      std::make_pair( "index",    "4" )
+  };
+
+  //----------------------------------------------------------------------------
+  // Test SetXAttr
+  //----------------------------------------------------------------------------
+  std::vector<xattr_t> attrs;
+  auto itr1 = attributes.begin();
+  for( ; itr1 != attributes.end() ; ++itr1 )
+    attrs.push_back( std::make_tuple( itr1->first, itr1->second ) );
+
+  std::vector<XAttrStatus> result1;
+  CPPUNIT_ASSERT_XRDST( file.SetXAttr( attrs, result1 ) );
+
+  auto itr2 = result1.begin();
+  for( ; itr2 != result1.end() ; ++itr2 )
+    CPPUNIT_ASSERT_XRDST( itr2->status );
+
+  //----------------------------------------------------------------------------
+  // Test GetXAttr
+  //----------------------------------------------------------------------------
+  std::vector<std::string> names;
+  itr1 = attributes.begin();
+  for( ; itr1 != attributes.end() ; ++itr1 )
+    names.push_back( itr1->first );
+
+  std::vector<XAttr> result2;
+  CPPUNIT_ASSERT_XRDST( file.GetXAttr( names, result2 ) );
+
+  auto itr3 = result2.begin();
+  for( ; itr3 != result2.end() ; ++itr3 )
+  {
+    CPPUNIT_ASSERT_XRDST( itr3->status );
+    auto match = attributes.find( itr3->name );
+    CPPUNIT_ASSERT( match != attributes.end() );
+    CPPUNIT_ASSERT( match->second == itr3->value );
+  }
+
+  //----------------------------------------------------------------------------
+  // Test ListXAttr
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( file.ListXAttr( result2 ) );
+
+  itr3 = result2.begin();
+  for( ; itr3 != result2.end() ; ++itr3 )
+  {
+    CPPUNIT_ASSERT_XRDST( itr3->status );
+    auto match = attributes.find( itr3->name );
+    CPPUNIT_ASSERT( match != attributes.end() );
+    CPPUNIT_ASSERT( match->second == itr3->value );
+  }
+
+  //----------------------------------------------------------------------------
+  // Test DelXAttr
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT_XRDST( file.DelXAttr( names, result1 ) );
+
+  itr2 = result1.begin();
+  for( ; itr2 != result1.end() ; ++itr2 )
+    CPPUNIT_ASSERT_XRDST( itr2->status );
+
+  CPPUNIT_ASSERT_XRDST( file.Close() );
+}
+
 //------------------------------------------------------------------------------
 // Plug-in test
 //------------------------------------------------------------------------------
